/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#if defined(macintosh)
#include <types.h>
#elif defined(WIN32)
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#endif
#include "olc.h"
#include "merc.h"
#include "recycle.h"


bool edit_desc( CHAR_DATA *ch, char **description, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    bool capital;

    if ( !strcmp(argument, "NULL") )
    {
	free_string( *description );
	*description = str_dup( "" );

	send_to_char( "OK.\n\r", ch );
	return TRUE;
    }

    if ( argument[0] == '-' )
    {
	bool found = FALSE;
	int len;

	if ( *description[0] == '\0' )
	{
	    send_to_char( "No lines left to remove.\n\r", ch );
	    return FALSE;
	}

	argument = cased_one_argument( argument, arg );
	if ( argument[0] != '\0' && is_number(argument) )
	{
	    int line, current;
	    
	    line = atoi( argument ) - 1;
	    
	    if ( line < 0 )
	    {
		send_to_char( "Line numbers start at 1.\n\r", ch );
		return FALSE;
	    }

	    for ( current = 0; (*description)[current] != '\0'; current++ )
	    {
		if ( line == 0 )
		{
		    int skip;

		    strncpy( buf, *description, current );
		    buf[ current ] = '\0';

		    for ( skip = current + 1; ; skip++ )
		    {
			if ( (*description)[skip] == '\r' )
			{
			    if ( buf[0] != '\0' )
				strcat( buf, "\r" );

			    strcat( buf, &((*description)[skip + 1]) );
			    break;
			}
			else if ( (*description)[skip] == '\0' )
			    break;
		    }

		    free_string( *description );
		    *description = str_dup( buf );

		    send_to_char( "OK.\n\r", ch );
		    return TRUE;
		}
		
		if ( (*description)[current] == '\r' )
		    line--;
	    }
	    
	    send_to_char( "No such line to remove.\n\r", ch );
	    return FALSE;
	}

	strcpy( buf, *description );

	for ( len = strlen(buf); len > 0; len-- )
	{
	    if (buf[len] == '\r')
	    {
		if ( !found )  /* back it up */
		{
		    if (len > 0)
			len--;
		    found = TRUE;
		}
		else /* found the second one */
		{
		    buf[len + 1] = '\0';
		    free_string( *description );
		    *description = str_dup( buf );
		    return TRUE;
		}
	    }
	}
	    
	free_string( *description );
	*description = str_dup( "" );
	send_to_char( "Description cleared.\n\r", ch );
	return TRUE;
    }

    capital  = isupper( argument[0] );
    argument = cased_one_argument( argument, arg );
    if ( !str_cmp(arg, "c") )
    {
	int line, current;

	argument = cased_one_argument( argument, arg );

	if ( arg[0] == '\0' || !is_number(arg) )
	{
	    send_to_char( "Change which line?", ch );
	    return FALSE;
	}
	
	line = atoi( arg ) - 1;
	
	if ( line < 0 )
	{
	    send_to_char( "Line numbers start at 1.\n\r", ch );
	    return FALSE;
	}

	for ( current = 0; (*description)[current] != '\0'; current++ )
	{
	    if ( line == 0 )
	    {
		int skip;

		strncpy( buf, *description, current );
		buf[ current ] = '\0';

		strcat( buf, argument );
		strcat( buf, "\n" );

		for ( skip = current + 1; ; skip++ )
		{
		    if ( (*description)[skip] == '\r' )
		    {
			if ( buf[0] != '\0' )
			    strcat( buf, "\r" );

			strcat( buf, &((*description)[skip + 1]) );
			break;
		    }
		    else if ( (*description)[skip] == '\0' )
			break;
		}

		free_string( *description );
		*description = str_dup( buf );

		send_to_char( "OK.\n\r", ch );
		return TRUE;
	    }
		
	    if ( (*description)[current] == '\r' )
		line--;
	}
	    
	send_to_char( "No such line to change.\n\r", ch );
	return FALSE;
    }
    else if ( !str_cmp(arg, "i") )
    {
	int line, current;

	argument = cased_one_argument( argument, arg );

	if ( arg[0] == '\0' || !is_number(arg) )
	{
	    send_to_char( "Insert at which line?", ch );
	    return FALSE;
	}
	
	line = atoi( arg ) - 1;
	
	if ( line < 0 )
	{
	    send_to_char( "Line numbers start at 1.\n\r", ch );
	    return FALSE;
	}

	for ( current = 0; (*description)[current] != '\0'; current++ )
	{
	    if ( line == 0 )
	    {
		strncpy( buf, *description, current );
		buf[ current ] = '\0';

		strcat( buf, argument );
		strcat( buf, "\n\r" );
		strcat( buf, &((*description)[current]) );

		free_string( *description );
		*description = str_dup( buf );

		send_to_char( "OK.\n\r", ch );
		return TRUE;
	    }
		
	    if ( (*description)[current] == '\r' )
		line--;
	}

	send_to_char( "There are not that many lines.\n\r", ch );
	return FALSE;
    }

    buf[0] = '\0';
    if ( arg[0] == '+' )
    {
	strcpy( buf, *description );
	while ( isspace(*argument) )
	    argument++;
    }
    else
    {
	sprintf( buf, "%s ", arg );
	if ( capital )
	    buf[0] = UPPER( buf[0] );
    }
	
    if ( strlen(buf) + strlen(argument) + 2 >= MAX_STRING_LENGTH )
    {
	send_to_char( "Description too long.\n\r", ch );
	return FALSE;
    }
	
    strcat( buf, argument );
    strcat( buf, "\n\r" );
    free_string( *description );
    *description = str_dup( buf );
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


EXTRA_DESCR_DATA *find_extra( EXTRA_DESCR_DATA *head, char *keywords )
{
    char keyword[MAX_INPUT_LENGTH];
    char *current;
    bool match;
    
    while ( head )
    {
	current	= keywords;
	match	= TRUE;

	for ( ; ; )
	{
	    current = one_argument( current, keyword );
	    if ( keyword[0] == '\0' )
		break;

	    if ( !is_exact_name(keyword, head->keyword) )
	    {
		match = FALSE;
		break;
	    }
	}
	
	if ( match )
	    return head;
	
	head = head->next;
    }
    
    return NULL;
}


void remove_extra( EXTRA_DESCR_DATA **head, EXTRA_DESCR_DATA *remove )
{
    EXTRA_DESCR_DATA *current, *previous;
    
    current = previous = *head;
    while ( current )
    {
	if ( current == remove )
	{
	    if ( previous == current )
		*head = current->next;
	    else
		previous->next = current->next;

	    free_extra_descr( current );
	    return;
	}
	
	previous	= current;
	current		= current->next;
    }
}
