/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#if defined(macintosh)
#include <types.h>
#elif defined(WIN32)
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#endif
#include "merc.h"
#include "tables.h"
#include "recycle.h"
#include "lookup.h"


/*
 * Global function prototypes.
 */

extern int flag_lookup  args( ( const char *name,
                                const struct flag_type *flag_table ) );


void do_trap( CHAR_DATA *ch, char *argument )
{
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    char arg3[MAX_INPUT_LENGTH];
    ROOM_INDEX_DATA *pRoomIndex;
    OBJ_INDEX_DATA *pObjIndex;
    RESET_DATA *pReset;
    AREA_DATA *pArea;
    OBJ_DATA *trap;
    bool room_trap;
    int vnum;

    pArea = ch->in_room->area;

    if ( !is_creator(ch, pArea) )
    {
	send_to_char( "You are not a creator in this area.\n\r", ch );
	return;
    }

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );
    argument = one_argument( argument, arg3 );
    
    if ( arg1[0] == '\0'
    ||   (str_prefix(arg1, "room")
    &&    str_prefix(arg1, "object"))
    ||   !is_number(arg2)
    ||   arg3[0] == '\0' )
    {
	send_to_char( "Syntax: trap <object|room> <vnum> charges <n>\n\r"
	              "        trap <object|room> <vnum> type <trap type>\n\r"
	              "        trap <object|room> <vnum> flags <flags>\n\r"
	              "        trap <object|room> <vnum> dest <room vnum>\n\r"
	              "        trap <object|room> <vnum> damage <damage>\n\r"
	              "        trap <object|room> <vnum> remove\n\r", ch );
	return;
    }
    
    room_trap = !str_prefix( arg1, "room" );
    
    vnum = atoi( arg2 );
    pRoomIndex = NULL;
    if ( room_trap )
    {
	if ( (pRoomIndex = get_room_index(vnum)) == NULL )
	{
	    send_to_char( "No such room.\n\r", ch );
	    return;
	}

	pObjIndex = NULL;
    }
    else if ( (pObjIndex = get_obj_index(vnum)) == NULL )
    {
	send_to_char( "No such object.\n\r", ch );
	return;
    }

    for ( pReset = pArea->reset_first; pReset != NULL; pReset = pReset->next )
    {
	if ( pReset->command == 'T'
	&&   ((IS_SET(pReset->arg3, TRAP_ROOM) && room_trap)
	||    (IS_SET(pReset->arg3, TRAP_OBJ) && !room_trap))
	&&   pReset->arg5 == vnum )
	    break;
    }

    if ( !str_prefix(arg3, "remove") )
    {
	if ( room_trap )
	{
	    trap = get_room_trap( pRoomIndex );
	    if ( trap )
		extract_obj( trap );
	}
	else
	{
	    OBJ_DATA *current;
	    
	    for ( current = pObjIndex->instances;
	          current != NULL;
	          current = current->next_instance )
	    {
		trap = get_trap( current );
		if ( trap )
		{
		    extract_obj( trap );
		    break;
		}
	    }
	}

	remove_reset( pArea, pReset );
	SET_BIT( pArea->flags, AREA_DIRTY );

	send_to_char( "OK.\n\r", ch );
	return;
    }
    else
	trap = NULL;

    if ( pReset == NULL )
    {
	RESET_DATA *pResetAfter = NULL;

	if ( room_trap )
	    pResetAfter = find_reset( pArea, 'O', vnum, ch->in_room->vnum );

	pReset			= create_reset( pArea, pResetAfter );
	pReset->command		= 'T';
	pReset->arg1		= -1;
	pReset->arg2		= room_trap
				? TRAP_TYPE_ARROW
				: TRAP_TYPE_POISON_NEEDLE;
	pReset->arg3		= room_trap ? TRAP_ROOM : TRAP_OBJ;
	pReset->arg4		= 0;
	pReset->arg5		= vnum;
    }
    
    if ( room_trap )
    {
	trap = get_room_trap( pRoomIndex );
	if ( trap == NULL )
	{
	    trap = create_object( get_obj_index(OBJ_VNUM_TRAP), 0, FALSE );
	    trap->value[0]	= -1;
	    trap->value[1]	= TRAP_TYPE_ARROW;
	    trap->value[2]	= TRAP_ROOM;
	    trap->value[3]	= 0;
	    
	    obj_to_room( trap, pRoomIndex );
	}
    }
    else
    {
	OBJ_DATA *current;

	for ( current = pObjIndex->instances;
	      current != NULL;
	      current = current->next_instance )
	{
	    trap = get_trap( current );
	    if ( trap )
		break;
	}

	if ( trap == NULL && pObjIndex->instances != NULL )
	{
	    trap = create_object( get_obj_index(OBJ_VNUM_TRAP), 0, FALSE );
	    trap->value[0]	= -1;
	    trap->value[1]	= TRAP_TYPE_POISON_NEEDLE;
	    trap->value[2]	= TRAP_OBJ;
	    trap->value[3]	= 0;

	    obj_to_obj( trap, pObjIndex->instances );
	}
    }

    if ( !str_prefix(arg3, "charges") )
    {
	int charges;
	
	if ( !is_number(argument) )
	{
	    send_to_char( "The number of charges must be numeric.\n\r", ch );
	    return;
	}
	
	charges = atoi( argument );
	if ( charges < -1 || charges == 0 )
	{
	    send_to_char( "The number of charges must be positive (or -1 for unlimited).\n\r", ch );
	    return;
	}
	
	pReset->arg1		= charges;
	if ( trap != NULL )
	    trap->value[0]	= charges;
    }
    else if ( !str_prefix(arg3, "type") )
    {
	int trap_type;
	bool old_dam;

	if ( !str_cmp(argument, "list") )
	{
	    char buf[MAX_STRING_LENGTH];
	    int i, col;
	    
	    col = 0;
	    for ( i = 0; trap_table[i].name != NULL; i++ )
	    {
		sprintf( buf, "%-30s     ", trap_table[i].name );
		send_to_char( buf, ch );
		if ( ++col % 2 == 0 )
		    send_to_char( "\n\r", ch );
	    }
	    
	    if ( col % 2 != 0 )
		send_to_char( "\n\r", ch );
		
	    return;
	}

	old_dam = can_trap_damage( pReset->arg2 );

	trap_type = trap_lookup( argument );
	if ( trap_type == -1 )
	{
	    send_to_char( "Unknown trap type (try using type 'list').\n\r", ch );
	    return;
	}
	
	pReset->arg2		= trap_type;
	if ( trap != NULL )
	    trap->value[1]	= trap_type;
	
	if ( old_dam != can_trap_damage(trap_type) )
	{
	    if ( IS_SET(pReset->arg3, TRAP_ROOM) )
	    {
		if ( get_room_index(pReset->arg4) == NULL )
		{
		    pReset->arg4 = ROOM_VNUM_ALTAR;
		    send_to_char( "Trap destination invalid. Changed to altar.\n\r", ch );
		}
	    }
	}
    }
    else if ( !str_prefix(arg3, "flags") )
    {
	char buf[MAX_STRING_LENGTH];
	long flags;
	int type;
	int pos;
	
	type = argument[0];
	if ( type == '=' || type == '-' || type == '+' )
	    argument++;
	    
	flags = 0;
	for ( ; ; )
	{
	    argument = one_argument( argument, arg1 );
	    if ( arg1[0] == '\0' )
		break;
	
	    pos = flag_lookup( arg1, trap_flags );
	    if ( pos == 0 )
	    {
		sprintf( buf, "Bad flag %s.\n\r", arg1 );
		send_to_char( buf, ch );
		return;
	    }
	    
	    SET_BIT( flags, pos );
	}
	
	for ( pos = 0; trap_flags[pos].name != NULL; pos++ )
	{
	    if ( !trap_flags[pos].settable 
	    &&   IS_SET(flags, trap_flags[pos].bit) )
	    {
		sprintf( buf, "Flag cannot be modified: %s.\n\r", 
		    trap_flags[pos].name );
		send_to_char( buf, ch );
		return;
	    }
	}
	
	switch ( type )
	{
	case '+':
	    SET_BIT( pReset->arg3, flags );
	    break;
	case '-':
	    REMOVE_BIT( pReset->arg3, flags );
	    break;
	case '=':
	    pReset->arg3 = flags;
	    break;
	default:
	    pReset->arg3 = pReset->arg3 ^ flags;
	}
	
	if ( trap != NULL )
	    trap->value[2] = pReset->arg3;
    }
    else if ( !str_prefix(arg3, "damage") )
    {
	int dam;

	if ( !is_number(argument) )
	{
	    send_to_char( "The damage value must be numeric.\n\r", ch );
	    return;
	}

	dam = atoi( argument );
	if ( dam < 0 )
	{
	    send_to_char( "The damage amount cannot be negative.\n\r", ch );
	    return;
	}

	if ( !can_trap_damage(pReset->arg2) )
	{
	    send_to_char( "That trap type cannot inflict damage.\n\r", ch );
	    return;
	}

	pReset->arg4		= dam;
	if ( trap != NULL )
	    trap->value[3]	= dam;
    }
    else if ( !str_prefix(arg3, "dest") )
    {
	int dest;

	if ( !is_number(argument) )
	{
	    send_to_char( "Invalid destination vnum.\n\r", ch );
	    return;
	}

	dest = atoi( argument );
	if ( get_room_index(dest) == NULL )
	{
	    send_to_char( "Invalid trap destination room.\n\r", ch );
	    return;
	}

	if ( can_trap_damage(pReset->arg2) )
	{
	    send_to_char( "This is a damaging trap, not a transporting one.\n\r", ch );
	    return;
	}

	pReset->arg4		= dest;
	if ( trap != NULL )
	    trap->value[3]	= dest;
    }
    else
    {
	send_to_char( "Invalid trap option.\n\r", ch );
	return;
    }
    
    SET_BIT( pArea->flags, AREA_DIRTY );
    
    send_to_char( "OK.\n\r", ch );
    return;
}

