/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#include <malloc.h>
#endif
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "merc.h"
#include "color.h"
#include "recycle.h"
#include "lookup.h"
#include "tables.h"
 
#if !defined(macintosh)
extern  int     _filbuf         args( (FILE *) );
#endif


int rename(const char *oldfname, const char *newfname);

char *print_flags(int flag)
{
    int count, pos = 0;
    static char buf[52];


    for (count = 0; count < 32;  count++)
    {
        if (IS_SET(flag,1<<count))
        {
            if (count < 26)
                buf[pos] = 'A' + count;
            else
                buf[pos] = 'a' + (count - 26);
            pos++;
        }
    }

    if (pos == 0)
    {
        buf[pos] = '0';
        pos++;
    }

    buf[pos] = '\0';

    return buf;
}


/*
 * Array of containers read for proper re-nesting of objects.
 */
#define MAX_NEST	100
static	OBJ_DATA *	rgObjNest	[MAX_NEST];



/*
 * Local functions.
 */
#define AD AFFECT_DATA
void	fwrite_char	args( ( CHAR_DATA *ch,  FILE *fp ) );
int 	fwrite_obj	args( ( CHAR_DATA *ch,  OBJ_DATA  *obj,
			        FILE *fp, int iNest, bool in_room, 
			        bool corpses_sink, int maximum ) );
void	fwrite_pet	args( ( CHAR_DATA *pet, FILE *fp) );
void    fwrite_cust     args( ( current_cust, fp ) );
void	fwrite_affects	args( ( AFFECT_DATA *paf, FILE *fp ) );
void	fread_char	args( ( CHAR_DATA *ch,  FILE *fp ) );
void    fread_pet	args( ( CHAR_DATA *ch,  FILE *fp ) );
void	fread_obj	args( ( CHAR_DATA *ch,  FILE *fp, bool load_rooms ) );
AD *	fread_affect	args( ( FILE *fp, int version ) );
#undef AD


void save_corpse_by_type( FILE *fp, CHAR_DATA *ch, bool extract, int vnum )
{
    OBJ_DATA *obj, *obj_next;

    for ( obj = get_obj_index(vnum)->instances; 
	  obj != NULL; 
	  obj = obj_next )
    {
	obj_next = obj->next_instance;
	
	if ( obj->item_type == ITEM_CORPSE_PC
	&&   !str_cmp(obj->owner, ch->name) )
        {
            if ( extract )
            {
                if ( obj->in_obj )
                {
                    OBJ_DATA *container = obj->in_obj;
                    while ( container->in_obj )
                        container = container->in_obj;
                    if ( (container->carried_by
                    &&    container->carried_by != ch)
                    ||   container->in_room )
                    {
                        obj_from_obj( obj );
                        if ( container->in_room )
                            obj_to_room( obj, container->in_room );
                        else if ( container->carried_by )
                        {
                            act( "$p drops to the ground.", 
                                 container->carried_by, obj, NULL, TO_ALL );
                            if ( container->carried_by->in_room )
                                obj_to_room( obj, container->carried_by->in_room );
                            else
                                obj_to_room( obj, get_random_room(ch) );
                        }
                    }
                }
                else if ( obj->carried_by && obj->carried_by != ch )
                {
                    ROOM_INDEX_DATA *location = obj->carried_by->in_room;
                    CHAR_DATA *holder = obj->carried_by;
                    obj_from_char( obj );
                    if ( !location )
                        location = get_random_room( ch );
                    obj_to_room( obj, location );
                    act( "$p drops to the ground.", holder, obj, NULL, TO_ALL );
                }
            }

	    if ( obj->in_room )
	    {
	        fwrite_obj( ch, obj, fp, 0, TRUE, extract, -1 );
                if ( extract )
                    extract_obj( obj );
            }
	}
    }
}


void save_owned_object( FILE *fp, CHAR_DATA *ch, bool extract, int vnum )
{
    OBJ_DATA *obj, *obj_next;

    for ( obj = get_obj_index(vnum)->instances; 
	  obj != NULL; 
	  obj = obj_next )
    {
	obj_next = obj->next_instance;
	
	if ( !str_cmp(obj->owner, ch->name) )
	{
            if ( extract )
            {
                if ( obj->in_obj )
                {
                    OBJ_DATA *container = obj->in_obj;
                    while ( container->in_obj )
                        container = container->in_obj;
                    if ( (container->carried_by
                    &&    container->carried_by != ch)
                    ||   container->in_room )
                    {
                        obj_from_obj( obj );
                        if ( container->in_room )
                            obj_to_room( obj, container->in_room );
                        else if ( container->carried_by )
                        {
                            act( "$p drops to the ground.", 
                                 container->carried_by, obj, NULL, TO_ALL );
                            if ( container->carried_by->in_room )
                                obj_to_room( obj, container->carried_by->in_room );
                            else
                                obj_to_room( obj, get_random_room(ch) );
                        }
                    }
                }
                else if ( obj->carried_by && obj->carried_by != ch )
                {
                    ROOM_INDEX_DATA *location = obj->carried_by->in_room;
                    CHAR_DATA *holder = obj->carried_by;
                    obj_from_char( obj );
                    if ( !location )
                        location = get_random_room( ch );
                    obj_to_room( obj, location );
                    act( "$p drops to the ground.", holder, obj, NULL, TO_ALL );
                }
            }

            if ( obj->in_room )
            {
                fwrite_obj( ch, obj, fp, 0, TRUE, extract, -1 );
                if ( extract )
                    extract_obj( obj );
            }
	}
    }
}


void save_room_objects( FILE *fp, CHAR_DATA *ch, bool extract )
{
    ROOM_INDEX_DATA *pRoomIndex = get_room_index( ch->homevnum );
    OBJ_DATA *obj, *obj_next;
    int numItems = MAX_ROOM_SAVE_OBJECTS;

    if ( pRoomIndex == NULL )
        return;

    for ( obj = pRoomIndex->contents; obj != NULL; obj = obj_next )
    {
        obj_next = obj->next_content;
        
        numItems -= fwrite_obj( ch, obj, fp, 0, TRUE, extract, numItems );
        if ( extract )
            extract_obj( obj );
    }

    if ( extract )
        REMOVE_BIT( pRoomIndex->room_flags, ROOM_EQUIPMENT_LOADED );
}


/*
 * Save a character and inventory.
 * Would be cool to save NPC's too for quest purposes,
 *   some of the infrastructure is provided.
 */
void save_char_obj( CHAR_DATA *ch, bool extract )
{
    char strsave[MAX_INPUT_LENGTH];
    FILE *fp;

    if ( IS_NPC(ch) )
	return;

    if ( ch->desc != NULL && ch->desc->original != NULL )
	ch = ch->desc->original;

#if defined(unix)
    /* create god log */
    if (IS_IMMORTAL(ch) || ch->level >= LEVEL_IMMORTAL)
    {
	fclose(fpReserve);
	sprintf(strsave, "%s%s",GOD_DIR, capitalize(ch->name));
	if ((fp = fopen(strsave,"w")) == NULL)
	{
	    bug("Save_char_obj: fopen",0);
	    perror(strsave);
 	}

	fprintf(fp,"Lev %2d Trust %2d  %s%s\n",
	    ch->level, get_trust(ch), ch->name, ch->pcdata->title);
	fclose( fp );
	fpReserve = fopen( NULL_FILE, "r" );
    }
#endif

    fclose( fpReserve );
    sprintf( strsave, "%s%s", PLAYER_DIR, capitalize( ch->name ) );
    if ( ( fp = fopen( TEMP_FILE, "w" ) ) == NULL )
    {
	bug( "Save_char_obj: fopen", 0 );
	perror( strsave );
    }
    else
    {
	fwrite_char( ch, fp );
	
	if ( ch->carrying != NULL )
	    fwrite_obj( ch, ch->carrying, fp, 0, FALSE, extract, -1 );
        
        /* save any owned unique items */
        save_owned_object( fp, ch, extract, OBJ_VNUM_DRUID_STAFF );
        save_owned_object( fp, ch, extract, OBJ_VNUM_SHADOWBLADE );
        save_owned_object( fp, ch, extract, OBJ_VNUM_DEFENDER );

	/* save any corpses */
        save_corpse_by_type( fp, ch, extract, OBJ_VNUM_CORPSE_PC_BASH );
        save_corpse_by_type( fp, ch, extract, OBJ_VNUM_CORPSE_PC_FIRE );
        save_corpse_by_type( fp, ch, extract, OBJ_VNUM_CORPSE_PC_COLD );
        save_corpse_by_type( fp, ch, extract, OBJ_VNUM_CORPSE_PC_LIGHTNING );
        save_corpse_by_type( fp, ch, extract, OBJ_VNUM_CORPSE_PC_ACID );
        save_corpse_by_type( fp, ch, extract, OBJ_VNUM_CORPSE_PC_POISON );
        save_corpse_by_type( fp, ch, extract, OBJ_VNUM_CORPSE_PC_NEGATIVE );
        save_corpse_by_type( fp, ch, extract, OBJ_VNUM_CORPSE_PC_HOLY );
        save_corpse_by_type( fp, ch, extract, OBJ_VNUM_CORPSE_PC_ENERGY );
        save_corpse_by_type( fp, ch, extract, OBJ_VNUM_CORPSE_PC_DISEASE );
        save_corpse_by_type( fp, ch, extract, OBJ_VNUM_CORPSE_PC_DROWNING );
        save_corpse_by_type( fp, ch, extract, OBJ_VNUM_CORPSE_PC_SLAY );
        save_corpse_by_type( fp, ch, extract, OBJ_VNUM_CORPSE_PC );
	
        save_room_objects( fp, ch, extract );

	/* save the pets */
	if ( ch->pet != NULL /*&& ch->pet->in_room == ch->in_room*/ )
	    fwrite_pet( ch->pet, fp );
	fprintf( fp, "#END\n" );
    }
    fclose( fp );
    #if defined(WIN32)
    remove(strsave);
    #endif
    rename(TEMP_FILE,strsave);
    fpReserve = fopen( NULL_FILE, "r" );
    return;
}


/*
 * Write the char.
 */
void fwrite_char( CHAR_DATA *ch, FILE *fp )
{
    int sn, gn, pos;
    int i;

    fprintf( fp, "#%s\n", IS_NPC(ch) ? "MOB" : "PLAYER" );

    fprintf( fp, "Name %s~\n",	ch->name		);
    fprintf( fp, "Id   %ld\n", ch->id			);
    fprintf( fp, "LogO %ld\n",	current_time		);
    fprintf( fp, "Vers %d\n",	8			);
    if (ch->short_descr[0] != '\0')
	fprintf( fp, "ShD  %s~\n",	ch->short_descr );
    if( ch->long_descr[0] != '\0')
	fprintf( fp, "LnD  %s~\n",	ch->long_descr	);
    if (ch->description[0] != '\0')
	fprintf( fp, "Desc %s~\n",	ch->description );
    if (ch->pcdata->who_name[0] != '\0')
	fprintf( fp, "Who  %s~\n",	ch->pcdata->who_name );
    if (ch->prompt != NULL || !str_cmp(ch->prompt,"<%hhp %mm %vmv> "))
	fprintf( fp, "Prom %s~\n",	ch->prompt	);
    if ( ch->pcdata->appearance[0] != '\0' )
	fprintf( fp, "App %s~\n",	ch->pcdata->appearance );
    if ( ch->appearance[0] != '\0' )
	fprintf( fp, "CurApp %s~\n",	ch->appearance	);
    if ( ch->priority )
	fprintf( fp, "Priority %d\n",	ch->priority	);
    fprintf( fp, "Race %s~\n", pc_race_table[ch->race].name );
    if ( ch->polymorphed )
    fprintf( fp, "Poly %d\n", ch->polymorphed->vnum );
    
    if ( ch->clan )
    {
	fprintf( fp, "Clan %s~\n",clan_table[ch->clan].name );
	if ( ch->pcdata->leader )
	    fprintf( fp, "Leader %d\n", ch->pcdata->leader );
    }
    if ( ch->pcdata->petition )
	fprintf( fp, "Petition %s~\n", clan_table[ch->pcdata->petition].name );

    fprintf( fp, "Sex  %d\n",	ch->sex			);
    fprintf( fp, "Cla  %d\n",	ch->class		);
    fprintf( fp, "Renown %d\n", ch->pcdata->renown	);
    fprintf( fp, "Home %d\n",	ch->hometown		);
    fprintf( fp, "Hnum %d\n",	ch->homevnum		);
    if ( ch->pcdata->former_professions[0] != -1 )
    {
	fprintf( fp, "Prof %d", MAX_PROFESSIONS - 1 );
	for ( i = 0; i < MAX_PROFESSIONS - 1; i++ )
	    fprintf( fp, " %d", ch->pcdata->former_professions[i] );
	fprintf( fp, "\n" );
    }
    fprintf( fp, "God  %d\n",   ch->pcdata->religion	);
    fprintf( fp, "Her  %d\n",	ch->pcdata->heresy	);
    fprintf( fp, "Levl %d\n",	ch->level		);
    if ( ch->pcdata->true_level != ch->level )
	fprintf( fp, "TLevl %d\n", ch->pcdata->true_level );
    if (ch->trust != 0)
	fprintf( fp, "Tru  %d\n",	ch->trust	);
    if (ch->pcdata->olc_level != 0)
	fprintf( fp, "OLC %d\n", ch->pcdata->olc_level );
    fprintf( fp, "Plyd %d\n",
	ch->played + (int) (current_time - ch->logon)	);
/*
 *   fprintf( fp, "Not  %ld %ld %ld %ld %ld\n",		
 *	ch->pcdata->last_note,ch->pcdata->last_idea,ch->pcdata->last_penalty,
 *	ch->pcdata->last_news,ch->pcdata->last_changes	);
 */
    fprintf( fp, "Not2 %ld %ld %ld %ld %ld %ld\n",		
	ch->pcdata->last_note,ch->pcdata->last_idea,ch->pcdata->last_penalty,
	ch->pcdata->last_news,ch->pcdata->last_changes,ch->pcdata->last_bug );
    fprintf( fp, "Scro %d\n",	ch->lines		);
    fprintf( fp, "Room %d\n",
	(ch->in_room == get_room_index(ROOM_VNUM_LIMBO)
	 && ch->was_in_room != NULL)
	? ch->was_in_room->vnum
	: ch->in_room == NULL ? ROOM_VNUM_TEMPLE : ch->in_room->vnum );

    fprintf( fp, "HMV  %d %d %d %d %d %d\n",
	ch->hit, ch->max_hit, ch->mana, ch->max_mana, ch->move, ch->max_move );
    if (ch->gold > 0)
	fprintf( fp, "Gold %ld\n",	ch->gold		);
    else
	fprintf( fp, "Gold %d\n", 0			); 
    if (ch->silver > 0)
	fprintf( fp, "Silv %ld\n",ch->silver		);
    else
	fprintf( fp, "Silv %d\n",0			);
    if (ch->pcdata->balance > 0)
	fprintf( fp, "Balance %ld\n", ch->pcdata->balance );
    fprintf( fp, "Exp  %d\n",	ch->exp			);
    if (ch->act != 0)
	fprintf( fp, "Act  %s\n",   print_flags(ch->act));
    if (ch->affected_by != 0)
	fprintf( fp, "AfBy %s\n",   print_flags(ch->affected_by));
    if (ch->affected_by2 != 0)
	fprintf( fp, "AfBy2 %s\n",  print_flags(ch->affected_by2));
    fprintf( fp, "Comm %s\n",	    print_flags(ch->comm));
    if (ch->wiznet)
	fprintf( fp, "Wizn %s\n",   print_flags(ch->wiznet));
    if (ch->invis_level)
	fprintf( fp, "Invi %d\n",	ch->invis_level );
    if (ch->incog_level)
	fprintf(fp,"Inco %d\n",ch->incog_level);
    fprintf( fp, "Pos  %d\n",	
	ch->position == POS_FIGHTING ? POS_STANDING : ch->position );
    if (ch->practice != 0)
	fprintf( fp, "Prac %d\n",	ch->practice	);
    if (ch->train != 0)
	fprintf( fp, "Trai %d\n",	ch->train	);
    if (ch->pcdata->meta_purchases != 0 )
	fprintf( fp, "Meta %d\n",	ch->pcdata->meta_purchases );
    if (ch->saving_throw != 0)
	fprintf( fp, "Save  %d\n",	ch->saving_throw);
    fprintf( fp, "Alig	%d\n",	ch->alignment		);
    if (ch->hitroll != 0)
	fprintf( fp, "Hit   %d\n",	ch->hitroll	);
    if (ch->damroll != 0)
	fprintf( fp, "Dam   %d\n",	ch->damroll	);
    if (ch->pcdata->style)
	fprintf( fp, "Style %ld\n",	ch->pcdata->style );
    fprintf( fp, "ACs %d %d %d %d\n",	
	ch->armor[0],ch->armor[1],ch->armor[2],ch->armor[3]);
    if (ch->wimpy !=0 )
	fprintf( fp, "Wimp  %d\n",	ch->wimpy	);
    fprintf( fp, "Attr %d %d %d %d %d %d %d\n",
	ch->perm_stat[STAT_STR],
	ch->perm_stat[STAT_INT],
	ch->perm_stat[STAT_WIS],
	ch->perm_stat[STAT_DEX],
	ch->perm_stat[STAT_CON],
	ch->perm_stat[STAT_PER],
	ch->perm_stat[STAT_CHA] );

    fprintf (fp, "AMod %d %d %d %d %d %d %d\n",
	ch->mod_stat[STAT_STR],
	ch->mod_stat[STAT_INT],
	ch->mod_stat[STAT_WIS],
	ch->mod_stat[STAT_DEX],
	ch->mod_stat[STAT_CON],
	ch->mod_stat[STAT_PER],
	ch->mod_stat[STAT_CHA] );

    if ( IS_NPC(ch) )
    {
	fprintf( fp, "Vnum %d\n",	ch->pIndexData->vnum	);
    }
    else
    {
        CUST_DATA *current_cust;
	int territory;
	
	fprintf( fp, "Pass %s~\n",	ch->pcdata->pwd		);
	if (ch->pcdata->bamfin[0] != '\0')
	    fprintf( fp, "Bin  %s~\n",	ch->pcdata->bamfin);
	if (ch->pcdata->bamfout[0] != '\0')
	    fprintf( fp, "Bout %s~\n",	ch->pcdata->bamfout);
	if (ch->pcdata->transin[0] != '\0')
	    fprintf( fp, "TIn	%s~\n", ch->pcdata->transin );
	if (ch->pcdata->transout[0] != '\0')
	    fprintf( fp, "TOut	%s~\n", ch->pcdata->transout );
	if (ch->pcdata->transvict[0] != '\0')
	    fprintf( fp, "TVict	 %s~\n", ch->pcdata->transvict );
	fprintf( fp, "Titl %s~\n",	ch->pcdata->title	);
	fprintf( fp, "Pnts %d\n",	ch->pcdata->points	);
	fprintf( fp, "TSex %d\n",	ch->pcdata->true_sex	);
	fprintf( fp, "LLev %d\n",	ch->pcdata->last_level	);
	fprintf( fp, "HMVP %d %d %d\n", ch->pcdata->perm_hit, 
						   ch->pcdata->perm_mana,
						   ch->pcdata->perm_move);
	fprintf( fp, "Cnd  %d %d %d %d\n",
	    ch->pcdata->condition[0],
	    ch->pcdata->condition[1],
	    ch->pcdata->condition[2],
	    ch->pcdata->condition[3] );

	/* write alias */
	for (pos = 0; pos < MAX_ALIAS; pos++)
	{
	    if ( ch->pcdata->alias[pos] == NULL
	    ||   ch->pcdata->alias_sub[pos] == NULL)
		break;

	    fprintf(fp,"Alias %s %s~\n",ch->pcdata->alias[pos],
		    ch->pcdata->alias_sub[pos]);
	}

	for ( i = 0; color_config[i].config_name != NULL; ++i )
	{
	    if ( strcmp(ch->pcdata->colors[i], color_config[i].config_default) )
	    {
		fprintf( fp, "Color %s %s\n",
		    color_config[i].config_name,
		    color_lookup_escape(ch->pcdata->colors[i]) );
	    }
	}

	for ( sn = 0; sn < max_player_skill; sn++ )
	{
	    if ( skill_table[sn].name != NULL && ch->pcdata->learned[sn] > 0 )
	    {
		fprintf( fp, "Sk %d '%s'\n",
		    ch->pcdata->learned[sn], skill_table[sn].name );
	    }
	}

	for ( gn = 0; gn < MAX_GROUP; gn++ )
	{
	    if ( group_table[gn].name != NULL && ch->pcdata->group_known[gn])
	    {
		fprintf( fp, "Gr '%s'\n",group_table[gn].name);
	    }
	}
	
	for ( sn = 0; sn < MAX_STYLES; sn++ )
	{
	    if ( style_table[sn].name != NULL && ch->pcdata->styles[sn] )
	    {
		fprintf( fp, "KnownStyle '%s'\n", style_table[sn].name );
	    }
	}
	
	for ( territory = 0; territory < MAX_TERRITORIES; territory++ )
	{
	    if ( IS_NULLSTR(territories[territory].name) )
		continue;
	
	    fprintf( fp, "Terr %d %ld %s~\n",
	        ch->pcdata->social_classes[territory],
                ch->pcdata->territory_flags[territory],
	        territories[territory].name );
	}
	
	for ( territory = 0; territory < MAX_TERRITORIES; territory++ )
	{
	    if ( IS_NULLSTR(territories[territory].name)
	    ||   ch->pcdata->crimes[territory] == NULL )
		continue;
	
	    fprintf( fp, "Crime %ld %s~ %s~\n",
	        ch->pcdata->crimes[territory]->crime,
	        territories[territory].name,
	        ch->pcdata->crimes[territory]->victim );
	}

        for ( current_cust = ch->pcdata->customizations;
              current_cust != NULL;
              current_cust = current_cust->next )
            fwrite_cust( current_cust, fp );
    }

    fwrite_affects( ch->affected, fp );

    fprintf( fp, "End\n\n" );
    return;
}

/* write a pet */
void fwrite_pet( CHAR_DATA *pet, FILE *fp)
{
    fprintf(fp,"#PET\n");
    
    fprintf(fp,"Vnum %d\n",pet->pIndexData->vnum);
    
    fprintf(fp,"Name %s~\n", pet->name);
    fprintf(fp,"LogO %ld\n", current_time);
    if (pet->short_descr != pet->pIndexData->short_descr)
	fprintf(fp,"ShD	 %s~\n", pet->short_descr);
    if (pet->long_descr != pet->pIndexData->long_descr)
	fprintf(fp,"LnD	 %s~\n", pet->long_descr);
    if (pet->description != pet->pIndexData->description)
	fprintf(fp,"Desc %s~\n", pet->description);
    if ( pet->appearance[0] != '\0' )
	fprintf( fp, "CurApp %s~\n",	pet->appearance );
    if (pet->race != pet->pIndexData->race)
	fprintf(fp,"Race %s~\n", race_table[pet->race].name);
    if ( pet->polymorphed )
        fprintf( fp, "Poly %d\n", pet->polymorphed->vnum );
    if (pet->clan)
	fprintf( fp, "Clan %s~\n",clan_table[pet->clan].name);
    fprintf(fp,"Sex  %d\n", pet->sex);
    if (pet->level != pet->pIndexData->level)
	fprintf(fp,"Levl %d\n", pet->level);
    fprintf(fp, "HMV  %d %d %d %d %d %d\n",
	pet->hit, pet->max_hit, pet->mana, pet->max_mana, pet->move, pet->max_move);
    fprintf( fp, "Room %d\n",
	pet->in_room == NULL ? ROOM_VNUM_TEMPLE : pet->in_room->vnum );
    if (pet->gold > 0)
	fprintf(fp,"Gold %ld\n",pet->gold);
    if (pet->silver > 0)
	fprintf(fp,"Silv %ld\n",pet->silver);
    if (pet->exp > 0)
	fprintf(fp, "Exp  %d\n", pet->exp);
    if (pet->act != pet->pIndexData->act)
	fprintf(fp, "Act  %s\n", print_flags(pet->act));
    if (pet->affected_by != pet->pIndexData->affected_by)
	fprintf(fp, "AfBy %s\n", print_flags(pet->affected_by));
    if (pet->affected_by2 != pet->pIndexData->affected_by2)
	fprintf(fp, "AfBy2 %s\n", print_flags(pet->affected_by2));
    if (pet->comm != 0)
	fprintf(fp, "Comm %s\n", print_flags(pet->comm));
    fprintf(fp,"Pos  %d\n", pet->position = POS_FIGHTING ? POS_STANDING : pet->position);
    if (pet->saving_throw != 0)
	fprintf(fp, "Save %d\n", pet->saving_throw);
    if (pet->alignment != pet->pIndexData->alignment)
	fprintf(fp, "Alig %d\n", pet->alignment);
    if (pet->hitroll != pet->pIndexData->hitroll)
	fprintf(fp, "Hit  %d\n", pet->hitroll);
    if (pet->damroll != pet->pIndexData->damage[DICE_BONUS])
	fprintf(fp, "Dam  %d\n", pet->damroll);
    fprintf(fp, "ACs  %d %d %d %d\n",
	pet->armor[0],pet->armor[1],pet->armor[2],pet->armor[3]);
    fprintf(fp, "Attr %d %d %d %d %d %d %d\n",
	pet->perm_stat[STAT_STR], pet->perm_stat[STAT_INT],
	pet->perm_stat[STAT_WIS], pet->perm_stat[STAT_DEX],
	pet->perm_stat[STAT_CON], pet->perm_stat[STAT_PER],
	pet->perm_stat[STAT_CHA]);
    fprintf(fp, "AMod %d %d %d %d %d %d %d\n",
	pet->mod_stat[STAT_STR], pet->mod_stat[STAT_INT],
	pet->mod_stat[STAT_WIS], pet->mod_stat[STAT_DEX],
	pet->mod_stat[STAT_CON], pet->mod_stat[STAT_PER],
	pet->mod_stat[STAT_CHA]);
    
	fwrite_affects( pet->affected, fp );
    
    fprintf(fp,"End\n");
    return;
}
    
/*
 * Write an object and its contents.
 */
int fwrite_obj( CHAR_DATA *ch, OBJ_DATA *obj, FILE *fp, int iNest, 
                bool in_room, bool corpses_sink, int maximum )
{
    EXTRA_DESCR_DATA *ed;
    int written = 0;

    if ( maximum == 0 )
        return written;

    /*
     * Slick recursion to write lists backwards,
     *   so loading them will load in forwards order.
     */
    if ( obj->next_content != NULL && !in_room )
	written += fwrite_obj( ch, obj->next_content, fp, iNest, FALSE, FALSE, maximum - 1 );

    /*
     * Castrate storage characters.
     */
    if ( (ch->level < obj->level - 2 && obj->item_type != ITEM_CONTAINER)
    ||   obj->item_type == ITEM_KEY
    ||   (obj->item_type == ITEM_MAP && !obj->value[0]))
	return written;

    if ( !IS_NULLSTR(obj->owner)
    &&   str_cmp(ch->name, obj->owner) )
    {
	if ( corpses_sink && !in_room )
	{
	    act( "$p drops to the ground.",
		 ch, obj, NULL, TO_ROOM );
	
	    obj_from_char( obj );
	    obj_to_room( obj, ch->in_room );
	}

	return written;
    }

    fprintf( fp, "#O\n" );
    fprintf( fp, "Vnum %d\n",   obj->pIndexData->vnum        );
    if (!obj->pIndexData->new_format)
	fprintf( fp, "Oldstyle\n");
    if (in_room)
	fprintf( fp, "Room %d\n", obj->in_room->vnum );
    if (obj->enchanted)
	fprintf( fp,"Enchanted\n");
    fprintf( fp, "Nest %d\n", iNest );

    /* these data are only used if they do not match the defaults */

    if ( obj->name != obj->pIndexData->name)
	fprintf( fp, "Name %s~\n",	obj->name );
    if ( obj->short_descr != obj->pIndexData->short_descr)
	fprintf( fp, "ShD  %s~\n",	obj->short_descr );
    if ( obj->description != obj->pIndexData->description)
	fprintf( fp, "Desc %s~\n",	obj->description );
    if ( obj->extra_flags != obj->pIndexData->extra_flags)
	fprintf( fp, "ExtF %d\n",	obj->extra_flags );
    if ( obj->wear_flags != obj->pIndexData->wear_flags)
	fprintf( fp, "WeaF %d\n",	obj->wear_flags );
    if ( obj->item_type != obj->pIndexData->item_type)
	fprintf( fp, "Ityp %d\n",	obj->item_type );
    if ( obj->weight != obj->pIndexData->weight)
	fprintf( fp, "Wt   %d\n",	obj->weight );
    if ( obj->condition != obj->pIndexData->condition)
	fprintf( fp, "Cond %d\n",	obj->condition );

    /* variable data */

    fprintf( fp, "Wear %d\n",   obj->wear_loc                );
    if (obj->level != obj->pIndexData->level)
	fprintf( fp, "Lev  %d\n",	obj->level		     );
    if (obj->timer != 0)
	fprintf( fp, "Time %d\n",	obj->timer	     );
    if ( !IS_NULLSTR(obj->owner) )
	fprintf( fp, "Owner %s~\n",	obj->owner );
    fprintf( fp, "Cost %d\n",	obj->cost		     );
    if (obj->value[0] != obj->pIndexData->value[0]
    ||  obj->value[1] != obj->pIndexData->value[1]
    ||  obj->value[2] != obj->pIndexData->value[2]
    ||  obj->value[3] != obj->pIndexData->value[3]
    ||  obj->value[4] != obj->pIndexData->value[4]) 
	fprintf( fp, "Val  %d %d %d %d %d\n",
	    obj->value[0], obj->value[1], obj->value[2], obj->value[3],
	    obj->value[4]	     );

    switch ( obj->item_type )
    {
    case ITEM_POTION:
    case ITEM_SCROLL:
    case ITEM_PILL:
	if ( obj->value[1] > 0 )
	{
	    fprintf( fp, "Spell 1 '%s'\n", 
		skill_table[obj->value[1]].name );
	}

	if ( obj->value[2] > 0 )
	{
	    fprintf( fp, "Spell 2 '%s'\n", 
		skill_table[obj->value[2]].name );
	}

	if ( obj->value[3] > 0 )
	{
	    fprintf( fp, "Spell 3 '%s'\n", 
		skill_table[obj->value[3]].name );
	}

	break;

    case ITEM_STAFF:
    case ITEM_WAND:
	if ( obj->value[3] > 0 )
	{
	    fprintf( fp, "Spell 3 '%s'\n", 
		skill_table[obj->value[3]].name );
	}
	break;
    }

    fwrite_affects( obj->affected, fp );
	
    for ( ed = obj->extra_descr; ed != NULL; ed = ed->next )
    {
	fprintf( fp, "ExDe %s~ %s~\n",
	    ed->keyword, ed->description );
    }

    if ( corpses_sink
    &&   in_room
//    &&   obj->item_type == ITEM_CORPSE_PC
    &&   obj->in_room->people != NULL )
    {
	act( "$p sinks into the ground.",
	    obj->in_room->people, obj, NULL, TO_ROOM );
	act( "$p sinks into the ground.",
	    obj->in_room->people, obj, NULL, TO_CHAR );
    }

    fprintf( fp, "End\n\n" );

    if ( obj->contains != NULL )
	written += fwrite_obj( ch, obj->contains, fp, iNest + 1, FALSE, FALSE, maximum - 1 );

    return written;
}


/*
 * Save a characters customization data.
 */
void fwrite_cust( CUST_DATA *cust, FILE *fp )
{
    const struct advantage_type *table;
    int type;
    if ( cust->type >= MIN_ADVANTAGE && cust->type <= MAX_ADVANTAGE )
    {
        table = advantage_table;
        type  = 1;
    }
    else if ( cust->type >= MIN_DISADVANTAGE && cust->type <= MAX_DISADVANTAGE )
    {
        table = disadvantage_table;
        type  = 2;
    }
    else
    {
        table = ancestor_table;
        type  = 3;
    }
    if ( cust->affect.where == -1 )
        fprintf( fp, "Cust %d '%s' -1\n",
            type,
            table[cust->index].name );
    else    
        fprintf( fp, "Cust %d '%s' %3d %10d '%s' %d\n",
            type,
            table[cust->index].name,
            cust->affect.where,
            cust->affect.bitvector,
            affect_procs[cust->affect.procNo].name,
            cust->affect.flags );
}


/*
 * Save a characters affect data.
 */
void fwrite_affects( AFFECT_DATA *paf, FILE *fp )
{
    for ( ; paf != NULL; paf = paf->next ) {
	if ( paf->type < 0 || paf->type >= MAX_SKILL ) {
	    continue;
	}
	
	fprintf( fp, "Affp '%s' %3d %3d %3d %3d %3d %10d '%s' %d %s~\n%s~\n",
	    skill_table[paf->type].name,
	    paf->where,
	    paf->level,
	    paf->duration,
	    paf->modifier,
	    paf->location,
	    paf->bitvector,
	    affect_procs[paf->procNo].name,
	    paf->flags,
	    paf->name,
	    paf->appearance
	);
    }
}


bool check_objs( const char *name, int vnum )
{
    OBJ_DATA *obj;

    for ( obj = get_obj_index(vnum)->instances; 
	  obj != NULL; 
	  obj = obj->next_instance )
    {
	if ( !str_cmp(obj->owner, name)
	&&   obj->in_room != NULL )
	{
            return TRUE;
	}
    }
    return FALSE;
}


/*
 * Load a char and inventory into a new ch structure.
 */
bool load_char_obj( DESCRIPTOR_DATA *d, char *name )
{
    char strsave[MAX_INPUT_LENGTH];
    char buf[100];
    CUST_DATA *cust;
    CHAR_DATA *ch;
    FILE *fp;
    bool found;
    int stat;
    int i;

    ch = new_char();
    ch->pcdata = new_pcdata();

    d->character			= ch;
    ch->desc				= d;
    ch->name				= str_dup( name );
    ch->id				= get_pc_id();
    ch->race				= race_lookup("human");
    ch->act				= PLR_NOSUMMON;
    ch->comm				= COMM_COMBINE 
					| COMM_PROMPT;
    ch->prompt 				= str_dup("<%hhp %mm %vmv> ");
    ch->pcdata->confirm_delete		= FALSE;
    ch->pcdata->pwd			= str_dup( "" );
    ch->pcdata->bamfin			= str_dup( "" );
    ch->pcdata->bamfout			= str_dup( "" );
    ch->pcdata->transin			= str_dup( "" );
    ch->pcdata->transout		= str_dup( "" );
    ch->pcdata->transvict		= str_dup( "" );
    ch->pcdata->title			= str_dup( "" );
    for (stat =0; stat < MAX_STATS; stat++)
	ch->perm_stat[stat]		= 52;
    ch->pcdata->condition[COND_THIRST]	= 48; 
    ch->pcdata->condition[COND_FULL]	= 48;
    ch->pcdata->condition[COND_HUNGER]	= 48;
    ch->pcdata->renown			= default_renown();
/*
 *  ch->pcdata->religion		= 0;
 *  ch->pcdata->heresy			= 0;
 *  ch->pcdata->balance			= 0;
 *  ch->pcdata->leader			= 0;
 *  ch->pcdata->petition		= 0;
 *  ch->pcdata->olc_level		= 0;
 *  ch->pcdata->true_level		= 0;
 *  ch->pcdata->style			= 0;
 */
    ch->pcdata->who_name		= str_dup( "" );
    ch->pcdata->appearance		= str_dup( "" );
    
    for (i = 0; color_config[i].config_name != NULL; ++i )
	ch->pcdata->colors[i] = color_config[i].config_default;

    for( i = 0; i < MAX_PROFESSIONS - 1; i++ )
	ch->pcdata->former_professions[i] = -1;
	
    for ( i = 0; i < MAX_TERRITORIES; i++ )
    {
	ch->pcdata->social_classes[i]  = 5;
        ch->pcdata->territory_flags[i] = 0;
    }

    for ( i = 0; i < MAX_PC_FIGHT_RECORD; ++i )
        ch->pcdata->fight_records[ i ] = 0;

    found = FALSE;
    fclose( fpReserve );
    
#if defined(unix)
    /* decompress if .gz file exists */
    sprintf( strsave, "%s%s%s", PLAYER_DIR, capitalize(name),".gz");
    if ( ( fp = fopen( strsave, "r" ) ) != NULL )
    {
	fclose(fp);
	sprintf(buf,"gzip -dfq %s",strsave);
	system(buf);
    }
#endif

    sprintf( strsave, "%s%s", PLAYER_DIR, capitalize( name ) );
    if ( ( fp = fopen( strsave, "r" ) ) != NULL )
    {
	bool load_rooms = FALSE;
        ROOM_INDEX_DATA *home;
	int iNest;

	/* check for existing room corpses - so we don't reload them */
        if ( check_objs(ch->name, OBJ_VNUM_CORPSE_PC_BASH)
        ||   check_objs(ch->name, OBJ_VNUM_CORPSE_PC_FIRE)
        ||   check_objs(ch->name, OBJ_VNUM_CORPSE_PC_COLD)
        ||   check_objs(ch->name, OBJ_VNUM_CORPSE_PC_LIGHTNING)
        ||   check_objs(ch->name, OBJ_VNUM_CORPSE_PC_ACID)
        ||   check_objs(ch->name, OBJ_VNUM_CORPSE_PC_POISON)
        ||   check_objs(ch->name, OBJ_VNUM_CORPSE_PC_NEGATIVE)
        ||   check_objs(ch->name, OBJ_VNUM_CORPSE_PC_HOLY)
        ||   check_objs(ch->name, OBJ_VNUM_CORPSE_PC_ENERGY)
        ||   check_objs(ch->name, OBJ_VNUM_CORPSE_PC_DISEASE)
        ||   check_objs(ch->name, OBJ_VNUM_CORPSE_PC_DROWNING)
        ||   check_objs(ch->name, OBJ_VNUM_CORPSE_PC_SLAY)
        ||   check_objs(ch->name, OBJ_VNUM_CORPSE_PC) )
            load_rooms = TRUE;

	for ( iNest = 0; iNest < MAX_NEST; iNest++ )
	    rgObjNest[iNest] = NULL;

	found = TRUE;
	for ( ; ; )
	{
	    char letter;
	    char *word;

	    letter = fread_letter( fp );
	    if ( letter == '*' )
	    {
		fread_to_eol( fp );
		continue;
	    }

	    if ( letter != '#' )
	    {
		bug( "Load_char_obj: # not found.", 0 );
		break;
	    }

	    word = fread_word( fp );
	    if      ( !str_cmp( word, "PLAYER" ) )
            {
                fread_char ( ch, fp );

                if ( ch->homevnum != 0
                &&   (home = get_room_index(ch->homevnum)) != NULL
                &&   IS_SET(home->room_flags, ROOM_EQUIPMENT_LOADED) )
                    load_rooms = TRUE;
            }
	    else if ( !str_cmp( word, "OBJECT" ) ) fread_obj  ( ch, fp, !load_rooms );
	    else if ( !str_cmp( word, "O"      ) ) fread_obj  ( ch, fp, !load_rooms );
	    else if ( !str_cmp( word, "PET"    ) ) fread_pet  ( ch, fp );
	    else if ( !str_cmp( word, "END"    ) ) break;
	    else
	    {
		bug( "Load_char_obj: bad section.", 0 );
		break;
	    }
	}
	fclose( fp );

        // If the player has a room set the equipment loaded flag on it to prevent
        // reloads.
        if ( ch->homevnum )
        {
            ROOM_INDEX_DATA *pRoomIndex = get_room_index( ch->homevnum );
            if ( pRoomIndex )
            {
                SET_BIT( pRoomIndex->room_flags, ROOM_EQUIPMENT_LOADED );
            }
        }
    }

    fpReserve = fopen( NULL_FILE, "r" );


    /* initialize race */
    if (found)
    {
	int i;

	if (ch->race == 0)
	    ch->race = race_lookup("human");

	ch->size = pc_race_table[ch->race].size;

	for (i = 0; i < 5; i++)
	{
	    if (pc_race_table[ch->race].skills[i] == NULL)
		break;
	    group_add(ch,pc_race_table[ch->race].skills[i],FALSE);
	}
	ch->affected_by = ch->affected_by|race_table[ch->race].aff;
	ch->affected_by2= ch->affected_by2|race_table[ch->race].aff2;
	ch->imm_flags	= ch->imm_flags | race_table[ch->race].imm;
	ch->res_flags	= ch->res_flags | race_table[ch->race].res;
	ch->vuln_flags	= ch->vuln_flags | race_table[ch->race].vuln;
	ch->form	= race_table[ch->race].form;
	ch->parts	= race_table[ch->race].parts;

#if 0	/* Replaced by dam_hand_to_hand dam_fun */
	if ( IS_SET(ch->parts, PART_CLAWS) )
	    ch->dam_type = 5;	/* attack_lookup( "claw" )  */
	else
#endif
	ch->dam_type = gdt_punch;

    }

    /* Add in any imm, res or vuln affects from customizations. */
    for ( cust = ch->pcdata->customizations;
          cust != NULL;
          cust = cust->next )
    {
        if ( cust->affect.where == TO_IMMUNE )
            ch->imm_flags |= cust->affect.bitvector;
        else if ( cust->affect.where = TO_RESIST )
            ch->res_flags |= cust->affect.bitvector;
        else
            ch->vuln_flags |= cust->affect.bitvector;
    }
	
    /* RT initialize skills */

    if (found && ch->version < 2)  /* need to add the new skills */
    {
	group_add(ch,"rom basics",FALSE);
	group_add(ch,class_table[ch->class].base_group,FALSE);
	group_add(ch,class_table[ch->class].default_group,TRUE);
	ch->pcdata->learned[gsn_recall] = 50;
    }
 
    /* fix levels */
    if (found && ch->version < 3 && (ch->level > 35 || ch->trust > 35))
    {
	switch (ch->level)
	{
	    case(40) : ch->level = 60;	break;  /* imp -> imp */
	    case(39) : ch->level = 58; 	break;	/* god -> supreme */
	    case(38) : ch->level = 56;  break;	/* deity -> god */
	    case(37) : ch->level = 53;  break;	/* angel -> demigod */
	}

        switch (ch->trust)
        {
            case(40) : ch->trust = 60;  break;	/* imp -> imp */
            case(39) : ch->trust = 58;  break;	/* god -> supreme */
            case(38) : ch->trust = 56;  break;	/* deity -> god */
            case(37) : ch->trust = 53;  break;	/* angel -> demigod */
            case(36) : ch->trust = 51;  break;	/* hero -> hero */
        }
    }

    /* ream gold */
    if (found && ch->version < 4)
    {
	    ch->gold   /= 100;
    }
    
    /* Set true level. */
    if ( found && ch->pcdata->true_level == 0 )
    {
	    ch->pcdata->true_level = ch->level;
    }
    
    return found;
}



/*
 * Read in a char.
 */

#if defined(KEY)
#undef KEY
#endif

#define KEY( literal, field, value )					\
				if ( !str_cmp( word, literal ) )	\
				{					\
				    field  = value;			\
				    fMatch = TRUE;			\
				    break;				\
				}

/* provided to free strings */
#if defined(KEYS)
#undef KEYS
#endif

#define KEYS( literal, field, value )					\
				if ( !str_cmp( word, literal ) )	\
				{					\
				    free_string(field);			\
				    field  = value;			\
				    fMatch = TRUE;			\
				    break;				\
				}

void fread_char( CHAR_DATA *ch, FILE *fp )
{
    char buf[MAX_STRING_LENGTH];
    char *word;
    bool fMatch;
    int count = 0;
    int lastlogoff = current_time;
    int percent;

    sprintf(buf,"Loading %s.",ch->name);
    log_string(buf);

    for ( ; ; )
    {
	word   = feof( fp ) ? "End" : fread_word( fp );
	fMatch = FALSE;

	switch ( UPPER(word[0]) )
	{
	case '*':
	    fMatch = TRUE;
	    fread_to_eol( fp );
	    break;

	case 'A':
	    KEY( "Act",		ch->act,		fread_flag( fp ) );
	    KEY( "AffectedBy",	ch->affected_by,	fread_flag( fp ) );
	    KEY( "AfBy",	ch->affected_by,	fread_flag( fp ) );
	    KEY( "AfBy2",	ch->affected_by2,	fread_flag( fp ) );
	    KEY( "Alignment",	ch->alignment,		fread_number( fp ) );
	    KEY( "Alig",	ch->alignment,		fread_number( fp ) );
	    KEY( "App",		ch->pcdata->appearance,	fread_string( fp ) );

	    if (!str_cmp( word, "Alia"))
	    {
		if (count >= MAX_ALIAS)
		{
		    fread_to_eol(fp);
		    fMatch = TRUE;
		    break;
		}

		ch->pcdata->alias[count] 	= str_dup(fread_word(fp));
		ch->pcdata->alias_sub[count]	= str_dup(fread_word(fp));
		count++;
		fMatch = TRUE;
		break;
	    }

            if (!str_cmp( word, "Alias"))
            {
                if (count >= MAX_ALIAS)
                {
                    fread_to_eol(fp);
                    fMatch = TRUE;
                    break;
                }
 
                ch->pcdata->alias[count]        = str_dup(fread_word(fp));
                ch->pcdata->alias_sub[count]    = fread_string(fp);
                count++;
                fMatch = TRUE;
                break;
            }

	    if (!str_cmp( word, "AC") || !str_cmp(word,"Armor"))
	    {
		fread_to_eol(fp);
		fMatch = TRUE;
		break;
	    }

	    if (!str_cmp(word,"ACs"))
	    {
		int i;

		for (i = 0; i < 4; i++)
		    ch->armor[i] = fread_number(fp);
		fMatch = TRUE;
		break;
	    }

	    if (!str_cmp(word, "AffD"))
	    {
		AFFECT_DATA *paf;
		int sn;

		paf = new_affect();

		sn = skill_lookup(fread_word(fp));
		if (sn < 0)
		    bug("Fread_char: unknown skill.",0);
		else
		    paf->type = sn;

		paf->where	= TO_AFFECTS;
		paf->level	= fread_number( fp );
		paf->duration	= fread_number( fp );
		paf->modifier	= fread_number( fp );
		paf->location	= fread_number( fp );
		paf->bitvector	= fread_number( fp );
		paf->procNo	= 0;
		paf->flags	= 0;
		paf->name	= str_dup( "" );
		paf->appearance	= str_dup( "" );
		paf->next	= ch->affected;
		ch->affected	= paf;
		fMatch = TRUE;
		break;
	    }

	    if (!str_cmp(word, "Affc"))
	    {
		AFFECT_DATA *paf;
		int sn;
 
		paf = new_affect();
 
		sn = skill_lookup(fread_word(fp));
		if (sn < 0)
		    bug("Fread_char: unknown skill.",0);
		else
		    paf->type = sn;
 
		paf->where	= fread_number(fp);
		paf->level	= fread_number( fp );
		paf->duration	= fread_number( fp );
		paf->modifier	= fread_number( fp );
		paf->location	= fread_number( fp );
		paf->bitvector	= fread_number( fp );
		paf->procNo	= 0;
		paf->flags	= 0;
		paf->name	= str_dup( "" );
		paf->appearance = str_dup( "" );
		paf->next	= ch->affected;
		ch->affected	= paf;
		fMatch = TRUE;
		break;
	    }

	    if ( !str_cmp(word, "Affp") )
	    {
		AFFECT_DATA *paf	= fread_affect( fp, ch->version );

		paf->next		= ch->affected;
		ch->affected		= paf;
		fMatch = TRUE;
		break;
	    }


	    if ( !str_cmp( word, "AttrMod"  ) || !str_cmp(word,"AMod"))
	    {
		int stat;
		for (stat = 0; stat < MAX_STATS; stat ++)
		   ch->mod_stat[stat] = fread_number(fp);
		fMatch = TRUE;
		break;
	    }

	    if ( !str_cmp( word, "AttrPerm" ) || !str_cmp(word,"Attr"))
	    {
		int stat;

		for (stat = 0; stat < MAX_STATS; stat++)
		    ch->perm_stat[stat] = fread_number(fp);
		fMatch = TRUE;
		break;
	    }
	    break;

	case 'B':
	    KEY( "Balance",	ch->pcdata->balance,	fread_number( fp ) );
	    KEY( "Bamfin",	ch->pcdata->bamfin,	fread_string( fp ) );
	    KEY( "Bamfout",	ch->pcdata->bamfout,	fread_string( fp ) );
	    KEY( "Bin",		ch->pcdata->bamfin,	fread_string( fp ) );
	    KEY( "Bout",	ch->pcdata->bamfout,	fread_string( fp ) );
	    break;

	case 'C':
	    KEY( "Class",	ch->class,		fread_number( fp ) );
	    KEY( "Cla",		ch->class,		fread_number( fp ) );
	    /*KEY( "Clan",	ch->clan,		clan_lookup( fread_string(fp) ) );*/
	    KEYS( "CurApp",	ch->appearance,		fread_string( fp ) );

	    /* Hopefully leak free version. */
	    if ( !str_cmp(word, "Clan") )
	    {
		char *temp	= fread_string( fp );
		ch->clan	= clan_lookup( temp );
		
		free_string( temp );
		fMatch = TRUE;
		break;
	    }
	    
	    if( !str_cmp( word, "Color" ) )
	    {
		char *which;
		char *color;
		char *escape;
		int index;

		which = fread_word( fp );
		index = color_lookup_config_index( which );

		color = fread_word( fp );
		escape = color_lookup_color( color );

		fMatch = TRUE;
		if( index == -1 || escape == NULL )
		{
		    bug( "Invalid color in fread_char()", 0 );
		    break;
		}
		ch->pcdata->colors[index] = escape;
                fMatch = TRUE;
		break;
	    }

	    if ( !str_cmp( word, "Condition" ) || !str_cmp(word,"Cond"))
	    {
		ch->pcdata->condition[0] = fread_number( fp );
		ch->pcdata->condition[1] = fread_number( fp );
		ch->pcdata->condition[2] = fread_number( fp );
		fMatch = TRUE;
		break;
	    }
	    
            if (!str_cmp(word,"Cnd"))
            {
                ch->pcdata->condition[0] = fread_number( fp );
                ch->pcdata->condition[1] = fread_number( fp );
                ch->pcdata->condition[2] = fread_number( fp );
		ch->pcdata->condition[3] = fread_number( fp );
                fMatch = TRUE;
                break;
            }
            
	    KEY("Comm",		ch->comm,		fread_flag( fp ) );
	    
	    if ( !str_cmp(word, "Crime") )
	    {
		int territory, crime_type;
		char *territory_name;
                CRIME_DATA *crime;
		char *victim;
		
		crime_type      = fread_number( fp );
		territory_name	= fread_string( fp );
		victim		    = fread_string( fp );
		
		territory	= territory_lookup( territory_name );
		
		if ( territory == -1 )
		    bug( "Fread_char: unknown territory in crime list - %s.", (long) territory_name );
		else
		{
		    crime		    = new_crime();
		    crime->crime	= crime_type;
		    crime->victim	= str_dup( victim );
		
		    crime->next		= ch->pcdata->crimes[territory];
		    ch->pcdata->crimes[territory]
					= crime;
		}
		
		free_string( territory_name );
		free_string( victim );
		fMatch = TRUE;
	    }

            if ( !str_cmp(word, "Cust") )
            {
                struct advantage_type *table = NULL;
                int  type       = fread_number( fp );
                char *temp      = fread_word( fp );
                char *cust_name = str_dup( temp );
                int  where      = fread_number( fp );
                int bitvector   = 0;
                int procNo      = 0;
                int flags       = 0;
                int index;

                if ( where != -1 )
                {
                    
                    bitvector  = fread_number( fp );
                    procNo     = affect_proc_lookup(fread_word(fp));
                    flags      = fread_number( fp );
                }
                

                switch ( type )
                {
                    case 1: table = advantage_table;    break;
                    case 2: table = disadvantage_table; break;
                    case 3: table = ancestor_table;     break;
                    default:
                        bug( "Fread_char: unknown customization type (%d)", (long) type );
                }

                if ( (index = advantage_lookup(table, cust_name, ch)) == -1 )
                {
                    bug( "Fread_char: unknown customization (%s)", (long) cust_name );
                }

                if ( table && index != -1 )
                {
                    CUST_DATA *cust             = new_cust_data();
                    cust->index                 = index;
                    cust->type                  = table[index].value;
                    cust->affect.where          = where;
                    if ( where != -1 )
                    {
                        cust->affect.type       = 0;
                        cust->affect.level      = 0;
                        cust->affect.duration   = -1;
                        cust->affect.location   = APPLY_NONE;
                        cust->affect.modifier   = 0;
                        cust->affect.bitvector  = bitvector;
                        cust->affect.procNo     = procNo;
                        cust->affect.flags      = flags;
                        cust->affect.name       = "";
                        cust->affect.appearance = "";
                    }
                    cust->next                  = ch->pcdata->customizations;
                    ch->pcdata->customizations  = cust;
                }
                free_string( cust_name );
                fMatch = TRUE;
            }
	    break;

	case 'D':
	    KEY( "Damroll",	ch->damroll,		fread_number( fp ) );
	    KEY( "Dam",		ch->damroll,		fread_number( fp ) );
	    KEY( "Description",	ch->description,	fread_string( fp ) );
	    KEY( "Desc",	ch->description,	fread_string( fp ) );
	    break;

	case 'E':
	    if ( !str_cmp( word, "End" ) )
	    {
    		/* adjust hp mana move up  -- here for speed's sake */
    		percent = (current_time - lastlogoff) * 25 / ( 2 * 60 * 60);

		percent = UMIN(percent,100);
 
    		if (percent > 0 && !IS_AFFECTED(ch,AFF_POISON)
    		&&  !IS_AFFECTED(ch,AFF_PLAGUE))
    		{
        	    ch->hit	+= (ch->max_hit - ch->hit) * percent / 100;
        	    ch->mana    += (ch->max_mana - ch->mana) * percent / 100;
        	    ch->move    += (ch->max_move - ch->move)* percent / 100;
    		}
		return;
	    }
	    KEY( "Exp",		ch->exp,		fread_number( fp ) );
	    break;

	case 'G':
	    KEY( "God",		ch->pcdata->religion,	fread_number( fp ) );
	    KEY( "Gold",	ch->gold,		fread_number( fp ) );
            if ( !str_cmp( word, "Group" )  || !str_cmp(word,"Gr"))
            {
                int gn;
                char *temp;
 
                temp = fread_word( fp ) ;

		if (temp == "maladictions")
		    sprintf(temp, "maledictions");
                /*gn = group_lookup_abs(temp);*/
                gn    = group_lookup( temp );
                if ( gn < 0 )
                {
                    fprintf(stderr,"%s",temp);
                    bug( "Fread_char: unknown group (%s)", (int) temp );
                }
                else
		    gn_add(ch,gn);
                fMatch = TRUE;
            }
	    break;

	case 'H':
	    KEY( "Hitroll",	ch->hitroll,		fread_number( fp ) );
	    KEY( "Hit",		ch->hitroll,		fread_number( fp ) );
	    KEY( "Her",		ch->pcdata->heresy,	fread_number( fp ) );
	    KEY( "Home",	ch->hometown,		fread_number( fp ) );
	    KEY( "Hnum",	ch->homevnum,		fread_number( fp ) );

	    if ( !str_cmp( word, "HpManaMove" ) || !str_cmp(word,"HMV"))
	    {
		ch->hit		= fread_number( fp );
		ch->max_hit	= fread_number( fp );
		ch->mana	= fread_number( fp );
		ch->max_mana	= fread_number( fp );
		ch->move	= fread_number( fp );
		ch->max_move	= fread_number( fp );
		fMatch = TRUE;
		break;
	    }

            if ( !str_cmp( word, "HpManaMovePerm" ) || !str_cmp(word,"HMVP"))
            {
                ch->pcdata->perm_hit	= fread_number( fp );
                ch->pcdata->perm_mana   = fread_number( fp );
                ch->pcdata->perm_move   = fread_number( fp );
                fMatch = TRUE;
                break;
            }
      
	    break;

	case 'I':
	    KEY( "Id",		ch->id,			fread_number( fp ) );
	    KEY( "InvisLevel",	ch->invis_level,	fread_number( fp ) );
	    KEY( "Inco",	ch->incog_level,	fread_number( fp ) );
	    KEY( "Invi",	ch->invis_level,	fread_number( fp ) );
	    break;

	case 'K':
            if ( !str_cmp( word, "KnownStyle" ) )
            {
                int sn;
                char *temp;
 
                temp = fread_word( fp ) ;
                sn = style_lookup(temp);
                if ( sn < 0 )
                {
                    fprintf( stderr, "%s", temp );
                    bug( "Fread_char: unknown style (%s).", (long) temp );
                }
                else
		    ch->pcdata->styles[sn] = 1;
                fMatch = TRUE;
            }
	    break;


	case 'L':
	    KEY( "LastLevel",	ch->pcdata->last_level, fread_number( fp ) );
	    KEY( "LLev",	ch->pcdata->last_level, fread_number( fp ) );
	    KEY( "Leader",	ch->pcdata->leader,	fread_number( fp ) );
	    KEY( "Level",	ch->level,		fread_number( fp ) );
	    KEY( "Lev",		ch->level,		fread_number( fp ) );
	    KEY( "Levl",	ch->level,		fread_number( fp ) );
	    KEY( "LogO",	lastlogoff,		fread_number( fp ) );
	    KEY( "LongDescr",	ch->long_descr,		fread_string( fp ) );
	    KEY( "LnD",		ch->long_descr,		fread_string( fp ) );
	    break;

	case 'M':
	    KEY( "Meta",	ch->pcdata->meta_purchases,
	    						fread_number( fp ) );
	    break;
	case 'N':
	    KEYS( "Name",	ch->name,		fread_string( fp ) );
	    KEY( "Note",	ch->pcdata->last_note,	fread_number( fp ) );
	    if (!str_cmp(word,"Not"))
	    {
		ch->pcdata->last_note			= fread_number(fp);
		ch->pcdata->last_idea			= fread_number(fp);
		ch->pcdata->last_penalty		= fread_number(fp);
		ch->pcdata->last_news			= fread_number(fp);
		ch->pcdata->last_changes		= fread_number(fp);
		fMatch = TRUE;
		break;
	    }
	    if (!str_cmp(word,"Not2"))
	    {
		ch->pcdata->last_note			= fread_number(fp);
		ch->pcdata->last_idea			= fread_number(fp);
		ch->pcdata->last_penalty		= fread_number(fp);
		ch->pcdata->last_news			= fread_number(fp);
		ch->pcdata->last_changes		= fread_number(fp);
		ch->pcdata->last_bug			= fread_number(fp);
		fMatch = TRUE;
		break;
	    }
	    break;
	
	case 'O':
	    KEY( "OLC",		ch->pcdata->olc_level,	fread_number( fp ) );
	    break;

	case 'P':
	    KEY( "Password",	ch->pcdata->pwd,	fread_string( fp ) );
	    KEY( "Pass",	ch->pcdata->pwd,	fread_string( fp ) );
	    /*KEY( "Petition",	ch->pcdata->petition,	
	         clan_lookup(fread_string(fp)) );*/
	    KEY( "Played",	ch->played,		fread_number( fp ) );
	    KEY( "Plyd",	ch->played,		fread_number( fp ) );
	    KEY( "Points",	ch->pcdata->points,	fread_number( fp ) );
	    KEY( "Pnts",	ch->pcdata->points,	fread_number( fp ) );
	    KEY( "Position",	ch->position,		fread_number( fp ) );
	    KEY( "Pos",		ch->position,		fread_number( fp ) );
	    KEY( "Practice",	ch->practice,		fread_number( fp ) );
	    KEY( "Prac",	ch->practice,		fread_number( fp ) );
	    KEY( "Priority",	ch->priority,		fread_number( fp ) );
	    KEY( "Prompt",	ch->prompt,		fread_string( fp ) );
	    KEY( "Prom",	ch->prompt,		fread_string( fp ) );
	    
	    /* Hopefully leak free version. */
	    if ( !str_cmp(word, "Petition") )
	    {
		char *temp		= fread_string( fp );
		ch->pcdata->petition	= clan_lookup( temp );
		
		free_string( temp );
		fMatch = TRUE;
		break;
	    }
            
            if ( !str_cmp(word, "Poly") )
            {
                int vnum = fread_number( fp );
                ch->polymorphed = get_mob_index( vnum );
                fMatch = TRUE;
                break;
            }
	    
	    if ( !str_cmp(word, "Prof") )
	    {
		int profession;
		int professions = fread_number( fp );
		
		for ( profession = 0; profession < professions; profession++ )
		    ch->pcdata->former_professions[profession] =
		        fread_number( fp );
		fMatch = TRUE;
	    }
	    
	    break;

	case 'R':
	    /*KEY( "Race",        ch->race,	
				race_lookup(fread_string( fp )) );*/
	    KEY( "Renown",	ch->pcdata->renown,
	    			fread_number(fp) );

	    if ( !str_cmp(word, "Race") )
	    {
		char *temp	= fread_string( fp );
		ch->race	= race_lookup( temp );
		
		free_string( temp );
		fMatch = TRUE;
		break;
	    }

	    if ( !str_cmp( word, "Room" ) )
	    {
		ch->in_room = get_room_index( fread_number( fp ) );
		if ( ch->in_room == NULL )
		    ch->in_room = get_room_index( ROOM_VNUM_LIMBO );
		fMatch = TRUE;
		break;
	    }

	    break;

	case 'S':

	    KEY( "SavingThrow",	ch->saving_throw,	fread_number( fp ) );
	    KEY( "Save",	ch->saving_throw,	fread_number( fp ) );
	    KEY( "Scro",	ch->lines,		fread_number( fp ) );
	    KEY( "Sex",		ch->sex,		fread_number( fp ) );
	    KEY( "ShortDescr",	ch->short_descr,	fread_string( fp ) );
	    KEY( "ShD",		ch->short_descr,	fread_string( fp ) );
            KEY( "Silv",        ch->silver,             fread_number( fp ) );
            KEY( "Style",	ch->pcdata->style,	fread_number( fp ) );

	    if ( !str_cmp( word, "Skill" ) || !str_cmp(word,"Sk"))
	    {
		int sn;
		int value;
		char *temp;

		fMatch = TRUE;

		value	= fread_number( fp );
		temp	= fread_word( fp );
		sn	= skill_lookup_abs(temp);
		/* sn    = skill_lookup( fread_word( fp ) ); */
		
		if ( sn < 0 )
		{
		    if ( !str_cmp(temp, "third attack") )
			sn = gsn_weapon_brawling;
		    else if ( !str_cmp(temp, "second attack") )
			sn = gsn_arms_mastery;
		    else
		    {
			fprintf(stderr,"%s",temp);
			bug( "Fread_char: unknown skill. ", 0 );
			break;
		    }
		}

		ch->pcdata->learned[sn] = value;
		break;
	    }
	    
	    if ( !str_cmp(word, "Social") )
	    {
		int territory;
		int social_class;
		char *territory_name;
		
		social_class = fread_number( fp );
		territory_name = fread_string( fp );
		territory = territory_lookup( territory_name );
		
		if ( territory == -1 )
		    bug( "Fread_char: unknown territory - %s.", (long) territory_name );
		else
		    ch->pcdata->social_classes[territory] = social_class;
		
		free_string( territory_name );
		fMatch = TRUE;
	    }
	    break;

	case 'T':
	    KEY( "TIn",		ch->pcdata->transin,	fread_string( fp ) );
	    KEY( "TOut",	ch->pcdata->transout,	fread_string( fp ) );
	    KEY( "TVict",	ch->pcdata->transvict,	fread_string( fp ) );
	    KEY( "TLevl",	ch->pcdata->true_level,	fread_number( fp ) );
            KEY( "TrueSex",     ch->pcdata->true_sex,	fread_number( fp ) );
	    KEY( "TSex",	ch->pcdata->true_sex,	fread_number( fp ) );
	    KEY( "Trai",	ch->train,		fread_number( fp ) );
	    KEY( "Trust",	ch->trust,		fread_number( fp ) );
	    KEY( "Tru",		ch->trust,		fread_number( fp ) );

	    if ( !str_cmp( word, "Title" )  || !str_cmp( word, "Titl"))
	    {
		ch->pcdata->title = fread_string( fp );
    		if (ch->pcdata->title[0] != '.' && ch->pcdata->title[0] != ',' 
		&&  ch->pcdata->title[0] != '!' && ch->pcdata->title[0] != '?')
		{
		    sprintf( buf, " %s", ch->pcdata->title );
		    free_string( ch->pcdata->title );
		    ch->pcdata->title = str_dup( buf );
		}
		fMatch = TRUE;
		break;
	    }

	    if ( !str_cmp(word, "Terr") )
	    {
		int territory;
		int social_class;
                long territory_flags;
		char *territory_name;
		
		social_class    = fread_number( fp );
                territory_flags = fread_number( fp );
                territory_name  = fread_string( fp );
		territory = territory_lookup( territory_name );
		
		if ( territory == -1 )
		    bug( "Fread_char: unknown territory - %s.", (long) territory_name );
		else
                {
		    ch->pcdata->social_classes[territory]  = social_class;
                    ch->pcdata->territory_flags[territory] = territory_flags;
                }
		
		free_string( territory_name );
		fMatch = TRUE;
	    }
	    break;

	case 'V':
	    KEY( "Version",     ch->version,		fread_number ( fp ) );
	    KEY( "Vers",	ch->version,		fread_number ( fp ) );
	    if ( !str_cmp( word, "Vnum" ) )
	    {
		ch->pIndexData = get_mob_index( fread_number( fp ) );
		fMatch = TRUE;
		break;
	    }
	    break;

	case 'W':
	    KEY( "Wimpy",	ch->wimpy,		fread_number( fp ) );
	    KEY( "Wimp",	ch->wimpy,		fread_number( fp ) );
	    KEY( "Wizn",	ch->wiznet,		fread_flag( fp ) );
	    KEY( "Who",		ch->pcdata->who_name,	fread_string( fp ) );
	    break;
	}

	if ( !fMatch )
	{
	    bug( "Fread_char: no match (%s).", (long) word );
	    fread_to_eol( fp );
	}
    }
}

/* load a pet from the forgotten reaches */
void fread_pet( CHAR_DATA *ch, FILE *fp )
{
    char *word;
    CHAR_DATA *pet;
    bool fMatch;
    int lastlogoff = current_time;
    int percent;

    /* first entry had BETTER be the vnum or we barf */
    word = feof(fp) ? "END" : fread_word(fp);
    if (!str_cmp(word,"Vnum"))
    {
    	int vnum;
    	
    	vnum = fread_number(fp);
    	if (get_mob_index(vnum) == NULL)
	{
    	    bug("Fread_pet: bad vnum %d.",vnum);
	    pet = create_mobile(get_mob_index(MOB_VNUM_FIDO));
	}
    	else
    	    pet = create_mobile(get_mob_index(vnum));
    }
    else
    {
        bug("Fread_pet: no vnum in file.",0);
        pet = create_mobile(get_mob_index(MOB_VNUM_FIDO));
    }
    
    for ( ; ; )
    {
    	word 	= feof(fp) ? "END" : fread_word(fp);
    	fMatch = FALSE;
    	
    	switch (UPPER(word[0]))
    	{
    	case '*':
    	    fMatch = TRUE;
    	    fread_to_eol(fp);
    	    break;
    		
    	case 'A':
    	    KEY( "Act",		pet->act,		fread_flag(fp));
    	    KEY( "AfBy",	pet->affected_by,	fread_flag(fp));
    	    KEY( "AfBy2",	pet->affected_by2,	fread_flag(fp));
    	    KEY( "Alig",	pet->alignment,		fread_number(fp));
    	    
	    if (!str_cmp(word,"ACs"))
	    {
		int i;
		
		for (i = 0; i < 4; i++)
		    pet->armor[i] = fread_number(fp);
		fMatch = TRUE;
		break;
	    }
	    
	    if (!str_cmp(word,"AffD"))
	    {
		AFFECT_DATA *paf;
		int sn;
		
		paf = new_affect();
		
		sn = skill_lookup(fread_word(fp));
		if (sn < 0)
		    bug("Fread_char: unknown skill.",0);
		else
		   paf->type = sn;
		
		paf->where	= TO_AFFECTS; 
		paf->level	= fread_number(fp);
		paf->duration	= fread_number(fp);
		paf->modifier	= fread_number(fp);
		paf->location	= fread_number(fp);
		paf->bitvector	= fread_number(fp);
		paf->procNo	= 0;
		paf->flags	= 0;
		paf->name	= str_dup( "" );
		paf->appearance = str_dup( "" );
		paf->next	= pet->affected;
		pet->affected	= paf;
		fMatch		= TRUE;
		break;
	    }

	    if (!str_cmp(word,"Affc"))
	    {
		AFFECT_DATA *paf;
		int sn;
 
		paf = new_affect();
 
		sn = skill_lookup(fread_word(fp));
		if (sn < 0)
		    bug("Fread_char: unknown skill.",0);
		else
		   paf->type = sn;
 
		paf->where		= fread_number(fp);
		paf->level	= fread_number(fp);
		paf->duration	= fread_number(fp);
		paf->modifier	= fread_number(fp);
		paf->location	= fread_number(fp);
		paf->bitvector	= fread_number(fp);
		paf->procNo	= 0;
		paf->flags	= 0;
		paf->name	= str_dup( "" );
		paf->appearance = str_dup( "" );
		paf->next	= pet->affected;
		pet->affected	= paf;
		fMatch		= TRUE;
		break;
	    }

	    if (!str_cmp(word, "Affp"))
	    {
		AFFECT_DATA *paf = fread_affect( fp, ch->version );;
	
		paf->next     = pet->affected;
		pet->affected = paf;
		fMatch = TRUE;
		break;
	    }
	     
	    if (!str_cmp(word,"AMod"))
	    {
		int stat;
		
		for (stat = 0; stat < MAX_STATS; stat++)
		    pet->mod_stat[stat] = fread_number(fp);
		fMatch = TRUE;
		break;
	    }
	     
	    if (!str_cmp(word,"Attr"))
	    {
		 int stat;
		 
		 for (stat = 0; stat < MAX_STATS; stat++)
		     pet->perm_stat[stat] = fread_number(fp);
		 fMatch = TRUE;
		 break;
	    }
	    break;
	     
	 case 'C':
	    KEY( "Clan",	pet->clan,		clan_lookup(fread_string(fp)) );
	    KEY( "Comm",	pet->comm,		fread_flag(fp) );
	    KEYS( "CurApp",	ch->appearance,		fread_string(fp) );
	    break;
	     
	 case 'D':
	     KEY( "Dam",	pet->damroll,		fread_number(fp));
	     KEY( "Desc",	pet->description,	fread_string(fp));
	     break;
	     
	 case 'E':
	     if (!str_cmp(word,"End"))
	     {
		pet->leader = ch;
		pet->master = ch;
		ch->pet = pet;
		/* adjust hp mana move up  -- here for speed's sake */
		percent = (current_time - lastlogoff) * 25 / ( 2 * 60 * 60);
 
		if (percent > 0 && !IS_AFFECTED(ch,AFF_POISON)
		&&  !IS_AFFECTED(ch,AFF_PLAGUE))
		{
		    percent = UMIN(percent,100);
		    pet->hit	+= (pet->max_hit - pet->hit) * percent / 100;
		    pet->mana	+= (pet->max_mana - pet->mana) * percent / 100;
		    pet->move	+= (pet->max_move - pet->move)* percent / 100;
		}
		return;
	     }
	     KEY( "Exp",	pet->exp,		fread_number(fp));
	     break;
	     
	 case 'G':
	     KEY( "Gold",	pet->gold,		fread_number(fp));
	     break;
	     
	 case 'H':
	     KEY( "Hit",	pet->hitroll,		fread_number(fp));
	     
	     if (!str_cmp(word,"HMV"))
	     {
		pet->hit	= fread_number(fp);
		pet->max_hit	= fread_number(fp);
		pet->mana	= fread_number(fp);
		pet->max_mana	= fread_number(fp);
		pet->move	= fread_number(fp);
		pet->max_move	= fread_number(fp);
		fMatch = TRUE;
		break;
	     }
	     break;
	     
	case 'L':
	     KEY( "Levl",	pet->level,		fread_number(fp));
	     KEY( "LnD",	pet->long_descr,	fread_string(fp));
	     KEY( "LogO",	lastlogoff,		fread_number(fp));
	     break;
	     
	case 'N':
	     KEY( "Name",	pet->name,		fread_string(fp));
	     break;
	     
	case 'P':
	    KEY( "Pos",	pet->position,		fread_number(fp));
            if ( !str_cmp(word, "Poly") )
            {
                int vnum = fread_number( fp );
                pet->polymorphed = get_mob_index( vnum );
                fMatch = TRUE;
                break;
            }
	    
	    break;
	     
	case 'R':
	    KEY( "Race",	pet->race, race_lookup(fread_string(fp)));

	    if ( !str_cmp( word, "Room" ) )
	    {
		pet->in_room = get_room_index( fread_number( fp ) );
		if ( pet->in_room == NULL )
		    pet->in_room = get_room_index( ROOM_VNUM_LIMBO );
		fMatch = TRUE;
		break;
	    }
	    break;

	case 'S' :
	    KEY( "Save",	pet->saving_throw,	fread_number(fp));
	    KEY( "Sex",		pet->sex,		fread_number(fp));
	    KEY( "ShD",		pet->short_descr,	fread_string(fp));
	    KEY( "Silv",	pet->silver,		fread_number( fp ) );
	    break;
	    
	if ( !fMatch )
	{
	    bug("Fread_pet: no match (%s).", (long) word);
	    fread_to_eol(fp);
	}
	
	}
    }
}



void fread_obj( CHAR_DATA *ch, FILE *fp, bool load_rooms )
{
    ROOM_INDEX_DATA *to_room;
    OBJ_DATA *obj;
    char *word;
    int iNest;
    bool fMatch;
    bool fNest;
    bool fVnum;
    bool first;
    bool new_format;  /* to prevent errors */
    bool make_new;    /* update object */
    
    obj = NULL;
    first = TRUE;  /* used to counter fp offset */
    fVnum = FALSE;
    to_room = NULL;
    make_new = FALSE;
    new_format = FALSE;

    word   = feof( fp ) ? "End" : fread_word( fp );
    if (!str_cmp(word,"Vnum" ))
    {
        int vnum;
	first = FALSE;  /* fp will be in right place */
 
        vnum = fread_number( fp );
        if (  get_obj_index( vnum )  == NULL )
	{
            bug( "Fread_obj: bad vnum %d.", vnum );
	}
        else
	{
	    obj = create_object(get_obj_index(vnum),-1, FALSE);
	    new_format = TRUE;
	}
	    
    }

    if (obj == NULL)  /* either not found or old style */
    {
    	obj = new_obj();
    	obj->name		= str_dup( "" );
    	obj->short_descr	= str_dup( "" );
    	obj->description	= str_dup( "" );
    }

    fNest		= FALSE;
    fVnum		= TRUE;
    iNest		= 0;

    for ( ; ; )
    {
	if (first)
	    first = FALSE;
	else
	    word   = feof( fp ) ? "End" : fread_word( fp );
	fMatch = FALSE;

	switch ( UPPER(word[0]) )
	{
	case '*':
	    fMatch = TRUE;
	    fread_to_eol( fp );
	    break;

	case 'A':
	    if (!str_cmp(word,"AffD"))
	    {
		AFFECT_DATA *paf;
		int sn;

		paf = new_affect();

		sn = skill_lookup(fread_word(fp));
		if (sn < 0)
		    bug("Fread_obj: unknown skill.",0);
		else
		    paf->type = sn;

		paf->where	= TO_AFFECTS;
		paf->level	= fread_number( fp );
		paf->duration	= fread_number( fp );
		paf->modifier	= fread_number( fp );
		paf->location	= fread_number( fp );
		paf->bitvector	= fread_number( fp );
		paf->flags	= 0;
		paf->procNo	= 0;
		paf->name	= str_dup( "" );
		paf->appearance	= str_dup( "" );
		paf->next	= obj->affected;
		obj->affected	= paf;
		fMatch		= TRUE;
		break;
	    }
	    
	    if (!str_cmp(word,"Affc"))
	    {
		AFFECT_DATA *paf;
		int sn;
 
		paf = new_affect();
 
		sn = skill_lookup(fread_word(fp));
		if (sn < 0)
		    bug("Fread_obj: unknown skill.",0);
		else
		    paf->type = sn;
 
		paf->where	= fread_number( fp );
		paf->level      = fread_number( fp );
		paf->duration   = fread_number( fp );
		paf->modifier   = fread_number( fp );
		paf->location   = fread_number( fp );
		paf->bitvector  = fread_number( fp );
		paf->procNo	= 0;
		paf->flags	= 0;
		paf->name	= str_dup( "" );
		paf->appearance	= str_dup( "" );
		paf->next       = obj->affected;
		obj->affected   = paf;
		fMatch          = TRUE;
		break;
	    }

	    if (!str_cmp(word,"Affp"))
	    {
		AFFECT_DATA *paf	= fread_affect( fp, ch->version );

		paf->next		= obj->affected;
		obj->affected		= paf;
		fMatch			= TRUE;
		break;
	    }
	    break;

	case 'C':
	    KEY( "Cond",	obj->condition,		fread_number( fp ) );
	    KEY( "Cost",	obj->cost,		fread_number( fp ) );
	    break;

	case 'D':
	    KEY( "Description",	obj->description,	fread_string( fp ) );
	    KEY( "Desc",	obj->description,	fread_string( fp ) );
	    break;

	case 'E':

	    if ( !str_cmp( word, "Enchanted"))
	    {
		obj->enchanted = TRUE;
	 	fMatch 	= TRUE;
		break;
	    }

	    KEY( "ExtraFlags",	obj->extra_flags,	fread_number( fp ) );
	    KEY( "ExtF",	obj->extra_flags,	fread_number( fp ) );

	    if ( !str_cmp( word, "ExtraDescr" ) || !str_cmp(word,"ExDe"))
	    {
		EXTRA_DESCR_DATA *ed;

		ed = new_extra_descr();

		ed->keyword		= fread_string( fp );
		ed->description		= fread_string( fp );
		ed->next		= obj->extra_descr;
		obj->extra_descr	= ed;
		fMatch = TRUE;
	    }

	    if ( !str_cmp( word, "End" ) )
	    {
		if ( !fNest || !fVnum || obj->pIndexData == NULL)
		{
		    bug( "Fread_obj: incomplete object.", 0 );
		    free_obj(obj);
		    return;
		}
		else
		{
		    if (!new_format)
		    {
			obj->next	= object_list;
			object_list	= obj;
			obj->pIndexData->count++;
		    }

		    if (!obj->pIndexData->new_format 
		    && obj->item_type == ITEM_ARMOR
		    &&  obj->value[1] == 0)
		    {
			obj->value[1] = obj->value[0];
			obj->value[2] = obj->value[0];
		    }

		    if (make_new)
		    {
			int wear;
			
			wear = obj->wear_loc;
			extract_obj(obj);

			obj = create_object(obj->pIndexData,0, FALSE);
			obj->wear_loc = wear;
		    }
		    else if ( HAS_TRIGGER(obj, OTRIG_RANDOM) )
		    {
			PERIODIC_CHECK_DATA *pcd;

			pcd			= new_periodic_check();
			pcd->data		= (void *) obj;

			pcd->next		= periodic_object_checks;
			periodic_object_checks	= pcd;
		    }

		    if ( to_room )
		    {
			if ( load_rooms )
			{
			    obj_to_room( obj, to_room );
			
			    if ( to_room->people )
			    {
				act( "$p rises from the ground.",
				    to_room->people, obj, NULL, TO_CHAR );
				act( "$p rises from the ground.",
				    to_room->people, obj, NULL, TO_ROOM );
                            }
                        }
                        else
                            extract_obj( obj );
		    }
		    else if ( iNest == 0 || rgObjNest[iNest] == NULL )
		    {
			if ( obj->item_type == ITEM_ARMOR
			&&   obj->wear_loc != WEAR_NONE )
			    ch->armor_weight += obj->weight;
			
			obj_to_char( obj, ch );

			if ( ch->version <= 6 )
			    obj->wear_loc = WEAR_NONE;
		    }
		    else
			obj_to_obj( obj, rgObjNest[iNest-1] );
		    return;
		}
	    }
	    break;

	case 'I':
	    KEY( "ItemType",	obj->item_type,		fread_number( fp ) );
	    KEY( "Ityp",	obj->item_type,		fread_number( fp ) );
	    break;

	case 'L':
	    KEY( "Level",	obj->level,		fread_number( fp ) );
	    KEY( "Lev",		obj->level,		fread_number( fp ) );
	    break;

	case 'N':
	    KEY( "Name",	obj->name,		fread_string( fp ) );

	    if ( !str_cmp( word, "Nest" ) )
	    {
		iNest = fread_number( fp );
		if ( iNest < 0 || iNest >= MAX_NEST )
		{
		    bug( "Fread_obj: bad nest %d.", iNest );
		}
		else
		{
		    rgObjNest[iNest] = obj;
		    fNest = TRUE;
		}
		fMatch = TRUE;
	    }
	    break;

   	case 'O':
	    if ( !str_cmp( word,"Oldstyle" ) )
	    {
		if (obj->pIndexData != NULL && obj->pIndexData->new_format)
		    make_new = TRUE;
		fMatch = TRUE;
	    }

	    if ( !str_cmp( word, "Owner" ) )
	    {
		obj->owner = fread_string( fp );
		fMatch = TRUE;
		break;
	    }
	    break;

	case 'R':
	    if ( !str_cmp( word, "Room") )
	    {
		to_room = get_room_index( fread_number(fp) );
		fMatch  = TRUE;
	    }
	    break;

	case 'S':
	    KEY( "ShortDescr",	obj->short_descr,	fread_string( fp ) );
	    KEY( "ShD",		obj->short_descr,	fread_string( fp ) );

	    if ( !str_cmp( word, "Spell" ) )
	    {
		int iValue;
		int sn;

		iValue = fread_number( fp );
		sn     = skill_lookup( fread_word( fp ) );
		if ( iValue < 0 || iValue > 3 )
		{
		    bug( "Fread_obj: bad iValue %d.", iValue );
		}
		else if ( sn < 0 )
		{
		    bug( "Fread_obj: unknown skill.", 0 );
		}
		else
		{
		    obj->value[iValue] = sn;
		}
		fMatch = TRUE;
		break;
	    }

	    break;

	case 'T':
	    KEY( "Timer",	obj->timer,		fread_number( fp ) );
	    KEY( "Time",	obj->timer,		fread_number( fp ) );
	    break;

	case 'V':
	    if ( !str_cmp( word, "Values" ) || !str_cmp(word,"Vals"))
	    {
		obj->value[0]	= fread_number( fp );
		obj->value[1]	= fread_number( fp );
		obj->value[2]	= fread_number( fp );
		obj->value[3]	= fread_number( fp );
		if (obj->item_type == ITEM_WEAPON && obj->value[0] == 0)
		   obj->value[0] = obj->pIndexData->value[0];
		fMatch		= TRUE;
		break;
	    }

	    if ( !str_cmp( word, "Val" ) )
	    {
		obj->value[0] 	= fread_number( fp );
	 	obj->value[1]	= fread_number( fp );
	 	obj->value[2] 	= fread_number( fp );
		obj->value[3]	= fread_number( fp );
		obj->value[4]	= fread_number( fp );
		fMatch = TRUE;
		break;
	    }

	    if ( !str_cmp( word, "Vnum" ) )
	    {
		int vnum;

		vnum = fread_number( fp );
		if ( ( obj->pIndexData = get_obj_index( vnum ) ) == NULL )
		    bug( "Fread_obj: bad vnum %d.", vnum );
		else
		    fVnum = TRUE;
		fMatch = TRUE;
		break;
	    }
	    break;

	case 'W':
	    KEY( "WearFlags",	obj->wear_flags,	fread_number( fp ) );
	    KEY( "WeaF",	obj->wear_flags,	fread_number( fp ) );

	    if ( !str_cmp( word, "WearLoc" ) || !str_cmp(word,"Wear"))
	    {
		int slotnum = 0;
		slotnum = fread_number( fp );
		if ( ch->version < 8 ) { // new slots
		    if ( slotnum < 7 )
			obj->wear_loc = slotnum;
		    else if ( slotnum == 7 )
			obj->wear_loc = 9;
		    else if ( slotnum == 8 )
			obj->wear_loc = 11;
		    else if ( slotnum == 9 )
			obj->wear_loc = 8;
		    else if ( slotnum == 10 )
			obj->wear_loc = 12;
		    else if ( (slotnum > 10) && (slotnum < 21) )
			obj->wear_loc = (slotnum + 3);
		    else if ( (slotnum == 21) || (slotnum == 22) )
			obj->wear_loc = (slotnum + 4);
		    else {
			fMatch = FALSE;
			break;
		    }
		    fMatch = TRUE;
		    break;
		} else {
		    obj->wear_loc = slotnum;
		    fMatch = TRUE;
		    break;
		}
	    }

	    KEY( "Weight",	obj->weight,		fread_number( fp ) );
	    KEY( "Wt",		obj->weight,		fread_number( fp ) );
	    break;

	}

	if ( !fMatch )
	{
	    bug( "Fread_obj: no match (%s).", (long) word );
	    fread_to_eol( fp );
	}
    }
}


AFFECT_DATA *fread_affect( FILE *fp, int version )
{
    AFFECT_DATA *paf;
    int sn;

    paf = new_affect();
	
    sn = skill_lookup(fread_word(fp));
    if (sn < 0)
	bug("Fread_char: unknown skill.",0);
    else
	paf->type = sn;
 
    paf->where		= fread_number( fp );
    paf->level		= fread_number( fp );
    paf->duration	= fread_number( fp );
    paf->modifier	= fread_number( fp );
    paf->location	= fread_number( fp );
    paf->bitvector	= fread_number( fp );
    paf->procNo		= affect_proc_lookup(fread_word(fp));
    paf->flags		= fread_number( fp );
    paf->name		= fread_string( fp );
    if ( version >= 6 )
	paf->appearance	= fread_string( fp );

    return paf;
}

	



