/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <time.h>
#include "merc.h"
#include "tables.h"

/* for clans */
const struct clan_type clan_table[MAX_CLAN] =
{
    /*  name,		who entry,		death-transfer,  independent, show_leaders, clan_hidden */
    {	"",		"",			ROOM_VNUM_ALTAR, TRUE,	FALSE,  FALSE	},
    {	"loner",	"[ Loner ] ",		ROOM_VNUM_ALTAR, TRUE,	FALSE,  FALSE	},
    {	"cernunnos",	"[ Cernunnos ]",	ROOM_VNUM_ALTAR, FALSE, TRUE,   FALSE	},
    {	"arkham",	"[ Arkham ]",		ROOM_VNUM_ALTAR, FALSE,	FALSE,  FALSE	},
    {	"invictus",	"[ Invictus ]",		ROOM_VNUM_ALTAR, FALSE, FALSE,  FALSE	},
    {	"honor",	"[ Honor ]",		ROOM_VNUM_ALTAR, FALSE,	TRUE,   FALSE	},
    {	"darkbreed",	"[ DarkBreed ]",	ROOM_VNUM_ALTAR, FALSE,	TRUE,   FALSE	},
    {   "sharper",      "[ Sharper ]",          ROOM_VNUM_ALTAR, FALSE, FALSE,  TRUE    },
    {   "ahrenreth",    "[ Jerak Ahrenreth ]",  ROOM_VNUM_ALTAR, FALSE, FALSE,  TRUE    }
/*    {	"rom",		"[  ROM  ] ",		ROOM_VNUM_ALTAR, FALSE,	FALSE	}*/
};

/* for position */
const struct position_type position_table[] =
{
    {	"dead",			"dead"	},
    {	"mortally wounded",	"mort"	},
    {	"incapacitated",	"incap"	},
    {	"stunned",		"stun"	},
    {	"sleeping",		"sleep"	},
    {	"resting",		"rest"	},
    {   "sitting",		"sit"   },
    {	"fighting",		"fight"	},
    {	"standing",		"stand"	},
    {	NULL,			NULL	}
};

/* for sex */
const struct sex_type sex_table[] =
{
   {	"none"		},
   {	"male"		},
   {	"female"	},
   {	"either"	},
   {	NULL		}
};

/* for sizes */
const struct size_type size_table[] =
{ 
    {	"tiny"		},
    {	"small" 	},
    {	"medium"	},
    {	"large"		},
    {	"huge", 	},
    {	"giant" 	},
    {	NULL		}
};

/* room sector names */
const struct sector_type sector_table[] =
{
    {	"inside"		},
    {	"city"			},
    {	"field"			},
    {	"forest"		},
    {	"hills"			},
    {	"mountain"		},
    {	"water_swim"		},
    {	"water_noswim"		},
    {	"unused"		},
    {	"air"			},
    {	"desert"		},
    {   "underwater"		},
    {	"underwater_bottom"	},
    {	NULL			}
};

/* various flag tables */
const struct flag_type act_flags[] =
{
    {	"npc",			ACT_IS_NPC,		FALSE	},
    {	"sentinel",		ACT_SENTINEL,		TRUE	},
    {	"scavenger",		ACT_SCAVENGER,		TRUE	},
    {	"mount",		ACT_MOUNT,		TRUE	},
    {	"aggressive",		ACT_AGGRESSIVE,		TRUE	},
    {	"stay_area",		ACT_STAY_AREA,		TRUE	},
    {	"wimpy",		ACT_WIMPY,		TRUE	},
    {	"pet",			ACT_PET,		TRUE	},
    {	"train",		ACT_TRAIN,		TRUE	},
    {	"practice",		ACT_PRACTICE,		TRUE	},
    {	"undead",		ACT_UNDEAD,		TRUE	},
    {	"cleric",		ACT_CLERIC,		TRUE	},
    {	"mage",			ACT_MAGE,		TRUE	},
    {	"thief",		ACT_THIEF,		TRUE	},
    {	"warrior",		ACT_WARRIOR,		TRUE	},
    {	"noalign",		ACT_NOALIGN,		TRUE	},
    {	"nopurge",		ACT_NOPURGE,		TRUE	},
    {	"outdoors",		ACT_OUTDOORS,		TRUE	},
    {	"indoors",		ACT_INDOORS,		TRUE	},
    {	"healer",		ACT_IS_HEALER,		TRUE	},
    {	"gain",			ACT_GAIN,		TRUE	},
    {	"update_always",	ACT_UPDATE_ALWAYS,	TRUE	},
    {	"changer",		ACT_IS_CHANGER,		TRUE	},
    {	"banker",		ACT_IS_BANKER,		TRUE	},
    {	"meta-physician",	ACT_META,		TRUE	},
    {	"crier",		ACT_CRIER,		TRUE	},
    {	"surrenders",		ACT_SURRENDERS,		TRUE	},
    {	"no_scan",		ACT_NO_SCAN,		TRUE	},
    {   "guard",                ACT_GUARD,              TRUE    },
    {	NULL,			0,			FALSE	}
};

const struct flag_type plr_flags[] =
{
    {	"npc",			A,	FALSE	},
    {	"justicar",		B,	TRUE	},
    {	"autoassist",		C,	FALSE	},
    {	"autoexit",		D,	FALSE	},
    {	"autoloot",		E,	FALSE	},
    {	"autosac",		F,	FALSE	},
    {	"autogold",		G,	FALSE	},
    {	"autosplit",		H,	FALSE	},
    {   "nodelete",             L,      TRUE    },
    {	"holylight",		N,	FALSE	},
    {	"can_loot",		P,	FALSE	},
    {	"nosummon",		Q,	FALSE	},
    {	"nofollow",		R,	FALSE	},
    {	"permit",		U,	TRUE	},
    {	"log",			W,	FALSE	},
    {	"deny",			X,	FALSE	},
    {	"freeze",		Y,	FALSE	},
    /*{	"thief",		Z,	FALSE	},*/
    /*{	"killer",		aa,	FALSE	},*/
    {	NULL,			0,	0	}
};

const struct flag_type affect_flags[] =
{
    {	"blind",		A,	TRUE	},
    {	"invisible",		B,	TRUE	},
    {	"detect_evil",		C,	TRUE	},
    {	"detect_invis",		D,	TRUE	},
    {	"detect_magic",		E,	TRUE	},
    {	"detect_hidden",	F,	TRUE	},
    {	"detect_good",		G,	TRUE	},
    {	"sanctuary",		H,	TRUE	},
    {	"faerie_fire",		I,	TRUE	},
    {	"infrared",		J,	TRUE	},
    {	"curse",		K,	TRUE	},
    {	"paralysis",		L,	TRUE	},
    {	"poison",		M,	TRUE	},
    {	"protect_evil",		N,	TRUE	},
    {	"protect_good",		O,	TRUE	},
    {	"sneak",		P,	TRUE	},
    {	"hide",			Q,	TRUE	},
    {	"sleep",		R,	TRUE	},
    {	"charm",		S,	TRUE	},
    {	"flying",		T,	TRUE	},
    {	"pass_door",		U,	TRUE	},
    {	"haste",		V,	TRUE	},
    {	"calm",			W,	TRUE	},
    {	"plague",		X,	TRUE	},
    {	"animal_call",		Y,	FALSE	},
    {	"dark_vision",		Z,	TRUE	},
    {	"berserk",		aa,	TRUE	},
    {	"swim",			bb,	TRUE	},
    {	"regeneration",		cc,	TRUE	},
    {	"slow",			dd,	TRUE	},
    {	"detect_traps",		ee,	TRUE	},
    {	NULL,			0,	0	}
};


const struct flag_type affect2_flags[] =
{
    {	"darkness_cloud",	A,	FALSE	},
    {	"fix_soul",		B,	FALSE	},
    {	"mark_tree",		C,	FALSE	},
    {	"water_speach",		D,	TRUE	},
    {	"water_breathing",	E,	TRUE	},
    {	"water_ways",		F,	TRUE	},
    {	"smoke_parasite",	G,	FALSE	},
    {	"spell_steal",		H,	FALSE	},
    {	"stolen_spell",		I,	FALSE	},
    {	"reflection",		J,	TRUE	},
    {	"stabilize_mana",	K,	FALSE	},
    {	"domination",		L,	FALSE	},
    {	"hallucination",	M,	TRUE	},
    {	"detect_illusion",	N,	TRUE	},
    {	"pacifist",		O,	TRUE	},
    {	"mirror_image",		P,	FALSE	},
    {	"delude",		Q,	TRUE	},
    {	"improved_invis",	R,	TRUE	},
    {	"alkar",		S,	TRUE	},
    {	"death_trance",		T,	TRUE	},
    {	"corpse_form",		U,	FALSE	},
    {	NULL,			0,	0	}
};


const struct flag_type off_flags[] =
{
    {	"area_attack",		A,	TRUE	},
    {	"backstab",		B,	TRUE	},
    {	"bash",			C,	TRUE	},
    {	"berserk",		D,	TRUE	},
    {	"disarm",		E,	TRUE	},
    {	"dodge",		F,	TRUE	},
    {	"fade",			G,	TRUE	},
    {	"fast",			H,	TRUE	},
    {	"kick",			I,	TRUE	},
    {	"dirt_kick",		J,	TRUE	},
    {	"parry",		K,	TRUE	},
    {	"rescue",		L,	TRUE	},
    {	"tail",			M,	TRUE	},
    {	"trip",			N,	TRUE	},
    {	"crush",		O,	TRUE	},
    {	"assist_all",		P,	TRUE	},
    {	"assist_align",		Q,	TRUE	},
    {	"assist_race",		R,	TRUE	},
    {	"assist_players",	S,	TRUE	},
    {	"assist_guard",		T,	TRUE	},
    {	"assist_vnum",		U,	TRUE	},
    {	"hunter",		V,	TRUE	},
    {	"unmount",		W,	TRUE	},
    {   "nasty_hunter",         X,      TRUE    },
    {	NULL,			0,	0	}
};

const struct flag_type imm_flags[] =
{
    {	"summon",		A,	TRUE	},
    {	"charm",		B,	TRUE	},
    {	"magic",		C,	TRUE	},
    {	"weapon",		D,	TRUE	},
    {	"bash",			E,	TRUE	},
    {	"pierce",		F,	TRUE	},
    {	"slash",		G,	TRUE	},
    {	"fire",			H,	TRUE	},
    {	"cold",			I,	TRUE	},
    {	"lightning",		J,	TRUE	},
    {	"acid",			K,	TRUE	},
    {	"poison",		L,	TRUE	},
    {	"negative",		M,	TRUE	},
    {	"holy",			N,	TRUE	},
    {	"energy",		O,	TRUE	},
    {	"mental",		P,	TRUE	},
    {	"disease",		Q,	TRUE	},
    {	"drowning",		R,	TRUE	},
    {	"light",		S,	TRUE	},
    {	"sound",		T,	TRUE	},
    {	"wood",			X,	TRUE	},
    {	"silver",		Y,	TRUE	},
    {	"iron",			Z,	TRUE	},
    {	NULL,			0,	0	}
};

const struct flag_type form_flags[] =
{
    {	"edible",		FORM_EDIBLE,		TRUE	},
    {	"poison",		FORM_POISON,		TRUE	},
    {	"magical",		FORM_MAGICAL,		TRUE	},
    {	"instant_decay",	FORM_INSTANT_DECAY,	TRUE	},
    {	"other",		FORM_OTHER,		TRUE	},
    {	"animal",		FORM_ANIMAL,		TRUE	},
    {	"sentient",		FORM_SENTIENT,		TRUE	},
    {	"undead",		FORM_UNDEAD,		TRUE	},
    {	"construct",		FORM_CONSTRUCT,		TRUE	},
    {	"mist",			FORM_MIST,		TRUE	},
    {	"intangible",		FORM_INTANGIBLE,	TRUE	},
    {	"biped",		FORM_BIPED,		TRUE	},
    {	"centaur",		FORM_CENTAUR,		TRUE	},
    {	"insect",		FORM_INSECT,		TRUE	},
    {	"spider",		FORM_SPIDER,		TRUE	},
    {	"crustacean",		FORM_CRUSTACEAN,	TRUE	},
    {	"worm",			FORM_WORM,		TRUE	},
    {	"blob",			FORM_BLOB,		TRUE	},
    {	"mammal",		FORM_MAMMAL,		TRUE	},
    {	"bird",			FORM_BIRD,		TRUE	},
    {	"reptile",		FORM_REPTILE,		TRUE	},
    {	"snake",		FORM_SNAKE,		TRUE	},
    {	"dragon",		FORM_DRAGON,		TRUE	},
    {	"amphibian",		FORM_AMPHIBIAN,		TRUE	},
    {	"fish",			FORM_FISH ,		TRUE	},
    {	"cold_blood",		FORM_COLD_BLOOD,	TRUE	},
    {	NULL,			0,			0	}
};

const struct flag_type part_flags[] =
{
    {	"head",			PART_HEAD,		TRUE	},
    {	"arms",			PART_ARMS,		TRUE	},
    {	"legs",			PART_LEGS,		TRUE	},
    {	"heart",		PART_HEART,		TRUE	},
    {	"brains",		PART_BRAINS,		TRUE	},
    {	"guts",			PART_GUTS,		TRUE	},
    {	"hands",		PART_HANDS,		TRUE	},
    {	"feet",			PART_FEET,		TRUE	},
    {	"fingers",		PART_FINGERS,		TRUE	},
    {	"ear",			PART_EAR,		TRUE	},
    {	"eye",			PART_EYE,		TRUE	},
    {	"long_tongue",		PART_LONG_TONGUE,	TRUE	},
    {	"eyestalks",		PART_EYESTALKS,		TRUE	},
    {	"tentacles",		PART_TENTACLES,		TRUE	},
    {	"fins",			PART_FINS,		TRUE	},
    {	"wings",		PART_WINGS,		TRUE	},
    {	"tail",			PART_TAIL,		TRUE	},
    {	"claws",		PART_CLAWS,		TRUE	},
    {	"fangs",		PART_FANGS,		TRUE	},
    {	"horns",		PART_HORNS,		TRUE	},
    {	"scales",		PART_SCALES,		TRUE	},
    {	"tusks",		PART_TUSKS,		TRUE	},
    {	NULL,			0,			0	}
};

const struct act_type act_table[] =
{
    { "mage",			ACT_MAGE,		10	},
    { "cleric",			ACT_CLERIC,		10	},
    { "warrior",		ACT_WARRIOR,		5	},
    { "thief",			ACT_THIEF,		5	},
    { "undead",			ACT_UNDEAD,		10	},
    { NULL,			0, 			0	}
};

const struct off_type off_table[] =
{
    { "area attack",		OFF_AREA_ATTACK,	7	},
    { "bash",                   OFF_BASH,               3       },
    { "berserk",                OFF_BERSERK,            2       },
    { "fast",                   OFF_FAST,               7       },
    { "dirt_kick",              OFF_KICK_DIRT,          3       },
    { "trip",                   OFF_TRIP,               3       },
    { "crush",                  OFF_CRUSH,              2       },
    { NULL,			0, 			0	}
};

const struct aff_type aff_table[] =
{
    { "sanctuary",		AFF_SANCTUARY,		15	},
    { "haste",			AFF_HASTE,		10	},
    { "regeneration",           AFF_REGENERATION,       2       },
    { NULL,			0,			0	}
};

const struct flag_type comm_flags[] =
{
    {	"quiet",		COMM_QUIET,		TRUE	},
    {   "deaf",			COMM_DEAF,		TRUE	},
    {   "nowiz",		COMM_NOWIZ,		TRUE	},
    {   "noclangossip",		COMM_NOAUCTION,		TRUE	},
    {   "nogossip",		COMM_NOGOSSIP,		TRUE	},
    {   "noquestion",		COMM_NOQUESTION,	TRUE	},
    {   "nomusic",		COMM_NOMUSIC,		TRUE	},
    {   "noclan",		COMM_NOCLAN,		TRUE	},
    {   "noquote",		COMM_NOQUOTE,		TRUE	},
    {   "shoutsoff",		COMM_SHOUTSOFF,		TRUE	},
    {   "compact",		COMM_COMPACT,		TRUE	},
    {   "brief",		COMM_BRIEF,		TRUE	},
    {   "prompt",		COMM_PROMPT,		TRUE	},
    {   "combine",		COMM_COMBINE,		TRUE	},
    {   "telnet_ga",		COMM_TELNET_GA,		TRUE	},
    {   "show_affects",		COMM_SHOW_AFFECTS,	TRUE	},
    {   "nograts",		COMM_NOGRATS,		TRUE	},
    {   "noemote",		COMM_NOEMOTE,		FALSE	},
    {   "noshout",		COMM_NOSHOUT,		FALSE	},
    {   "notell",		COMM_NOTELL,		FALSE	},
    {   "nochannels",		COMM_NOCHANNELS,	FALSE	},
    {   "snoop_proof",		COMM_SNOOP_PROOF,	FALSE	},
    {   "afk",			COMM_AFK,		TRUE	},
    {	NULL,			0,			0	}
};


const struct flag_type aggression_flags[] =
{
    {   "cast_mage",		CHAR_CAST_MAGE,		TRUE	},
    {   "cast_cleric",		CHAR_CAST_CLERIC,	TRUE	},
    {   "non_caster",		CHAR_NON_CASTER,	TRUE	},
    {   "semi_caster",		CHAR_SEMI_CASTER,	TRUE	},
    {   "pure_caster",		CHAR_PURE_CASTER,	TRUE	},
    {   "male",			CHAR_MALE,		TRUE	},
    {   "female",		CHAR_FEMALE,		TRUE	},
    {   "eunich",		CHAR_EUNICH,		TRUE	},
    {   "good",			CHAR_GOOD,		TRUE	},
    {   "neutral",		CHAR_NEUTRAL,		TRUE	},
    {   "evil",			CHAR_EVIL,		TRUE	},
    {   "human",		CHAR_HUMAN,		TRUE	},
    {   "elf",			CHAR_ELF,		TRUE	},
    {   "dwarf",		CHAR_DWARF,		TRUE	},
    {   "giant",		CHAR_GIANT,		TRUE	},
    {   "centaur",		CHAR_CENTAUR,		TRUE	},
    {   "gnome",		CHAR_GNOME,		TRUE	},
    {   "orc",			CHAR_ORC,		TRUE	},
    {   "cambion",		CHAR_CAMBION,		TRUE	},
    {   "everyone",		AGG_EVERYONE,		TRUE	},
    {   "chicken",		AGG_CHICKEN,		TRUE	},
    {   "psycho",		AGG_PSYCHO,		TRUE	},
    {	"non-ruler",		CHAR_NON_RULER,		TRUE	},
    {	"noble",		CHAR_NOBLE,		TRUE	},
    {	"peasant",		CHAR_PEASANT,		TRUE	},
    {	NULL,			0,			0	}
};


const struct flag_type extra_flags[] =
{
    {	"glow",			ITEM_GLOW,		TRUE	},
    {	"hum",			ITEM_HUM,		TRUE	},
    {	"dark",			ITEM_DARK,		TRUE	},
    {	"lock",			ITEM_LOCK,		TRUE	},
    {	"evil",			ITEM_EVIL,		TRUE	},
    {	"invis",		ITEM_INVIS,		TRUE	},
    {	"magic",		ITEM_MAGIC,		TRUE	},
    {	"nodrop",		ITEM_NODROP,		TRUE	},
    {	"bless",		ITEM_BLESS,		TRUE	},
    {	"mirror",		ITEM_MIRROR,		TRUE	},
    {	"anti_good",		ITEM_ANTI_GOOD,		FALSE	},
    {	"anti_evil",		ITEM_ANTI_EVIL,		FALSE	},
    {	"anti_neutral",		ITEM_ANTI_NEUTRAL,	FALSE	},
    {	"noremove",		ITEM_NOREMOVE,		TRUE	},
    {	"inventory",		ITEM_INVENTORY,		FALSE	},
    {	"nopurge",		ITEM_NOPURGE,		TRUE	},
    {	"rot_death",		ITEM_ROT_DEATH,		TRUE	},
    {	"vis_death",		ITEM_VIS_DEATH,		TRUE	},
    {	"nonmetal",		ITEM_NONMETAL,		TRUE	},
    {	"nolocate",		ITEM_NOLOCATE,		TRUE	},
    {	"melt_drop",		ITEM_MELT_DROP,		TRUE	},
    {	"had_timer",		ITEM_HAD_TIMER,		FALSE	},
    {	"sell_extract",		ITEM_SELL_EXTRACT,	TRUE	},
    {	"burn_proof",		ITEM_BURN_PROOF,	TRUE	},
    {	"nouncurse",		ITEM_NOUNCURSE,		TRUE	},
    {	"nosac",		ITEM_NO_SAC,		TRUE	},
    {	"dispelled",		ITEM_DISPELLED,		FALSE	},
    {	"lodged",		ITEM_LODGED,		TRUE    },
    {	"first_socket",		ITEM_FIRST_SOCKET,	TRUE    },
    {	"second_socket",	ITEM_SECOND_SOCKET,	TRUE    },
    {	"third_socket",		ITEM_THIRD_SOCKET,	TRUE    },
    {	 NULL,			0,			0	}
};


const struct flag_type wear_flags[] =
{
    {	"take",			ITEM_TAKE,		TRUE	},
    {	"finger",		ITEM_WEAR_FINGER,	TRUE	},
    {	"neck",			ITEM_WEAR_NECK,		TRUE	},
    {	"shoulder",		ITEM_WEAR_SHOULDER,	TRUE	},
    {	"body",			ITEM_WEAR_BODY,		TRUE	},
    {	"head",			ITEM_WEAR_HEAD,		TRUE	},
    {	"legs",			ITEM_WEAR_LEGS,		TRUE	},
    {	"feet",			ITEM_WEAR_FEET,		TRUE	},
    {	"hands",		ITEM_WEAR_HANDS,	TRUE	},
    {	"arms",			ITEM_WEAR_ARMS,		TRUE	},
    {	"shield",		ITEM_WEAR_SHIELD,	TRUE	},
    {	"about",		ITEM_WEAR_ABOUT,	TRUE	},
    {	"waist",		ITEM_WEAR_WAIST,	TRUE	},
    {	"wrist",		ITEM_WEAR_WRIST,	TRUE	},
    {	"wield",		ITEM_WIELD,		TRUE	},
    {	"hold",			ITEM_HOLD,		TRUE	},
    {	"no_sac",		ITEM_OLD_NO_SAC,	TRUE	},
    {	"float",		ITEM_WEAR_FLOAT,	TRUE	},
    {	"hair",			ITEM_WEAR_HAIR,		TRUE	},
    {   "face",			ITEM_WEAR_FACE,		TRUE	},
    {	"ears",			ITEM_WEAR_EARS,		TRUE	},
    {	"lodge_leg",		ITEM_LODGE_LEG,		TRUE	},
    {	"lodge_arm",		ITEM_LODGE_ARM,		TRUE	},
    {	"lodge_rib",		ITEM_LODGE_RIB,		TRUE	},
    {	NULL,			0,			0	}
};


const struct flag_type fit_flags[] =
{
    {   "no_tiny",		FIT_NO_TINY,		TRUE	},
    {   "no_small",		FIT_NO_SMALL,		TRUE	},
    {   "no_medium",		FIT_NO_MEDIUM,		TRUE	},
    {   "no_large",		FIT_NO_LARGE,		TRUE	},
    {   "no_huge",		FIT_NO_HUGE,		TRUE	},
    {   "no_giant",		FIT_NO_GIANT,		TRUE	},
    {   "no_human",		FIT_NO_HUMAN,		TRUE	},
    {   "no_elf",		FIT_NO_ELF,		TRUE	},
    {   "no_dwarf",		FIT_NO_DWARF,		TRUE	},
    {   "no_giant_race",	FIT_NO_GIANT_RACE,	TRUE	},
    {   "no_centaur",		FIT_NO_CENTAUR,		TRUE	},
    {   "no_gnome",		FIT_NO_GNOME,		TRUE	},
    {   "no_orc",		FIT_NO_ORC,		TRUE	},
    {   "no_cambion",		FIT_NO_CAMBION,		TRUE	},
    {	NULL,			0,			0	}
};


const struct flag_type restriction_flags[] =
{
    {   "cast_mage",		CHAR_CAST_MAGE,		TRUE	},
    {   "cast_cleric",		CHAR_CAST_CLERIC,	TRUE	},
    {   "non_caster",		CHAR_NON_CASTER,	TRUE	},
    {   "semi_caster",		CHAR_SEMI_CASTER,	TRUE	},
    {   "pure_caster",		CHAR_PURE_CASTER,	TRUE	},
    {   "male",			CHAR_MALE,		TRUE	},
    {   "female",		CHAR_FEMALE,		TRUE	},
    {   "eunich",		CHAR_EUNICH,		TRUE	},
    {   "good",			CHAR_GOOD,		TRUE	},
    {   "neutral",		CHAR_NEUTRAL,		TRUE	},
    {   "evil",			CHAR_EVIL,		TRUE	},
    {   "human",		CHAR_HUMAN,		TRUE	},
    {   "elf",			CHAR_ELF,		TRUE	},
    {   "dwarf",		CHAR_DWARF,		TRUE	},
    {   "giant",		CHAR_GIANT,		TRUE	},
    {   "centaur",		CHAR_CENTAUR,		TRUE	},
    {   "gnome",		CHAR_GNOME,		TRUE	},
    {   "orc",			CHAR_ORC,		TRUE	},
    {   "cambion",		CHAR_CAMBION,		TRUE	},
    {   "harmful",		RESTRICT_HARMFUL,	TRUE	},
    {   "wasting",		RESTRICT_WASTING,	TRUE	},
    {   "corrupting",		RESTRICT_CORRUPTING,	TRUE	},
    {	"non-ruler",		CHAR_NON_RULER,		TRUE	},
    {	"noble",		CHAR_NOBLE,		TRUE	},
    {	"peasant",		CHAR_PEASANT,		TRUE	},
    {	NULL,			0,			0	}
};


const struct flag_type furniture_flags[] =
{
    {	"stand_at",		STAND_AT,		TRUE	},
    {	"stand_on",		STAND_ON,		TRUE	},
    {	"stand_in",		STAND_IN,		TRUE	},
    {	"sit_at",		SIT_AT	,		TRUE	},
    {	"sit_on",		SIT_ON,			TRUE	},
    {	"sit_in",		SIT_IN	,		TRUE	},
    {	"rest_at",		REST_AT,		TRUE	},
    {	"rest_on",		REST_ON,		TRUE	},
    {	"rest_in",		REST_IN,		TRUE	},
    {	"sleep_at",		SLEEP_AT,		TRUE	},
    {	"sleep_on",		SLEEP_ON,		TRUE	},
    {	"sleep_in",		SLEEP_IN,		TRUE	},
    {	"put_at",		PUT_AT,			TRUE	},
    {	"put_on",		PUT_ON,			TRUE	},
    {	"put_in",		PUT_IN,			TRUE	},
    {	"put_inside",		PUT_INSIDE,		TRUE	},
    {	NULL,			0,			0	}
};


const struct flag_type weapon_flags[] =
{
    {	"flaming",		WEAPON_FLAMING,		TRUE	},
    {	"frost",		WEAPON_FROST,		TRUE	},
    {	"vampiric",		WEAPON_VAMPIRIC,	TRUE	},
    {	"sharp",		WEAPON_SHARP,		TRUE	},
    {	"vorpal",		WEAPON_VORPAL,		TRUE	},
    {	"two_hands",		WEAPON_TWO_HANDS,	TRUE	},
    {	"shocking",		WEAPON_SHOCKING,	TRUE	},
    {	"poison",		WEAPON_POISON,		TRUE	},
    {	"fading",		WEAPON_FADING,		FALSE	},
    {   "defender",		WEAPON_DEFENDER,	FALSE	},
    {   "curse",                WEAPON_CURSE,           FALSE   },
    {   "dispel",               WEAPON_DISPEL,          TRUE    },
    {	NULL,			0,			0	}
};


const struct flag_type portal_flags[] =
{
    {	"normal",		GATE_NORMAL_EXIT,	TRUE	},
    {	"nocurse",		GATE_NOCURSE,		TRUE	},
    {	"gowith",		GATE_GOWITH,		TRUE	},
    {	"buggy",		GATE_BUGGY,		TRUE	},
    {	"random",		GATE_RANDOM,		TRUE	},
    {	"sendhome",		GATE_SENDHOME,		TRUE	},
    {	NULL,			0,			0	}
};


const struct flag_type trap_flags[] =
{
    {	"object",		TRAP_OBJ,		FALSE	},
    {	"room",			TRAP_ROOM,		FALSE	},
    {	"enter",		TRAP_ENTER,		TRUE	},
    {	"leave",		TRAP_LEAVE,		TRUE	},
    {	"open",			TRAP_OPEN,		TRUE	},
    {	"close",		TRAP_CLOSE,		TRUE	},
    {	"unlock",		TRAP_UNLOCK,		TRUE	},
    {	"pick",			TRAP_PICK,		TRUE	},
    {	"get",			TRAP_GET,		TRUE	},
    {	"put",			TRAP_PUT,		TRUE	},
    {	"examine",		TRAP_EXAMINE,		TRUE	},
    {	"north",		TRAP_N,			TRUE	},
    {	"south",		TRAP_S,			TRUE	},
    {	"east",			TRAP_E,			TRUE	},
    {	"west",			TRAP_W,			TRUE	},
    {	"up",			TRAP_U,			TRUE	},
    {	"down",			TRAP_D,			TRUE	},
    {	"northeast",		TRAP_NW,		TRUE	},
    {	"southeast",		TRAP_SW,		TRUE	},
    {	"southwest",		TRAP_SW,		TRUE	},
    {	"northwest",		TRAP_NW,		TRUE	},
    {	"clockwise",		TRAP_CW,		TRUE	},
    {	"counterclockwise",	TRAP_CCW,		TRUE	},
    {	"easy",			TRAP_EASY,		TRUE	},
    {	"hard",			TRAP_HARD,		TRUE	},
    {	"infuriating",		TRAP_INFURIATING,	TRUE	},
    {	"impossible",		TRAP_IMPOSSIBLE,	TRUE	},
    {	"magical",		TRAP_MAGICAL,		TRUE	},
    {	NULL,			0,			0	}
};


const struct flag_type room_flags[] =
{
    {	"dark",			ROOM_DARK,		TRUE	},
    {   "court",                ROOM_COURT,             TRUE    },
    {	"no_mob",		ROOM_NO_MOB,		TRUE	},
    {	"indoors",		ROOM_INDOORS,		TRUE	},
    {	"no_scan",		ROOM_NO_SCAN,		TRUE	},
    {	"private",		ROOM_PRIVATE,		TRUE	},
    {	"safe",			ROOM_SAFE,		TRUE	},
    {	"solitary",		ROOM_SOLITARY,		TRUE	},
    {	"pet_shop",		ROOM_PET_SHOP,		TRUE	},
    {	"no_recall",		ROOM_NO_RECALL,		TRUE	},
    {	"imp_only",		ROOM_IMP_ONLY,		TRUE	},
    {	"gods_only",		ROOM_GODS_ONLY,		TRUE	},
    {	"heroes_only",		ROOM_HEROES_ONLY,	TRUE	},
    {	"newbies_only",		ROOM_NEWBIES_ONLY,	TRUE	},
    {	"law",			ROOM_LAW,		TRUE	},
    {	"nowhere",		ROOM_NOWHERE,		TRUE	},
    {	"no_cast_mage",		ROOM_NO_CAST_MAGE,	TRUE	},
    {	"no_cast_cleric",	ROOM_NO_CAST_CLERIC,	TRUE	},
    {   "no_amusic",            ROOM_NO_AMUSIC,         TRUE    },
    {	NULL,			0,			0	}
};


const struct flag_type area_flags[] =
{
    {   "dirty",                AREA_DIRTY,                 FALSE	},
    /*{	"lawless",              AREA_LAWLESS,               TRUE	},*/
    {   "open_slather",         AREA_OPEN_SLATHER,          TRUE	},
    {   "virtual",              AREA_VIRTUAL,               TRUE	},
    {   "invis",                AREA_INVIS,                 TRUE	},
    {   "arena",                AREA_ARENA,		    TRUE	},
    {	"everload",		AREA_EVERLOAD,		    FALSE	},
    {	"free",			AREA_FREE,		    FALSE	},
    {	"quiet",		AREA_QUIET,		    FALSE	},
    {	"revamp",		AREA_REVAMP,		    TRUE	},
    {   NULL,                   0,                          0           }
};


const struct flag_type exit_flags[] =
{
    {	"isdoor",		EX_ISDOOR,		TRUE	},
    {	"closed",		EX_CLOSED,		TRUE	},
    {	"locked",		EX_LOCKED,		TRUE	},
    {	"pickproof",		EX_PICKPROOF,		TRUE	},
    {	"passproof",		EX_NOPASS,		TRUE	},
    {	"stay",			EX_NGBACK,		TRUE	},
    {	"easy",			EX_EASY,		TRUE	},
    {	"hard",			EX_HARD,		TRUE	},
    {	"infuriating",		EX_INFURIATING,		TRUE	},
    {	"noclose",		EX_NOCLOSE,		TRUE	},
    {	"nolock",		EX_NOLOCK,		TRUE	},
    {	NULL,			0,			0	}
};


const struct apply_type apply_table[] =
{
    {	"none",			APPLY_NONE,		},
    {	"strength",		APPLY_STR,		},
    {	"dexterity",		APPLY_DEX,		},
    {	"intelligence",		APPLY_INT,		},
    {	"wisdom",		APPLY_WIS,		},
    {	"constitution",		APPLY_CON,		},
    {	"sex",			APPLY_SEX,		},
    {	"mana",			APPLY_MANA		},
    {	"hit",			APPLY_HIT		},
    {	"move",			APPLY_MOVE		},
    {	"ac",			APPLY_AC		},
    {	"hitroll",		APPLY_HITROLL		},
    {	"damroll",		APPLY_DAMROLL		},
    {	"saves",		APPLY_SAVES		},
    {	"saves_paralyzation",	APPLY_SAVING_PARA	},
    {	"saves_rods",		APPLY_SAVING_ROD	},
    {	"saves_petrification",	APPLY_SAVING_PETRI	},
    {	"saves_breath",		APPLY_SAVING_BREATH	},
    {	"saves_spell",		APPLY_SAVING_SPELL	},
    {	"perception",		APPLY_PER		},
    {	"charisma",		APPLY_CHA		},
    {	NULL						},
};


const struct flag_type mprog_flags[] =
{
    {	"act",			TRIG_ACT,		TRUE	},
    {	"bribe",		TRIG_BRIBE,		TRUE	},
    {	"death",		TRIG_DEATH,		TRUE	},
    {	"entry",		TRIG_ENTRY,		TRUE	},
    {	"fight",		TRIG_FIGHT,		TRUE	},
    {	"give",			TRIG_GIVE,		TRUE	},
    {	"greet",		TRIG_GREET,		TRUE	},
    {	"grall",		TRIG_GRALL,		TRUE	},
    {	"kill",			TRIG_KILL,		TRUE	},
    {	"hpcnt",		TRIG_HPCNT,		TRUE	},
    {	"random",		TRIG_RANDOM,		TRUE	},
    {	"speech",		TRIG_SPEECH,		TRUE	},
    {	"exit",			TRIG_EXIT,		TRUE	},
    {	"exall",		TRIG_EXALL,		TRUE	},
    {	"delay",		TRIG_DELAY,		TRUE	},
    {	"surr",			TRIG_SURR,		TRUE	},
    {	NULL							}
};


const struct flag_type oprog_flags[] =
{
    {	"act",			OTRIG_ACT,		TRUE	},
    {	"random",		OTRIG_RANDOM,		TRUE	},
    {	"greet",		OTRIG_GREET,		TRUE	},
    {	"grall",		OTRIG_GRALL,		TRUE	},
    {	"wear",			OTRIG_WEAR,		TRUE	},
    {	"remove",		OTRIG_REMOVE,		TRUE	},
    {	"sac",			OTRIG_SAC,		TRUE	},
    {	"look",			OTRIG_LOOK,		TRUE	},
    {	"examine",		OTRIG_EXAMINE,		TRUE	},
    {	"use",			OTRIG_USE,		TRUE	},
    {	"get",			OTRIG_GET,		TRUE	},
    {	"put",			OTRIG_PUT,		TRUE	},
    {	"drop",			OTRIG_DROP,		TRUE	},
    {	"speech",		OTRIG_SPEECH,		TRUE	},
    {	"get_content",		OTRIG_GET_CONTENT,	TRUE	},
    {	"put_content",		OTRIG_PUT_CONTENT,	TRUE	},
    {	"steal",		OTRIG_STEAL,		TRUE	},
    {	"open",			OTRIG_OPEN,		TRUE	},
    {	"close",		OTRIG_CLOSE,		TRUE	},
    {	NULL							}
};


const struct flag_type rprog_flags[] =
{
    {	"act",			RTRIG_ACT,		TRUE	},
    {	"random",		RTRIG_RANDOM,		TRUE	},
    {	"greet",		RTRIG_GREET,		TRUE	},
    {	"grall",		RTRIG_GRALL,		TRUE	},
    {	"speech",		RTRIG_SPEECH,		TRUE	},
    {	"exall",		RTRIG_EXALL,		TRUE	},
    {	NULL							}
};


const struct trap_type trap_table[] =
{
    {	"arrow",		TRAP_TYPE_ARROW,		TRUE	},
    {	"poisoned arrow",	TRAP_TYPE_POISONED_ARROW,	TRUE	},
    {	"ceiling block",	TRAP_TYPE_CEILING_BLOCK,	TRUE	},
    {	"chute",		TRAP_TYPE_CHUTE,		FALSE	},
    {	"gas blinding",		TRAP_TYPE_GAS_BLINDING,		TRUE	},
    {	"gas poison",		TRAP_TYPE_GAS_POISON,		TRUE	},
    {	"gas sleep",		TRAP_TYPE_GAS_SLEEP,		TRUE	},
    {	"gas slow",		TRAP_TYPE_GAS_SLOW,		TRUE	},
    {	"gas weakness",		TRAP_TYPE_GAS_WEAKNESS,		TRUE	},
    {	"lightning",		TRAP_TYPE_LIGHTNING,		TRUE	},
    {	"blade",		TRAP_TYPE_BLADE,		TRUE	},
    {	"teleporter",		TRAP_TYPE_TELEPORTER,		FALSE	},
    {	"vent acid",		TRAP_TYPE_VENT_ACID,		TRUE	},
    {	"vent fire",		TRAP_TYPE_VENT_FIRE,		TRUE	},
    {	"dart",			TRAP_TYPE_DART,			TRUE	},
    {	"poisoned dart",	TRAP_TYPE_POISON_DART,		TRUE	},
    {	"poisoned needle",	TRAP_TYPE_POISON_NEEDLE,	TRUE	},
    {	"sex change",		TRAP_TYPE_SEX_CHANGE,		FALSE	},
    {	"petrification",	TRAP_TYPE_PETRIFICATION,	FALSE	},
    {	NULL,			0,				0	}
};


const struct flag_type crime_table[] =
{
    {	"pkilling",		CRIME_PKILLING,			TRUE	},
    {	"pstealing",		CRIME_PSTEALING,		TRUE	},
    {   "justicar_aligned",     TERRITORY_JUSTICAR_ALIGNED,     TRUE	},
    {	NULL,			0,				0	}
};


const struct flag_type territory_flags[] =
{
    {   "captain_guard",        TERR_CAPTAIN_GUARD,             TRUE    },
    {   "guard",                TERR_GUARD,                     TRUE    },
    {   "tresurer",             TERRITORY_JUSTICAR_ALIGNED,     TRUE    },
    {   NULL,                   0,                              0       }
};


const struct advantage_type advantage_table[] =
{
    { "bland", 1, ADVANTAGE_BLAND, -1, 0 },
    { "blood of mtaviar", 1, ADVANTAGE_BLOOD_OF_MTAVIAR, -1, CHAR_DWARF|CHAR_GIANT|CHAR_CENTAUR|CHAR_GNOME },
    { "clear thinker", 1, ADVANTAGE_CLEAR_THINKER, TO_RESIST, 0, RES_CHARM, "", 0 },
    { "crafty", 1, ADVANTAGE_CRAFTY, -1, 0 },
    { "danger sense", 3, ADVANTAGE_DANGER_SENSE, -1, 0 },
    { "dangerous beauty", 3, ADVANTAGE_DANGEROUS_BEAUTY, -1, 0 },
    { "eidetic memory", 3, ADVANTAGE_EIDETIC_MEMORY, -1, 0 },
    { "hands of stone", 1, ADVANTAGE_HANDS_OF_STONE, -1, 0 },
    { "high pain threshold", 2, ADVANTAGE_HIGH_PAIN_THRESHOLD, -1, 0 },
    { "large", 1, ADVANTAGE_LARGE, -1, 0 },
    { "luck", 2, ADVANTAGE_LUCK, -1, 0 },
    { "magical attunement", 1, ADVANTAGE_MAGICAL_ATTUNEMENT, TO_AFFECTS, 0, AFF_DETECT_MAGIC },
    { "meek", 2, ADVANTAGE_MEEK, -1, 0 },
    { "natural fighter", 1, ADVANTAGE_NATURAL_FIGHTER, -1, 0 },
    { "quick healer", 3, ADVANTAGE_QUICK_HEALER, -1, 0 },
    { "renshais blessing", 2, ADVANTAGE_RENSHAIS_BLESSING, -1, 0 },
    { "strong will", 2, ADVANTAGE_STRONG_WILL, -1, 0 },
    { "tough", 2, ADVANTAGE_TOUGH, -1, 0 },
    { "" }
};


const struct advantage_type disadvantage_table[] =
{
    { "agrophobia", -2, DISADVANTAGE_AGROPHOBIA, TO_AFFECTS, 0, 0, "agrophobia", AFF_CONT_UPDATE },
    { "alcoholic", -4, DISADVANTAGE_ALCOHOLIC, TO_AFFECTS, 0, 0, "dipsomania", AFF_CONT_UPDATE },
    { "bad back", -3, DISADVANTAGE_BAD_BACK, -1, 0 },
    { "bad eyesight", -2, DISADVANTAGE_BAD_EYESIGHT, -1, 0 },
    { "bad health", -2, DISADVANTAGE_BAD_HEALTH, -1, 0 },
    { "claustrophobia", -2, DISADVANTAGE_CLAUSTROPHOBIA, TO_AFFECTS, 0, 0, "claustrophobia", AFF_CONT_UPDATE },
    { "elderly", -7, DISADVANTAGE_ELDERLY, -1, 0 },
    { "introversion", -5, DISADVANTAGE_INTROVERSION, -1, 0 },
    { "hideously scared", -3, DISADVANTAGE_HIDEOUSLY_SCARED, -1, 0 },
    { "klutz", -2, DISADVANTAGE_KLUTZ, -1, 0 },
    { "lame", -2, DISADVANTAGE_LAME, -1, 0 },
    { "light intolerance", -3, DISADVANTAGE_LIGHT_INTOLERANCE, TO_AFFECTS, 0, 0, "light intolerance", AFF_CONT_UPDATE },
    { "low pain threshold", -2, DISADVANTAGE_LOW_PAIN_THRESHOLD, -1, 0 },
    { "mania", -5, DISADVANTAGE_MANIA, -1, 0 },
    { "melancholia", -2, DISADVANTAGE_MELANCHOLIA, TO_AFFECTS, 0, 0, "melancholia", AFF_CONT_UPDATE },
    { "mute", -10, DISADVANTAGE_MUTE, -1, 0 },
    { "overconfident", -3, DISADVANTAGE_OVERCONFIDENT, -1, 0 },
    { "permanent wound", -2, DISADVANTAGE_PERMANENT_WOUND, -1, 0 },
    { "renshais curse", -2, DISADVANTAGE_RENSHAIS_CURSE, -1, 0 },
    { "scared", -2, DISADVANTAGE_SCARED, -1, 0 },
    { "small", -1, DISADVANTAGE_SMALL, -1, 0 },
    { "weak willed", -2, DISADVANTAGE_WEAK_WILLED, -1, 0 },
    { "" }
};


const struct advantage_type ancestor_table[] =
{
    { "Yrngrath", -1, ANCESTOR_YRNGRATH, -1, CHAR_DWARF|CHAR_GIANT|CHAR_CENTAUR|CHAR_GNOME },
    { "Xyllian", 2, ANCESTOR_XYLLIAN, TO_RESIST, CHAR_DWARF|CHAR_GIANT|CHAR_CENTAUR|CHAR_GNOME, RES_NEGATIVE, "", 0 },
    { "Andaras", 1, ANCESTOR_ANDARAS, -1 },
    { "Baton", 0, ANCESTOR_BATON, TO_VULN, CHAR_DWARF|CHAR_GIANT|CHAR_CENTAUR|CHAR_GNOME, VULN_HOLY, "", 0 },
    { "Loranhir", 1, ANCESTOR_LORANHIR, -1, CHAR_DWARF|CHAR_GIANT|CHAR_CENTAUR|CHAR_GNOME },
    { "Eslaine", 4, ANCESTOR_ESLAINE, TO_RESIST, 0, RES_FIRE|RES_COLD, "", 0 },
    { "Fionavar", 2, ANCESTOR_FIONAVAR, -1, 0 },
    { "Sumandar", 2, ANCESTOR_SUMANDAR, -1, CHAR_DWARF|CHAR_GIANT|CHAR_CENTAUR|CHAR_GNOME },
    { "Ondoval", 0, ANCESTOR_ONDOVAL, -1, CHAR_DWARF|CHAR_GIANT|CHAR_CENTAUR|CHAR_GNOME },
    { "Ailas", 3, ANCESTOR_AILAS, -1, CHAR_DWARF|CHAR_GIANT|CHAR_CENTAUR|CHAR_GNOME },
    { "Dmoon", 2, ANCESTOR_DMOON, -1, CHAR_DWARF|CHAR_GIANT|CHAR_CENTAUR|CHAR_GNOME },
    { "Ryna", 2, ANCESTOR_RYNA, -1, CHAR_DWARF|CHAR_GIANT|CHAR_CENTAUR|CHAR_GNOME },
    { "" }
};
