/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <time.h>
#include "merc.h"
#include "magic.h"
#include "interp.h"
#include "damage.h"


#define DEFAULT_SKILL_CLASS	{ FALSE, { 0, 0 }, { 0, 0 } }
#define DEFAULT_SPELL_CLASS     { FALSE, { 0, 0 }, { 0, -25 } }


const struct use_msg_type use_msgs[] =
{
    {	"wear_loc",		USE_WEAR_LOC,		0		},
    {	"wear_self",		USE_WEAR_SELF,		0		},
    {	"wear_room",		USE_WEAR_ROOM,		0		},
    {	"remove_self",		USE_REMOVE_SELF,	0		},
    {	"remove_room",		USE_REMOVE_ROOM,	0		},
    {	"zap_self",		USE_ZAP_SELF,		ITEM_WAND	},
    {	"zap_vict",		USE_ZAP_VICT,		ITEM_WAND	},
    {	"zap_room",		USE_ZAP_ROOM,		ITEM_WAND	},
    {	"zap_obj_self",		USE_ZAP_OBJ_SELF,	ITEM_WAND	},
    {	"zap_obj_room",		USE_ZAP_OBJ_ROOM,	ITEM_WAND	},
    {	"brandish_self",	USE_BRANDISH_SELF,	ITEM_STAFF	},
    {	"brandish_room",	USE_BRANDISH_ROOM,	ITEM_STAFF	},
    {	"contain_open_self",	USE_CONTAIN_OPEN_SELF,	ITEM_CONTAINER	},
    {	"contain_open_room",	USE_CONTAIN_OPEN_ROOM,	ITEM_CONTAINER	},
    {	"contain_close_self",	USE_CONTAIN_CLOSE_SELF,	ITEM_CONTAINER	},
    {	"contain_close_room",	USE_CONTAIN_CLOSE_ROOM,	ITEM_CONTAINER	},
    {	"contain_lock_self",	USE_CONTAIN_LOCK_SELF,	ITEM_CONTAINER	},
    {	"contain_lock_room",	USE_CONTAIN_LOCK_ROOM,	ITEM_CONTAINER	},
    {	"contain_unlock_self",	USE_CONTAIN_UNLOCK_SELF,ITEM_CONTAINER	},
    {	"contain_unlock_room",	USE_CONTAIN_UNLOCK_ROOM,ITEM_CONTAINER	},
    {	"furn_sleep_self",	USE_FURN_SLEEP_SELF,	ITEM_FURNITURE	},
    {	"furn_sleep_room",	USE_FURN_SLEEP_ROOM,	ITEM_FURNITURE	},
    {	"furn_rest_self",	USE_FURN_REST_SELF,	ITEM_FURNITURE	},
    {	"furn_rest_room",	USE_FURN_REST_ROOM,	ITEM_FURNITURE	},
    {	"furn_sit_self",	USE_FURN_SIT_SELF,	ITEM_FURNITURE	},
    {	"furn_sit_room",	USE_FURN_SIT_ROOM,	ITEM_FURNITURE	},
    {	"furn_stand_self",	USE_FURN_STAND_SELF,	ITEM_FURNITURE	},
    {	"furn_stand_room",	USE_FURN_STAND_ROOM,	ITEM_FURNITURE	},
    
    {	NULL								}
};


/* item type list */
const struct item_type		item_table	[]	=
{
    {	ITEM_LIGHT,	"light"		},
    {	ITEM_SCROLL,	"scroll"	},
    {	ITEM_WAND,	"wand"		},
    {	ITEM_STAFF,	"staff"		},
    {	ITEM_WEAPON,	"weapon"	},
    {	ITEM_TREASURE,	"treasure"	},
    {	ITEM_ARMOR,	"armor"		},
    {	ITEM_POTION,	"potion"	},
    {	ITEM_CLOTHING,	"clothing"	},
    {	ITEM_FURNITURE, "furniture"	},
    {	ITEM_TRASH,	"trash"		},
    {	ITEM_CONTAINER, "container"	},
    {	ITEM_DRINK_CON, "drink"		},
    {	ITEM_KEY,	"key"		},
    {	ITEM_FOOD,	"food"		},
    {	ITEM_MONEY,	"money"		},
    {	ITEM_BOAT,	"boat"		},
    {	ITEM_CORPSE_NPC,"npc_corpse"	},
    {	ITEM_CORPSE_PC, "pc_corpse"	},
    {	ITEM_FOUNTAIN,	"fountain"	},
    {	ITEM_PILL,	"pill"		},
    {	ITEM_PROTECT,	"protect"	},
    {	ITEM_MAP,	"map"		},
    {	ITEM_PORTAL,	"portal"	},
    {	ITEM_WARP_STONE,"warp_stone"	},
    {	ITEM_ROOM_KEY,	"room_key"	},
    {	ITEM_GEM,	"gem"		},
    {	ITEM_JEWELRY,	"jewelry"	},
    {	ITEM_JUKEBOX,	"jukebox"	},
    {	ITEM_SYMBOL,	"symbol"	},
    {	ITEM_FOCUS,	"focus"		},
    {	ITEM_TRAP,	"trap"		},
    {	ITEM_POISON,	"poison"	},
    {	ITEM_INSTRUMENT,"instrument"	},
    {	ITEM_CHECKERS_BOARD, "checkers_board" },
    {	ITEM_QUIVER,	"quiver"	},
    {	ITEM_ARROW,	"arrow"		},
    {	ITEM_SOCKETED_GEM, "socketed_gem" },
    {	0,		NULL		}
};


/* weapon selection table */
const	struct	weapon_type	weapon_table	[]	=
{
   { "sword",	OBJ_VNUM_SCHOOL_SWORD,	WEAPON_SWORD,	&gsn_sword	},
   { "bow",	OBJ_VNUM_SCHOOL_BOW,	WEAPON_BOW,	&gsn_bow	},
   { "mace",	OBJ_VNUM_SCHOOL_MACE,	WEAPON_MACE,	&gsn_mace 	},
   { "dagger",	OBJ_VNUM_SCHOOL_DAGGER,	WEAPON_DAGGER,	&gsn_dagger	},
   { "axe",	OBJ_VNUM_SCHOOL_AXE,	WEAPON_AXE,	&gsn_axe	},
   { "staff",	OBJ_VNUM_SCHOOL_STAFF,	WEAPON_SPEAR,	&gsn_spear	},
   { "flail",	OBJ_VNUM_SCHOOL_FLAIL,	WEAPON_FLAIL,	&gsn_flail	},
   { "whip",	OBJ_VNUM_SCHOOL_WHIP,	WEAPON_WHIP,	&gsn_whip	},
   { "polearm",	OBJ_VNUM_SCHOOL_POLEARM,WEAPON_POLEARM,	&gsn_polearm	},
   { NULL,	0,				0,	NULL		}
};


 
/* poison type table */
const	struct	poison_type	poison_table	[]	=
{
   { POISON_NORMAL,		"normal" },
   { POISON_WEAKNESS,		"weakness" },
   { POISON_NAUSEA,		"nausea" },
   { POISON_BLINDNESS,		"blindness" },
   { POISON_BLEEDING,		"bleeding" },
   { POISON_PARALYSIS,		"paralysis" },
   { POISON_HALLUCINATION,	"hallucination" },
   { POISON_COMA,		"coma" },
   { POISON_MADNESS,		"madness" },
   { POISON_WITHER,		"wither" },
   { POISON_DEATH,		"death" },
   { 0,				NULL }
};


 
/* wiznet table and prototype for future flag setting */
const struct wiznet_type wiznet_table[] =
{
   {	"on",		WIZ_ON,		IM },
   {	"prefix",	WIZ_PREFIX,	IM },
   {	"ticks",	WIZ_TICKS,	IM },
   {	"logins",	WIZ_LOGINS,	IM },
   {	"sites",	WIZ_SITES,	L4 },
   {	"links",	WIZ_LINKS,	L7 },
   {	"newbies",	WIZ_NEWBIE,	IM },
   {	"spam",		WIZ_SPAM,	L5 },
   {	"deaths",	WIZ_DEATHS,	IM },
   {	"resets",	WIZ_RESETS,	L4 },
   {	"mobdeaths",	WIZ_MOBDEATHS,	L4 },
   {	"flags",	WIZ_FLAGS,	L5 },
   {	"penalties",	WIZ_PENALTIES,	L5 },
   {	"saccing",	WIZ_SACCING,	L5 },
   {	"levels",	WIZ_LEVELS,	IM },
   {	"load",		WIZ_LOAD,	L2 },
   {	"restore",	WIZ_RESTORE,	L2 },
   {	"snoops",	WIZ_SNOOPS,	L2 },
   {	"switches",	WIZ_SWITCHES,	L2 },
   {	"secure",	WIZ_SECURE,	L1 },
   {	"mobactions",	WIZ_MOBACTIONS,	L7 },
   {	NULL,		0,		0  }
};

/* attack table  -- not very organized :( */
const struct attack_type attack_table[MAX_DAMAGE_MESSAGE]=
{
    {	"none",		"hit",		-1,		NULL,	NULL	},  /*	0 */
    {	"slice",	"slice",	DAM_SLASH,	NULL,	NULL	},	
    {	"stab",		"stab",		DAM_PIERCE,	NULL,	NULL	},
    {	"slash",	"slash",	DAM_SLASH,	NULL,	NULL	},
    {	"whip",		"whip",		DAM_SLASH,	NULL,	NULL	},
    {	"claw",		"claw",		DAM_SLASH,	NULL,	NULL	},  /*	5 */
    {	"blast",	"blast",	DAM_BASH,	NULL,	NULL	},
    {	"pound",	"pound",	DAM_BASH,	NULL,	NULL	},
    {	"crush",	"crush",	DAM_BASH,	NULL,	NULL	},
    {	"grep",		"grep",		DAM_SLASH,	NULL,	NULL	},
    {	"bite",		"bite",		DAM_PIERCE,	NULL,	NULL	},  /* 10 */
    {	"pierce",	"pierce",	DAM_PIERCE,	NULL,	NULL	},
    {	"suction",	"suction",	DAM_BASH,	NULL,	NULL	},
    {	"beating",	"beating",	DAM_BASH,	NULL,	NULL	},
    {	"digestion",	"digestion",	DAM_ACID,	NULL,	NULL	},
    {	"charge",	"charge",	DAM_BASH,	NULL,	NULL	},  /* 15 */
    {	"slap",		"slap",		DAM_BASH,	NULL,	NULL	},
    {	"punch",	"punch",	DAM_BASH,	dam_hand_to_hand,
    								&gdt_punch },
    {	"wrath",	"wrath",	DAM_ENERGY,	NULL,	NULL	},
    {	"magic",	"magic",	DAM_ENERGY,	NULL,	NULL	},
    {	"divine",	"divine power",	DAM_HOLY,	NULL,	NULL	},  /* 20 */
    {	"cleave",	"cleave",	DAM_SLASH,	NULL,	NULL	},
    {	"scratch",	"scratch",	DAM_PIERCE,	NULL,	NULL	},
    {	"peck",		"peck",		DAM_PIERCE,	NULL,	NULL	},
    {	"peckb",	"peck",		DAM_BASH,	NULL,	NULL	},
    {	"chop",		"chop",		DAM_SLASH,	NULL,	NULL	},  /* 25 */
    {	"sting",	"sting",	DAM_PIERCE,	NULL,	NULL	},
    {	"smash",	"smash",	DAM_BASH,	NULL,	&gdt_smash },
    {	"shbite",	"shocking bite",DAM_LIGHTNING,	NULL,	NULL	},
    {	"flbite",	"flaming bite",	DAM_FIRE,	NULL,	NULL	},
    {	"frbite",	"freezing bite",DAM_COLD,	NULL,	NULL	},  /* 30 */
    {	"acbite",	"acidic bite",	DAM_ACID,	NULL,	NULL	},
    {	"chomp",	"chomp",	DAM_PIERCE,	NULL,	NULL	},
    {	"drain",	"life drain",	DAM_NEGATIVE,	NULL,	NULL	},
    {	"thrust",	"thrust",	DAM_PIERCE,	NULL,	NULL	},
    {	"slime",	"slime",	DAM_ACID,	NULL,	NULL	},
    {	"shock",	"shock",	DAM_LIGHTNING,	NULL,	NULL	},
    {	"thwack",	"thwack",	DAM_BASH,	NULL,	NULL	},
    {	"flame",	"flame",	DAM_FIRE,	NULL,	NULL	},
    {	"chill",	"chill",	DAM_COLD,	NULL,	NULL	},
    {	NULL,		NULL,		0,		NULL,	NULL	}
};


/* Donation rooms. */
const int donation_rooms[MAX_DONATION] =
{ 
	12080, 9610, 5554
};


/* race table */
const struct race_type race_table[] =
{
/*
    {
	name,		pc_race?,
	act bits,	aff_by bits,	aff_by2 bits,	off bits,
	imm,		res,		vuln,
	form,		parts,
	char_flags
    },
*/
    { "unique",		FALSE, 0, 0, 0, 0, 0, 0, 0, 0, 0 },

    { 
	"human",		TRUE, 
	0,		0, 		0,		0,
	0, 		0,		0,
	A|H|M|V,	A|B|C|D|E|F|G|H|I|J|K,
	CHAR_HUMAN
    },

    {
	"elf",		TRUE,
	0,		AFF_INFRARED,	0,		0,
	0,		RES_CHARM,      VULN_IRON,
	A|H|M|V,	A|B|C|D|E|F|G|H|I|J|K,
	CHAR_ELF
    },

    {
	"dwarf",		TRUE,
	0,		AFF_INFRARED,	0,		0,
	0,		RES_POISON|RES_DISEASE, VULN_DROWNING,
	A|H|M|V,	A|B|C|D|E|F|G|H|I|J|K,
	CHAR_DWARF
    },

    {
	"demon",		FALSE, 
	0,		AFF_DARK_VISION|AFF_REGENERATION|AFF_PROTECT_GOOD,	0,	OFF_FAST,
	0,		RES_FIRE|RES_MAGIC|RES_NEGATIVE,	VULN_HOLY,
	A|H|M|V,	A|B|C|D|E|F|G|H|I|J|K|P|Q|U|V|X,
	CHAR_DEMON	//0
    },

    {
	"giant",		FALSE,
	0,		0,		0,		0,
	0,		RES_FIRE|RES_COLD,	VULN_MENTAL|VULN_LIGHTNING,
	A|H|M|V,	A|B|C|D|E|F|G|H|I|J|K,
	CHAR_GIANT
    },

    {
	"bat",			FALSE,
	0,	AFF_FLYING|AFF_DARK_VISION,	0,	OFF_DODGE|OFF_FAST,
	0,		0,		VULN_LIGHT,
	A|G|V,		A|C|D|E|F|H|J|K|P,
	0
    },

    {
	"bear",			FALSE,
	0,		0,	0,	OFF_CRUSH|OFF_DISARM|OFF_BERSERK,
	0,		RES_BASH|RES_COLD,	0,
	A|G|V,		A|B|C|D|E|F|H|J|K|U|V,
	0
    },

    {
	"cat",			FALSE,
	0,		AFF_DARK_VISION,	0,	OFF_FAST|OFF_DODGE,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|H|J|K|Q|U|V,
	0
    },

    {
	"centipede",		FALSE,
	0,		AFF_DARK_VISION,	0,		0,
	0,		RES_PIERCE|RES_COLD,	VULN_BASH,
 	A|B|G|O,		A|C|K,
	0
    },

    {
	"dog",			FALSE,
	0,		0,		0,	OFF_FAST,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|H|J|K|U|V,
	0
    },

    {
	"doll",			FALSE,
	0,		0,		0,		0,
	IMM_COLD|IMM_POISON|IMM_HOLY|IMM_NEGATIVE|IMM_MENTAL|IMM_DISEASE
	|IMM_DROWNING,	RES_BASH|RES_LIGHT,
	VULN_SLASH|VULN_FIRE|VULN_ACID|VULN_LIGHTNING|VULN_ENERGY,
	E|J|M|cc,	A|B|C|G|H|K,
	0
    },

    { 	"dragon", 		FALSE, 
	0, 		AFF_INFRARED|AFF_FLYING,	0,		0,
	0,			RES_FIRE|RES_BASH|RES_CHARM, 
	VULN_PIERCE|VULN_COLD,
	A|H|Z,		A|C|D|E|F|G|H|I|J|K|P|Q|U|V|X,
	0
    },

    {
	"fido",			FALSE,
	0,		0,		0,		OFF_DODGE|ASSIST_RACE,
	0,		0,			VULN_MAGIC,
	A|B|G|V,	A|C|D|E|F|H|J|K|Q|V,
	0
    },		
   
    {
	"fox",			FALSE,
	0,	AFF_DARK_VISION,	0,		OFF_FAST|OFF_DODGE,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|H|J|K|Q|V,
	0
    },

    {
	"goblin",		FALSE,
	0,		AFF_INFRARED,		0,		0,
	0,		RES_DISEASE,	VULN_MAGIC,
	A|H|M|V,	A|B|C|D|E|F|G|H|I|J|K,
	CHAR_ORC
    },

    {
	"hobgoblin",		FALSE,
	0,		AFF_INFRARED,		0,		0,
	0,		RES_DISEASE|RES_POISON,	0,
	A|H|M|V,        A|B|C|D|E|F|G|H|I|J|K|Y,
	CHAR_ORC
    },

    {
	"kobold",		FALSE,
	0,		AFF_INFRARED,		0,		0,
	0,		RES_POISON,	VULN_MAGIC,
	A|B|H|M|V,	A|B|C|D|E|F|G|H|I|J|K|Q,
	0
    },

    {
	"lizard",		FALSE,
	0,		0,		0,		0,
	0,		RES_POISON,	VULN_COLD,
	A|G|X|cc,	A|C|D|E|F|H|K|Q|V,
	0
    },

    {
	"modron",		FALSE,
	0,		AFF_INFRARED,	0,	ASSIST_RACE|ASSIST_ALIGN,
	IMM_CHARM|IMM_DISEASE|IMM_MENTAL|IMM_HOLY|IMM_NEGATIVE,
			RES_FIRE|RES_COLD|RES_ACID,	0,
	H,		A|B|C|G|H|J|K,
	0
    },

    {
	"orc",			FALSE,
	0,		AFF_INFRARED,		0,		0,
	0,		RES_DISEASE,	VULN_LIGHT,
	A|H|M|V,	A|B|C|D|E|F|G|H|I|J|K,
	CHAR_ORC
    },

    {
	"pig",			FALSE,
	0,		0,		0,		0,
	0,		0,		0,
	A|G|V,	 	A|C|D|E|F|H|J|K,
	0
    },	

    {
	"rabbit",		FALSE,
	0,		0,		0,		OFF_DODGE|OFF_FAST,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|H|J|K,
	0
    },
    
    {
	"school monster",	FALSE,
	ACT_NOALIGN,		0,		0,		0,
	IMM_CHARM|IMM_SUMMON,	0,		VULN_MAGIC,
	A|M|V,		A|B|C|D|E|F|H|J|K|Q|U,
	0
    },	

    {
	"snake",		FALSE,
	0,		0,		0,		0,
	0,		RES_POISON,	VULN_COLD,
	A|G|X|Y|cc,	A|D|E|F|K|L|Q|V|X,
	0
    },
 
    {
	"song bird",		FALSE,
	0,		AFF_FLYING,		0,	OFF_FAST|OFF_DODGE,	
	0,		0,		0,
	A|G|W,		A|C|D|E|F|H|K|P,
	0
    },

    {
	"troll",		FALSE,
	0,		AFF_REGENERATION|AFF_INFRARED|AFF_DETECT_HIDDEN,
	0,		OFF_BERSERK,
 	0,	RES_CHARM|RES_BASH,	VULN_FIRE|VULN_ACID,
	A|B|H|M|V,		A|B|C|D|E|F|G|H|I|J|K|U|V,
	0
    },

    {
	"water fowl",		FALSE,
	0,		AFF_SWIM|AFF_FLYING,	0,		0,
	0,		RES_DROWNING,		0,
	A|G|W,		A|C|D|E|F|H|K|P,
	0
    },		
  
    {
	"wolf",			FALSE,
	0,		AFF_DARK_VISION,	0,	OFF_FAST|OFF_DODGE,
	0,		0,		0,	
	A|G|V,		A|C|D|E|F|J|K|Q|V,
	0
    },

    {
	"wyvern",		FALSE,
	0,		AFF_FLYING|AFF_DETECT_INVIS|AFF_DETECT_HIDDEN,
	0,		OFF_BASH|OFF_FAST|OFF_DODGE,
	IMM_POISON,	0,	VULN_LIGHT,
	A|B|G|Z,		A|C|D|E|F|H|J|K|Q|V|X,
	0
    },

    {
	"cow",		FALSE,
	0,		0,		0,		0,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|J|K|W,
	0
    },

    {
	"horse",	FALSE,
	0,		0,		0,		0,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|J|K,
	0
    },

    {
	"deer",		FALSE,
	0,		0,		0,		0,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|J|K|W,
	0
    },

    {
	"goat",		FALSE,
	0,		0,		0,		0,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|J|K|W,
	0
    },

    {
	"spectre",	FALSE,
	O,	AFF_DARK_VISION|AFF_REGENERATION,	0,	OFF_FADE,
	IMM_NEGATIVE,	RES_COLD|RES_PIERCE|RES_SLASH,	VULN_HOLY|VULN_LIGHT,
	B|I|L,		A|B|C|G|I|J|U|V,
	0
    },

    {
	"vampire",	FALSE,
	O,	AFF_DARK_VISION|AFF_REGENERATION,	0,
	OFF_FADE|OFF_FAST,
	IMM_CHARM|IMM_NEGATIVE,	RES_BASH|RES_SLASH|RES_MENTAL,
	VULN_HOLY|VULN_WOOD|VULN_LIGHT|VULN_FIRE,
	D|H|I|M,	A|B|C|G|H|I|J|K|U|V,
	0
    },

    {
	"wraith",	FALSE,
	O,		AFF_DARK_VISION|AFF_REGENERATION,
	0,		OFF_FADE|OFF_FAST,
	IMM_COLD,	RES_CHARM|RES_NEGATIVE|RES_MENTAL,
	VULN_HOLY|VULN_LIGHT,
	D|I|L,		A|B|G|K|U|V,
	0
    },

    {
	"ghoul",	FALSE,
	O,		AFF_DARK_VISION|AFF_REGENERATION,	0,
	OFF_CRUSH,
	IMM_BASH|IMM_DISEASE,	RES_MENTAL|RES_COLD|RES_NEGATIVE,
	VULN_HOLY|VULN_LIGHT,
	A|B|I,		A|B|C|F|G|H|I|U,
	0
    },

    {
	"skeleton",	FALSE,
	O,		AFF_DARK_VISION,	0,		0,
	IMM_COLD|IMM_CHARM,	RES_PIERCE|RES_FIRE|RES_NEGATIVE,
	VULN_HOLY|VULN_MAGIC,
	I|M,		A|B|C|G|H|I|U,
	0
    },

    {
	"unique",		FALSE,
	0,		0,		0,		0,
	0,		0,		0,
	0,		0,
	0
    },

    {
	NULL, FALSE, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    }
};

const	struct	pc_race_type	pc_race_table	[]	=
{
    { "null race", "", 0,
      { 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100 },
      { "" }, { 52, 52, 52, 52, 52 }, { 72, 72, 72, 72, 72 }, 0, 0, 0 },
 
/*
    {
	"race name", 	short name, 	points,	{ class multipliers },
	{ bonus skills },
	{ base stats },		{ max stats },		size,
        race_age, sexes,
        { religious restrictions }
    },
*/

/*  Murgens point guidelines:
 *    1 for infra/dark vision
 *    2 for non-human (??)
 *    1 per skill (0 if the skill is useless - like hide)
 *    1 more if the skill is sneak
 *    1 if +con and +str over humans
 *    1 if +wis over humans
 *   -1 if -wis over humans
 *   -1 if -con over humans
 *    1 for major resistance (eg. magic, blunt, sharp, pierce)
 *   -1 for major vulnerability
 *    Use your discretion
 */
    {
	"human",	"Human",	0,
	{ 100, 100, 100, 100, 100, 100, 100, 100,
	  100, 100, 100, 100, 100, 100, 100, 100 },
	{ "" },
	{ 50, 50, 50, 50, 50, 50, 50 },	{ 70, 70, 70, 70, 70, 70, 70 },
// Human Max Total: 500
	SIZE_MEDIUM,	0,      SEX_FLAG_MALE|SEX_FLAG_FEMALE,
        { TRUE, TRUE, TRUE, TRUE }
    },

    { 
	"elf",	"HElf",		3,	
	{ 100, 125, 100, 120, 123, 110, 123, 130,
	  120, 115, 125, 100, 100, 150, 100, 110 },
	{ "hide" },
	{ 48, 56, 52, 60, 48, 56, 60 }, { 68, 76, 72, 80, 68, 76, 80  },
// Elf Max Total: 520
	SIZE_MEDIUM,	158,    SEX_FLAG_MALE|SEX_FLAG_FEMALE,
        { TRUE, TRUE, TRUE, TRUE }
    },

    {
	"dwarf",	"Dwarf",	6,
	{ 150, 100, 120, 100, 130, 150, 100, 140,
	  115, 125, 100, 150, 150, 200, 150, 150 },
	{ "berserk" },
	{ 60, 44, 60, 36, 64, 49, 44 },	{ 80, 64, 80, 56, 84, 69, 64 },
// Dwarf Max Total: 497
	SIZE_MEDIUM,	33,     SEX_FLAG_MALE|SEX_FLAG_FEMALE,
        { TRUE, TRUE, TRUE, TRUE }
    }
};




/*
 * Class table.
 */
const	struct	class_type	class_table	[MAX_CLASS]	=
{
    {
	"mage", "Mag",  STAT_INT,  OBJ_VNUM_SCHOOL_DAGGER,
	{ 9618 },  75,  20, 6,  6,  8, 100,
	"mage basics", "mage default",
	0
    },

    {
	"cleric", "Cle",  STAT_WIS,  OBJ_VNUM_SCHOOL_MACE,
	{ 9619 },  75,  20, 2,  7, 10, 100,
	"cleric basics", "cleric default",
	CLASS_NEEDS_SYMBOL | CLASS_RELIGIOUS
    },

    {
	"thief", "Thi",  STAT_DEX,  OBJ_VNUM_SCHOOL_DAGGER,
    	{ 9639 },  75,  20,  -4,  8, 13, 50,
	"thief basics", "thief default",
	CLASS_SNEAKY
    },

    {
	"warrior", "War",  STAT_STR,  OBJ_VNUM_SCHOOL_SWORD,
	{ 9633 },  75,  20,  -10,  11, 15, 50,
	"warrior basics", "warrior default",
	CLASS_NEEDS_SYMBOL
    },

    {
	"ranger", "Ran",  STAT_PER,  OBJ_VNUM_SCHOOL_SWORD,
	{ 9633 }, 75, 20, -8, 11, 15, 60,
	"ranger basics", "ranger default",
	CLASS_OUTDOORS | CLASS_NEEDS_SYMBOL
    },
    
    {
	"ninja", "Nin", STAT_DEX,  OBJ_VNUM_SCHOOL_DAGGER,
	{ 9639 }, 75, 20, -4, 8, 13, 60,
	"ninja basics", "ninja default",
	CLASS_SNEAKY
    },
    
    {
	"templar", "Tem", STAT_WIS,  OBJ_VNUM_SCHOOL_SWORD,
	{ 9619 }, 75, 20, -8, 11, 15, 70,
	"templar basics", "templar default",
	CLASS_NEEDS_SYMBOL | CLASS_RELIGIOUS
    },
    
    {
	"monk", "Mon", STAT_DEX, 0,
	{ 9619 }, 75, 20, -6, 7, 10, 40,
	"monk basics", "monk default",
	CLASS_MARTIAL_ARTIST
    },

    {
	"warden", "Wdn",  STAT_WIS, 0,
	{ 9619 }, 75, 20, 2, 7, 10, 100,
	"warden basics", "warden default",
	CLASS_OUTDOORS|CLASS_NEEDS_SYMBOL
    },

    {
	"antipaladin", "Ant", STAT_CON, 0,
	{ 9633 }, 75, 20, -8, 11, 15, 70,
	"antipaladin basics", "antipaladin default",
	0
    },

    {
	"paladin", "Pal", STAT_WIS, 0,
	{ 9619 }, 75, 20, -8, 11, 15, 70,
	"paladin basics", "paladin default",
	CLASS_NEEDS_SYMBOL | CLASS_RELIGIOUS
    },

    {
	"sorcerer", "Sor", STAT_INT, 0,
	{ 9618 },  75,  20, 6,  6,  8, 100,
	"sorcerer basics", "sorcerer default",
	CLASS_MARTIAL_ARTIST
    },

    {
	"bard", "Bar", STAT_CHA, OBJ_VNUM_SCHOOL_SWORD,
	{ 9639 }, 75, 20, -2, 7, 11, 75,
	"bard basics", "bard default",
	0
    },

    {
	"necromancer", "Nec", STAT_INT, OBJ_VNUM_SCHOOL_DAGGER,
	{ 9618 }, 75, 20, 6, 6, 8, 100,
	"necromancer basics", "necromancer default",
	0
    },

    {
	"augurer", "Aug", STAT_PER, OBJ_VNUM_SCHOOL_DAGGER,
	{ 9618 }, 75, 20, 6, 6, 8, 100,
	"augurer basics", "augurer default",
	0
    },

    {
	"illusionist", "Ill", STAT_INT, OBJ_VNUM_SCHOOL_DAGGER,
	{ 9618 }, 75, 20, 6, 6, 8, 100,
	"illusionist basics", "illusionist default",
	0
    }
};


/*
 * Attribute bonus tables.
 */
const struct str_app_type str_app[26] =
// tohit tohit_mob todam todam_mob carry wield
{
    { -5, -6, -4, -5,   0,  0 },  /* 0  */
    { -5, -6, -4, -5,   3,  1 },  /* 1  */
    { -3, -5, -2, -4,   3,  2 },
    { -3, -5, -1, -3,  10,  3 },  /* 3  */
    { -2, -4, -1, -2,  25,  4 },
    { -2, -4, -1, -1,  55,  5 },  /* 5  */
    { -1, -3,  0,  0,  80,  6 },
    { -1, -3,  0,  0,  90,  7 },
    {  0, -2,  0,  0, 100,  8 },
    {  0, -2,  0,  0, 100,  9 },
    {  0, -1,  0,  0, 115, 10 }, /* 10  */
    {  0, -1,  0,  0, 115, 11 },
    {  0,  0,  0,  0, 130, 12 },
    {  0,  0,  0,  0, 130, 13 }, /* 13  */
    {  0,  0,  1,  0, 140, 14 },
    {  1,  0,  1,  0, 150, 15 }, /* 15  */
    {  2,  0,  2,  0, 165, 16 },
    {  2,  1,  3,  1, 180, 22 },
    {  3,  1,  3,  2, 200, 25 }, /* 18  */
    {  4,  2,  4,  3, 225, 30 },
    {  5,  2,  5,  4, 250, 35 }, /* 20  */
    {  6,  3,  6,  5, 300, 40 },
    {  7,  3,  8,  6, 350, 45 },
    {  8,  4,  11, 7, 400, 50 },
    {  9,  4,  14, 8, 450, 55 },
    { 10,  5,  16, 9, 500, 60 }  /* 25   */
};



const	struct	int_app_type	int_app		[26]		=
{
/*   Learn, Percentage Spell Modifier */
    {  3 , -84},/*  0 */
    {  5 , -79},/*  1 */
    {  7 , -74},
    {  8 , -69},/*  3 */
    {  9 , -64},
    { 10 , -59},/*  5 */
    { 11 , -54},
    { 12 , -49},
    { 13 , -44},
    { 15 , -35},
    { 17 , -27},/* 10 */
    { 19 , -20},
    { 22 , -14},
    { 25 , -9 },
    { 28 , -5 },
    { 31 , 0 },	/* 15 */
    { 34 , 0 },
    { 37 , 0  },
    { 40 , 0  },/* 18 */
    { 44 , 5  },
    { 49 , 9  },/* 20 */
    { 55 , 14 },
    { 60 , 20 },
    { 70 , 27 },
    { 80 , 35 },
    { 85 , 45 }	/* 25 */
};



const	struct	wis_app_type	wis_app		[26]		=
{
    { 0 , -84},	/*  0 */
    { 0 , -79},	/*  1 */
    { 0 , -74},
    { 0 , -69},	/*  3 */
    { 0 , -64},
    { 0 , -59},	/*  5 */
    { 0 , -54},
    { 1 , -49},
    { 1 , -44},
    { 1 , -39},
    { 1 , -33},	/* 10 */
    { 1 , -25},
    { 2 , -20},
    { 2 , -12},
    { 2 , -5},
    { 2 , 0},	/* 15 */
    { 3 , 0},
    { 3 , 0},
    { 3 , 0},	/* 18 */
    { 3 , 5},
    { 4 , 10},	/* 20 */
    { 4 , 16},
    { 4 , 25},
    { 5 , 33},
    { 5 , 39},
    { 6 , 45}	/* 25 */
};



const	struct	dex_app_type	dex_app		[26]		=
{
    {   60 },   /* 0 */
    {   50 },   /* 1 */
    {   50 },
    {   40 },
    {   30 },
    {   20 },   /* 5 */
    {   10 },
    {    0 },
    {    0 },
    {    0 },
    {    0 },   /* 10 */
    {    0 },
    {    0 },
    {    0 },
    {    0 },
    { - 10 },   /* 15 */
    { - 15 },
    { - 20 },
    { - 30 },
    { - 40 },
    { - 50 },   /* 20 */
    { - 60 },
    { - 75 },
    { - 90 },
    { -105 },
    { -120 }    /* 25 */
};


const	struct	con_app_type	con_app		[26]		=
{
    { -4, 20 },   /*  0 */
    { -3, 25 },   /*  1 */
    { -2, 30 },
    { -2, 35 },	  /*  3 */
    { -1, 40 },
    { -1, 45 },   /*  5 */
    { -1, 50 },
    {  0, 55 },
    {  0, 60 },
    {  0, 65 },
    {  0, 70 },   /* 10 */
    {  0, 75 },
    {  0, 80 },
    {  0, 85 },
    {  0, 88 },
    {  1, 90 },   /* 15 */
    {  2, 95 },
    {  2, 97 },
    {  3, 99 },   /* 18 */
    {  3, 99 },
    {  4, 99 },   /* 20 */
    {  4, 99 },
    {  5, 99 },
    {  6, 99 },
    {  7, 99 },
    {  8, 99 }    /* 25 */
};


const	struct	per_app_type	per_app		[26]		=
{
    {  0, -20 },   /*  0 */
    {  0, -15 },   /*  1 */
    {  0, -10 },
    {  0,  -8 },   /*  3 */
    {  0,  -6 },
    {  0,  -4 },   /*  5 */
    {  0,  -2 },
    {  0,  -1 },
    {  0,   0 },
    {  0,   0 },
    {  0,   0 },   /* 10 */
    {  0,   0 },
    {  0,   0 },
    {  0,   0 },
    {  0,   0 },
    {  0,   1 },   /* 15 */
    {  0,   1 },
    {  0,   2 },
    {  0,   3 },   /* 18 */
    {  0,   3 },
    {  0,   4 },   /* 20 */
    {  0,   4 },
    {  0,   5 },
    {  0,   6 },
    {  0,   8 },
    {  1,  10 }    /* 25 */
};


const char *str_names[ MAX_STAT_NAME ] =
{
    "Minimal", "Pitiful", "Feeble", "Weak", "Average",
    "Athletic", "Muscled", "Strong", "Powerful",
    "Massive", "Legendary", "Godlike"
};
const char *int_names[ MAX_STAT_NAME ] =
{
    "Unintelligent", "Moronic", "Stupid", "Slow",
    "Average", "Resourceful", "Clever", "Intelligent",
    "Ingenious", "Transcendent", "Legendary", "Godlike"
};
const char *wis_names[ MAX_STAT_NAME ] =
{
    "None", "Asinine", "Ignorant", "Foolhardy",
    "Average", "Sensible", "Intuitive", "Savvy",
    "Wise", "Enlightened", "Legendary", "Godlike"
};
const char *dex_names[ MAX_STAT_NAME ] =
{
    "Unmoving", "Disasterous", "Clumsy", "Awkward",
    "Average", "Flexible", "Balanced", "Agile",
    "Acrobatic", "Effortless", "Legendary", "Godlike"
};
const char *con_names[ MAX_STAT_NAME ] =
{
    "None", "Diseased", "Sickly", "Frail", "Average",
    "Healthy", "Energetic", "Robust", "Vigorous", 
    "Inexhaustible", "Legendary", "Godlike"
};
const char *per_names[ MAX_STAT_NAME ] =
{
    "None", "Oblivious", "Distracted", "Dull", "Average",
    "Aware", "Keen", "Honed", "Piercing", "Omniscient",
    "Legendary", "Godlike"
};
const char *cha_names[ MAX_STAT_NAME ] =
{
    "Alien", "Loathsome", "Repulsive", "Graceless",
    "Average", "Amiable", "Charming", "Persuasive",
    "Influential", "Commanding", "Legendary", "Godlike"
};

const char *con_types[] =
{
    "Adventuring",
    "Choose name",
    "Choose PW",
    "Choose name",
    "Choose PW",
    "Choose race",
    "Choose sex",
    "Choose god",
    "Choose class",
    "Choose align",
    "Customize?",
    "Customizing",
    "Pick weapon",
    "Read IMOTD",
    "Read MOTD",
    "Closing link",
    "Choose home",
    "Choose home",
    "Recovering",
    "\n"
};

/*
 * Hometowns. Original code.
 */
const struct hometown_type hometown_table [MAX_HOMETOWN] =
{
//                                         human  dwarf  neut
//      name            vnum    avail    zero  elf   good  evil
      { NULL,		0,	FALSE,	{ 0, 0, 0, 0, 0, 0, 0 } },
      { "Malathar",	8191,	TRUE,	{ 0, 1, 1, 1, 1, 1, 1 } },
      { "Lorinandor",	1061,	TRUE,	{ 0, 0, 1, 1, 1, 1, 0 } },
      { "Kalassen",	18107,	TRUE,	{ 0, 1, 1, 1, 1, 1, 1 } },
      { "Mahn-Tor",	2364,	TRUE,	{ 0, 1, 0, 0, 0, 0, 1 } },
      { "New Thalos",	9506,	TRUE,	{ 0, 1, 0, 0, 1, 1, 1 } },
      { "Anon",		31000,	TRUE,	{ 0, 1, 1, 1, 1, 1, 1 } },
      { "Underdark",	4293,	TRUE,	{ 0, 1, 1, 1, 0, 1, 1 } },
      { "Diamondaxe",	12003,	FALSE,	{ 0, 0, 0, 0, 1, 1, 1 } },
      { "Regalis",	12003,	TRUE,	{ 0, 1, 1, 1, 1, 1, 0 } },
      { NULL,		0,	FALSE,	{ 0, 0, 0, 0, 0, 0, 0 } }
};

/*
 * Liquid properties.
 * Used in world.obj.
 */
const	struct	liq_type	liq_table	[]	=
{
/*    name			color	proof, full, thirst, food, ssize */
    { "water",			"clear",	{   0, 1, 10, 0, 16 }	},
    { "beer",			"amber",	{  12, 1,  8, 1, 12 }	},
    { "red wine",		"burgundy",	{  30, 1,  8, 1,  5 }	},
    { "ale",			"brown",	{  15, 1,  8, 1, 12 }	},
    { "dark ale",		"dark",		{  16, 1,  8, 1, 12 }	},

    { "whisky",			"golden",	{ 120, 1,  5, 0,  2 }	},
    { "lemonade",		"pink",		{   0, 1,  9, 2, 12 }	},
    { "firebreather",		"boiling",	{ 190, 0,  4, 0,  2 }	},
    { "local specialty",	"clear",	{ 151, 1,  3, 0,  2 }	},
    { "slime mold juice",	"green",	{   0, 2, -8, 1,  2 }	},

    { "milk",			"white",	{   0, 2,  9, 3, 12 }	},
    { "tea",			"tan",		{   0, 1,  8, 0,  6 }	},
    { "coffee",			"black",	{   0, 1,  8, 0,  6 }	},
    { "blood",			"red",		{   0, 2, -1, 2,  6 }	},
    { "salt water",		"clear",	{   0, 1, -2, 0,  1 }	},

    { "coke",			"brown",	{   0, 2,  9, 2, 12 }	}, 
    { "root beer",		"brown",	{   0, 2,  9, 2, 12 }   },
    { "elvish wine",		"green",	{  35, 2,  8, 1,  5 }   },
    { "white wine",		"golden",	{  28, 1,  8, 1,  5 }   },
    { "champagne",		"golden",	{  32, 1,  8, 1,  5 }   },

    { "mead",			"honey-colored",{  34, 2,  8, 2, 12 }   },
    { "rose wine",		"pink",		{  26, 1,  8, 1,  5 }	},
    { "benedictine wine",	"burgundy",	{  40, 1,  8, 1,  5 }   },
    { "vodka",			"clear",	{ 130, 1,  5, 0,  2 }   },
    { "cranberry juice",	"red",		{   0, 1,  9, 2, 12 }	},

    { "orange juice",		"orange",	{   0, 2,  9, 3, 12 }   },
    { "cider",			"golden brown", {   0, 2,  9, 3, 12 }   },
    { "hard cider",		"golden brown", {  16, 2,  9, 2,  8 }   },
    { "absinthe",		"green",	{ 200, 1,  4, 0,  2 }	},
    { "brandy",			"golden",	{  80, 1,  5, 0,  4 }	},
    { "aquavit",		"clear",	{ 140, 1,  5, 0,  2 }	},
    { "schnapps",		"clear",	{  90, 1,  5, 0,  2 }   },

    { "icewine",		"purple",	{  50, 2,  6, 1,  5 }	},
    { "amontillado",		"burgundy",	{  35, 2,  8, 1,  5 }	},
    { "sherry",			"red",		{  38, 2,  7, 1,  5 }   },	
    { "framboise",		"red",		{  50, 1,  7, 1,  5 }   },
    { "rum",			"amber",	{ 151, 1,  4, 0,  2 }	},

    { "cordial",		"clear",	{ 100, 1,  5, 0,  2 }   },
    { "hot chocolate",          "brown",        {   0, 5,  9, 9, 12 }   },
    { "blackberry juice",       "purple",       {   0, 1,  9, 2, 12 }   },
    { "dragon blood",           "boiling red",  { 199, 0,  4, 0,  2 }   },
    { "rain water", 		"clear", 	{   0, 1, 10, 0,  1 }   },
    { NULL,			NULL,		{   0, 0,  0, 0,  0 }	}
};



/*
 * The skill and spell table.
 * Slot numbers must never be changed as they appear in #OBJECTS sections.
 */
#define SLOT(n)		n

const struct skill_type skill_table[MAX_SKILL] =
{

/*
 * Magic spells.
 */

    {
	"reserved",
	{ 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99 },
	{ 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99 },
	0,			TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT( 0),	 0,	 0,
	"",
	"",
	"",
	SCLASS_NONE,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"absolution",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_absolution,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(0),	20,	12,
	"",
	"!Absolution!",
	"",
        SCLASS_CHANNELING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"acid blast",
	{ 28, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 34, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2 },
	spell_acid_blast,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(70),	20,	12,
	"acid blast",
	"!Acid Blast!",
	"",
        SCLASS_ELEMENTAL,	SSCLASS_ACID,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"alkar",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 29 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  1,  2,  1,  2,  2,  2,  1 },
	spell_alkar,	TAR_CHAR_SELF,	POS_STANDING,
	&gsn_alkar,	SLOT(0),	20,	12,
	"",
	"The aura of power surrounding you disapates.",
	"",
        SCLASS_ILLUSION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },


    {
	"animal call",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 15, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_animal_call,	TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT(40),	50,	18,
	"",
	"",
	"",
        SCLASS_ANIMAL,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"animal form",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 34, 53, 53, 53, 53, 53, 53, 53 },
	{  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_animal_form,	TAR_CHAR_DEFENSIVE,	POS_STANDING,
	NULL,			SLOT(0),	50,	18,
	"",
	"Your body returns to its normal form.",
	"",
        SCLASS_ANIMAL,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },
    
    {
	"antifreeze",
	{  53, 53, 53, 53, 53, 53, 53, 53, 53, 20, 53, 53, 53, 53, 53, 53 },
	{   3,  1,  4,  2,  3,  4,  1,  4,  2,  1,  2,  2,  2,  2,  2,  2 },
	spell_antifreeze,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),	10,	12,
	"",
	"A momentary chill sends shivers through your body.",
	"",
        SCLASS_PROTECTIVE,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"armor",
	{  7,  2, 10,  5,  4, 53,  4, 53, 53, 53, 53, 53,  9, 53,  5,  7 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1 },
	spell_armor,		TAR_CHAR_DEFENSIVE,	POS_STANDING,
	&gsn_armor,		SLOT( 1),	 5,	12,
	"",
	"You feel less armored.",
	"",
        SCLASS_PROTECTIVE,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"avatar",
	{  53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 31, 53, 53, 53, 53, 53 },
	{   3,  1,  4,  3,  3,  4,  1,  4,  2,  2,  3,  2,  2,  2,  2,  2 },
	spell_avatar,		TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),	100,	18,
	"",
	"The power of your god fades.",
	"",
        SCLASS_PROTECTIVE,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"awakening",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53,  8, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2 },
	spell_awakening,	TAR_CHAR_DEFENSIVE,	POS_STANDING,
	NULL,			SLOT(0),	10,	12,
	"",
	"",
	"",
	SCLASS_HEALING,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"backlash",
	{ 25, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 25, 25, 25 },
	{  1,  2,  2,  2,  2,  2,  2 , 2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_backlash,         TAR_CHAR_OFFENSIVE,     POS_FIGHTING,
	&gsn_backlash,		SLOT(0),	20,	12,
	"backlash",
	"", 
	"",
        SCLASS_MALEDICTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        "Underdark", NULL
    },

    {
	"banish dazzle",
	{  5, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53,  4,  7 },
	{  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_banish_dazzle,	TAR_CHAR_DEFENSIVE,	POS_STANDING,
	NULL,		        SLOT(0),	 5,	5,
	"",
	"You protection against harsh light fades.", 
	"",
        SCLASS_WEATHER,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        "'New Thalos'", NULL
    },

    {
	"banishment",
	{  53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 11, 53, 53, 53, 53, 53 },
	{   3,  1,  4,  2,  3,  4,  1,  4,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_banishment,	TAR_CHAR_OFFENSIVE,		POS_FIGHTING,
	NULL,			SLOT(0),	150,	12,
	"",
	"",
	"",
	SSCLASS_HOLY,           0,
	0, 0, 0, 0,
        FALSE, FALSE,
        NULL, NULL
    },

    {
	"befriend",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 20, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_befriend,		TAR_CHAR_DEFENSIVE,	POS_STANDING,
	&gsn_befriend,		SLOT(0),	 5,	12,
	"",
	"",
	"",
        SCLASS_BEGUILING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"blade of darkness",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 30, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  1,  2,  2,  2,  2,  2,  2 },
	spell_blade_of_darkness,TAR_OBJ_INV,		POS_FIGHTING,
	NULL,			SLOT(0),	15,	12,
	"",
	"!blade of darkness!",
	"$p comes into clear view.",
        SCLASS_ENCHANTMENT,	SSCLASS_UNHOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"bless",
	{ 53,  7, 53,  8, 53, 53,  7, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_bless,		TAR_OBJ_CHAR_DEF,	POS_STANDING,
	&gsn_bless,		SLOT( 3),	 5,	12,
	"",
	"You feel less righteous.", 
	"$p's holy aura fades.",
        SCLASS_BENEDICTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"blindness",
	{ 12,  8, 17, 15, 53, 53, 53, 53, 53, 53, 53, 53, 23,  6, 53, 11 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2 },
	spell_blindness,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	&gsn_blindness,		SLOT( 4),	 5,	12,
	"",
	"You can see again.",
	"",
        SCLASS_MALEDICTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
        "bright moonkiss touch",
        { 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53,  5, 53, 53, 53 },
        {  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2 },
	spell_bright_moonkiss_touch,TAR_IGNORE, POS_STANDING,
        &gsn_bright_moonkiss_touch,SLOT(0), 26,  12,
	"",
	"You have lost your peace of mind.",
	"",
	SCLASS_BEGUILING,   0,
	0, 0, 0, 0,
	FALSE, TRUE,
        NULL, NULL
    },

    {
	"brittle bones",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 45, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_brittle_bones,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(0),	15,	12,
	"",
	"",
	"",
        SCLASS_EARTH,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"burning hands",
	{  7, 53, 10,  9, 53, 53, 53, 53, 53, 53, 53, 53, 53,  7, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2 },
	spell_burning_hands,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT( 5),	15,	12,
	"burning hands",
	"!Burning Hands!",
	"",
        SCLASS_ELEMENTAL,	SSCLASS_FIRE,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"call lightning",
	{ 26, 18, 31, 22, 20, 53, 53, 53, 53, 53, 53, 53, 53, 53, 24, 53 },
	{  1,  1,  2,  2,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2 },
	spell_call_lightning,	TAR_IGNORE,		POS_FIGHTING,
	NULL,			SLOT( 6),	15,	12,
	"lightning bolt",
	"!Call Lightning!",
	"",
        SCLASS_WEATHER,		SSCLASS_LIGHTNING,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"calm",
	{ 48, 16, 50, 20, 53, 53, 19, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_calm,		TAR_IGNORE,		POS_FIGHTING,
	NULL,			SLOT(509),	30,	12,
	"",
	"You have lost your peace of mind.",
	"",
	SCLASS_BENEDICTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"cancellation",
	{ 18, 26, 34, 34, 32, 53, 32, 53, 53, 53, 53, 53, 30, 53, 16, 18 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  3,  2,  1,  1 },
	spell_cancellation,	TAR_CHAR_DEFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(507),	20,	12,
	"",
	"!cancellation!",
	"",
        SCLASS_PROTECTIVE,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"capture",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1, 53, 53, 53, 53 },
	{  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_capture,	        TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(0),	100,	24,
	"",
	"!Capture!",
	"",
        SCLASS_CHANNELING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"cause critical",
	{ 53, 13, 53, 19, 53, 20, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_cause_critical,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(63),	20,	12,
	"spell",
	"!Cause Critical!",
	"",
        SCLASS_CHANNELING,	SSCLASS_UNHOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"cause light",
	{ 53,  1, 53,  3, 53,  7, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_cause_light,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(62),	15,	12,
	"spell",
	"!Cause Light!",
	"",
        SCLASS_CHANNELING,	SSCLASS_UNHOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"cause serious",
	{ 53,  7, 53, 10, 53, 15, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_cause_serious,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(64),	17,	12,
	"spell",
	"!Cause Serious!",
	"",
        SCLASS_CHANNELING,	SSCLASS_UNHOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {   
	"chain lightning",
	{ 33, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 37, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2 },
	spell_chain_lightning,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	&gsn_chain_lightning,	SLOT(500),	25,	12,
	"lightning",
	"!Chain Lightning!",
	"",
        SCLASS_ELEMENTAL,	SSCLASS_LIGHTNING,
	0, 0, 0, 0,
        FALSE, FALSE,
        NULL, NULL
    }, 

    {
	"change sex",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_change_sex,	TAR_CHAR_DEFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(82),	15,	12,
	"",
	"Your body feels familiar again.",	
	"",
        SCLASS_MALEDICTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"charm person",
	{ 20, 53, 25, 53, 53, 24, 53, 53, 53, 53, 53, 53, 22, 53, 25, 25 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2 },
	spell_charm_person,	TAR_CHAR_OFFENSIVE,	POS_STANDING,
	&gsn_charm_person,	SLOT( 7),	 5,	12,
	"",
	"You feel more self-confident.",
	"",
        SCLASS_BEGUILING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"chill touch",
	{  4, 53,  6,  6, 53, 53, 53, 53, 53,  3, 53, 53, 53,  3, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  1,  2,  2 },
	spell_chill_touch,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT( 8),	15,	12,
	"chilling touch",
	"You feel less cold.",
	"",
        SCLASS_ELEMENTAL,	SSCLASS_COLD,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"cloud of darkness",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 17, 53, 53, 53, 53 },
	{  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2,  2 },
	spell_cloud_of_darkness,TAR_IGNORE,	POS_STANDING,
	NULL,			SLOT(0),	25,	24,
	"",
	"The darkness surrounding you disappears.",
	"",
	SCLASS_ILLUSION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"colour spray",
	{ 16, 53, 22, 20, 53, 53, 53, 53, 53, 53, 53, 53, 53, 14, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_colour_spray,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(10),	15,	12,
	"colour spray",
	"!Colour Spray!",
	"",
        SCLASS_ELEMENTAL,	SSCLASS_LIGHT,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
        "come clarion rage",
        { 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 20, 53, 53, 53 },
        {  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2 },
	spell_come_clarion_rage,TAR_IGNORE, POS_STANDING,
        &gsn_come_clarion_rage,SLOT(0), 26,  12,
	"",
	"Your burning desire to kill fades.",
	"",
	SCLASS_BEGUILING,   0,
	0, 0, 0, 0,
	FALSE, TRUE,
        NULL, NULL
    },

    {
	"confiscate",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	{  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_confiscate,       TAR_IGNORE,	POS_STANDING,
	NULL,			SLOT(0),	100,	12,
	"",
	"!Confiscate!",
	"",
        SCLASS_CHANNELING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"conjuration",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 30, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2,  2 },
	spell_conjuration,	TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT(0),	80,		24,
	"",
	"",
	"",
        SCLASS_DEMONIC,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },	

    {
	"conjuration true",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 50, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2,  2 },
	spell_conjuration_true,	TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT(0),	90,		24,
	"",
	"",
	"",
        SCLASS_DEMONIC,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },	

    {
	"contingency",
	{ 16, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 18, 15, 18 },
	{  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  2,  3 },
	spell_contingency,	TAR_IGNORE,		POS_STANDING,
	&gsn_contingency,	SLOT(0),	20,	24,
	"",
	"",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
        FALSE, FALSE,
        "Anon", NULL
    },

    {
	"continual light",
	{  6,  4,  6,  9,  9, 53,  8, 53, 53, 53, 53, 53,  7, 53,  7, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2 },
	spell_continual_light,	TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT(57),	 7,	12,
	"",
	"!Continual Light!",
	"",
        SCLASS_CREATION,	0,
	0, 0, 0, 0,
        FALSE, FALSE,
        NULL, NULL
    },

    {
	"control undead",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 26, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  1,  2,  2,  1,  2,  2 },
	spell_control_undead,	TAR_CHAR_OFFENSIVE,	POS_STANDING,
	NULL,			SLOT(0),	5,	12,
	"",
	"You regain control of your own will.",
	"",
        SCLASS_NECROMANCY,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"control weather",
	{ 15, 19, 28, 22, 20, 53, 53, 53, 53, 53, 53, 53, 53, 53, 15, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2 },
	spell_control_weather,	TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT(11),	25,	12,
	"",
	"!Control Weather!",
	"",
        SCLASS_WEATHER,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"corpse form",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53,  7, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  1,  2,  2,  1,  2,  2 },
	spell_corpse_form,	TAR_CHAR_SELF,		POS_STANDING,
	&gsn_corpse_form,	SLOT(0),	5,	12,
	"",
	"Your normal form returns.",
	"",
        SCLASS_NECROMANCY,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"create focus",
	{  1, 53, 53, 53, 53, 53, 53, 53, 53,  1, 53,  1,  1,  1,  1,  1 },
	{  2,  4,  2,  4,  4,  4,  4,  4,  2,  3,  2,  2,  3,  1,  1,  1 },
	spell_create_focus,	TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(0),	100,	48,
	"",
	"You slowly begin to feel the powers around you again.",
	"",
	SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

   {
	"create food",
	{ 10,  5, 11, 12, 12, 53, 11, 53, 53, 53, 53, 53,  12, 53, 11, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,   2,  2,  1,  2 },
	spell_create_food,	TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT(12),	 5,	12,
	"",
	"!Create Food!",
	"",
        SCLASS_CREATION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"create rose",
	{ 16, 11, 10, 24, 24, 53, 24, 53, 53, 53, 53, 53,  1, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2 },
	spell_create_rose,	TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT(511),	30, 	12,
	"",
	"!Create Rose!",
	"",
        SCLASS_CREATION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },  

    {
	"create spring",
	{ 14, 17, 23, 20, 15, 53, 19, 53, 53, 53, 53, 53, 53, 53, 16, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2 },
	spell_create_spring,	TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT(80),	20,	12,
	"",
	"!Create Spring!",
	"",
         SCLASS_CREATION,	0,
	0, 0, 0, 0,
        FALSE, FALSE,
        NULL, NULL
   },

     {
	"create symbol",
	{ 53,  1, 53,  1,  1, 53,  1, 53,  1, 53,  1, 53, 53, 53, 53, 53 },
	{  2,  1,  4,  4,  4,  4,  2,  4,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_create_symbol,	TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT(0),	100,	48,
	"",
	"!Create Symbol!",
	"",
	SCLASS_CHANNELING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"create water",
	{  8,  3, 12, 11, 11, 53, 10, 53, 53, 53, 53, 53, 10, 53,  9, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2 },
	spell_create_water,	TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(13),	 5,	12,
	"",
	"!Create Water!",
	"",
        SCLASS_CREATION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"cure blindness",
	{ 53,  6, 53,  8,  7, 53,  7, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_cure_blindness,	TAR_CHAR_DEFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(14),	 5,	12,
	"",
	"!Cure Blindness!",
	"",
        SCLASS_HEALING,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"cure critical",
	{ 53, 13, 53, 19, 18, 53, 17, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_cure_critical,	TAR_CHAR_DEFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(15),	20,	12,
	"",
	"!Cure Critical!",
	"",
        SCLASS_HEALING,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"cure disease",
	{ 53, 13, 53, 14, 14, 53, 13, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_cure_disease,	TAR_CHAR_DEFENSIVE,	POS_STANDING,
	NULL,			SLOT(501),	20,	12,
	"",
	"!Cure Disease!",
	"",
        SCLASS_HEALING,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"cure light",
	{ 53,  1, 53,  3,  2, 53,  2, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_cure_light,	TAR_CHAR_DEFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(16),	10,	12,
	"",
	"!Cure Light!",
	"",
        SCLASS_HEALING,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"cure poison",
	{ 53, 14, 53, 16, 15, 53, 15, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_cure_poison,	TAR_CHAR_DEFENSIVE,	POS_STANDING,
	NULL,			SLOT(43),	 5,	12,
	"",
	"!Cure Poison!",
	"",
        SCLASS_HEALING,		0,
	0, 0, 0, 0,
        FALSE, FALSE,
        NULL, NULL
    },

    {
	"cure serious",
	{ 53,  7, 53, 10,  9, 53,  9, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_cure_serious,	TAR_CHAR_DEFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(61),	15,	12,
	"",
	"!Cure Serious!",
	"",
        SCLASS_HEALING,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"curse",
	{ 18, 18, 26, 22, 53, 53, 53, 53, 53, 53, 53, 53, 29, 16, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2 , 2,  2,  2,  2,  2,  2,  1,  2,  2 },
	spell_curse,		TAR_OBJ_CHAR_OFF,	POS_FIGHTING,
	&gsn_curse,		SLOT(17),	20,	12,
	"curse",
	"The curse wears off.", 
	"$p is no longer impure.",
        SCLASS_MALEDICTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"dark vision",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 10, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2 },
	spell_dark_vision,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),	 5,	12,
	"",
	"The red glow in your eyes fades.",
	"",
        SCLASS_DETECTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"death trance",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 35, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  1,  2,  2,  1,  2,  2 },
	spell_death_trance,	TAR_CHAR_SELF,		POS_STANDING,
	&gsn_death_trance,	SLOT(0),	50,	24,
	"",
	"Your resistance to death fades.",
	"",
        SCLASS_NECROMANCY,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"defender true",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 50, 53, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  1,  2,  2,  2,  2,  2 },
	spell_defender_true,	TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(0),	700,	96,
	"",
	"!defender true!",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"delude",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 15 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  1,  2,  1,  2,  2,  2,  1 },
	spell_delude,           TAR_CHAR_SELF,          POS_STANDING,
	NULL,                   SLOT(0),        10,     12,
	"",
	"Your aura is no longer masked.",
	"",
        SCLASS_ILLUSION,        0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"demonfire",
	{ 53, 34, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 42, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_demonfire,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(505),	20,	12,
	"torments",
	"!Demonfire!",
	"",
        SCLASS_CHANNELING,	SSCLASS_UNHOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },	

    {
	"demonskin",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 30, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2,  2 },
	spell_demonskin,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),	200,	36,
	"demonskin",
	"",
	"",
        SCLASS_DEMONIC,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },	

    {
	"demonskin true",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 50, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2,  2 },
	spell_demonskin_true,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),	300,	36,
	"demonskin",
	"",
	"",
        SCLASS_DEMONIC,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },	

    {
	"detect evil",
	{ 11,  4, 12, 53, 12, 11, 12, 53, 53, 53, 53, 53,  8, 11,  2,  9 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_detect_evil,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(18),	 5,	12,
	"",
	"The red in your vision disappears.",
	"",
        SCLASS_DETECTION,	SSCLASS_HOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
        "detect good",
        { 11,  4, 12, 53, 12, 11, 12, 53, 53, 53, 53, 53,  8, 11,  2,  9 },
        {  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
        spell_detect_good,	TAR_CHAR_SELF,		POS_STANDING,
        NULL,			SLOT(513),	 5,	12,
        "",
        "The gold in your vision disappears.",
        "",
        SCLASS_DETECTION,	SSCLASS_UNHOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"detect hidden",
	{ 15, 11, 12, 53, 19, 11, 19, 53, 53, 53, 53, 53, 17, 15,  9, 13 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1 },
	spell_detect_hidden,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(44),	 5,	12,
	"",
	"You feel less aware of your surroundings.",
	"",
        SCLASS_DETECTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"detect illusion",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53,  8, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2 },
	spell_detect_illusion,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),	5,	12,
	"",
	"A momentary ache burns at the back of your eyes.",
	"",
        SCLASS_DETECTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"detect invis",
	{  3,  8,  6, 53, 16,  5, 16, 53, 53, 53, 53, 53,  5,  3,  3,  3 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1 },
	spell_detect_invis,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(19),	 5,	12,
	"",
	"You no longer see invisible objects.",
	"",
        SCLASS_DETECTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"detect magic",
	{  2,  6,  5, 53, 14,  4, 14, 53, 53, 53, 53, 53,  4,  2,  1,  2 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  3,  1,  1,  1 },
	spell_detect_magic,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(20),	 5,	12,
	"",
	"The detect magic wears off.",
	"",
        SCLASS_DETECTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"detect poison",
	{ 15,  7,  9, 53, 15,  8, 15, 53, 53, 53, 53, 53, 12, 12, 15, 20 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  2,  2 },
	spell_detect_poison,	TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(21),	 5,	12,
	"",
	"!Detect Poison!",
	"",
        SCLASS_DETECTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"detect traps",
	{ 18, 15, 30, 53, 53, 53, 26, 28, 53, 53, 53, 53, 53, 20, 16, 20 },
	{  1,  1,  3,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2 },
	spell_detect_traps,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),	 5,	12,
	"",
	"Your ability to detect traps wears off.",
	"",
        SCLASS_DETECTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"dispel evil",
	{ 53, 15, 53, 21, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 17, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2 },
	spell_dispel_evil,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(22),	15,	12,
	"dispel evil",
	"!Dispel Evil!",
	"",
        SCLASS_CHANNELING,	SSCLASS_HOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
        "dispel good",
        { 53, 15, 53, 21, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 17, 53 },
        {  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2 },
        spell_dispel_good,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
        NULL,			SLOT(512),	15,	12,
        "dispel good",
        "!Dispel Good!",
        "",
        SCLASS_CHANNELING,	SSCLASS_UNHOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"dispel magic",
	{ 16, 24, 30, 30, 29, 30, 28, 53, 53, 53, 53, 53, 32, 53, 15, 16 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1 },
	spell_dispel_magic,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	&gsn_dispel_magic,	SLOT(59),	15,	12,
	"",
	"!Dispel Magic!",
	"",
        SCLASS_PROTECTIVE,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"distraction",
	{ 52, 52, 52, 52, 53, 53, 53, 53, 53, 53, 53, 53, 12, 53, 53, 53 },
	{  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2 },
	spell_distraction,      TAR_CHAR_OFFENSIVE,     POS_FIGHTING,
	NULL,                   0,              10,     18,
	"",
	"The feeling that someone is watching you passes.",
	"",
        SCLASS_BEGUILING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"druidstaff true",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 50, 53, 53, 53, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  2,  2,  2,  2,  2,  2 },
	spell_druidstaff_true,	TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(0),	700,	96,
	"",
	"!druidstaff true!",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"earth elemental",
	{ 52, 52, 52, 52, 53, 53, 53, 53, 32, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  3,  2,  2,  2,  2,  2,  2,  2 },
	spell_earth_elemental,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),	50,	24,
	"",
	"",
	"",
        SCLASS_EARTH,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"earthquake",
	{ 53, 10, 53, 14, 53, 53, 53, 53,  8, 53, 53, 53, 53, 53, 12, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2,  1,  2 },
	spell_earthquake,	TAR_IGNORE,		POS_FIGHTING,
	NULL,			SLOT(23),	15,	12,
	"earthquake",
	"!Earthquake!",
	"",
        SCLASS_CHANNELING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"embalm",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53,  1, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  1,  2,  2,  1,  2,  2 },
	spell_embalm,		TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(0),	5,	12,
	"",
	"!embalm!",
	"",
        SCLASS_NECROMANCY,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"embed",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	{  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_embed,	        TAR_IGNORE,	POS_STANDING,
	NULL,			SLOT(0),	100,	12,
	"",
	"You are surrounded by a bright glow as you are released from your prison.",
	"",
        SCLASS_CHANNELING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"empathic healing",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 16, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2 },
	spell_empathic_healing,	TAR_CHAR_DEFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(0),	50,	24,
	"",
	"",
	"",
        SCLASS_HEALING,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"enchant armor",
	{ 16, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 14, 16 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  2,  2,  2,  2,  2,  2,  1,  1 },
	spell_enchant_armor,	TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(510),	100,	24,
	"",
	"!Enchant Armor!",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
        FALSE, FALSE,
        NULL, NULL
    },

    {
	"enchant weapon",
	{ 17, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 15, 17 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  2,  2,  2,  2,  2,  2,  1,  1 },
	spell_enchant_weapon,	TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(24),	100,	24,
	"",
	"!Enchant Weapon!",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"energy drain",
	{ 19, 22, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 17, 53, 18 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2 },
	spell_energy_drain,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(25),	35,	12,
	"energy drain",
	"!Energy Drain!",
	"",
        SCLASS_MALEDICTION,	SSCLASS_UNHOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
        "ere faerie glamour",
        { 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 25, 53, 53, 53 },
        {  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2 },
	spell_ere_faerie_glamour,TAR_IGNORE, POS_STANDING,
        &gsn_ere_faerie_glamour,SLOT(0), 26,  12,
	"",
	"Your sense of self returns.",
	"",
	SCLASS_BEGUILING,   0,
	0, 0, 0, 0,
	FALSE, TRUE,
        NULL, NULL
    },

    {
	"eye bite",
	{ 52, 52, 52, 52, 53, 53, 53, 53, 53, 53, 53, 53, 26, 53, 53, 53 },
	{  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2 },
	spell_eye_bite,         TAR_CHAR_OFFENSIVE,     POS_FIGHTING,
	NULL,                   0,      10,     18,
	"",
	"Your vision returns.",
	"",
        SCLASS_BEGUILING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"execute",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	{  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_execute,	        TAR_IGNORE,	POS_STANDING,
	NULL,			SLOT(0),	100,	12,
	"",
	"!Execute!",
	"",
        SCLASS_CHANNELING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"extend",
	{ 25, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 27, 23, 27 },
	{  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  2,  3 },
	spell_extend,	        TAR_IGNORE,		POS_STANDING,
	NULL,                   SLOT(0),        10,     12,
	"",
	"",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
        FALSE, FALSE,
        "Anon", NULL
    },

    {
	"extend true",
	{ 45, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 47, 43, 47 },
	{  4,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  3,  4,  3,  4 },
	spell_extend_true,      TAR_CHAR_SELF,          POS_STANDING,
	NULL,                   SLOT(0),        100,    24,
	"",
	"",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
        FALSE, FALSE,
        "Anon", NULL
    },

    {
	"extinguish",
	{ 15, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 15, 15, 15 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_extinguish,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,                   SLOT(0),	 5,	12,
	"",
	"",
	"Light starts to glow from $p",
        SCLASS_MALEDICTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        "Underdark", NULL
    },

    {
	"extradite",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	{  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_extradite,        TAR_CHAR_OFFENSIVE,	POS_STANDING,
	NULL,			SLOT(0),	100,	12,
	"",
	"!Extradite!",
	"",
        SCLASS_CHANNELING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"eyes of the undead",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 10, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2 },
	spell_eyes_of_the_undead,TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),	 5,	12,
	"",
	"The gray tinge disappears from your vision.",
	"",
        SCLASS_DETECTION,	SSCLASS_UNHOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"faerie fire",
	{  6,  3,  5,  8,  7, 53, 53, 53, 53, 53, 53, 53,  4, 53,  6, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  1,  2 },
	spell_faerie_fire,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(72),	 5,	12,
	"faerie fire",
	"The pink aura around you fades away.",
	"",
        SCLASS_WEATHER,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"faerie fog",
	{ 14, 21, 16, 24, 23, 53, 53, 53, 53, 53, 53, 53, 15, 53, 14, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  1,  2 },
	spell_faerie_fog,	TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT(73),	12,	12,
	"faerie fog",
	"!Faerie Fog!",
	"",
        SCLASS_WEATHER,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

#if 0
    {
	"farsight",		{ 14, 16, 16, 53 },	{ 1, 1, 2, 2},
	spell_farsight,		TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT(521),	36,	20,
	"farsight",		"!Farsight!",		"",
        SCLASS_DETECTION,	0
    },
#endif

    {
	"fine",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	{  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_fine,	        TAR_IGNORE,	POS_STANDING,
	NULL,			SLOT(0),	100,	12,
	"",
	"!Fine!",
	"",
        SCLASS_CHANNELING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"fire track",
	{ 38, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 38, 38, 38 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_fire_track,	TAR_IGNORE,     POS_STANDING,
	&gsn_fire_track,	SLOT(0),	50,	12,
	"",
	"",
	"",
        SCLASS_TRANSPORT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        "'New Thalos'", NULL
    },

    {
	"fireball",
	{ 22, 53, 30, 26, 53, 53, 53, 53, 53, 53, 53, 53, 53, 24, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2 },
	spell_fireball,		TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(26),	15,	12,
	"fireball",
	"!Fireball!",
	"",
        SCLASS_ELEMENTAL,	SSCLASS_FIRE,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, "'New Thalos'"
    },
  
    {
	"fireproof",
	{ 13, 12, 19, 18, 17, 53, 16, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_fireproof,	TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(523),	10,	12,
	"",
	"",
	"$p's protective aura fades.",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"fix soul",
	{  5,  9, 53, 53, 53, 53, 53, 53, 53, 53, 53,  7, 53, 53,  4,  6 },
	{  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_fix_soul,		TAR_CHAR_SELF,		POS_STANDING,
	&gsn_fix_soul,		SLOT(0),	15,	24,
	"",
	"",
	"",
	SCLASS_TRANSPORT,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"flamestrike",
	{ 53, 20, 53, 27, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 23, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2 },
	spell_flamestrike,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(65),	20,	12,
	"flamestrike",
	"!Flamestrike!",
	"",
        SCLASS_CHANNELING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"fly",
	{ 10, 18, 20, 22, 53, 53, 53, 53, 53, 53, 53, 53, 53, 12, 10, 10 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_fly,		TAR_CHAR_DEFENSIVE,	POS_STANDING,
	NULL,			SLOT(56),	10,	18,
	"",
	"You slowly float to the ground.",
	"",
        SCLASS_TRANSPORT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"floating disc",
	{  4, 10,  7, 16, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_floating_disc,	TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT(522),	40,	24,
	"",
	"!Floating disc!",
	"",
        SCLASS_CREATION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"fortitude",
	{  53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 11, 53, 53, 53, 53, 53 },
	{   3,  1,  4,  2,  3,  4,  1,  4,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_fortitude,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),	10,	12,
	"",
	"Your resistance to evil weakens.",
	"",
        SCLASS_PROTECTIVE,	SSCLASS_HOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"fossilize",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_fossilize,		TAR_IGNORE,		POS_FIGHTING,
	NULL,			SLOT(0),	50,	12,
	"",
	"Your flesh returns to its normal form.",
	"",
        SCLASS_EARTH,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
        "frenzy",
        { 53, 24, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
        {  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
        spell_frenzy,		TAR_CHAR_DEFENSIVE,	POS_FIGHTING,
        NULL,			SLOT(504),	30,	24,
        "",
        "Your rage ebbs.",
        "",
        SCLASS_BENEDICTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"gate",
	{ 27, 17, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 30, 23, 28 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_gate,		TAR_IGNORE,		POS_FIGHTING,
	NULL,			SLOT(83),	80,	12,
	"",
	"!Gate!",
	"",
        SCLASS_TRANSPORT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"giant strength",
	{ 11, 53, 22, 20, 25, 53, 53, 53, 53, 53, 53, 53, 22, 15, 11, 11 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_giant_strength,	TAR_CHAR_DEFENSIVE,	POS_STANDING,
	NULL,			SLOT(39),	20,	12,
	"",
	"You feel weaker.",
	"",
        SCLASS_ENHANCEMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"golden defender",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 20, 53, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  1,  2,  2,  2,  2,  2 },
	spell_golden_defender,TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(0),	500,	96,
	"",
	"!golden defender!",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"golden druidstaff",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 20, 53, 53, 53, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  2,  2,  2,  2,  2,  2 },
	spell_golden_druidstaff,TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(0),	500,	96,
	"",
	"!golden druidstaff!",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"greater conjuration",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 40, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2,  2 },
	spell_greater_conjuration,	TAR_IGNORE,	POS_STANDING,
	NULL,			SLOT(0),	90,	24,
	"",
	"",
	"",
	SCLASS_DEMONIC,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },	

    {
	"greater defender",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53,  8, 53, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  1,  2,  2,  2,  2,  2 },
	spell_greater_defender,TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(0),	300,	96,
	"",
	"!greater defender!",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"greater demonskin",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 40, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2,  2 },
	spell_greater_demonskin,TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),	250,	36,
	"demonskin",
	"",
	"",
        SCLASS_DEMONIC,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"greater druidstaff",
	{ 53, 53, 53, 53, 53, 53, 53, 53,  8, 53, 53, 53, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  2,  2,  2,  2,  2,  2 },
	spell_greater_druidstaff,TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(0),	300,	96,
	"",
	"!greater druidstaff!",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"greater healing",
	{ 53, 35, 53, 53, 53, 53, 40, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_greater_healing,	TAR_CHAR_DEFENSIVE,	POS_STANDING,
	NULL,			SLOT(0),	100,	18,
	"",
	"You feel your strength return.",
	"",
	SCLASS_HEALING,		SSCLASS_GTR_HEALS,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"greater shadowblade",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 30, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  1,  2,  1,  2,  2,  2,  2 },
	spell_greater_shadowblade,TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(0),	300,	96,
	"",
	"!greater shadowblade!",
	"",
        SCLASS_ENCHANTMENT,	SSCLASS_UNHOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"hallucinate",
	{ 53, 53, 53, 53, 53, 28, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_hallucinate,	TAR_CHAR_DEFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(0),	20,	12,
	"",
	"Your perspective on things returns to \"normal\".",
	"",
        SCLASS_BEGUILING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"harden focus",
	{ 25, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 25, 25, 25 },
	{  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_harden_focus,     TAR_OBJ_INV,          POS_STANDING,
	NULL,			SLOT(0),        200,	24,
	"",
	"",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        "Anon", NULL
    },

    {
	"harm",
	{ 53, 23, 53, 53, 53, 28, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_harm,		TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(27),	35,	12,
	"harm spell",
	"!Harm!",
	"",
        SCLASS_CHANNELING,	SSCLASS_UNHOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },
  
    {
	"haste",
	{ 21, 53, 26, 29, 34, 53, 53, 53, 53, 53, 53, 53, 26, 24, 21, 21 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_haste,		TAR_CHAR_DEFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(502),	30,	12,
	"",
	"You feel yourself slow down.",
	"",
        SCLASS_ENHANCEMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"heal",
	{ 53, 21, 53, 53, 53, 53, 30, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_heal,		TAR_CHAR_DEFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(28),	50,	12,
	"",
	"!Heal!",
	"",
        SCLASS_HEALING,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },
  
    {
	"heat metal",
	{ 53, 16, 53, 23, 53, 53, 53, 53, 14, 53, 53, 53, 53, 53, 18, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2,  1,  2 },
	spell_heat_metal,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(516), 	25,	18,
	"spell",
	"!Heat Metal!",
	"",
        SCLASS_CHANNELING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"holy defender",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 17, 53, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  1,  2,  2,  2,  2,  2 },
	spell_holy_defender,	TAR_OBJ_INV,		POS_FIGHTING,
	NULL,			SLOT(0),	15,	12,
	"",
	"!holy defender!",
	"The protective aura surrounding $p disappears.",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"holy word",
	{ 53, 36, 53, 53, 53, 53, 53, 53, 53, 53, 44, 53, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  2,  2,  3,  2,  2,  2,  2,  2 },
	spell_holy_word,	TAR_IGNORE,	POS_FIGHTING,
	NULL,			SLOT(506), 	200,	24,
	"divine wrath",
	"!Holy Word!",
	"",
        SCLASS_CHANNELING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"identify",
	{ 15, 16, 18, 53, 24, 17, 24, 53, 53, 53, 53, 53, 18, 15, 10, 15 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_identify,		TAR_OBJ_INV,	POS_STANDING,
	NULL,			SLOT(53),	12,	24,
	"",
	"!Identify!",
	"",
        SCLASS_DETECTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"improved invisibility",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 21 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  1,  2,  1,  2,  2,  2,  1 },
	spell_improved_invisibility,TAR_CHAR_SELF,POS_STANDING,
	&gsn_improved_invis,	SLOT(0),	15,	12,
	"",
	"You are no longer cloaked.",
	"",
        SCLASS_ILLUSION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"infravision",
	{  9, 13, 10, 16, 15, 53, 53, 53, 53, 53, 53, 53, 10, 10,  9,  9 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_infravision,	TAR_CHAR_DEFENSIVE,	POS_STANDING,
	NULL,			SLOT(77),	 5,	18,
	"",
	"You no longer see in the dark.",
	"",
        SCLASS_ENHANCEMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"invisibility",
	{  5, 53,  9, 53, 53,  8, 53, 53, 53, 53, 53, 53, 12, 53,  5,  3 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1 },
	spell_invis,		TAR_OBJ_CHAR_DEF,	POS_STANDING,
	&gsn_invis,		SLOT(29),	 5,	12,
	"",
	"You are no longer invisible.",		
	"$p fades into view.",
        SCLASS_ILLUSION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"know alignment",
	{  12, 9, 20, 53, 17, 19, 17, 53, 53, 53, 53, 53, 16, 12,  7, 11 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_know_alignment,	TAR_CHAR_DEFENSIVE,	POS_STANDING,
	NULL,			SLOT(58),	 9,	12,
	"",
	"!Know Alignment!",
	"",
        SCLASS_DETECTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"lesser conjuration",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 20, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2,  2 },
	spell_lesser_conjuration,	TAR_IGNORE,	POS_STANDING,
	NULL,			SLOT(0),	70,	24,
	"",
	"",
	"",
        SCLASS_DEMONIC,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },	

    {
	"lesser defender",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53,  4, 53, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  1,  2,  2,  2,  2,  2 },
	spell_lesser_defender,TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(0),	200,	96,
	"",
	"!lesser defender!",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"lesser demonskin",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 20, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2,  2 },
	spell_lesser_demonskin,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),	150,	36,
	"demonskin",
	"",
	"",
        SCLASS_DEMONIC,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"lesser druidstaff",
	{ 53, 53, 53, 53, 53, 53, 53, 53,  4, 53, 53, 53, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  2,  2,  2,  2,  2,  2 },
	spell_lesser_druidstaff,TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(0),	200,	96,
	"",
	"!lesser druidstaff!",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"lesser shadowblade",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 20, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  1,  2,  2,  2,  2,  2,  2 },
	spell_lesser_shadowblade,TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(0),	200,	96,
	"",
	"!lesser shadowblade!",
	"",
        SCLASS_ENCHANTMENT,	SSCLASS_UNHOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"lethargy",
	{ 52, 52, 52, 52, 53, 53, 53, 53, 53, 53, 53, 53, 34, 53, 53, 53 },
	{  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2 },
	spell_lethargy,         TAR_CHAR_OFFENSIVE,     POS_FIGHTING,
	NULL,                   0,      10,     18,
	"",
	"Your body starts to move at the normal rate.",
	"",
        SCLASS_BEGUILING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"lightning bolt",
	{ 13, 23, 18, 16, 15, 53, 53, 53, 53, 53, 53, 53, 18, 15, 13, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1,  2 },
	spell_lightning_bolt,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(30),	15,	12,
	"lightning bolt",
	"!Lightning Bolt!",
	"",
        SCLASS_WEATHER,		SSCLASS_LIGHTNING,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"lightning storm",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_lightning_storm,	TAR_IGNORE,		POS_FIGHTING,
	NULL,			SLOT(0),	50,	12,
	"lightning storm",
	"!Lightning storm!",
	"",
        SCLASS_ELEMENTAL,	SSCLASS_LIGHTNING,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"locate object",
	{  9, 15, 11, 53, 23, 10, 23, 53, 53, 53, 53, 53, 11, 10,  7,  9 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1,  1 },
	spell_locate_object,	TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT(31),	20,	18,
	"",
	"!Locate Object!",
	"",
        SCLASS_DETECTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"long eye",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 19, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2 },
	spell_long_eye,		    TAR_IGNORE,		    POS_STANDING,
	NULL,			SLOT(0),	10,	18,
	"",
	"!Long Eye!",
	"",
        SCLASS_DETECTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"lord defender",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 30, 53, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  1,  2,  2,  2,  2,  2 },
	spell_lord_defender,	TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(0),	600,	96,
	"",
	"!lord defender!",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"lord druidstaff",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 30, 53, 53, 53, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  2,  2,  2,  2,  2,  2 },
	spell_lord_druidstaff,TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(0),	600,	96,
	"",
	"!lord druidstaff!",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"lord shadowblade",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 40, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  1,  2,  1,  2,  2,  2,  2 },
	spell_lord_shadowblade,	TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(0),	400,	96,
	"",
	"!lord shadowblade!",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
        "love",
        { 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
        {  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
        spell_love,		TAR_CHAR_DEFENSIVE,	POS_STANDING,
        NULL,			SLOT(0),	30,	12,
        "",
        "",
        "",
        SCLASS_BEGUILING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"magic missile",
	{  1, 53,  2,  2, 53, 53, 53, 53, 53, 53, 53, 53, 53,  2, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2 },
	spell_magic_missile,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(32),	15,	12,
	"magic missile",
	"!Magic Missile!",
	"",
        SCLASS_ELEMENTAL,	SSCLASS_ENERGY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"mana stabilization",
	{  30, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 24, 53, 35, 22, 30 },
	{   2,  2,  3,  3,  3,  3,  3,  3,  3,  3,  3,  1,  2,  2,  1,  1 },
	spell_mana_stabilization,TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),	10,	12,
	"",
	"Your vitality no longer acts as a substitute for mana.",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"mark tree",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 25, 53, 53, 53, 53, 53, 53, 53 },
	{  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2,  2,  2 },
	spell_mark_tree,	TAR_CHAR_SELF,		POS_STANDING,
	&gsn_mark_tree,		SLOT(0),	15,	24,
	"",
	"",
	"",
	SCLASS_TRANSPORT,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"mass healing",
	{ 53, 38, 53, 53, 53, 53, 45, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_mass_healing,	TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT(508),	100,	36,
	"",
	"!Mass Healing!",
	"",
        SCLASS_HEALING,		0,
	0, 0, 0, 0,
        FALSE, FALSE,
        NULL, NULL
    },

    {
	"mass invis",
	{ 22, 25, 31, 53, 53, 30, 53, 53, 53, 53, 53, 53, 53, 53, 53, 20 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1 },
	spell_mass_invis,	TAR_IGNORE,		POS_STANDING,
	&gsn_mass_invis,	SLOT(69),	20,	24,
	"",
	"You are no longer invisible.",
	"",
        SCLASS_ILLUSION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"merge soul",
	{  7, 11, 53, 53, 53, 53, 53, 53, 53, 53, 53,  9, 53, 53,  6,  8 },
	{  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_merge_soul,			TAR_CHAR_SELF,			POS_STANDING,
	NULL,			SLOT(0),		10,	24,
	"",
	"",
	"",
	SCLASS_TRANSPORT,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"mind scan",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 26, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2 },
	spell_mind_scan,	    TAR_IGNORE, 	POS_STANDING,
	NULL,			SLOT(0),	10,	18,
	"",
	"!Mind Scan!",
	"",
        SCLASS_DETECTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"minor conjuration",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 10, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2,  2 },
	spell_minor_conjuration,	TAR_IGNORE,	POS_STANDING,
	NULL,			SLOT(0),	60,	24,
	"",
	"",
	"",
        SCLASS_DEMONIC,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },	

    {
	"minor defender",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53,  1, 53, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  1,  2,  2,  2,  2,  2 },
	spell_minor_defender,	TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT(0),	100,	96,
	"",
	"!minor defender!",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"minor demonskin",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 10, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2,  2 },
	spell_minor_demonskin,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),	100,	36,
	"demonskin",
	"",
	"",
        SCLASS_DEMONIC,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"minor druidstaff",
	{ 53, 53, 53, 53, 53, 53, 53, 53,  1, 53, 53, 53, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  2,  2,  2,  2,  2,  2 },
	spell_minor_druidstaff,	TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT(0),	100,	96,
	"",
	"!minor druidstaff!",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"minor shadowblade",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 10, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  1,  2,  1,  2,  2,  2,  2 },
	spell_minor_shadowblade,TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT(0),	100,	96,
	"",
	"!minor shadowblade!",
	"",
        SCLASS_ENCHANTMENT,	SSCLASS_UNHOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"mirror image",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 23 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  1,  2,  1,  2,  2,  2,  1 },
	spell_mirror_image,     TAR_CHAR_SELF,	POS_STANDING,
	&gsn_mirror_image,	SLOT(0),	15,	12,
	"",
	"What was many is now one.",
	"",
        SCLASS_ILLUSION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
        "nexus",
        { 40, 35, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 43, 35, 40 },
        {  2,  2,  4,  4,  4,  4,  4,  2,  2,  2,  2,  2,  2,  2,  1,  1 },
        spell_nexus,		TAR_IGNORE,		POS_STANDING,
        NULL,			SLOT(520),	150,	36,
        "",
        "!Nexus!",
        "",
        SCLASS_TRANSPORT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"numb",
	{ 52, 52, 52, 52, 53, 53, 53, 53, 53, 53, 53, 53, 18, 53, 53, 53 },
	{  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2 },
	spell_numb,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,	                0,	10,	18,
	"",
	"The feeling in your limbs returns.",
	"",
        SCLASS_BEGUILING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"overload protection",
	{ 15, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 15, 15, 15 },
	{  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_overload_protection,TAR_OBJ_INV,          POS_STANDING,
	NULL,			SLOT(0),        50,	18,
	"",
	"",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        "Anon", NULL
    },

    {
	"pass door",
	{ 24, 32, 25, 37, 53, 53, 53, 53, 53, 53, 53, 53, 53, 27, 21, 24 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1 },
	spell_pass_door,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(74),	20,	12,
	"",
	"You feel solid again.",
	"",
        SCLASS_TRANSPORT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"petrify",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 45, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_petrify,		TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	&gsn_petrify,		SLOT(0),	50,	18,
	"petrification",
	"A weight lifts from you, your bones returning to normal.",
	"",
        SCLASS_EARTH,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"plague",
	{ 23, 17, 36, 26, 53, 53, 53, 53, 53, 53, 53, 53, 53, 15, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2 },
	spell_plague,		TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	&gsn_plague,		SLOT(503),	20,	12,
	"sickness",
	"Your sores vanish.",
	"",
        SCLASS_MALEDICTION,	SSCLASS_UNHOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"poison",
	{ 17, 12, 15, 21, 53, 53, 53, 53, 53, 53, 53, 53, 53, 10, 53, 19 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2 },
	spell_poison,		TAR_OBJ_CHAR_OFF,	POS_FIGHTING,
	&gsn_poison,		SLOT(33),	10,	12,
	"poison",
	"You feel less sick.",	
	"The poison on $p dries up.",
        SCLASS_MALEDICTION,	SSCLASS_UNHOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },
    
    {
	"poison resistance",
	{  5, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53,  4, 53,  7 },
	{  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2 },
	spell_poison_resistance,TAR_CHAR_DEFENSIVE,	POS_STANDING,
	NULL,		        SLOT( 0),	 5,	5,
	"",
	"You protection against poison fades.", 
	"",
        SCLASS_WEATHER,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        "Underdark", NULL
    },

    {
	"polar wind",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 27, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2,  2 },
	spell_polar_wind,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(0),	15,	12,
	"polar wind",
	"!polar wind!",
	"",
        SCLASS_ELEMENTAL,	SSCLASS_COLD,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
        "portal",
        { 35, 30, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 38, 28, 35 },
        {  2,  2,  4,  4,  4,  4,  4,  2,  2,  2,  2,  2,  2,  2,  1,  1 }, 
        spell_portal,		TAR_IGNORE,		POS_STANDING,
        NULL,			SLOT(519),	100,	24,
        "",
        "!Portal!",
        "",
        SCLASS_TRANSPORT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"prayer",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 16, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2 },
	spell_prayer,		TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT(0),	15,	12,
	"",
	"The strength of your blessing fades.",
	"",
	SCLASS_BENEDICTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"presence",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 35, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2 },
	spell_presence,		TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),	15,	24,
	"",
	"!Presence!",
	"",
        SCLASS_DETECTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"protection evil",
	{ 12,  9, 17, 11, 10, 53, 10, 53, 53, 53, 53, 53, 15, 53, 10, 12 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1 },
	spell_protection_evil,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(34), 	5,	12,
	"",
	"You feel less protected.",
	"",
        SCLASS_PROTECTIVE,	SSCLASS_HOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
        "protection good",
        { 12,  9, 17, 11, 53, 53, 53, 53, 53, 53, 53, 53, 15, 53, 10, 12 },
        {  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1 },
        spell_protection_good,	TAR_CHAR_SELF,		POS_STANDING,
        NULL,			SLOT(514),	 5,	12,
        "",
        "You feel less protected.",
        "",
        SCLASS_PROTECTIVE,	SSCLASS_UNHOLY,
	0, 0, 0, 0,
        FALSE, FALSE,
        NULL, NULL
    },

    {
	"ray of dissolution",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_ray_of_dissolution,TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(0),	20,	12,
	"",
	"!Ray of dissolution!",
	"",
        SCLASS_CHANNELING,	SSCLASS_UNHOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
        "ray of truth",
        { 53, 35, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
        {  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
        spell_ray_of_truth,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
        NULL,			SLOT(518),	20,	12,
        "ray of truth",
        "!Ray of Truth!",
        "",
        SCLASS_CHANNELING,	SSCLASS_HOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"raze",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_raze,		TAR_IGNORE,		POS_FIGHTING,
	NULL,			SLOT(0),	50,	12,
	"inferno",
	"!Raze!",
	"",
        SCLASS_ELEMENTAL,	SSCLASS_FIRE,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"recall druidstaff",
	{ 53, 53, 53, 53, 53, 53, 53, 53,  9, 53, 53, 53, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  2,  2,  2,  2,  2,  2 },
	spell_recall_druidstaff,TAR_IGNORE,		POS_FIGHTING,
	NULL,			SLOT(0),	90,	12,
	"",
	"!recall druidstaff!",
	"",
        SCLASS_TRANSPORT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"recall shadowblade",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53,  9, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  1,  2,  1,  2,  2,  2,  2 },
	spell_recall_shadowblade,TAR_IGNORE,		POS_FIGHTING,
	NULL,			SLOT(0),	90,	12,
	"",
	"!recall shadowblade!",
	"",
        SCLASS_TRANSPORT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"recharge",
	{  9, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53,  6, 10 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1 },
	spell_recharge,		TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(517),	60,	24,
	"",
	"!Recharge!",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"reflection",
	{  21, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 19, 53, 23, 18, 21 },
	{   2,  1,  3,  3,  3,  3,  3,  3,  3,  3,  3,  1,  2,  2,  1,  1 },
	spell_reflection,	TAR_CHAR_SELF,		POS_STANDING,
	&gsn_reflection,	SLOT(0),	15,	12,
	"",
	"The reflective magical shield surrounding you fades into nothingness.",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },


    {
	"refresh",
	{  8,  5, 12,  9,  8, 53, 53, 53, 53, 53, 53, 53, 10, 53,  8,  8 },
	{  1,  1,  2,  2,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1 },
	spell_refresh,		TAR_CHAR_DEFENSIVE,	POS_STANDING,
	NULL,			SLOT(81),	12,	18,
	"refresh",
	"!Refresh!",
	"",
        SCLASS_ENHANCEMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"rejoin soul",
	{ 34, 30, 53, 53, 53, 53, 53, 53, 53, 53, 53, 32, 53, 53, 53, 53 },
	{  2,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2, 32, 37 },
	spell_rejoin_soul,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),		100,	24,
	"",
	"",
	"",
	SCLASS_TRANSPORT,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"release mana",
	{  5, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53,  5,  5,  5 },
	{  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_release_mana,	TAR_IGNORE,             POS_STANDING,
	NULL,			SLOT(0),	 1,	12,
	"",
	"",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        "Anon", NULL
    },

    {
	"remove curse",
	{ 53, 18, 53, 53, 53, 53, 25, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_remove_curse,	TAR_OBJ_CHAR_DEF,	POS_STANDING,
	NULL,			SLOT(35),	 5,	12,
	"",
	"!Remove Curse!",
	"",
        SCLASS_BENEDICTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"sanctuary",
	{ 36, 20, 53, 53, 53, 53, 30, 53, 53, 53, 53, 53, 53, 53, 18, 36 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1 },
	spell_sanctuary,	TAR_CHAR_DEFENSIVE,	POS_STANDING,
	&gsn_sanctuary,		SLOT(36),	75,	12,
	"",
	"The white aura around your body fades.",
	"",
        SCLASS_PROTECTIVE,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"sand shadow",
	{  18, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 16, 20, 14 },
	{   1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_sand_shadow,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,		        SLOT(0),	25,	12,
	"",
	"You leave the world of shadows.",
	"",
        SCLASS_TRANSPORT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        "'New Thalos'", NULL
    },

    {
	"shadow",
	{  53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53,  5, 53, 53, 53, 53 },
	{   1,  2,  1,  4,  4,  1,  4,  4,  2,  2,  2,  1,  2,  2,  2,  2 },
	spell_shadow,		TAR_CHAR_SELF,		POS_STANDING,
	&gsn_shadow,		SLOT(0),	20,	12,
	"",
	"You reenter the real world.",
	"",
        SCLASS_TRANSPORT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"shadowblade true",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 50, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  1,  2,  1,  2,  2,  2,  2 },
	spell_shadowblade_true,	TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(0),	500,	96,
	"",
	"!shadowblade true!",
	"",
        SCLASS_ENCHANTMENT,	SSCLASS_UNHOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"shadowfire",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 25, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2 },
	spell_shadowfire,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(0),	15,	12,
	"shadowfire",
	"!Shadowfire!",
	"",
        SCLASS_ELEMENTAL,	SSCLASS_NEGATIVE,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"shield",
	{ 20, 35, 35, 40, 39, 53, 40, 53, 53, 53, 53, 53, 53, 53, 17, 20 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1 },
	spell_shield,		TAR_CHAR_DEFENSIVE,	POS_STANDING,
	NULL,			SLOT(67),	12,	18,
	"",
	"Your force shield shimmers then fades away.",
	"",
        SCLASS_PROTECTIVE,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"shocking grasp",
	{ 10, 53, 14, 13, 53, 53, 53, 53, 53, 53, 53, 53, 53, 10, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2 },
	spell_shocking_grasp,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(53),	15,	12,
	"shocking grasp",
	"!Shocking Grasp!",
	"",
        SCLASS_ELEMENTAL,	SSCLASS_LIGHTNING,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"silver defender",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 14, 53, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  2,  2,  2,  2,  2,  2 },
	spell_silver_defender,TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(0),	400,	96,
	"",
	"!silver defender!",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"silver druidstaff",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 14, 53, 53, 53, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  2,  2,  2,  2,  2,  2 },
	spell_silver_druidstaff,TAR_OBJ_INV,		POS_STANDING,
	NULL,			SLOT(0),	400,	96,
	"",
	"!silver druidstaff!",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"sleep",
	{ 10, 53, 11, 53, 53, 10, 53, 53, 53, 53, 53, 53,  8, 53, 13, 13 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  1 },
	spell_sleep,		TAR_CHAR_OFFENSIVE,	POS_STANDING,
	&gsn_sleep,		SLOT(38),	15,	12,
	"",
	"You feel less tired.",
	"",
        SCLASS_BEGUILING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
        "slow",
        { 23, 30, 29, 32, 53, 53, 53, 53, 53, 53, 53, 53, 24, 21, 53, 23 },
        {  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  1 },
        spell_slow,		TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
        &gsn_slow,		SLOT(515),	30,	12,
        "",
        "You feel yourself speed up.",
        "",
        SCLASS_MALEDICTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"smoke parasite",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 45, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_smoke_parasite,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	&gsn_smoke_parasite,	SLOT(0),	50,	12,
	"",
	"The tightness in your chest seems to pass.",
	"",
        SCLASS_MALEDICTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },
    
    {
	"soul vision",
	{ 23, 27, 53, 53, 53, 53, 53, 53, 53, 53, 53, 25, 53, 53, 21, 25 },
	{  1,  3,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_soul_vision,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),		20,	24,
	"",
	"",
	"",
	SCLASS_TRANSPORT,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"spell haste",
	{ 33, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 33, 33, 33 },
	{  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_spell_haste,      TAR_CHAR_SELF,          POS_FIGHTING,
	NULL,			SLOT(0),	35,	12,
	"",
	"Your enhanced casting skills fade.",
	"",
        SCLASS_ENHANCEMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        "Anon", NULL
    },

    {
	"spell steal",
	{  38, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 42, 53, 40, 30, 38 },
	{   1,  1,  3,  3,  3,  3,  3,  3,  3,  3,  3,  1,  2,  2,  1,  1 },
	spell_spell_steal,	TAR_CHAR_SELF,		POS_STANDING,
	&gsn_spell_steal,	SLOT(0),	30,	12,
	"",
	"The tingling of the magical trap fades as the enchantment disapates.",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },
    
    {
	"spider familier",
	{  2, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53,  1, 53,  2 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_spider_familier,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),	50,	24,
	"",
	"",
	"",
        SCLASS_ANIMAL,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        "Underdark", NULL
    },

    {
	"spider form",
	{ 42, 53, 53, 53, 35, 53, 53, 53, 53, 53, 53, 53, 53, 38, 53, 42 },
	{  2,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  2 },
	spell_spider_form,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),	40,	18,
	"",
	"Your body returns to its normal form.",
	"",
        SCLASS_NECROMANCY,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        "Underdark", NULL
    },
    
    {
	"spider plague",
	{ 22, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 15, 53, 24 },
	{  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2 },
	spell_spider_plague,	TAR_OBJ_CHAR_OFF,	POS_FIGHTING,
	NULL,   		SLOT(0),	15,	12,
	"spider plague",
	"You feel less sick.",	
	"",
        SCLASS_MALEDICTION,	SSCLASS_UNHOLY,
	0, 0, 0, 0,
	FALSE, FALSE,
        "Underdark", NULL
    },

    {
    	"starvation",
    	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
    	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
    	spell_starvation,	TAR_CHAR_OFFENSIVE,	POS_STANDING,
    	&gsn_starvation,	SLOT(0),	0,	12,
    	"",
    	"The gnawing hunger in your belly fades.",
    	"",
        SCLASS_MALEDICTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"stone skin",
	{ 25, 40, 40, 45, 35, 53, 53, 53, 53, 53, 53, 53, 40, 53, 23, 25 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1 },
	spell_stone_skin,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(66),	12,	18,
	"",
	"Your skin feels soft again.",
	"",
        SCLASS_PROTECTIVE,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"stone to flesh",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 25, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2 },
	spell_stone_to_flesh,	TAR_CHAR_DEFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(0),	25,	24,
	"",
	"",
	"",
        SCLASS_HEALING,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"store mana",
	{ 10, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 10, 10, 10 },
	{  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_store_mana,	TAR_OBJ_INV,          POS_STANDING,
	NULL,			SLOT(0),        150,	24,
	"",
	"",
	"",
        SCLASS_ENCHANTMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        "Anon", NULL
    },

    {
	"summon",
	{ 24, 12, 29, 22, 53, 53, 53, 53, 53, 53, 53, 53, 29, 53, 10, 24 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1 },
	spell_summon,		TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT(40),	50,	12,
	"",
	"!Summon!",
	"",
        SCLASS_TRANSPORT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"sundazzle",
	{ 28, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 31, 30 },
	{  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_sundazzle,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(0),	15,	12,
	"",
	"Your blinding glare leaves your eyes.",
	"",
        SCLASS_WEATHER,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        "'New Thalos'", NULL
    },

    {
	"sunfire",
	{ 22, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 24, 23 },
	{  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_sunfire,	        TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(0),	20,	12,
	"",
	"The smoke leaves your eyes.",
	"",
        SCLASS_WEATHER,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        "'New Thalos'", NULL
    },

    {
	"swarm",
	{ 53, 53, 53, 53, 53, 53, 53, 53,  8, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2,  2,  2 },
	spell_swarm,		TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(0),	50,	24,
	"",
	"",
	"",
        SCLASS_ANIMAL,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"teleport",
	{  13, 22, 25, 36, 53, 53, 53, 53, 53, 53, 53, 53, 53, 15, 11, 13 },
	{   1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_teleport,		TAR_CHAR_SELF,		POS_FIGHTING,
	NULL,	 		SLOT( 2),	35,	12,
	"",
	"!Teleport!",
	"",
        SCLASS_TRANSPORT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"the dragon flight thunder",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 15, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2 },
	spell_the_dragon_flight_thunder,TAR_IGNORE,	POS_STANDING,
	&gsn_the_dragon_flight_thunder,SLOT(0), 26,	12,
	"",
	"",
	"",
	SCLASS_BEGUILING,   0,
	0, 0, 0, 0,
	FALSE, TRUE,
        NULL, NULL
    },

    {
	"the voice of nations calling",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 10, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2 },
	spell_the_voice_of_nations_calling,TAR_IGNORE, POS_STANDING,
	&gsn_the_voice_of_nations_calling,SLOT(0), 26,  12,
	"",
	"You have lost your peace of mind.",
	"",
	SCLASS_BEGUILING,   0,
	0, 0, 0, 0,
	FALSE, TRUE,
        NULL, NULL
    },

    {
	"thunderstaff",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 17, 53, 53, 53, 53, 53, 53, 53 },
	{  2,  2,  4,  4,  4,  4,  4,  2,  1,  2,  2,  2,  2,  2,  2,  2 },
	spell_thunderstaff,	TAR_OBJ_INV,		POS_FIGHTING,
	NULL,			SLOT(0),	15,	12,
	"",
	"!thunderstaff!",
	"The arcing current surrounding $p disappears.",
        SCLASS_ELEMENTAL,	SSCLASS_LIGHTNING,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"traceless travel",
	{   9, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53,  8, 10,  7 },
	{   1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_traceless_travel,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,		        SLOT(0),	5,	12,
	"",
	"Your tracks are no longer masked.",
	"",
        SCLASS_TRANSPORT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        "'New Thalos'", NULL
    },

    {
	"tree transport",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 25, 53, 53, 53, 53, 53, 53, 53 },
	{  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2,  2,  2 },
	spell_tree_transport,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),	100,	24,
	"",
	"",
	"",
	SCLASS_TRANSPORT,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"turn undead",
	{  53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 15, 53, 53, 53, 53, 53 },
	{   3,  1,  4,  2,  3,  4,  1,  4,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_turn_undead	,TAR_IGNORE,		POS_FIGHTING,
	NULL,			SLOT(0),	30,	24,
	"turn attempt",
	"!Turn Undead!",
	"",
	SCLASS_CHANNELING,	SSCLASS_HOLY,			
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"unmark tree",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 25, 53, 53, 53, 53, 53, 53, 53 },
	{  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2,  2,  2 },
	spell_unmark_tree,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),		10,	24,
	"",
	"",
	"",
	SCLASS_TRANSPORT,		0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"ventriloquate",
	{  1, 53,  2, 53, 53, 1, 53, 53, 53, 53, 53, 53,  1, 53,  2,  1 },
	{  1,  1,  2,  2,  2, 1,  2,  2,  2,  2,  2,  2,  1,  2,  2,  1 },
	spell_ventriloquate,	TAR_IGNORE,		POS_STANDING,
	NULL,			SLOT(41),	 5,	12,
	"",
	"!Ventriloquate!",
	"",
        SCLASS_ILLUSION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

   {
	"water breathing",
	{ 15, 13, 37, 33, 21, 23, 53, 23, 53, 53, 53, 53, 53, 15, 15, 15 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_water_breathing,	TAR_CHAR_DEFENSIVE,	POS_STANDING,
	NULL,			SLOT(0),	 10,	12,
	"",
	"You can no longer breathe water.",
	"",
        SCLASS_ENHANCEMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

   {
	"water speech",
	{ 11,  9, 32, 28, 17, 18, 53, 18, 53, 53, 53, 53, 53, 11, 11, 11 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_water_speech,	TAR_CHAR_DEFENSIVE,	POS_STANDING,
	NULL,			SLOT(0),	 10,	12,
	"",
	"You can no longer speak beneath water.",
	"",
        SCLASS_ENHANCEMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

   {
	"waters ways",
	{ 25, 23, 53, 48, 36, 37, 53, 37, 53, 53, 53, 53, 53, 25, 25, 25 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_water_speech,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,			SLOT(0),	 10,	12,
	"",
	"You can no longer move freely beneath water water.",
	"",
        SCLASS_ENHANCEMENT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },        
    {
	"weaken",
	{ 11, 14, 16, 17, 53, 53, 53, 53, 53, 53, 53, 53, 16,  9, 53, 11 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  2,  1 },
	spell_weaken,		TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(68),	20,	12,
	"spell",
	"You feel stronger.",
	"",
        SCLASS_MALEDICTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"weapon curse",
	{ 15, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 13, 53, 16 },
	{  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  1 },
	spell_weapon_curse,	TAR_OBJ_INV,		POS_FIGHTING,
	&gsn_weapon_curse,	SLOT(0),	20,	12,
	"",
	"!weapon curse!",
	"The dark aura surrounding $p disappears.",
        SCLASS_MALEDICTION,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        "Underdark", NULL
    },

    {
	"wind shadow",
	{  32, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 32, 32, 32 },
	{   1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1 },
	spell_wind_shadow,	TAR_CHAR_SELF,		POS_STANDING,
	NULL,		        SLOT(0),        50,	12,
	"",
	"You return to your normal form.",
	"",
        SCLASS_TRANSPORT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        "'New Thalos'", NULL
    },

    {
	"winters touch",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 14, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2,  2 },
	spell_winters_touch,	TAR_OBJ_INV,		POS_FIGHTING,
	NULL,			SLOT(0),	10,	12,
	"winters touch",
	"!winters touch!",
	"The chilling mist surrounding $p disapates.",
        SCLASS_ELEMENTAL,	SSCLASS_COLD,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"withering touch",
	{ 53, 53, 53, 53, 53, 35, 53, 53, 53, 53, 53, 53, 53, 30, 53, 53 },
	{  1,  1,  3,  3,  4,  2,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2 },
	spell_withering_touch,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(0),	30,	18,
	"withering touch",
	"The gnawing cold leaves your bones.",
	"",
	SCLASS_CHANNELING,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"word of recall",
	{ 32, 28, 40, 30, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_word_of_recall,	TAR_CHAR_SELF,		POS_RESTING,
	NULL,			SLOT(42),	 5,	12,
	"",
	"!Word of Recall!",
	"",
        SCLASS_TRANSPORT,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

/*
 * Dragon breath
 */
    {
	"acid breath",
	{ 31, 32, 33, 34, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_acid_breath,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(200),	50/*100*/,	24,
	"blast of acid",
	"!Acid Breath!",
	"",
        SCLASS_ELEMENTAL,	SSCLASS_ACID,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"fire breath",
	{ 40, 45, 50, 51, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_fire_breath,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(201),	150/*200*/,	24,
	"blast of flame",
	"The smoke leaves your eyes.",
	"",
        SCLASS_ELEMENTAL,	SSCLASS_FIRE,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"frost breath",
	{ 34, 36, 38, 40, 53, 53, 53, 53, 53, 30, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  1,  2,  2,  2,  2,  2,  2 },
	spell_frost_breath,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(202),	100/*125*/,	24,
	"blast of frost",
	"!Frost Breath!",
	"",
        SCLASS_ELEMENTAL,	SSCLASS_COLD,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"gas breath",
	{ 39, 43, 47, 50, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_gas_breath,	TAR_IGNORE,		POS_FIGHTING,
	NULL,			SLOT(203),	60/*175*/,	24,
	"blast of gas",
	"!Gas Breath!",
	"",
        SCLASS_ELEMENTAL,	SSCLASS_POISON,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

    {
	"lightning breath",
	{ 37, 40, 43, 46, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2 },
	spell_lightning_breath,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(204),	75/*150*/,	24,
	"blast of lightning",
	"!Lightning Breath!",
	"",
        SCLASS_ELEMENTAL,	SSCLASS_LIGHTNING,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },

/*
 * Spells for mega1.are from Glop/Erkenbrand.
 */
    {
	"general purpose",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  2,  2,  2,  2 },
	spell_general_purpose,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(401),	 0,	12,
	"general purpose ammo",
	"!General Purpose Ammo!",
	"",
	SCLASS_ELEMENTAL,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },
 
    {
	"high explosive",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  2,  2,  2,  2 },
	spell_high_explosive,	TAR_CHAR_OFFENSIVE,	POS_FIGHTING,
	NULL,			SLOT(402),	 0,	12,
	"high explosive ammo",
	"!High Explosive Ammo!",
	"",
	SCLASS_ELEMENTAL,	0,
	0, 0, 0, 0,
	FALSE, FALSE,
        NULL, NULL
    },


/* combat and weapons skills */


    {
	"axe",
	{  1,  1,  1,  1,  1,  1,  1,  1, 53, 53, 53, 53,  1,  1,  1,  1 },
	{  6,  6,  5,  4,  4,  5,  4,  3,  0,  0,  0,  0,  5,  3,  6,  6 },
	spell_null,		TAR_IGNORE,		POS_FIGHTING,
	&gsn_axe,		SLOT( 0),	0,	0,
	"",
	"!Axe!",
	"",
	SCLASS_WEAPON,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"bow",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	{  6,  6,  4,  5,  3,  4,  6,  5,  4,  6,  6,  4,  4,  6,  6,  6 },
	spell_null,		TAR_IGNORE,		POS_FIGHTING,
	&gsn_bow,		SLOT(0),	0,	0,
	"arrow",
	"!Bow!",
	"",
	SCLASS_WEAPON,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"dagger",
	{  1,  1,  1,  1,  1,  1,  1,  1, 53, 53, 53,  1,  1,  1,  1,  1 },
	{  2,  3,  2,  2,  2,  2,  2,  4,  0,  0,  0,  2,  2,  2,  2,  2 },
	spell_null,		TAR_IGNORE,		POS_FIGHTING,
	&gsn_dagger,		SLOT( 0),	0,	0,
	"",
	"!Dagger!",
	"",
	SCLASS_WEAPON,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },
 
    {
	"flail",
	{  1,  1,  1,  1,  1,  1,  1,  1, 53, 53, 53, 53,  1,  1,  1,  1 },
	{  6,  3,  6,  4,  4,  6,  3,  3,  0,  0,  0,  0,  6,  6,  6,  6 },
        spell_null,             TAR_IGNORE,             POS_FIGHTING,
        &gsn_flail,            	SLOT( 0),       0,      0,
        "",
        "!Flail!",
        "",
        SCLASS_WEAPON,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"mace",
	{  1,  1,  1,  1,  1,  1,  1,  1, 53, 53, 53, 53,  1,  1,  1,  1 },
	{  5,  2,  3,  3,  3,  3,  2,  2,  0,  0,  0,  0,  3,  5,  5,  5 },
        spell_null,		TAR_IGNORE,             POS_FIGHTING,
        &gsn_mace,		SLOT( 0),       0,      0,
        "",
        "!Mace!",
        "",
        SCLASS_WEAPON,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"polearm",
	{  1,  1,  1,  1,  1,  1,  1,  1, 53, 53,  1, 53,  1,  1,  1,  1 },
	{  6,  6,  6,  4,  6,  6,  4,  6,  0,  0,  4,  0,  6,  6,  6,  6 },
        spell_null,             TAR_IGNORE,             POS_FIGHTING,
        &gsn_polearm,           SLOT( 0),       0,      0,
        "",
        "!Polearm!",
        "",
        SCLASS_WEAPON,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },
    
    {
	"shield block",
	{  1,  1,  1,  1,  1, 53,  1, 53, 53, 53, 53, 53, 53,  1,  1,  1 },
	{  6,  4,  6,  2,  2,  6,  2,  0,  0,  0,  0,  0,  0,  6,  6,  6 },
	spell_null,		TAR_IGNORE,		POS_FIGHTING,
	&gsn_shield_block,	SLOT(0),	0,	0,
	"",
	"!Shield!",
	"",
        SCLASS_COMBAT,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },
 
    {
	"spear",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1, 53, 53, 53,  1,  1,  1,  1 },
	{  4,  4,  4,  3,  3,  4,  3,  3,  3,  0,  0,  0,  4,  4,  4,  4 },
        spell_null,             TAR_IGNORE,             POS_FIGHTING,
        &gsn_spear,            	SLOT( 0),       0,      0,
        "",
        "!Spear!",
        "",
        SCLASS_WEAPON,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"sword",
	{  1,  1,  1,  1,  1,  1,  1,  1, 53, 53,  1, 53,  1,  1,  1,  1 },
	{  5,  6,  3,  2,  2,  3,  2,  5,  0,  0,  2,  0,  3,  5,  5,  5 },
        spell_null,             TAR_IGNORE,             POS_FIGHTING,
        &gsn_sword,            	SLOT( 0),       0,      0,
        "",
        "!sword!",
        "",
        SCLASS_WEAPON,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"whip",
	{  1,  1,  1,  1,  1,  1,  1,  1, 53,  1, 53, 53,  1,  1,  1,  1 },
	{  6,  5,  5,  4,  4,  5,  5,  5,  0,  3,  0,  0,  3,  6,  6,  6 },
        spell_null,             TAR_IGNORE,             POS_FIGHTING,
        &gsn_whip,            	SLOT( 0),       0,      0,
        "",
        "!Whip!",
        "",
        SCLASS_WEAPON,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
   },

    {
	"backstab",
	{ 53, 53,  1, 53, 53,  1, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  0,  0,  5,  0,  0,  5,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
	spell_null,             TAR_IGNORE,             POS_STANDING,
	&gsn_backstab,          SLOT( 0),        0,     24,
	"backstab",
	"!Backstab!",
	"",
	SCLASS_SUBTERFUGE,	0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"bash",
	{ 53, 53, 53,  1, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  0,  0,  0,  4,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
        spell_null,             TAR_IGNORE,             POS_FIGHTING,
        &gsn_bash,            	SLOT( 0),       0,      24,
        "bash",
        "!Bash!",
        "",
        SCLASS_COMBAT,		SSCLASS_DISHONORABLE,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"berserk",
	{ 53, 53, 53, 28, 26, 23, 53, 17, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  0,  0,  0,  4,  4,  3,  0,  3,  0,  0,  0,  0,  0,  0,  0,  0 },
        spell_null,             TAR_IGNORE,             POS_FIGHTING,
        &gsn_berserk,        	SLOT(0),	0,      0,
        "",
        "You feel your pulse slow down.",
        "",
        SCLASS_COMBAT,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"bite",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	{  3,  3,  2,  1,  1,  2,  2,  1,  2,  1,  3,  3,  3,  3,  3,  3 },
        spell_null,             TAR_IGNORE,             POS_FIGHTING,
        &gsn_bite,        	SLOT(0),	0,      24,
        "bite",
        "",
        "",
        SCLASS_COMBAT,		0,
	0, 0, PART_FANGS, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"blind fighting",
	{ 53, 53, 53, 30, 53, 53, 35, 19, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  0,  0,  0,  4,  0,  0,  4,  2,  0,  0,  0,  0,  0,  0,  0,  0 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_blind_fighting,	SLOT(0),	0,	32,
	"blind fighting",
	"!Blind Fighting!",
	"",
	SCLASS_COMBAT,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"circle",
	{ 53, 53, 10, 53, 53, 10, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  0,  0,  5,  0,  0,  5,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
	spell_null,		TAR_IGNORE,		POS_FIGHTING,
	&gsn_circle,		SLOT( 0),	0,	36,
	"backstab",
	"!Circle!",
	"",
	SCLASS_SUBTERFUGE,	0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"dirt kicking",
	{ 53, 53,  3,  3, 53,  3, 53, 53, 53, 53, 53, 53,  8, 53, 53, 53 },
	{  0,  0,  3,  4,  0,  3,  0,  0,  0,  0,  0,  0,  4,  0,  0,  0 }, 
	spell_null,		TAR_IGNORE,		POS_FIGHTING,
	&gsn_dirt,		SLOT( 0),	0,	24,
	"kicked dirt",
	"You rub the dirt out of your eyes.",
	"",
        SCLASS_COMBAT,		SSCLASS_DISHONORABLE,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
        "disarm",
        { 53, 53, 12, 11, 12, 12, 10,  5, 53, 53, 53, 53, 53, 53, 53, 53 },
        {  0,  0,  6,  4,  5,  3,  4,  2,  0,  0,  0,  0,  0,  0,  0,  0 },
        spell_null,             TAR_IGNORE,             POS_FIGHTING,
        &gsn_disarm,            SLOT( 0),        0,     24,
        "",
        "!Disarm!",
        "",
        SCLASS_COMBAT,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },
 
    {
        "dodge",
        { 20, 22,  1, 13, 13,  1, 15,  1, 53, 53, 53, 53,  5, 25, 20, 20 },
        {  8,  8,  4,  6,  5,  4,  6,  3,  0,  0,  0,  0,  5,  8,  8,  8 },
        spell_null,             TAR_IGNORE,             POS_FIGHTING,
        &gsn_dodge,             SLOT( 0),        0,     0,
        "",
        "!Dodge!",
        "",
        SCLASS_COMBAT,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"dual wield",
	{ 53, 53, 53, 30, 25, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  0,  0,  0,  6,  6,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
	spell_null,		TAR_IGNORE,		POS_FIGHTING,
	&gsn_dual_wield,	SLOT(0),	0,	0,
	"",
	"!Dual Wield!",
	"",
	SCLASS_COMBAT,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"eagles claw",
	{ 53, 53, 53, 53, 53, 53, 53, 40, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  0,  0,  0,  0,  0,  0,  0,  6,  0,  0,  0,  0,  0,  0,  0,  0 },
	spell_null,		TAR_IGNORE,		POS_FIGHTING,
	&gsn_eagles_claw,	SLOT(0),	50,	24,
	"",
	"!Eagles Claw!",
	"",
	SCLASS_COMBAT,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
        "enhanced damage",
        { 53, 53, 25,  1,  3, 25,  1, 15, 53, 53, 53, 53, 53, 53, 53, 53 },
        {  0,  0,  5,  3,  4,  5,  3,  5,  0,  0,  0,  0,  0,  0,  0,  0 },
        spell_null,             TAR_IGNORE,             POS_FIGHTING,
        &gsn_enhanced_damage,   SLOT( 0),        0,     0,
        "",
        "!Enhanced Damage!",
        "",
        SCLASS_COMBAT,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"envenom",
	{ 53, 53, 10, 53, 53,  9, 53, 53, 53, 53, 53, 53, 53, 15, 53, 53 },
	{  0,  0,  4,  0,  0,  3,  0,  0,  0,  0,  0,  0,  0,  6,  0,  0 },
	spell_null,		TAR_IGNORE,	  	POS_RESTING,
	&gsn_envenom,		SLOT(0),	0,	36,
	"",
	"!Envenom!",
	"",
        SCLASS_SUBTERFUGE,	0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"escape",
	{ 53, 53,  1, 53, 53,  1, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  0,  0,  3,  0,  0,  2,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
	spell_null,		TAR_IGNORE,		POS_FIGHTING,
	&gsn_escape,		SLOT(0),	0,	24,
	"",
	"!Escape!",
	"",
        SCLASS_SUBTERFUGE,	0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"hand to hand",
	{ 25, 10, 15,  6,  7, 14, 53,  1, 53, 53, 53, 53, 13, 27, 25, 25 },
	{  8,  5,  6,  4,  5,  5,  0,  2,  0,  0,  0,  0,  5,  8,  8,  8 },
	spell_null,		TAR_IGNORE,		POS_FIGHTING,
	&gsn_hand_to_hand,	SLOT( 0),	0,	0,
	"",
	"!Hand to Hand!",
	"",
        SCLASS_COMBAT,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"joust",
	{ 53, 53, 53, 30, 53, 53, 35, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  0,  0,  0,  4,  0,  0,  4,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_joust,		SLOT(0),	0,	32,
	"joust",
	"!Joust!",
	"",
	SCLASS_COMBAT,		0,
	0, FORM_CENTAUR, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"ki punch",
	{ 53, 53, 53, 53, 53, 53, 53, 30, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  0,  0,  0,  0,  0,  0,  0,  4,  0,  0,  0,  0,  0,  0,  0,  0 },
	spell_null,		TAR_IGNORE,		POS_FIGHTING,
	&gsn_ki_punch,		SLOT(0),	20,	18,
	"",
	"!Ki Punch!",
	"",
	SCLASS_COMBAT,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"ki shout",
	{ 53, 53, 53, 53, 53, 53, 53, 20, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  0,  0,  0,  0,  0,  0,  0,  3,  0,  0,  0,  0,  0,  0,  0,  0 },
	spell_null,		TAR_IGNORE,		POS_FIGHTING,
	&gsn_ki_shout,		SLOT(0),	10,	24,
	"",
	"!Ki Shout!",
	"",
	SCLASS_COMBAT,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
        "kick",
        { 53, 12, 14,  8, 10, 14, 10,  3, 53, 53, 53, 53, 53, 53, 53, 53 },
        {  0,  4,  6,  3,  4,  6,  4,  3,  0,  0,  0,  0,  0,  0,  0,  0 },
        spell_null,             TAR_CHAR_OFFENSIVE,     POS_FIGHTING,
        &gsn_kick,              SLOT( 0),        0,     12,
        "kick",
        "!Kick!",
        "",
        SCLASS_COMBAT,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
        "parry",
        { 22, 20, 13,  1,  1, 13,  5,  8, 53, 53, 53, 53, 13, 24, 22, 22 },
        {  8,  8,  6,  4,  5,  6,  4,  4,  0,  0,  0,  0,  7,  8,  8,  8 },
        spell_null,             TAR_IGNORE,             POS_FIGHTING,
        &gsn_parry,             SLOT( 0),        0,     0,
        "",
        "!Parry!",
        "",
        SCLASS_COMBAT,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },


    {
        "pummeling",
        { 53, 53, 25,  1,  3, 25,  1, 15, 53, 53, 53, 53, 53, 53, 53, 53 },
        {  0,  0,  5,  3,  4,  5,  3,  5,  0,  0,  0,  0,  0,  0,  0,  0 },
        spell_null,		TAR_IGNORE,		POS_FIGHTING,
        &gsn_pummeling,		SLOT( 0),	0,	0,
        "",
        "!Pummeling!",
        "",
        SCLASS_COMBAT,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
        "rescue",
        { 53, 53, 53,  1,  4, 53,  1, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
        {  0,  0,  0,  4,  5,  0,  3,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
        spell_null,             TAR_IGNORE,             POS_FIGHTING,
        &gsn_rescue,            SLOT( 0),        0,     12,
        "",
        "!Rescue!",
        "",
        SCLASS_COMBAT,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"sap",
	{ 53, 53,  3, 53, 53,  3, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  0,  0,  4,  0,  0,  4,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_sap,		SLOT( 0),	0,	24,
	"backstab",
	"!Backstab!",
	"",
	SCLASS_SUBTERFUGE,	0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"trip",
	{ 53, 53,  1, 15, 53,  1, 53, 12, 53, 53, 53, 53,  1, 53, 53, 53 },
	{  0,  0,  4,  8,  0,  4,  0,  4,  0,  0,  0,  0,  4,  0,  0,  0 },
	spell_null,		TAR_IGNORE,		POS_FIGHTING,
	&gsn_trip,		SLOT( 0),	0,	24,
	"trip",
	"!Trip!",
	"",
        SCLASS_COMBAT,		SSCLASS_DISHONORABLE,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
        "arms mastery",
        { 53, 24, 12,  5,  7, 12,  5, 10, 53, 53, 53, 53, 15, 53, 53, 53 },
        {  0,  8,  5,  3,  4,  5,  5,  5,  0,  0,  0,  0,  5,  0,  0,  0 },
        spell_null,		TAR_IGNORE,		POS_FIGHTING,
        &gsn_arms_mastery,	SLOT(0),	0,	0,
        "",
        "!Arms Mastery!",
        "",
        SCLASS_COMBAT,	0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
        "weapon brawling",
        { 53, 53, 24, 12, 14, 53, 14, 53, 53, 53, 53, 53, 24, 53, 53, 53 },
        {  0,  0, 10,  4,  5,  0,  6,  0,  0,  0,  0,  0, 10,  0,  0,  0 },
        spell_null,		TAR_IGNORE,		POS_FIGHTING,
        &gsn_weapon_brawling,	SLOT( 0),	0,	0,
        "smash",
        "!Weapon Brawling!",
        "",
        SCLASS_COMBAT,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },
    
    {
	"trample",
	{ 53, 53, 53, 25, 20, 53, 30, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  0,  0,  0,  5,  4,  0,  5,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
	spell_null,		TAR_IGNORE,		POS_FIGHTING,
	&gsn_trample,		SLOT(0),	0,	24,
	"",
	"!Trample!",
	"",
	SCLASS_COMBAT,		0,
	0, FORM_CENTAUR, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"unmount",
	{ 53, 25, 20, 12, 10, 20, 53, 12, 53, 53, 53, 53, 20, 53, 53, 53 },
	{  0,  5,  4,  3,  2,  4,  0,  3,  0,  0,  0,  0,  4,  0,  0,  0 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_unmount,		SLOT(0),	0,	24,
	"unmount",
	"!Unmount!",
	"",
	SCLASS_COMBAT,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

/* non-combat skills */

    {
	"detrap",
	{ 53, 53,  1, 53, 53,  1, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  0,  0,  4,  0,  0,  4,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_detrap,		SLOT( 0),	 0,	36,
	"",
	"!Detrap!",
	"",
        SCLASS_SUBTERFUGE,	0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    { 
	"fast healing",
	{ 15,  9, 16,  6,  5, 53,  3,  5, 53, 53, 53, 53, 16, 21, 18, 18 },
	{  8,  5,  6,  4,  3,  0,  4,  4,  0,  0,  0,  0,  6,  9,  8,  8 },
	spell_null,		TAR_IGNORE,		POS_SLEEPING,
	&gsn_fast_healing,	SLOT( 0),	0,	0,
	"",
	"!Fast Healing!",
	"",
        SCLASS_HEALING_SKILLS,	0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"forage",
	{ 53, 53, 53, 25,  8, 53, 53, 53, 53, 53, 53, 53, 15, 53, 53, 53 },
	{  0,  0,  0,  4,  2,  0,  0,  0,  0,  0,  0,  0,  3,  0,  0,  0 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_forage,		SLOT( 0),	 0,	60,
	"",
	"!Forage!",
	"",
	SCLASS_OUTDOORS,	0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"haggle",
	{  7, 18,  1, 14, 53,  5, 53, 53, 53, 53, 53, 53,  1, 20, 15,  4 },
	{  5,  8,  3,  6,  0,  4,  0,  0,  0,  0,  0,  0,  3,  7,  6,  4 },
	spell_null,		TAR_IGNORE,		POS_RESTING,
	&gsn_haggle,		SLOT( 0),	0,	0,
	"",
	"!Haggle!",
	"",
        SCLASS_SOCIAL,	0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"hide",
	{ 53, 53,  1, 36, 20,  1, 53, 53, 53, 53, 53, 53,  5, 53, 53, 20 },
	{  0,  0,  4,  6,  5,  4,  0,  0,  0,  0,  0,  0,  3,  0,  0,  6 },
	spell_null,		TAR_IGNORE,		POS_RESTING,
	&gsn_hide,		SLOT( 0),	 0,	12,
	"",
	"!Hide!",
	"",
        SCLASS_SUBTERFUGE,	0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"lore",
	{ 10, 10,  6, 20, 53, 53, 53, 25, 53, 53, 53, 53,  5, 10,  8, 10 },
	{  6,  6,  4,  8,  0,  0,  0,  4,  0,  0,  0,  0,  3,  6,  3,  6 },
	spell_null,		TAR_IGNORE,		POS_RESTING,
	&gsn_lore,		SLOT( 0),	0,	36,
	"",
	"!Lore!",
	"",
        SCLASS_MISC_SKILL,	0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"meditation",
	{  6,  6, 15, 15, 53, 53, 13, 15, 53, 53, 53, 53, 15,  7,  6,  6 },
	{  5,  5,  8,  8,  0,  0,  8,  6,  0,  0,  0,  0,  8,  5,  5,  5 },
	spell_null,		TAR_IGNORE,		POS_SLEEPING,
	&gsn_meditation,	SLOT( 0),	0,	0,
	"",
	"Meditation",
	"",
        SCLASS_MAGICAL,	0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"music",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53,  1, 53, 53, 53 },
	{  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  3,  0,  0,  0 },
	spell_null,         TAR_IGNORE,     POS_STANDING,
	&gsn_music,         SLOT( 0),   0,  12,
	"",
	"!Music!",
	"",
        SCLASS_SOCIAL,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"peek",
	{  8, 21,  1, 14, 53, 1, 53, 53, 53, 53, 53, 53,  1,  8,  6,  8 },
	{  5,  7,  3,  6,  0,  3, 0,  0,  0,  0,  0,  0,  3,  5,  5,  5 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_peek,		SLOT( 0),	 0,	 0,
	"",
	"!Peek!",
	"",
        SCLASS_SUBTERFUGE,	0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"pick lock",
	{ 53, 53,  7, 53, 53,  7, 53, 53, 53, 53, 53, 53, 10, 53, 53, 53 },
	{  0,  0,  4,  0,  0,  4,  0,  0,  0,  0,  0,  0,  5,  0,  0,  0 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_pick_lock,		SLOT( 0),	 0,	12,
	"",
	"!Pick!",
	"",
        SCLASS_SUBTERFUGE,	0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"sing",
	{ 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53,  1, 53, 53, 53 },
	{  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  3,  0,  0,  0 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_sing,		SLOT( 0),	0,	12,
	"",
	"!Sing!",
	"",
        SCLASS_SOCIAL,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },


    {
	"sneak",
	{ 53, 53,  4, 30, 15,  4, 53, 53, 53, 53, 53, 53,  4, 53, 53, 25 },
	{  0,  0,  4,  6,  5,  4,  0,  0,  0,  0,  0,  0,  4,  0,  0,  5 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_sneak,		SLOT( 0),	 0,	12,
	"",
	"You no longer feel stealthy.",
	"",
        SCLASS_SUBTERFUGE,	0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"steal",
	{ 53, 53,  5, 53, 53,  6, 53, 53, 53, 53, 53, 53, 10, 53, 53, 53 },
	{  0,  0,  4,  0,  0,  5,  0,  0,  0,  0,  0,  0,  5,  0,  0,  0 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_steal,		SLOT( 0),	 0,	24,
	"",
	"!Steal!",
	"",
        SCLASS_SUBTERFUGE,	0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"tail",
	{ 53, 53, 53, 53, 53, 15, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  0,  0,  0,  0,  0,  5,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_tail,		SLOT(0),	0,	24,
	"",
	"!Tail!",
	"",
	SCLASS_SUBTERFUGE,	0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"track",		
	{ 53, 53, 53, 53, 15, 25, 53, 53, 53, 53, 53, 53, 53, 53, 53, 53 },
	{  0,  0,  0,  0,  4,  5,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_track,		SLOT(0),	0,	12,
	"",
	"!Track!",
	"",
	SCLASS_OUTDOORS,	0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"consider",
	{  1,  1,  1,  1,  1,  1,  1,  1, 53, 53, 53, 53,  1,  1,  1,  1 },
	{  3,  2,  1,  1,  1,  2,  1,  1,  0,  0,  0,  0,  1,  3,  3,  3 },
	spell_null,		TAR_IGNORE,		POS_RESTING,
	&gsn_consider,		SLOT(0),	0,	12,
	"",
	"!Consider!",
	"",
	SCLASS_COMBAT,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"scan",
	{ 10, 10,  4,  5,  3,  4,  6,  5, 53, 53, 53, 53,  4, 12,  8, 10 },
	{  2,  3,  1,  2,  1,  1,  2,  1,  0,  0,  0,  0,  2,  4,  3,  3 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_scan,		SLOT(0),	0,	12,
	"",
	"!Scan!",
	"",
	SCLASS_OUTDOORS,	0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"scrolls",
	{  1,  1,  1,  1,  1,  1,  1,  1, 53, 53, 53, 53,  1,  1,  1,  1 },
	{  2,  3,  5,  8,  7,  5,  7,  7,  0,  0,  0,  0,  5,  2,  2,  2 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_scrolls,		SLOT( 0),	0,	24,
	"",
	"!Scrolls!",
	"",
        SCLASS_MAGICAL,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"staves",
	{  1,  1,  1,  1,  1,  1,  1,  1, 53, 53, 53, 53,  1,  1,  1,  1 },
	{  2,  3,  5,  8,  7,  5,  7,  5,  0,  0,  0,  0,  5,  2,  2,  2 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_staves,		SLOT( 0),	0,	12,
	"",
	"!Staves!",
	"",
        SCLASS_MAGICAL,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },
    
    {
	"wands",
	{  1,  1,  1,  1,  1,  1,  1,  1, 53, 53, 53, 53,  1,  1,  1,  1 },
	{  2,  3,  5,  8,  7,  5,  8,  6,  0,  0,  0,  0,  5,  2,  2,  2 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_wands,		SLOT( 0),	0,	12,
	"",
	"!Wands!",
	"",
        SCLASS_MAGICAL,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"recall",
	{  1,  1,  1,  1,  1,  1,  1,  1, 53, 53, 53, 53,  1,  1,  1,  1 },
	{  2,  2,  2,  2,  2,  2,  2,  2,  0,  0,  0,  0,  2,  2,  2,  2 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_recall,		SLOT( 0),	0,	12,
	"",
	"!Recall!",
	"",
        SCLASS_MISC_SKILL,      0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },
        
    {
	"ride",
	{  1,  1,  1,  1,  1,  1,  1,  1, 53, 53, 53, 53,  1,  1,  1,  1 },
	{  3,  2,  3,  2,  1,  3,  2,  4,  0,  0,  0,  0,  3,  4,  3,  3 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_ride,		SLOT(0),	0,	12,
	"",
	"!Ride!",
	"",
	SCLASS_OUTDOORS,	0,
	0, FORM_CENTAUR, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

/*
 * Entries from this point cannot be used by players as skills/spells.
 */
    {
	"place holder",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&max_player_skill,	SLOT(0),	0,	0,
	"",			"",			"",
	SCLASS_NONE,		0,
	0, 0, 0, 0,
        FALSE, FALSE,
        NULL, NULL
    },
    
/*
 * Spell like affects.
 */
    {
	"petrification",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_petrification,	SLOT(0),	0,	0,
	"",
	"Your sense of numbness leaves your body.",
	"",
        SCLASS_NONE,		0,
	0, 0, 0, 0,
	TRUE, FALSE,
        NULL, NULL
    },

    {
	"aging",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_aging,		SLOT(0),	0,	0,
	"",
	"Strength returns to you, your youth regained.",
	"",
	SCLASS_NONE,		0,
	0, 0, 0, 0,
        FALSE, FALSE,
        NULL, NULL
    },

    {
	"cadaverous appearance",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_cadaverous_appearance, SLOT(0),	0,	0,
	"",			"",			"",
	SCLASS_NONE,		0,
        0, 0, 0, 0,
        FALSE, FALSE,
        NULL, NULL
    },

    {
	"intolerance to light",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_light_intolerance,	SLOT(0),	0,	0,
	"",			"",			"",
	SCLASS_NONE,		0,
	0, 0, 0, 0,
        FALSE, FALSE,
        NULL, NULL
    },

    {
	"agrophobia",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_agrophobia,	SLOT(0),	0,	0,
	"",			"",			"",
	SCLASS_NONE,		0,
	0, 0, 0, 0,
        FALSE, FALSE,
        NULL, NULL
    },

    {
	"catatonia",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_catatonia,		SLOT(0),	0,	0,
	"",			"",			"",
	SCLASS_NONE,		0,
	0, 0, 0, 0,
        FALSE, FALSE,
        NULL, NULL
    },

    {
	"claustrophobia",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_claustrophobia,	SLOT(0),	0,	0,
	"",			"",			"",
	SCLASS_NONE,		0,
	0, 0, 0, 0,
        FALSE, FALSE,
        NULL, NULL
    },

    {
	"dipsomania",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_dipsomania,	SLOT(0),	0,	0,
	"",			"",			"",
	SCLASS_NONE,		0,
	0, 0, 0, 0,
        FALSE, FALSE,
        NULL, NULL
    },

    {
	"introversion",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_introversion,	SLOT(0),	0,	0,
	"",			"",			"",
	SCLASS_NONE,		0,
	0, 0, 0, 0,
        FALSE, FALSE,
        NULL, NULL
    },

    {
	"mania",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_mania,		SLOT(0),	0,	0,
	"",			"",			"",
	SCLASS_NONE,		0,
	0, 0, 0, 0,
        FALSE, FALSE,
        NULL, NULL
    },

    {
	"melancholia",
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	{  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },
	spell_null,		TAR_IGNORE,		POS_STANDING,
	&gsn_melancholia,	SLOT(0),	0,	0,
	"",			"",			"",
	SCLASS_NONE,		0,
	0, 0, 0, 0,
        FALSE, FALSE,
        NULL, NULL
    },
};

const   struct  group_type      group_table     [MAX_GROUP]     =
{

    {
	"rom basics",		{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
	{ "scrolls", "staves", "wands", "recall", "consider", "scan",
	  "ride" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"mage basics",		{ 0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "dagger", "create focus" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"cleric basics",	{ -1, 0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "mace", "create symbol" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },
   
    {
	"thief basics",		{ -1, -1, 0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "dagger", "steal", "escape" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"warrior basics",	{ -1, -1, -1, 0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "sword", "arms mastery", "create symbol" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },
    
    {
	"ranger basics",	{ -1, -1, -1, -1, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "sword", "create symbol", "track" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"ninja basics",		{ -1, -1, -1, -1, -1, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "dagger", "backstab", "escape" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"templar basics",	{ -1, -1, -1, -1, -1, -1, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "sword", "fast healing", "create symbol" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"monk basics",		{ -1, -1, -1, -1, -1, -1, -1, 10, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "hand to hand", "arms mastery", "dodge" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"warden basics",	{ -1, -1, -1, -1, -1, -1, -1, -1, 10, -1, -1, -1, -1, -1, -1, -1 },
	{ "" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"antipaladin basics",	{ -1, -1, -1, -1, -1, -1, -1, -1, -1, 10, -1, -1, -1, -1, -1, -1 },
	{ "" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"paladin basics",	{ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 10, -1, -1, -1, -1, -1 },
	{ "" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"sorcerer basics",	{ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 10, -1, -1, -1, -1 },
	{ "" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"bard basics",		{ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 10, -1, -1, -1 },
	{ "dagger", "sing", "music", "create focus" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"necromancer basics",	{ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 10, -1, -1 },
	{ "dagger", "create focus", "lore" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"augurer basics",	{ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 10, -1 },
	{ "dagger", "create focus", "lore" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"illusionist basics",	{ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 10 },
	{ "dagger", "create focus", "lore" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"mage default",		{ 40, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "lore", "beguiling", "combat", "detection", "enhancement", "illusion",
	  "maledictions", "protective", "transportation", "weather" },
        SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"cleric default",	{ -1, 40, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "flail", "attack", "creation", "curative",  "benedictions", 
	  "detection", "healing", "maledictions", "protective", "shield block", 
	  "transportation", "weather" },
        SCLASS_NONE, TRUE,
        NULL, NULL
    },
 
    {
	"thief default",	{ -1, -1, 40, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "mace", "sword", "backstab", "detrap", "disarm", "dodge",
	  "trip", "hide", "peek", "pick lock", "sneak" },
        SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"warrior default",	{ -1, -1, -1, 40, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "weaponsmaster", "shield block", "bash", "disarm", "enhanced damage", 
	  "parry", "rescue", "weapon brawling" },
        SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"ranger default",	{ -1, -1, -1, -1, 40, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "disarm", "dodge", "enhanced damage", "dual wield", "parry",
	  "arms mastery", "weapon brawling", "sneak", "fast healing",
	  "forage" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"ninja default",	{ -1, -1, -1, -1, -1, 40, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "enhanced damage", "parry", "dodge", "arms mastery", 
	  "detrap", "tail", "track", "hand to hand", "envenom", 
	  "hide", "sneak" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"templar default",	{ -1, -1, -1, -1, -1, -1, 40, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "disarm", "enhanced damage", "parry", "rescue", "arms mastery",
	  "weapon brawling", "meditation", "healing",
	  "curative" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"monk default",		{ -1, -1, -1, -1, -1, -1, -1, 40, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "spear", "disarm", "pummeling", "kick", "parry",
	  "trip", "fast healing", "meditation",
	  "ki punch", "ki shout", "eagles claw" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"bard default",		{ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 40, -1, -1, -1 },
	{ "dagger", "dodge", "trip", "pick lock", "sneak", "forage", "steal",
	  "bard song", "sword" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"necromancer default",	{ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 40, -1, -1 },
	{ "combat", "detection", "enhancement", "maledictions", 
	  "transportation", "necromancy", "water mastery" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"augurer default",	{ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 40, -1 },
	{ "detection", "enhancement", "enchantment", "creation",
	  "protective", "transportation", "weather", "augury", 
	  "beguiling", "water mastery" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"illusionist default",	{ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 40 },
	{ "detection", "enhancement", "enchantment",
	  "protective", "transportation", "greater illusions", 
	  "illusion", "maledictions" },
	SCLASS_NONE, TRUE,
        NULL, NULL
    },

    {
	"weaponsmaster",	{ 38, 33, 28, 23, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "axe", "bow", "dagger", "flail", "mace", "polearm", "spear", "sword","whip" },
	SCLASS_WEAPON, TRUE,
        NULL, NULL
    },

    {
	"animal mastery",	{ -1, -1, -1, -1, -1, -1, -1, -1,  8, -1, -1, -1, -1, -1, -1, -1 },
	{ "befriend", "animal call", "swarm", "animal form" },
	SCLASS_ANIMAL, TRUE,
        NULL, NULL
    },
    
    {
        "arachnomancy",         {  7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  6, -1,  7 },
        { "spider familier", "poison resistance", "spider plague", "spider form" },
        SCLASS_ANIMAL, TRUE,
        "Underdark", ""
    },

    {
	"attack",		{ -1,  5, -1, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 7, -1 },
	{ "demonfire", "dispel evil", "dispel good", "earthquake", 
	  "flamestrike", "heat metal", "ray of truth" },
	SCLASS_CHANNELING, TRUE,
        NULL, NULL
    },

    {
	"augury",		{ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  6, -1 },
	{ "dark vision", "long eye", "detect illusion",
	  "presence", "mind scan" },
	SCLASS_DETECTION, TRUE,
        NULL, NULL
    },

    {
	"bard song",	        { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 10, -1, -1, -1 },
	{ "bright moonkiss touch", "the voice of nations calling",
          "the dragon flight thunder", "come clarion rage",
          "ere faerie glamour" },
	SCLASS_SPIRIT, TRUE,
        NULL, NULL
    },

    {
	"banishments",	        { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  3, -1, -1, -1, -1, -1 },
	{ "banishment", "turn undead" },
	SSCLASS_HOLY, TRUE,
        NULL, NULL
    },

    {
	"beguiling",		{  4, -1,  8, -1, -1,  6, -1, -1, -1, -1, -1, -1,  6, -1,  5,  5 },
	{ "calm", "charm person", "sleep" },
	SCLASS_BEGUILING, TRUE,
        NULL, NULL
    },

    {
	"benedictions",		{ -1,  4, -1,  6, -1, -1,  4, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "bless", "calm", "frenzy", "holy word", "remove curse" },
	SCLASS_BENEDICTION, TRUE,
        NULL, NULL
    },

    {
	"combat",		{  6, -1, 12, 12, -1, -1, -1, -1, -1, -1, -1, -1, -1,  7, -1, -1 },
	{ "acid blast", "burning hands", "chain lightning", "chill touch",
	  "colour spray", "fireball", "lightning bolt", "magic missile",
	  "shocking grasp", "sunfire"  },
        SCLASS_ELEMENTAL, TRUE,
        NULL, NULL
    },

   {
	"conjurations",		{  -1, -1, -1, -1, -1, -1, -1, -1, -1,  8, -1, -1, -1, -1, -1, -1 },
	{ "minor conjuration", "lesser conjuration", "conjuration",
	  "greater conjuration", "conjuration true" },
	SCLASS_DEMONIC, TRUE,
        NULL, NULL
    },

    {
	"creation",		{  4,  4,  8,  8,  6, -1,  5, -1, -1, -1, -1, -1,  6, -1,  5, -1 },
	{ "continual light", "create food", "create spring", "create water",
	  "create rose", "floating disc" },
	SCLASS_CREATION, TRUE,
        NULL, NULL
    },

    {
	"curative",		{ -1,  4, -1, 10,  7, -1,  5, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "cure blindness", "cure disease", "cure poison" },
	SCLASS_HEALING, TRUE,
        NULL, NULL
    }, 

    {
	"defender",		{ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 10, -1, -1, -1, -1, -1 },
	{ "minor defender", "lesser defender", "greater defender",
	  "silver defender", "golden defender", "lord defender",
	  "defender true", "holy defender" },
	SCLASS_ENCHANTMENT, TRUE,
        NULL, NULL
    },

    {
	"delusions",		{ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  6, -1, -1, -1 },
	{ "distraction", "numb", "eye bite", "lethargy" },
	SCLASS_BEGUILING, TRUE,
        NULL, NULL
    },

    {
	"detection",		{  5,  4,  8, -1, 10,  7, 10, -1, -1, -1, -1, -1,  6,  5,  3,  4 },
	{ "detect evil", "detect good", "detect hidden", "detect invis", 
	  "detect magic", "detect poison", "detect traps", /*"farsight",*/
	  "identify", "know alignment", "locate object" },
	SCLASS_DETECTION, TRUE,
        NULL, NULL
    },

    {
	"demonic skin",		{ -1, -1, -1, -1, -1, -1, -1, -1, -1, 12, -1, -1, -1, -1, -1, -1 },
	{ "minor demonskin", "lesser demonskin", "demonskin",
	  "greater demonskin", "demonskin true" },
	SCLASS_DEMONIC, TRUE,
        NULL, NULL
    },

    {
	"draconian",		{  8, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "acid breath", "fire breath", "frost breath", "gas breath",
	  "lightning breath"  },
	SCLASS_ELEMENTAL, TRUE,
        NULL, NULL
    },

    {
	"druids staff",		{ -1, -1, -1, -1, -1, -1, -1, -1, 10, -1, -1, -1, -1, -1, -1, -1 },
	{ "minor druidstaff", "lesser druidstaff", "greater druidstaff",
	  "silver druidstaff", "golden druidstaff", "lord druidstaff",
	  "druidstaff true", "recall druidstaff", "thunderstaff" },
	SCLASS_ENCHANTMENT, TRUE,
        NULL, NULL
    },

    {
	"enchantment",		{ 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  5,  7 },
	{ "enchant armor", "enchant weapon", "fireproof", "recharge" },
	SCLASS_ENCHANTMENT, TRUE,
        NULL, NULL
    },

    { 
	"enhancement",		{  5, -1, 10, 10,  9, -1, -1, -1, -1, -1, -1, -1,  9,  7,  5,  5 },
	{ "giant strength", "haste", "infravision", "refresh" },
	SCLASS_ENHANCEMENT, TRUE,
        NULL, NULL
    },

    { 
	"focus magic",		{  5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  5,  5,  5 },
	{ "release mana", "store mana", "overload protection", "harden focus" },
	SCLASS_ENCHANTMENT, TRUE,
        "Anon", NULL
    },

    {
	"greater illusions",	{ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  8 },
	{ "mirror image", "delude", "improved invisibility", "alkar" },
	SCLASS_ILLUSION, TRUE,
        NULL, NULL
    },

    {
	"harmful",		{ -1,  3, -1,  8, -1,  6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "cause critical", "cause light", "cause serious", "harm",
	  "withering touch" },
	SCLASS_CHANNELING, TRUE,
        NULL, NULL
    },

    {   
	"healing",		{ -1,  3, -1,  8,  5, -1,  4, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
	{ "cure critical", "cure light", "cure serious", "heal", 
	  "mass healing", "greater healing", "refresh" },
	SCLASS_HEALING, TRUE,
        NULL, NULL
    },

    {
	"holy warrior",		{ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  7, -1, -1, -1, -1, -1 },
	{ "prayer", "fortitude", "avatar", "holy word" },
	SCLASS_CHANNELING, TRUE,
        NULL, NULL
    },

    {
	"illusion",		{  4, -1,  7, -1, -1,  6, -1, -1, -1, -1, -1, -1,  5, -1,  4,  3 },
	{ "invisibility", "mass invis", "ventriloquate" },
	SCLASS_ILLUSION, TRUE,
        NULL, NULL
    },

    {
        "journeys ways",        {  7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  7,  7,  7 },
        { "fire track", "sand shadow", "traceless travel", "wind shadow" },
        SCLASS_TRANSPORT, TRUE,
        "'New Thalos'", NULL
    },

    {
	"laying of hands",	{ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  5, -1, -1, -1, -1, -1 },
	{ "awakening", "empathic healing", "stone to flesh" },
	SCLASS_HEALING, TRUE,
        NULL, NULL
    },

    {
	"maledictions",		{  5,  5,  6, 10, -1, -1, -1, -1, -1, -1, -1, -1,  7,  4, -1,  5 },
	{ "blindness", "change sex", "curse", "energy drain", "plague", 
	  "poison", "slow", "weaken" },
	SCLASS_MALEDICTION, TRUE,
        NULL, NULL
    },

    {
	"necromancy",		{ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  7, -1, -1 },
	{ "embalm", "death trance", "control undead", "corpse form",
	  "withering touch" },
	SCLASS_NECROMANCY, TRUE,
        NULL, NULL
    },

    { 
	"protective",		{  4,  4, 9, 10,  7, -1,  7, -1, -1, -1, -1, -1,  9, -1,  3,  4 },
	{ "armor", "cancellation", "dispel magic", "fireproof",
	  "protection evil", "protection good", "sanctuary", "shield", 
	  "stone skin" },
	SCLASS_PROTECTIVE, TRUE,
        NULL, NULL
    },

    {
	"shadow lore",		{ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 6, -1, -1, -1, -1 },
	{ "shadow", "eyes of the undead", "shadowfire", "cloud of darkness" },
	SCLASS_DEMONIC, TRUE,
        NULL, NULL
    },

    {
	"shadowblade",		{ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 10, -1, -1, -1, -1 },
	{ "blade of darkness", "greater shadowblade", "lesser shadowblade",
	  "lord shadowblade", "minor shadowblade", "recall shadowblade",
	  "shadowblade true" },
	SCLASS_ENCHANTMENT, TRUE,
        NULL, NULL
    },
    
    {
        "spell coordination",   {  7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  7,  7,  7 },
        { "contingency", "extend", "extend true" },
        SCLASS_ENCHANTMENT, TRUE,
        "Anon", NULL
    },

    {
	"stone mastery",	{  -1, -1, -1, -1, -1, -1, -1, -1,  8, -1, -1, -1, -1, -1, -1, -1 },
	{ "petrify", "earthquake", "earth elemental", "heat metal" },
	SCLASS_EARTH, TRUE,
        NULL, NULL
    },

    {
	"suns dazzle",	{  5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  5,  5 },
	{ "banish dazzle", "sundazzle", "sunfire" },
	SCLASS_WEATHER, TRUE,
        "'New Thalos'", NULL
    },

    {
	"Tenser's travels",	{  5,  7, -1, -1, -1, -1, -1, -1, -1, -1, -1,  4, -1, -1,  4,  6 },
	{ "fix soul", "rejoin soul", "soul vision", "merge soul" },
	SCLASS_TRANSPORT, FALSE,
        NULL, NULL
    },
    
    {
        "the void",             {  5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  5, -1,  5 },
        { "extinguish", "backlash", "weapon curse" },
        SCLASS_MALEDICTION, TRUE,
        "Underdark", ""
    },

    {
	"transportation",	{  4,  4,  9, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1,  5,  4,  4 },
	{ "fly", "gate", "nexus", "pass door", "portal",
	  "summon", "teleport", "word of recall" },
	SCLASS_TRANSPORT, TRUE,
        NULL, NULL
    },
    
    {
	"water mastery",	{  4,  3,  7,  6,  5,  6, -1,  5,  3,  6,  4,  5, -1,  4,  4,  4 },
	{ "water breathing", "water speech", "waters ways" },
	SCLASS_ENHANCEMENT, TRUE,
        NULL, NULL
    },

    {
	"weather",		{  4,  4,  8,  8,  6, -1, -1, -1, -1, -1, -1, -1,  6, -1,  4, -1 },
	{ "call lightning", "control weather", "faerie fire", "faerie fog",
	  "lightning bolt" },
	SCLASS_WEATHER, TRUE,
        NULL, NULL
    },

    {
	"winters ways",		{ -1, -1, -1, -1, -1, -1, -1, -1, -1,  7, -1, -1, -1, -1, -1, -1 },
	{ "chill touch", "antifreeze", "winters touch", "polar wind",
	  "frost breath" },
	SCLASS_ELEMENTAL, TRUE,
        NULL, NULL
    },

    {
	"wizards ways",		{   9, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  9, -1,  9,  9,  9 },
	{ "mana stabilization", "reflection", "spell haste", "spell steal" },
	SCLASS_ENCHANTMENT, FALSE,
        NULL, NULL
    },
    
    {
	"wood lore",		{  -1, -1, -1, -1, -1, -1, -1, -1, 8, -1, -1, -1, -1, -1, -1, -1 },
	{ "mark tree", "tree transport", "unmark tree" },
	SCLASS_TRANSPORT, TRUE,
        NULL, NULL
    }
};


/*
 * Specializations.
 */
const struct style_type style_table[MAX_STYLES] =
{
    { "single weapon combat",
      { -1, -1,  3,  2,  3, -1,  3, -1, -1, -1, -1, -1 },
      SPEC_STYLE | SPEC_NO_REMORT,
      &gsn_single_weapon_combat,
      "dodge" },
    
    { "two handed combat",
      { -1, -1, -1,  2,  3, -1,  3, -1, -1, -1, -1, -1 },
      SPEC_STYLE | SPEC_NO_REMORT,
      &gsn_two_handed_combat,
      "" },
    
    { "weapon and shield combat",
      { -1, -1, -1,  2,  3, -1,  3, -1, -1, -1, -1, -1 },
      SPEC_STYLE | SPEC_NO_REMORT,
      &gsn_weapon_and_shield_combat,
      "shield block" },
    
    { "two weapon combat",
      { -1, -1, -1,  2,  3, -1,  3, -1, -1, -1, -1, -1 },
      SPEC_STYLE | SPEC_NO_REMORT,
      &gsn_two_weapon_combat,
      "dual wield" },

    { "windmill",
      { -1, -1, -1,  1,  1, -1,  1, -1, -1, -1, -1, -1 },
      SPEC_STYLE | SPEC_NO_REMORT,
      &gsn_windmill,
      "" },

    { "viper",
      { -1, -1, -1,  1,  1, -1,  1, -1, -1, -1, -1, -1 },
      SPEC_STYLE | SPEC_NO_REMORT,
      &gsn_viper,
      "" },

    { "fan",
      { -1, -1, -1,  1,  1, -1,  1, -1, -1, -1, -1, -1 },
      SPEC_STYLE | SPEC_NO_REMORT,
      &gsn_fan,
      "" },

    { "stone claw",
      { -1, -1, -1,  1,  1, -1,  1, -1, -1, -1, -1, -1 },
      SPEC_STYLE | SPEC_NO_REMORT,
      &gsn_stone_claw,
      "pummeling" },

    { "rainbow",
      { -1, -1, -1,  1,  1, -1,  1, -1, -1, -1, -1, -1 },
      SPEC_STYLE | SPEC_NO_REMORT,
      &gsn_rainbow,
      "dual wield" },

    { "vertigo",
      { -1, -1, -1,  1,  1, -1,  1, -1, -1, -1, -1, -1 },
      SPEC_STYLE | SPEC_NO_REMORT,
      &gsn_vertigo,
      "" }
};


/*
 * The gods of the game.
 */
const struct religion_type religion_table[MAX_GODS] =
{
    {
	"The Triad","The Triad",        FALSE,  ALIGN_ANY,
	0, 0, 0, 0, 
	0,
	{ TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, 
	  TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE },
	{ { -1, -1 }, { -1, -1 } },
	"", "", "", "", "", ""
    },

    {
	"GoodReligion",   "GoodGod", 	TRUE,   ALIGN_GOOD|ALIGN_NEUTRAL,            
	12000, OBJ_VNUM_VAL_SYM, OBJ_VNUM_VAL_ORB, 0, 
	GOD_SUMMER,
	{ TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, TRUE, TRUE,
	  FALSE, FALSE, FALSE, FALSE, TRUE, FALSE, TRUE, TRUE },
	{ { 4, 6, 7, 12, 14, 15 }, { 8, 10, -1, -1, -1, -1 } },
	"The room seems to fade as you are bathed in a pool of golden light.",
	"$n seems to fade from view as $e is bathed in a pool of golden light.",
	"As the golden glow fades from your eyes you realize you are elsewhere.",
	"$n appears suddenly in a pool of golden light.",
	"$p seems to fade from view as it is bathed in a pool of golden light.",
	"$p seems to fade from view as it is bathed in a pool of golden light.",
	{ DEFAULT_SKILL_CLASS,				/* Weapons.	*/
	  { FALSE, { SSCLASS_DISHONORABLE, 0 }, { SSCLASS_DISHONORABLE, 0 } },
	                                                /* Combat.	*/
	  { FALSE, { 0, 10, }, { 0, 10 } },		/* Healing.	*/
	  DEFAULT_SKILL_CLASS,				/* Magical.	*/
	  { FALSE, { 0, -10 }, { 0, -10 } },		/* Subterfuge.	*/
	  DEFAULT_SKILL_CLASS,				/* Social.	*/
	  DEFAULT_SKILL_CLASS,				/* Outdoors.	*/
	  DEFAULT_SKILL_CLASS,				/* Misc.	*/
	  { FALSE, { SSCLASS_UNHOLY, 0 }, { SSCLASS_UNHOLY, -25 } },	
	                                                /* Channeling.	*/
	  DEFAULT_SPELL_CLASS,			        /* Beguiling.	*/
	  DEFAULT_SPELL_CLASS,			        /* Benedictions	*/
	  DEFAULT_SPELL_CLASS,			        /* Elemental.	*/
	  { FALSE, { 0, 5 }, { 0, -10 } },	        /* Creation.	*/
	  { FALSE, { 0, 10 }, { 0, 5 } },	        /* Healing.	*/
	  DEFAULT_SPELL_CLASS,		        	/* Detection.	*/
          { FALSE, { 0, 10 }, { 0, -25 } },             /* Enchantment. */
	  DEFAULT_SPELL_CLASS,		        	/* Enhancement.	*/
	  DEFAULT_SPELL_CLASS,		        	/* Illusion.	*/
	  { FALSE, { SSCLASS_UNHOLY, -10 }, { SSCLASS_UNHOLY, -50 } },
							/* Malediction.	*/	  
	  { FALSE, { 0, 5 }, { 0, -10 } },		/* Protective.	*/
	  DEFAULT_SPELL_CLASS,			        /* Transport.	*/
	  DEFAULT_SPELL_CLASS,			        /* Weather.	*/
	  { TRUE },					/* Demonic.	*/
	  DEFAULT_SPELL_CLASS,				/* Animal.	*/
	  DEFAULT_SPELL_CLASS,				/* Earth.	*/
          DEFAULT_SPELL_CLASS,                          /* Spirit.      */
	  { TRUE }					/* Necromancy.	*/
	}
    },

    {
	"EvilReligion",   "EvilGod",        TRUE,   ALIGN_EVIL|ALIGN_NEUTRAL,
	12000, OBJ_VNUM_ARI_SYM, OBJ_VNUM_ARI_ORB, 0, 
	GOD_WINTER,
	{ TRUE, TRUE, TRUE, TRUE, FALSE, TRUE, FALSE, TRUE,
	  FALSE, FALSE, FALSE, FALSE, TRUE, TRUE, FALSE, TRUE },
	{ { 5, 7, 12, 13, 14, -1 }, { 9, 11, -1, -1, -1, -1, -1 } },
	"The room fades from view as a swirling maelstrom of ice surrounds you.",
	"$n disappears in a swirling maelstrom of ice.",
	"As the storm eases you look around, realising you have shifted.",
	"A swirling mass of ice springs from the ground, then scatters as $n steps from its centre.",
	"$p turns a cold blue, then shatters into thousands of tiny shards of coldness.",
	"$p turns a cold blue, then shatters into thousands of tiny shards of coldness.",
	{ DEFAULT_SKILL_CLASS,				/* Weapons.	*/
	  DEFAULT_SKILL_CLASS,				/* Combat.	*/
	  DEFAULT_SKILL_CLASS,				/* Healing.	*/
	  DEFAULT_SKILL_CLASS,				/* Magical.	*/
	  DEFAULT_SKILL_CLASS,				/* Subterfuge.	*/
	  DEFAULT_SKILL_CLASS,				/* Social.	*/
	  DEFAULT_SKILL_CLASS,				/* Outdoors.	*/
	  DEFAULT_SKILL_CLASS,				/* Misc.	*/
	  { FALSE, { SSCLASS_HOLY, 0 }, { SSCLASS_HOLY, -25 } },	
	                                                /* Channeling.	*/
	  DEFAULT_SPELL_CLASS,			        /* Beguiling.	*/
	  DEFAULT_SPELL_CLASS,			        /* Benedictions	*/
	  { FALSE, { SSCLASS_FIRE, 5 }, { SSCLASS_FIRE, -15 } },	
	                                                /* Elemental.	*/
	  { TRUE },					/* Creation.	*/
	  { TRUE },					/* Healing.	*/
	  DEFAULT_SPELL_CLASS,			        /* Detection.	*/
          DEFAULT_SPELL_CLASS,                          /* Enchantment. */
	  DEFAULT_SPELL_CLASS,			        /* Enhancement.	*/
	  DEFAULT_SPELL_CLASS,			        /* Illusion.	*/
	  { FALSE, { 0, 7 }, { 0, -3 } },		/* Malediction.	*/
	  DEFAULT_SPELL_CLASS,			        /* Protective.	*/
	  DEFAULT_SPELL_CLASS,			        /* Transport.	*/
	  { FALSE, { 0, 15 }, { 0, 0 } },		/* Weather.	*/
	  { FALSE, { 0, 10 }, { 0, 0 } },		/* Demonic.	*/
	  { FALSE, { 0, -35 }, { 0, -60 } },		/* Animal.	*/
	  DEFAULT_SPELL_CLASS,				/* Earth.	*/
          DEFAULT_SPELL_CLASS,                          /* Spirit.      */
	  DEFAULT_SPELL_CLASS			        /* Necromancy.	*/
	}
    }

};





