/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "merc.h"
#include "color.h"
#include "tables.h"
#include "recycle.h"
#include "magic.h"


#define ATT_NORMAL	0
#define ATT_SECONDARY	1
#define ATT_RIPOSTE	2


/* command procedures needed */
DECLARE_DO_FUN(do_backstab	);
DECLARE_DO_FUN(do_emote		);
DECLARE_DO_FUN(do_berserk	);
DECLARE_DO_FUN(do_bash		);
DECLARE_DO_FUN(do_trip		);
DECLARE_DO_FUN(do_dirt		);
DECLARE_DO_FUN(do_flee		);
DECLARE_DO_FUN(do_kick		);
DECLARE_DO_FUN(do_disarm	);
DECLARE_DO_FUN(do_get		);
DECLARE_DO_FUN(do_recall	);
DECLARE_DO_FUN(do_yell		);
DECLARE_DO_FUN(do_sacrifice	);
DECLARE_DO_FUN(do_unmount	);
DECLARE_DO_FUN(do_circle	);
DECLARE_DO_FUN(do_surrender	);

DECLARE_SPELL_FUN(spell_slow    );


/*
 * Local functions.
 */
void	check_assist	args( ( CHAR_DATA *ch, CHAR_DATA *victim ) );
void	check_killer	args( ( CHAR_DATA *ch, CHAR_DATA *victim ) );
bool	check_dodge	args( ( CHAR_DATA *ch, CHAR_DATA *victim ) );
bool	check_parry	args( ( CHAR_DATA *ch, OBJ_DATA *wield, 
				CHAR_DATA *victim ) );
bool	check_fade	args( ( CHAR_DATA *ch, OBJ_DATA *wield,
				CHAR_DATA *victim ) );
bool	check_shield_block
			args( ( CHAR_DATA *ch, OBJ_DATA *wield, 
				CHAR_DATA *victim ) );
bool	check_mirror_image
			args( ( CHAR_DATA *ch, OBJ_DATA *wield, CHAR_DATA *victim ) );
void    dam_message 	args( ( CHAR_DATA *ch, CHAR_DATA *victim, int dam,
				int dt, bool immune ) );
void    dam_message_new args( ( CHAR_DATA *ch, CHAR_DATA *victim, int dam,
				int dt, bool immune ) );
void	death_cry	args( ( CHAR_DATA *ch, bool make_parts ) );
void	group_gain	args( ( CHAR_DATA *ch, CHAR_DATA *victim ) );
int	xp_compute	args( ( CHAR_DATA *gch, CHAR_DATA *victim, 
				int total_levels, int max_level,
                                int members ) );
bool	is_safe		args( ( CHAR_DATA *ch, CHAR_DATA *victim ) );
void	make_corpse	args( ( CHAR_DATA *ch, int damage_type ) );
bool	one_hit		args( ( CHAR_DATA *ch, CHAR_DATA *victim, int dt,
				int attack_type ) );
void    mob_hit		args( ( CHAR_DATA *ch, CHAR_DATA *victim, int dt ) );
void	raw_kill	args( ( CHAR_DATA *victim, bool make_parts, int damage_type ) );
void	disarm		args( ( CHAR_DATA *ch, CHAR_DATA *victim ) );
char *	attack_noun	args( ( int dam_type, CHAR_DATA *ch, 
				CHAR_DATA *victim ) );
void	crush		args( ( CHAR_DATA *ch, int hits ) );


#define OTHER_COLOR(ch) args( IS_NPC(ch) ? C_COMBAT_MOBS : C_COMBAT_PLAYER )


/*
 * Control the fights going on.
 * Called periodically by update_handler.
 */
void violence_update( void )
{
    CHAR_DATA *ch;
    CHAR_DATA *ch_next;
    CHAR_DATA *victim;

    for ( ch = char_list; ch != NULL; ch = ch_next )
    {
	ch_next	= ch->next;

	if ( IS_AFFECTED(ch, AFF_PARALYSIS) )
	{
	    if ( ch->fighting )
		stop_fighting( ch, FALSE );

	    continue;
	}
	
	if ( !IS_NPC(ch)
	&&   !IS_AFFECTED2(ch, AFF2_WATER_BREATHING)
	&&   ch->level <= LEVEL_HERO
	&&   !is_creator(ch, ch->in_room->area)
	&&   UNDERWATER(ch) )
	{
	    damage( ch, ch, number_range(1,10), 0, DAM_DROWNING, FALSE, NULL );
	    if ( CHAR_KILLED(ch) )
	    	continue;
	    
	    check_contingency( ch, CONTIN_DROWNING );
	    
	    if ( ch->fighting == NULL )
	    {
		send_to_char( "\n\r", ch );
		continue;
	    }
	}
	
	if ( !IS_NPC(ch)
	&&   IS_SET(ch->act, PLR_SINGING)
	&&   !IS_AFFECTED2(ch, AFF2_WATER_SPEECH)
	&&   !IS_AFFECTED2(ch, AFF2_WATERS_WAYS)
	&&   ch->level <= LEVEL_HERO
	&&   UNDERWATER(ch) )
	    remove_bard_song( ch, FALSE );

	if ( ( victim = ch->fighting ) == NULL || ch->in_room == NULL )
	    continue;

        if ( IS_AFFECTED2(ch, AFF2_CONTINGENCY) )
            check_contingency( ch, CONTIN_ATTACKED );
        
	if ( IS_AWAKE(ch) && ch->in_room == victim->in_room )
	{
	    multi_hit( ch, victim, TYPE_UNDEFINED );
	    remove_bard_song( ch, FALSE );
	}
	else
	    stop_fighting( ch, FALSE );

	if ( ( victim = ch->fighting ) == NULL )
	    continue;

	/*
	 * Fun for the whole family!
	 */
	check_assist(ch,victim);

	if ( IS_NPC( ch ) )
	{
	    if ( HAS_TRIGGER( ch, TRIG_FIGHT ) )
		mp_percent_trigger( ch, victim, NULL, NULL, TRIG_FIGHT );
	    if ( HAS_TRIGGER( ch, TRIG_HPCNT ) )
		mp_hprct_trigger( ch, victim );
	}
    }

    return;
}

/* for auto assisting */
void check_assist(CHAR_DATA *ch,CHAR_DATA *victim)
{
    CHAR_DATA *rch, *rch_next;

    for (rch = ch->in_room->people; rch != NULL; rch = rch_next)
    {
	rch_next = rch->next_in_room;
	
	if ( IS_AFFECTED(rch, AFF_PARALYSIS) )
	    continue;
	
	if (IS_AWAKE(rch) && rch->fighting == NULL)
	{
	    int level_mod = 0;

	    if ( !IS_NPC(victim)
	    &&   victim->pcdata->renown >= 60 )
		level_mod = 3;
	    else if ( !IS_NPC(victim)
	    &&   victim->pcdata->renown <= 20 )
		level_mod = 7;
	    else
		level_mod = 5;

	    if ( IS_AFFECTED2(victim, AFF2_ALKAR)
	    &&   !IS_AFFECTED2(rch, AFF2_DETECT_ILLUSION) )
	    {
		AFFECT_DATA *paf = affect_find( victim->affected, gsn_alkar );

		if ( paf == NULL )
		    bug( "check_assist: NULL paf.", 0 );
		else
		    level_mod += paf->modifier;
	    }

	    /* quick check for ASSIST_PLAYER */
	    if (!IS_NPC(ch) && IS_NPC(rch) 
	    && IS_SET(rch->off_flags,ASSIST_PLAYERS)
	    &&  rch->level > victim->level - level_mod)
	    {
		do_emote(rch,"screams and attacks!");
		multi_hit(rch,victim,TYPE_UNDEFINED);
		continue;
	    }

	    /* PCs next */
	    if (!IS_NPC(ch) || IS_AFFECTED(ch,AFF_CHARM))
	    {
		if ( ( (!IS_NPC(rch) && IS_SET(rch->act,PLR_AUTOASSIST))
		||     IS_AFFECTED(rch,AFF_CHARM)) 
		&&   is_same_group(ch,rch) 
		&&   !is_safe(rch, victim))
		    multi_hit (rch,victim,TYPE_UNDEFINED);
		
		continue;
	    }
  	
	    /* now check the NPC cases */
	    
 	    if (IS_NPC(ch) && !IS_AFFECTED(ch,AFF_CHARM))
	
	    {
		if ( (IS_NPC(rch) && IS_SET(rch->off_flags,ASSIST_ALL))
		||   (IS_NPC(rch) && rch->group && rch->group == ch->group)
		||   (IS_NPC(rch) && CHAR_RACE(rch) == CHAR_RACE(ch)
		&&    IS_SET(rch->off_flags,ASSIST_RACE))
		||   (IS_NPC(rch) && IS_SET(rch->off_flags,ASSIST_ALIGN)
		&&    ((IS_GOOD(rch)    && IS_GOOD(ch))
		||     (IS_EVIL(rch)    && IS_EVIL(ch))
		||     (IS_NEUTRAL(rch) && IS_NEUTRAL(ch)))) 
		||   (rch->pIndexData == ch->pIndexData 
		&&    IS_SET(rch->off_flags,ASSIST_VNUM)))

	   	{
		    CHAR_DATA *vch;
		    CHAR_DATA *target;
		    int number;

		    if (number_bits(1) == 0)
			continue;
		
		    target = NULL;
		    number = 0;
		    for (vch = ch->in_room->people; vch; vch = vch->next)
		    {
			if (can_see(rch,vch)
			&&  is_same_group(vch,victim)
			&&  number_range(0,number) == 0)
			{
			    target = vch;
			    number++;
			}
		    }

		    if (target != NULL)
		    {
			do_emote(rch,"screams and attacks!");
			multi_hit(rch,target,TYPE_UNDEFINED);
		    }
		}
	    }
	}
    }
}


/*
 * Do one group of attacks.
 */
void multi_hit( CHAR_DATA *ch, CHAR_DATA *victim, int dt )
{
    OBJ_DATA *primary = NULL, *secondary = NULL;
    int chance;
    bool armed;

    /* decrement the wait */
    if (ch->desc == NULL)
	ch->wait = UMAX(0,ch->wait - PULSE_VIOLENCE);

    if (ch->desc == NULL)
	ch->daze = UMAX(0,ch->daze - PULSE_VIOLENCE); 
	
    if ( IS_AFFECTED(ch, AFF_PARALYSIS) )
    {
	stop_fighting( ch, FALSE );
	return;
    }

    /* no attacks for stunnies -- just a check */
    if (ch->position < POS_RESTING)
	return;

    if ( IS_NPC(ch) )
    {
	mob_hit( ch, victim, dt );
	return;
    }

    if ( dt == gsn_joust )
    {
	one_hit( ch, victim, dt, ATT_NORMAL );
	return;
    }
    
    if ( dt == gsn_backstab )
    {
	one_hit( ch, victim, dt, FALSE );
	if ( IS_AFFECTED(ch, AFF_HASTE) )
	    one_hit( ch, victim, dt, ATT_NORMAL );
	
	return;
    }

    primary	= get_eq_char( ch, WEAR_WIELD );
    secondary	= get_eq_char( ch, WEAR_SECONDARY );
    armed	= primary != NULL || secondary != NULL;

    if ( armed && primary != NULL )
    {
	int attack_rate, second, second_chance;
	bool gets_second;

	switch ( primary->value[0] )
	{
	case WEAPON_SWORD:
	case WEAPON_SPEAR:
	case WEAPON_MACE:
	    attack_rate = 100;
	    break;
	case WEAPON_BOW:
	    attack_rate = 0;
	    break;
	case WEAPON_DAGGER:
	    attack_rate = 110;
	    break;
	case WEAPON_AXE:
	    attack_rate = 90;
	    break;
	case WEAPON_FLAIL:
	case WEAPON_WHIP:
	    attack_rate = 85;
	    break;
	case WEAPON_POLEARM:
	    attack_rate = 80;
	    break;
	default:
	    attack_rate = 75;
	    break;
	}

             if ( COMPLEX_STYLE(ch, STYLE_WINDMILL) )
            attack_rate -= 30;
        else if ( COMPLEX_STYLE(ch, STYLE_VIPER) )
            attack_rate += 20;
        else if ( COMPLEX_STYLE(ch, STYLE_FAN) )
            attack_rate += 10;
        else if ( COMPLEX_STYLE(ch, STYLE_STONE_CLAW) )
            attack_rate -= 20;
        else if ( COMPLEX_STYLE(ch, STYLE_RAINBOW) )
            attack_rate -= 20;
        else if ( COMPLEX_STYLE(ch, STYLE_VERTIGO) )
            attack_rate += 50;

	if ( IS_AFFECTED(ch, AFF_HASTE) )
	    attack_rate *= 2;
        else if ( IS_AFFECTED(ch, AFF_SLOW) )
            attack_rate /= 2;

	second = get_skill( ch, gsn_arms_mastery );

	second_chance = second / 2;
	if ( IS_AFFECTED(ch, AFF_SLOW) )
	    second_chance /= 2;
	if ( second )
	    second_chance = UMAX( 1, second_chance );

	if ( number_percent() < second_chance )
	{
	    attack_rate += second;
	    gets_second  = TRUE;
	}
	else
	    gets_second  = FALSE;
	
	if ( IS_SET(ch->pcdata->style, STYLE_TWO_HANDED)
	&&   secondary == NULL
	&&   (primary->value[0] == WEAPON_SWORD
	||    primary->value[0] == WEAPON_MACE
	||    primary->value[0] == WEAPON_AXE
	||    primary->value[0] == WEAPON_FLAIL)
	&&   get_eq_char(ch, WEAR_SHIELD) == NULL
	&&   get_eq_char(ch, WEAR_HOLD) == NULL )
	    attack_rate += 30;
	    
	while ( attack_rate > 0 )
	{
	    if ( attack_rate > 100
	    ||   number_percent() <= attack_rate )
		one_hit( ch, victim, dt, ATT_NORMAL );
		
	    attack_rate -= 100;
	    
	    if ( ch->fighting != victim )
		break;
	}

	if ( gets_second )
	    check_improve( ch, gsn_arms_mastery, TRUE, 3, TRUE );
	
	if ( ch->fighting != victim )
	    return;
    }
    else if ( !armed )
    {
	one_hit( ch, victim, dt, ATT_NORMAL );
	
	if ( ch->fighting != victim )
	    return;

        if ( IS_AFFECTED(ch, AFF_HASTE) )
        {
            one_hit( ch, victim, dt, ATT_NORMAL );
            if ( ch->fighting != victim )
                return;
        }
	    
	if ( martial_artist(ch) )
	{
	    int hand_to_hand, second_chance;
	    
	    hand_to_hand = get_skill( ch, gsn_hand_to_hand );
	    
	    second_chance = hand_to_hand / 2;
	    if ( IS_AFFECTED(ch, AFF_SLOW) )
		second_chance /= 2;
	    
	    if ( ch->level >= 6
	    &&   number_percent() <= second_chance )
	    {
		int third_chance;
		
		one_hit( ch, victim, dt, ATT_NORMAL );
		
		if ( ch->fighting != victim
		||   ch->level < 12 )
		    return;
		
		third_chance = second_chance / 2;
		if ( IS_AFFECTED(ch, AFF_SLOW) )
		    third_chance = 0;
		
		if ( number_percent() <= third_chance )
		    one_hit( ch, victim, dt, ATT_NORMAL );
	    }
	}
	
	return;
    }

    chance = get_skill( ch, gsn_weapon_brawling ) / 4;

    if ( IS_AFFECTED(ch, AFF_SLOW) )
	chance = 0;

    if ( number_percent() < chance )
    {
	one_hit( ch, victim, gsn_weapon_brawling, ATT_NORMAL );
	check_improve( ch, gsn_weapon_brawling, TRUE, 6, TRUE );
	if ( ch->fighting != victim )
	    return;
    }

    if ( secondary != NULL
    &&   IS_SET(ch->pcdata->style, STYLE_DUAL_OFFENSIVE) )
    {
	bool hit = one_hit( ch, victim, dt, ATT_SECONDARY );
	check_improve( ch, gsn_dual_wield, hit, 6, TRUE );
	if ( ch->fighting != victim )
	    return;
    }

    return;
}

/* procedure for all mobile attacks */
void mob_hit( CHAR_DATA *ch, CHAR_DATA *victim, int dt )
{
    CHAR_DATA *vch, *vch_next;
    int chance, number, hits;

    if ( IS_AFFECTED(victim, AFF_PARALYSIS) )
    {
	stop_fighting( ch, FALSE );
	return;
    }

    hits = 0;

    /* Mounts can only make special attacks. */
    if ( ch->mounted_by == NULL )
    {
	bool armed;
	
	if ( one_hit(ch, victim, dt, ATT_NORMAL) )
	    hits++;

	if ( ch->fighting != victim || dt == gsn_joust )
	    return;

	/* Area attack -- BALLS nasty! */
	if (IS_SET(ch->off_flags, OFF_AREA_ATTACK))
	{
	    for ( vch = ch->in_room->people; vch != NULL; vch = vch_next )
	    {
		vch_next = vch->next;
		if ( vch != victim && vch->fighting == ch )
		    one_hit( ch, vch, dt, ATT_NORMAL );
	    }
	}

	if ( IS_AFFECTED(ch,AFF_HASTE) 
	||   (IS_SET(ch->off_flags, OFF_FAST) && !IS_AFFECTED(ch, AFF_SLOW)) )
	    if ( one_hit(ch, victim, dt, ATT_NORMAL) )
		hits++;

	if (ch->fighting != victim || dt == gsn_backstab)
	    return;

	chance = get_skill( ch, gsn_arms_mastery ) / 2;
 
	if ( IS_AFFECTED(ch, AFF_SLOW) && !IS_SET(ch->off_flags, OFF_FAST) )
	    chance /= 2;

	if ( number_percent() < chance )
	{
	    if ( one_hit(ch, victim, dt, ATT_NORMAL) )
		hits++;
	    if ( ch->fighting != victim )
		return;
	}

	armed = get_eq_char( ch, WEAR_WIELD ) != NULL || get_eq_char( ch, WEAR_SECONDARY ) != NULL;

	chance = get_skill( ch, gsn_weapon_brawling ) / 4;

	if ( IS_AFFECTED(ch, AFF_SLOW) && !IS_SET(ch->off_flags, OFF_FAST) )
	    chance = 0;

	if (number_percent() < chance)
	{
	    if ( one_hit(ch, victim, gsn_weapon_brawling, ATT_NORMAL) )
		hits++;
		
	    if (ch->fighting != victim)
		return;
	}
    }

    /* oh boy!  Fun stuff! */

    if (ch->wait > 0)
	return;

    number = number_range(0,2);

    if (number == 1 && IS_SET(ch->act,ACT_MAGE))
    {
	/*  { mob_cast_mage(ch,victim); return; } */ ;
    }

    if (number == 2 && IS_SET(ch->act,ACT_CLERIC))
    {	
	/* { mob_cast_cleric(ch,victim); return; } */ ;
    }

    /* now for the skills */

    number = number_range(0,8);

    switch(number) 
    {
    case (0) :
	if (IS_SET(ch->off_flags,OFF_BASH))
	    do_bash(ch,"");
	break;

    case (1) :
	if (IS_SET(ch->off_flags,OFF_BERSERK) && !IS_AFFECTED(ch,AFF_BERSERK))
	    do_berserk(ch,"");
	break;

    case (2) :
	if (IS_SET(ch->off_flags,OFF_DISARM) 
	|| (get_weapon_sn(ch) != gsn_hand_to_hand 
	&& (IS_SET(ch->act,ACT_WARRIOR)
   	||  IS_SET(ch->act,ACT_THIEF))))
	    do_disarm(ch,"");
	break;

    case (3) :
	if (IS_SET(ch->off_flags,OFF_KICK))
	    do_kick(ch,"");
	break;

    case (4) :
	if (IS_SET(ch->off_flags,OFF_KICK_DIRT))
	    do_dirt(ch,"");
	break;

    case (5) :
	if (IS_SET(ch->off_flags,OFF_TAIL))
	{
	    /* do_tail(ch,"") */ ;
	}
	break; 

    case (6) :
	if (IS_SET(ch->off_flags,OFF_TRIP))
	    do_trip(ch,"");
	break;

    case (7) :
	if ( IS_SET(ch->off_flags,OFF_CRUSH) && hits )
	{
	    crush( ch, hits );
	}
	break;
    case (8) :
	if (IS_SET(ch->off_flags,OFF_UNMOUNT))
	{
	    do_unmount(ch,"");
	}
	break;
    }

    tail_chain();
    return;
}
	

/*
 * Hit one guy once.
 */
bool one_hit( CHAR_DATA *ch, CHAR_DATA *victim, int dt, int att_type )
{
    char buf[MAX_STRING_LENGTH];
    int pummeling_skill;
    OBJ_DATA *wield;
    int victim_ac;
    bool enhanced;
    int diceroll;
    int sn,skill;
    int dam_type;
    int thac0_00;
    int thac0_32;
    bool damages;
    int thac0;
    int dam;
    
    sn = -1;

    /* just in case */ 
    if ( victim == ch || ch == NULL || victim == NULL )
	return FALSE;

    /*
     * Can't beat a dead char!
     * Guard against weird room-leavings.
     */
    if ( victim->position == POS_DEAD || ch->in_room != victim->in_room )
	return FALSE;

    /*
     * Figure out the type of damage message.
     */
    if ( att_type == ATT_NORMAL )
	wield = get_eq_char( ch, WEAR_WIELD );
    else
	wield = get_eq_char( ch, WEAR_SECONDARY );

    if ( wield != NULL
    &&   !IS_OBJ_STAT(wield, ITEM_NOREMOVE)
    &&   !IS_NULLSTR(wield->owner)
    &&   str_cmp(wield->owner, ch->name)
    &&   number_percent() < 5 )
    {
	act_color( "You fumble and drop $p!",  ch, wield, NULL, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	act_color( "$n fumbles and drops $p!", ch, wield, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );

	unequip_char( ch, wield );
	if ( !IS_OBJ_STAT(wield, ITEM_NODROP) )
	{
	    obj_from_char( wield );
	    obj_to_room( wield, ch->in_room );
	}
    }

    if ( dt == TYPE_UNDEFINED )
    {
	dt = TYPE_HIT;
	if ( wield != NULL && wield->item_type == ITEM_WEAPON )
	    dt += wield->value[3];
	else 
	    dt += CHAR_DAM_TYPE(ch);
    }

    if (dt < TYPE_HIT)
	if ( wield != NULL )
	    if ( dt == gsn_weapon_brawling )
		dam_type = DAM_BASH;
	    else
		dam_type = attack_table[wield->value[3]].damage;
	else
	    dam_type = attack_table[CHAR_DAM_TYPE(ch)].damage;
    else
	dam_type = attack_table[dt - TYPE_HIT].damage;

    if ( dam_type == -1 )
	dam_type = DAM_BASH;

    /* get the weapon skill */
    switch ( att_type )
    {
    case ATT_SECONDARY:
	sn		= get_secondary_sn( ch );
	skill		= get_weapon_skill( ch, sn );
	if ( has_specialization(ch, gsn_two_weapon_combat) )
	    skill	= 85 * skill * get_skill( ch, gsn_dual_wield ) / 10000;
	else
	    skill	= 75 * skill * get_skill( ch, gsn_dual_wield ) / 40000;
	break;
    
    case ATT_RIPOSTE:
	sn		= get_secondary_sn( ch );
	skill		= 20 + get_weapon_skill( ch, sn );
	break;
    
    default:
	sn		= get_weapon_sn( ch );
	skill		= 20 + get_weapon_skill( ch, sn );
	break;
    }
    
    if ( ch->riding != NULL )
	skill		= skill * get_skill( ch, gsn_ride ) / 100;

    /*
     * Calculate to-hit-armor-class-0 versus armor.
     */
    if ( IS_NPC(ch) )
    {
	thac0_00 = 20;
	thac0_32 = -4;   /* as good as a thief */ 
	if (IS_SET(ch->act,ACT_WARRIOR))
	    thac0_32 = -10;
	else if (IS_SET(ch->act,ACT_THIEF))
	    thac0_32 = -4;
	else if (IS_SET(ch->act,ACT_CLERIC))
	    thac0_32 = 2;
	else if (IS_SET(ch->act,ACT_MAGE))
	    thac0_32 = 6;
    }
    else
    {
	thac0_00 = class_table[ch->class].thac0_00;
	thac0_32 = class_table[ch->class].thac0_32;
    }
    thac0  = interpolate( effective_level(ch, sn), thac0_00, thac0_32 );

    if (thac0 < 0)
        thac0 = thac0/2;

    if (thac0 < -5)
        thac0 = -5 + (thac0 + 5) / 2;

    if ( !IS_NPC(ch) )
	thac0 -= GET_HITROLL(ch) * skill/100;
    else
	thac0 -= GET_HITROLL_MOB(ch) * skill/100;

    thac0 += 5 * (100 - skill) / 100;

    if (dt == gsn_backstab)
	thac0 -= 10 * (100 - get_skill(ch,gsn_backstab));

    if (dt == gsn_circle)
	thac0 -= 9 * (100 - get_skill(ch,gsn_circle));	// circle is harder than backstab?

    if ( wield != NULL )
    {
        if ( COMPLEX_STYLE(ch, STYLE_VERTIGO) )
            thac0 += 10;
        else if ( COMPLEX_STYLE(ch, STYLE_RAINBOW)
        &&   (att_type == ATT_SECONDARY
        ||    att_type == ATT_RIPOSTE) )
            thac0 -= 5;
        else if ( COMPLEX_STYLE(ch, STYLE_VIPER) )
            thac0 -= 5;
    }

    switch(dam_type)
    {
        case(DAM_PIERCE):victim_ac = get_ac(victim, AC_PIERCE);  break;
        case(DAM_BASH):  victim_ac = get_ac(victim, AC_BASH);  break;
        case(DAM_SLASH): victim_ac = get_ac(victim, AC_SLASH);  break;
        default:         victim_ac = get_ac(victim, AC_EXOTIC);  break;
    };

//    sprintf(buf,"%s's base AC is %d.", victim->name, victim_ac );
//    wiznet(buf,NULL,NULL,WIZ_SACCING,0,0);
    

    if (!IS_NPC(victim))
    {
        victim_ac /= 6;  //if victim is a player, lower ac by factor 5
    }
    else 
    {
	victim_ac /= 8; //if victim is a MOB, lower ac by factor 8
    }

    if (victim_ac < -15)
	victim_ac = (victim_ac + 15) / 5 - 15;
     
    if ( !can_see(ch, victim) )
    {
	int chance, roll;
	
	chance = get_skill( ch, gsn_blind_fighting );
	
	if ( (roll = number_percent()) < chance )
	{
	    victim_ac -= 4 - 3 * roll * chance / 10000;
	    
	    check_improve( victim, gsn_blind_fighting, TRUE, 10, TRUE );
	}
	else
	    victim_ac -= 4;
    }
	
    if ( victim->riding != NULL )
	victim_ac -= 2;

    if ( victim->position < POS_FIGHTING)
	victim_ac += 4;
 
    if (victim->position < POS_RESTING)
	 victim_ac += 6;

    /*
     * The moment of excitement!
     */
    while ( ( diceroll = number_bits( 5 ) ) >= 20 )
	;

    sprintf(buf,"%s's thac0 is %d.", ch->name, thac0 );
    wiznet(buf,NULL,NULL,WIZ_SACCING,0,0);
    sprintf(buf,"%s's AC is %d.", victim->name, victim_ac );
    wiznet(buf,NULL,NULL,WIZ_SACCING,0,0);
    sprintf(buf,"%s needs %d and over to hit.", ch->name, (thac0 - victim_ac) );
    wiznet(buf,NULL,NULL,WIZ_SACCING,0,0);
    sprintf(buf,"%s rolls %d.\n\r", ch->name, diceroll );
    wiznet(buf,NULL,NULL,WIZ_SACCING,0,0);
 

    if ( ch->riding != NULL )
	check_improve( ch, gsn_ride, TRUE, 6, TRUE );

    if ( diceroll == 0
    || ( diceroll != 19 && diceroll < thac0 - victim_ac ) )
    {
	/* Miss. */
	damage( ch, victim, 0, dt, dam_type, TRUE, wield );
	tail_chain( );
	return FALSE;
    }

    /*
     * Hit.
     * Calc damage.
     */
    if ( IS_NPC(ch) && (!ch->pIndexData->new_format || wield == NULL))
	if (!ch->pIndexData->new_format)
	{
	    dam = number_range( ch->level / 2, ch->level * 3 / 2 );
	    if ( wield != NULL )
		dam += dam / 2;
	}
	else
	    dam = dice( ch->damage[DICE_NUMBER], ch->damage[DICE_TYPE] );
	
    else
    {
	if (sn != -1)
	    check_improve( ch, sn, TRUE, 5, TRUE );
	if ( wield != NULL )
	{
	    if (wield->pIndexData->new_format)
		dam = dice( wield->value[1], wield->value[2] ) * skill/100;
	    else
	    	dam = number_range( wield->value[1] * skill/100, 
				    wield->value[2] * skill/100 );
				    
	    /* no shield = more */
/*
 *	    if ( get_eq_char(ch, WEAR_SHIELD) == NULL )  
 *		dam = dam * 11/10;
 */
 	    if ( !IS_NPC(ch)
 	    &&   IS_SET(ch->pcdata->style, STYLE_TWO_HANDED)
 	    &&   !IS_OBJ_STAT(wield, WEAPON_TWO_HANDS)
 	    &&   has_specialization(ch, gsn_two_handed_combat)
	    &&   get_eq_char(ch, WEAR_HOLD) == NULL
 	    &&   get_eq_char(ch, WEAR_SECONDARY) == NULL
 	    &&   get_eq_char(ch, WEAR_SHIELD) == NULL )
 		dam = dam * 115 / 100;
 
	    /* sharpness! */
	    if (IS_WEAPON_STAT(wield,WEAPON_SHARP))
	    {
		int percent;

		if ((percent = number_percent()) <= (skill / 8))
		    dam = 2 * dam + (dam * 2 * percent / 100);
	    }
	    /* windmill style! */
	    if (COMPLEX_STYLE(ch, STYLE_WINDMILL))
	    {
		int percent;

		if ((percent = number_percent()) <= (skill / 2))
		    dam += 2 * dam * skill / 100;
	    }

            if ( COMPLEX_STYLE(ch, STYLE_VIPER) )
                dam -= 2;
	}
	else
        {
            if ( martial_artist(ch) )
	        dam = number_range( 1 + 4 * skill/100,
				    effective_level(ch, sn) * skill/100 );
	    else
	        dam = number_range( 1 + 4 * skill/100, 
				    2 * effective_level(ch, sn)/3 * skill/100 );
            if ( has_customization(ch, ADVANTAGE_HANDS_OF_STONE) )
                dam += number_range( 1, 5 );
        }

        if ( has_customization(ch, ADVANTAGE_LARGE) )
            dam += number_range( 1, 5 );
        if ( has_customization(ch, ANCESTOR_FIONAVAR) )
            dam += number_range( 1, 5 );
        if ( has_customization(ch, DISADVANTAGE_SMALL) )
            dam = 9 * dam / 10;
    }

    /* Modify for weapon type. */
    if ( wield != NULL )
    {
	switch ( wield->value[0] )
	{
	case WEAPON_BOW:	dam = dam *	0;	break;
	case WEAPON_DAGGER:	dam = dam *  7/10;	break;
	case WEAPON_SPEAR:	dam = dam *  9/10;	break;
	case WEAPON_POLEARM:	dam = dam * 12/10;	break;
	case WEAPON_WHIP:	dam = dam *  6/10;	break;
	case WEAPON_MACE:	dam = dam *  9/10;	break;
	case WEAPON_FLAIL:	dam = dam *  9/10;	break;
	default:				break;
	}
    }

    /* Non-piercing weapons underwater do less damage. */
    if ( dam > 1 && wield != NULL
    &&   UNDERWATER(ch)
    &&   !IS_AFFECTED2(ch, AFF2_WATERS_WAYS) )
    {
	if ( wield->value[0] == WEAPON_WHIP )
	    dam  = 0;
	else if ( wield->value[0] == WEAPON_FLAIL )
	    dam /= 4;
	else if ( dam_type != DAM_PIERCE )
	    dam /= 2;
    }

    /*
     * Bonuses.
     */
    if ( wield != NULL
    &&   get_skill(ch, gsn_enhanced_damage) > 0
    &&   dt != gsn_weapon_brawling
    &&   (dam_type == DAM_PIERCE
    ||    dam_type == DAM_SLASH
    ||    (dam_type != DAM_BASH
    &&     (wield->value[0] == WEAPON_DAGGER
    ||      wield->value[0] == WEAPON_SWORD
    ||      wield->value[0] == WEAPON_AXE
    ||      wield->value[0] == WEAPON_POLEARM
    ||      wield->value[0] == WEAPON_SPEAR))) )
    {
        diceroll = number_percent();
        if ( diceroll < get_skill(ch, gsn_enhanced_damage) )
        {
            dam += 2 * ( dam * diceroll/300 );
            enhanced = TRUE;
        }
        else
            enhanced = FALSE;
    }
    else
	enhanced = FALSE;

    if ( !IS_AWAKE(victim) )
	dam *= 2;
     else if (victim->position < POS_FIGHTING)
	dam = dam * 3 / 2;

    if ( dt == gsn_backstab && wield != NULL ) 
    {
        int multiplier;
	if ( wield->value[0] != WEAPON_DAGGER )
	    multiplier = 2 + (effective_level(ch, gsn_backstab) / 10); 
	else 
	    multiplier = 2 + (effective_level(ch, gsn_backstab) / 8);
        if ( has_customization(ch, ANCESTOR_RYNA) )
            multiplier += 1;

	dam *= multiplier;
    }

    if ( dt == gsn_circle && wield != NULL ) 
    {
	if ( wield->value[0] != WEAPON_DAGGER )
	    dam *= 1 + (effective_level(ch, gsn_circle) / 10); 
	else 
	    dam *= 1 + (effective_level(ch, gsn_circle) / 8);
    }

//joust used to be a flat 3x damage, now it should be 2x for a weapon other
//than a spear. for a spear/lance it should range from 2-5x or so.
    if ( dt == gsn_joust && wield != NULL )
    {
    if ( wield->value[0] == WEAPON_SPEAR )
		dam *= 1 + (effective_level(ch, gsn_joust) / 12);
	else
		dam *= 2;
	}
	//dam *= 3;

// Modified to multiple damroll bonus by 1.5. Avvrax July 27, 2001
    dam += GET_DAMROLL( ch ) * 1.5 * UMIN(100, skill) / 100;

    if ( wield )
    {
	if ( IS_OBJ_STAT(wield, ITEM_BLESS) && IS_EVIL(victim) )
	    dam++;
	else if ( IS_OBJ_STAT(wield, ITEM_EVIL) && IS_GOOD(victim) )
	    dam++;
    }

    if ( dam <= 0 )
	dam = 1;

    damages = damage( ch, victim, dam, dt, dam_type, TRUE, wield );
    
    /* display the enhanced damage message */
    if ( damages
    &&   enhanced
    &&   wield != NULL
    &&   ch->fighting == victim )
    {
        act_color( "You twist your weapon viciously in the wound!", ch, NULL, NULL, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	act_color( "$n twists $s weapon viciously in the wound!", ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );

        check_improve( ch, gsn_enhanced_damage, TRUE, 6, TRUE );
    }
    
    /* check for pummeling stun */
    if ( damages
    &&   !enhanced
    &&   ch->fighting == victim
    &&   (pummeling_skill = get_skill(ch, gsn_pummeling)) > 0
    &&   ((wield != NULL
    &&     (dam_type == DAM_BASH
    ||      (dam_type != DAM_PIERCE
    &&       dam_type != DAM_SLASH
    &&       (wield->value[0] == WEAPON_MACE
    ||        wield->value[0] == WEAPON_FLAIL
    ||        wield->value[0] == WEAPON_WHIP))))
    ||    martial_artist(ch))
    &&   number_percent() <= pummeling_skill
    &&   !saves_spell(ch->level / 4, victim, DAM_BASH) )
    {
	act_color( "Your powerful blow sends $N reeling!", ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	act_color( "$ns powerful blow sends you reeling!", ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	
	DAZE_STATE( victim, UMAX(12, ch->level / 10) );

	check_improve( ch, gsn_pummeling, TRUE, 6, TRUE );
    }
    
    /* but do we have a funky weapon? */
    if ( damages )
    {
	int dam;

	if ( wield )
	{
            if ( ch->fighting == victim
            &&   IS_WEAPON_STAT(wield, WEAPON_POISON) )
            {
                int level, poison_type;
                AFFECT_DATA *poison, af;

                if ( (poison = affect_find(wield->affected, gsn_poison)) == NULL )
                {
                    level = wield->level;
                    poison_type = POISON_NORMAL;
                }
                else
                {
                    level = poison->level;
                    poison_type = (poison->modifier >> 8);
                }

                if (!saves_spell(level / 2,victim,DAM_POISON)) 
                {
                    /* check if poison_type = POISON_DEATH and handle accordingly */
                    if ( poison_type == POISON_DEATH )
                    {
                        act(
                            "You feel poison coursing through your veins and realise with a shock that this is the end, you are dead!",
                            victim, NULL, NULL, TO_CHAR );
                        act( 
                            "$n is poisoned by the venom on $p and keels over dead!.",
	                    victim, wield, NULL, TO_ROOM );

                        sprintf(buf,"%s kills %s by death poison.", PERS_TRUE(ch), PERS_TRUE(victim));
                        wiznet(buf,NULL,NULL,WIZ_DEATHS,0,0);
                        log_string( buf );
                        handle_death( ch, victim, DAM_POISON, TRUE );
                    }
                    else if ((poison == NULL) || (poison_type == POISON_NORMAL))
                    /* original default weapon_poison handling code */
                    {
                        af.where		= TO_AFFECTS;
                        af.type			= gsn_poison;
                        af.level		= level * 3/4;
                        af.duration		= level / 2;
                        af.location		= APPLY_STR;
                        af.modifier		= -4;
                        af.bitvector		= AFF_POISON;
                        af.procNo		= affect_proc_lookup("poison");
                        af.flags		= AFF_CONT_UPDATE;
                        af.name			= "";
                        if ( IS_NPC(victim) )
	                    af.appearance	= "";
                        else
	                    af.appearance	= 
	                        "%n %p, a sickly green tinge coloring %s skin.";
	                    
                        act_color(
                            "You feel poison coursing through your veins.",
                            victim, NULL, NULL, TO_CHAR, POS_RESTING, OTHER_COLOR(ch) );
                        act_color( "$n is poisoned by the venom on $p.", 
                            victim, wield, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );

                        affect_join( victim, &af );
                    }
                    else
                    /* new poison types except for poison_death */
                    {
                        af.where		= TO_AFFECTS;
                        af.type			= gsn_poison;
                        af.level		= poison->level;
                        af.duration		= (poison->modifier & 0xFF);
                        // If duration > 128 then should really be negative - twos complement stuff
                        // if duration = 0, default to standard old duration values of level/2
                        if (af.duration > 128)
                            af.duration		= -1;
                        else if (af.duration == 0)
                            af.duration		= level / 2;
                        af.bitvector		= AFF_POISON;
                        af.modifier		= 0;
                        af.procNo		= affect_proc_lookup("poison");
                        af.flags		= AFF_CONT_UPDATE;	//poison->flags;
                        af.name			= poison->name;
                        if ( IS_NPC(victim) )
                            af.appearance	= "";
                        else
                            af.appearance	= poison->appearance;

                        switch (poison_type)
                        {
                            default:	/* shouldn't get this...but just in case */
                                af.location = APPLY_STR;
                                af.modifier = -4;
                                affect_join( victim, &af );
      
                                act_color(
                                    "You feel poison coursing through your veins.",
                                    victim, NULL, NULL, TO_CHAR, POS_RESTING, OTHER_COLOR(ch) );
                                act_color( "$n is poisoned by the venom on $p.", 
                                    victim, wield, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );
                                break;

                            case POISON_NORMAL:
                                af.location = APPLY_STR;
                                af.modifier = -4;
                                affect_join( victim, &af );

                                act_color(
                                    "You feel poison coursing through your veins.",
                                    victim, NULL, NULL, TO_CHAR, POS_RESTING, OTHER_COLOR(ch) );
                                act_color( "$n is poisoned by the venom on $p.", 
                                    victim, wield, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );
                                break;
					
                            case POISON_WEAKNESS:
                                af.location = APPLY_STR;
                                af.modifier = -8;
                                if ( !strcmp(af.name, "") )
	                            af.name = "poison - weakness";
                                affect_join( victim, &af );

                                act_color(
                                    "You feel poison coursing through your veins.",
                                    victim, NULL, NULL, TO_CHAR, POS_RESTING, OTHER_COLOR(ch) );
                                act_color( "$n is poisoned by the venom on $p.", 
                                    victim, wield, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );
                                break;
					
                            case POISON_NAUSEA:
                                af.location = APPLY_HITROLL;
                                af.modifier = -4;
                                af.procNo	= affect_proc_lookup("nausea");
                                if ( !strcmp(af.name, "") )
	                            af.name = "poison - nausea";
                                affect_join( victim, &af );
	                                
                                af.location = APPLY_AC;
                                af.modifier = 10;
                                af.procNo = 0;
                                af.flags = 0;
                                affect_join( victim, &af );
	                                
                                af.location = APPLY_DEX;
                                af.modifier = -4;
                                affect_join( victim, &af );
	                                
                                act_color(
                                    "You feel poison coursing through your veins.",
                                    victim, NULL, NULL, TO_CHAR, POS_RESTING, OTHER_COLOR(ch) );
                                act_color( "$n is poisoned by the venom on $p.", 
                                    victim, wield, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );
                                break;

                            case POISON_BLINDNESS:
                                af.location = APPLY_NONE;
                                af.modifier = 0;
                                af.bitvector = AFF_POISON + AFF_BLIND;
                                if ( !strcmp(af.name, "") )
	                            af.name = "poison - blindness";
                                affect_join( victim, &af );

                                act_color(
                                    "You feel poison coursing through your veins.",
                                    victim, NULL, NULL, TO_CHAR, POS_RESTING, OTHER_COLOR(ch) );
                                act_color( "$n is poisoned by the venom on $p.", 
                                    victim, wield, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );
                                break;
					
                            case POISON_BLEEDING:
                                af.location = APPLY_STR;
                                af.modifier = -4;
                                af.procNo	= affect_proc_lookup("bleeding");
                                if ( !strcmp(af.name, "") )
	                            af.name = "poison - bleeding";
                                affect_join( victim, &af );

                                act_color(
                                    "You feel poison coursing through your veins.",
                                    victim, NULL, NULL, TO_CHAR, POS_RESTING, OTHER_COLOR(ch) );
                                act_color( "$n is poisoned by the venom on $p.", 
                                    victim, wield, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );
                                break;	

                            case POISON_PARALYSIS:
                                af.location = APPLY_STR;
                                af.modifier = -4;
                                af.bitvector = AFF_POISON + AFF_PARALYSIS;
                                af.procNo = 0;
                                af.flags = 0;
                                if ( !strcmp(af.name, "") )
	                            af.name = "poison - paralysis";
                                affect_join( victim, &af );

                                act_color(
                                    "You feel poison coursing through your veins.",
                                    victim, NULL, NULL, TO_CHAR, POS_RESTING, OTHER_COLOR(ch) );
                                act_color( "$n is poisoned by the venom on $p.", 
                                    victim, wield, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );

                                stop_fighting( ch, TRUE );
                                break;
/*					
                            case POISON_HALLUCINATION:
                                af.location = APPLY_NONE;
                                af.modifier = 0;
                                af.bitvector = AFF_POISON + AFF_HALLUCINATION;
                                if ( !strcmp(af.name, "") )
	                            af.name = "poison - hallucination";
                                affect_join( victim, &af );

                                send_to_char(
	                            "You feel poison coursing through your veins and the world starts to twist unconcertingly.\n\r",
	                            victim );
                                act( "$n is poisoned by the venom on $p and looks quite confused.",
	                            victim, wield, NULL, TO_ROOM );
                                break;
*/					
                            case POISON_COMA:
                                af.location = APPLY_NONE;
                                af.modifier = 0;
                                af.bitvector = AFF_SLEEP;
                                af.procNo = 0;
                                af.flags = 0;
                                if ( !strcmp(af.name, "") )
	                            af.name = "poison - coma";
                                affect_join( victim, &af );

                                if ( IS_AWAKE(victim) )
                                {
	                            act_color(
                                        "You feel poison coursing through your veins.",
                                        victim, NULL, NULL, TO_CHAR, POS_RESTING, OTHER_COLOR(ch) );
	                            act_color( "$n is poisoned by the venom on $p.", 
                                        victim, wield, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );
	                                
	                            stop_fighting( ch, TRUE );
	                            victim->position = POS_SLEEPING;
                                }
                                break;
/*
                            case POISON_MADNESS:
                                af.location = APPLY_INT;
                                af.modifier = -16;
                                if ( !strcmp(af.name, "") )
	                            af.name = "poison - madness";
                                affect_join( victim, &af );
	                                
                                af.location = APPLY_WIS;
                                af.flags = 0;
                                affect_join( victim, &af );

                                send_to_char(
	                            "You feel poison coursing through your veins and everything suddenly goes black.",
	                            victim );
                                act( "$n is poisoned by the venom on $p and collapses unconscious.",
	                             victim, wield, NULL, TO_ROOM );
                                break;
*/
                            case POISON_WITHER:
                                af.location = APPLY_STR;
                                af.modifier = -16;
                                if ( !strcmp(af.name, "") )
	                            af.name = "poison - wither";
                                if ( !strcmp(af.appearance, "") && !IS_NPC(victim) )
	                            af.appearance = "%n %p, all withered and shrunken.";
                                affect_join( victim, &af );

                                af.location = APPLY_DEX;
                                af.flags = 0;
                                affect_join( victim, &af );

                                af.location = APPLY_CON;
                                affect_join( victim, &af );

                                act_color(
                                    "You feel poison coursing through your veins.",
                                    victim, NULL, NULL, TO_CHAR, POS_RESTING, OTHER_COLOR(ch) );
                                act_color( "$n is poisoned by the venom on $p.", 
                                    victim, wield, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );
                                break;
					
                            case POISON_DEATH:
                                bug( "One_Hit: POISON_DEATH reached in switch.", 0 );
                                break;
			}	// switch
		    }	// new poison types
		}

                /* weaken the poison if it's temporary */
                if (poison != NULL)
                {
                    poison->level = UMAX(0,poison->level - 2);
                    poison->duration = UMAX(0,poison->duration - 1);

                    if ( poison->level == 0 || poison->duration == 0 )
	                act( "The poison on $p has worn off.",
	                     ch, wield, NULL, TO_CHAR );
                }
 	    }

	    if ( ch->fighting == victim
	    &&   IS_WEAPON_STAT(wield, WEAPON_VAMPIRIC)
	    &&   !IS_SET(CHAR_FORM(victim), FORM_UNDEAD) )
	    {
		dam = number_range( 1, wield->level / 5 + 1 );
		act_color( "$p draws life from $n.", victim, wield, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );
		act_color( "You feel $p drawing your life away.", victim, wield, NULL, TO_CHAR, POS_RESTING, OTHER_COLOR(victim) );
		damage_old( ch, victim, dam, 0, DAM_NEGATIVE, FALSE, wield );
		ch->alignment = UMAX(-1000,ch->alignment - 1);
		ch->hit += dam/2;
	    }

	    if ( ch->fighting == victim
	    &&   IS_WEAPON_STAT(wield, WEAPON_FLAMING) )
	    {
		dam = number_range( 1,wield->level / 4 + 1 );
		act_color( "$n is burned by $p.", victim, wield, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );
		act_color( "$p sears your flesh.", victim, wield, NULL, TO_CHAR, POS_RESTING, OTHER_COLOR(ch) );
		fire_effect( (void *) victim, wield->level/2, 
		             dam, TARGET_CHAR );
		damage( ch, victim, dam, 0, DAM_FIRE, FALSE, wield );
	    }

	    if ( ch->fighting == victim
	    &&   IS_WEAPON_STAT(wield, WEAPON_FROST) )
	    {
		dam = number_range( 1, wield->level / 6 + 2 );
		act_color("$p freezes $n.",victim,wield,NULL,TO_ROOM, POS_RESTING, OTHER_COLOR(ch));
		act_color( "The cold touch of $p surrounds you with ice.",
		     victim, wield, NULL, TO_CHAR, POS_RESTING, OTHER_COLOR(ch) );
		cold_effect( victim, wield->level/2, dam, TARGET_CHAR );
		damage( ch, victim, dam, 0, DAM_COLD, FALSE, wield );
	    }

	    if ( ch->fighting == victim
	    &&   IS_WEAPON_STAT(wield, WEAPON_SHOCKING) )
	    {
		dam = number_range( 1, wield->level/5 + 2 );
		act_color( "$n is struck by lightning from $p.",
		     victim, wield, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );
		act_color( "You are shocked by $p.",
		     victim, wield, NULL, TO_CHAR, POS_RESTING, OTHER_COLOR(ch) );
		shock_effect( victim, wield->level/2, dam, TARGET_CHAR );
		damage( ch, victim, dam, 0, DAM_LIGHTNING, FALSE, wield );
	    }
            
            if ( ch->fighting == victim
            &&   IS_WEAPON_STAT(wield, WEAPON_DISPEL) )
                spell_dispel_magic_silently( gsn_dispel_magic, wield->level / 2, ch, victim, TAR_CHAR_OFFENSIVE );

	    if ( ch->fighting == victim
	    &&   IS_WEAPON_STAT(wield, WEAPON_VORPAL)
	    &&   IS_SET(CHAR_PARTS(victim), PART_HEAD)
	    &&   number_bits(IS_NPC(ch) ? 7 : 8) == 0
	    &&   !saves_spell(wield->level, victim, DAM_SLASH) )
		decapitate( victim, ch );
            
            if ( ch->fighting == victim
            &&   IS_WEAPON_STAT(wield, WEAPON_CURSE) )
                curse_effect( victim, wield->level / 2, 0, TARGET_CHAR );

            if ( ch->fighting == victim
            &&   COMPLEX_STYLE(ch, STYLE_STONE_CLAW)
            &&   number_range(1, 10) == 1
            &&   !saves_spell(ch->level, victim, DAM_BASH) )
            {
                act_color( "Your powerful blow staggers $N.",
                    ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
                act_color( "$N is staggered by $ns powerful blow.",
                    ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
                act_color( "$ns powerful blow staggers you!",
                    ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	        DAZE_STATE( victim, 2 * PULSE_VIOLENCE );
            }
	}
	else if ( number_bits(6) == 0
	&&   ch->fighting == victim
	&&   victim->hit > 0
	&&   !saves_spell(effective_level(ch, gsn_hand_to_hand), victim, DAM_BASH)
	&&   martial_artist(ch) )
	{
	    act_color( "$ns powerful strike to the midriff makes $N spit blood!",
	         ch, NULL, victim, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );
	    act_color( "$n shouts out 'HAI' as he drives $s fist into your midriff!",
	         ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	    damage( ch, victim, victim->hit / 2, TYPE_HIT + CHAR_DAM_TYPE(ch), 
	            DAM_BASH, TRUE, NULL );
	}
    }

    tail_chain( );
    return damages;
}


/*
 * Inflict damage from a hit.
 */
bool damage(CHAR_DATA *ch,CHAR_DATA *victim,int dam,int dt,int dam_type,
	    bool show, OBJ_DATA *weapon) 
{
    bool immune;
    
    if ( victim->position == POS_DEAD )
	return FALSE;

    /*
     * Stop up any residual loopholes.
     */
    if ( dam > 1200 && dt >= TYPE_HIT )
    {
	bug( "Damage: %d: more than 1200 points!", dam );
	dam = 1200;
	if (!IS_IMMORTAL(ch))
	{
	    send_to_char("You really shouldn't cheat.\n\r",ch);
	    if (weapon != NULL)
	    	extract_obj(weapon);
	}

    }

    
    /* damage reduction */
    if ( dam > 35)
	dam = (dam - 35)/2 + 35;
    if ( dam > 80)
	dam = (dam - 80)/2 + 80; 



   
    if ( victim != ch )
    {
	/*
	 * Certain attacks are forbidden.
	 * Most other attacks are returned.
	 */
	if ( is_safe( ch, victim ) )
	    return FALSE;
	check_killer( ch, victim );

	if ( victim->position > POS_STUNNED )
	{
	    if ( victim->fighting == NULL )
	    {
		set_fighting( victim, ch, FALSE );
		if ( IS_NPC( victim ) && HAS_TRIGGER( victim, TRIG_KILL ) )
		    mp_percent_trigger( victim, ch, NULL, NULL, TRIG_KILL );
	    }
	    if (victim->timer <= 4)
		victim->position = POS_FIGHTING;
	}

	if ( victim->position > POS_STUNNED )
	{
	    if ( ch->fighting == NULL )
		set_fighting( ch, victim, TRUE );

	    /*
	     * If victim is charmed, ch might attack victim's master.
	     taken out by Russ! */
/*
	    if ( IS_NPC(ch)
	    &&   IS_NPC(victim)
	    &&   IS_AFFECTED(victim, AFF_CHARM)
	    &&   victim->master != NULL
	    &&   victim->master->in_room == ch->in_room
	    &&   number_bits( 3 ) == 0 )
	    {
		stop_fighting( ch, FALSE );
		multi_hit( ch, victim->master, TYPE_UNDEFINED );
		return FALSE;
	    }
*/
	}

	/*
	 * More charm stuff.
	 */
	if ( victim->master == ch )
	    stop_follower( victim );
    }

    /*
     * Inviso attacks ... not.
     */
    if ( IS_AFFECTED(ch, AFF_INVISIBLE) )
    {
	affect_strip( ch, gsn_invis );
	affect_strip( ch, gsn_mass_invis );
	REMOVE_BIT( ch->affected_by, AFF_INVISIBLE );
	act_color( "$n fades into existence.", ch, NULL, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );
    }

    /*
     * Damage modifiers.
     */

    if ( dam > 1 && !IS_NPC(victim) 
    &&   victim->pcdata->condition[COND_DRUNK]  > 10 )
	dam = 9 * dam / 10;

    if ( dam > 1 && IS_AFFECTED(victim, AFF_SANCTUARY) )
	dam /= 2;

    if ( dam > 1 && ((IS_AFFECTED(victim, AFF_PROTECT_EVIL) && IS_EVIL(ch) )
    ||		     (IS_AFFECTED(victim, AFF_PROTECT_GOOD) && IS_GOOD(ch) )))
	dam -= dam / 4;

    immune = FALSE;


    /*
     * Check for parry, and dodge.
     */
    if ( dt >= TYPE_HIT && ch != victim )
    {
        if ( check_parry( ch, weapon, victim ) )
	    return FALSE;
	if ( check_dodge( ch, victim ) )
	    return FALSE;
	if ( check_shield_block(ch, weapon, victim) )
	    return FALSE;
	if ( check_fade(ch, weapon, victim) )
	    return FALSE;
    }
    else if ( ( dt == gsn_backstab || dt == gsn_circle ) && ch != victim )
    {
	if ( check_fade(ch, weapon, victim) )
	    return FALSE;
    }
    else if ( dt == gsn_joust && ch != victim )
    {
	if ( check_shield_block(ch, weapon, victim) )
	    return FALSE;
	if ( check_fade(ch, weapon, victim) )
	    return FALSE;
    }
    else if ( ch != victim )
    {
	if ( check_mirror_image(ch, weapon, victim) )
	    return FALSE;
    }

    switch(check_immune(victim,dam_type))
    {
	case(IS_IMMUNE):
	    immune = TRUE;
	    dam = 0;
	    break;
	case(IS_RESISTANT):
	    dam -= dam/3;
	    break;
	case(IS_VULNERABLE):
	    dam += dam/2;
	    break;
    }

    if ( weapon != NULL && dt == gsn_weapon_brawling )
    {
	if ( weapon->value[0] == WEAPON_EXOTIC )
	{
	    act_color( "You smash $N with your weapon!", ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	    act_color( "$n smashes you with $s weapon!", ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	}
	else
	{
	    char buf[ MAX_STRING_LENGTH ];
	    char *attack, *weaponName;
	    
	    switch ( weapon->value[0] )
	    {
		case WEAPON_SWORD:
		    attack = "hilt";
		    weaponName = "sword";
		    break;
		case WEAPON_BOW:
		    attack = "shaft";
		    weaponName = "bow";
		    break;
		case WEAPON_DAGGER:
		    attack = "pommel";
		    weaponName = "dagger";
		    break;
		case WEAPON_SPEAR:
		    attack = "shaft";
		    weaponName = "spear";
		    break;
		case WEAPON_MACE:
		    attack = "handle";
		    weaponName = "mace";
		    break;
		case WEAPON_AXE:
		    attack = "handle";
		    weaponName = "axe";
		    break;
		case WEAPON_FLAIL:
		    attack = "handle";
		    weaponName = "flail";
		    break;
		case WEAPON_WHIP:
		    attack = "handle";
		    weaponName = "whip";
		    break;
		case WEAPON_POLEARM:
		    attack = "shaft";
		    weaponName = "polearm";
		    break;
		default:
		    attack = weaponName = "????";
		    break;		
	    }
	    
	    sprintf( buf, "You smash the %s of your %s into $N.",
	        attack, weaponName );
	    act_color( buf, ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	    sprintf( buf, "$n smashes the %s of $s %s into your face!",
	        attack, weaponName );
	    act_color( buf, ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	}
    }

    if (show)
	dam_message_new( ch, victim, dam, dt, immune );

    if ( !IS_NPC(ch)
    &&   IS_SET(ch->act, PLR_SINGING) )
	remove_bard_song( ch, FALSE );

    if (dam == 0)
	return FALSE;

    /*
     * Hurt the victim.
     * Inform the victim of his new state.
     */
    victim->hit -= dam;
    if ( !IS_NPC(victim)
    &&   victim->level >= LEVEL_IMMORTAL
    &&   victim->hit < 1 )
	victim->hit = 1;
    update_pos( victim );

    switch( victim->position )
    {
    case POS_MORTAL:
	act( "$n is mortally wounded, and will die soon, if not aided.",
	    victim, NULL, NULL, TO_ROOM );
	send_to_char( 
	    "You are mortally wounded, and will die soon, if not aided.\n\r",
	    victim );
	break;

    case POS_INCAP:
	act( "$n is incapacitated and will slowly die, if not aided.",
	    victim, NULL, NULL, TO_ROOM );
	send_to_char(
	    "You are incapacitated and will slowly die, if not aided.\n\r",
	    victim );
	break;

    case POS_STUNNED:
	act( "$n is stunned, but will probably recover.",
	    victim, NULL, NULL, TO_ROOM );
	send_to_char("You are stunned, but will probably recover.\n\r",
	    victim );
	break;

    case POS_DEAD:
	act( "$n is DEAD!!", victim, 0, 0, TO_ROOM );
	send_to_char( "You have been KILLED!!\n\r\n\r", victim );
	break;

    default:
	if ( dam > victim->max_hit / 4 )
	    act_color( "That really did HURT!", victim, NULL, NULL, TO_CHAR, POS_RESTING, OTHER_COLOR(ch) );
	if ( victim->hit < victim->max_hit / 4 )
	    act_color( "You sure are BLEEDING!", victim, NULL, NULL, TO_CHAR, POS_RESTING, OTHER_COLOR(ch) );
	break;
    }

    /*
     * Sleep spells and extremely wounded folks.
     */
    if ( !IS_AWAKE(victim) )
	stop_fighting( victim, FALSE );

    /* Kill them. */
    if ( victim->position == POS_DEAD )
    {
	handle_death( ch, victim, dam_type, TRUE );
	return TRUE;
    }

    if ( victim == ch )
	return TRUE;

    /*
     * Take care of link dead people.
     */
    if ( !IS_NPC(victim) && victim->desc == NULL 
    &&   !IS_AFFECTED(victim, AFF_PARALYSIS) )
    {
	if ( number_range( 0, victim->wait ) == 0 )
	{
	    do_recall( victim, "" );
	    return TRUE;
	}
    }

    /*
     * Wimp out?
     */
    if ( IS_NPC(victim) && dam > 0 && victim->wait < PULSE_VIOLENCE / 2)
    {
	if ( (IS_SET(victim->act, ACT_WIMPY)
	&&    number_bits(IS_SET(victim->act, ACT_SURRENDERS) ? 3 : 2) == 0
	&&    !IS_AFFECTED(victim, AFF_PARALYSIS)
	&&    victim->hit < victim->max_hit / 5) 
	||   (IS_AFFECTED(victim, AFF_CHARM)
	&&    victim->master != NULL
	&&    victim->master->in_room != victim->in_room) )
	    do_flee( victim, "" );
	
	if ( (IS_SET(victim->act, ACT_SURRENDERS)
	&&    number_bits(3) == 0
	&&    !IS_AFFECTED(victim, AFF_PARALYSIS)
	&&    victim->hit < victim->max_hit / 2)
	||   (IS_AFFECTED(victim, AFF_CHARM)
	&&    victim->master != NULL
	&&    victim->master->in_room != victim->in_room) )
	    do_surrender( victim, "" );
    }

    if ( !IS_NPC(victim)
    &&   !IS_AFFECTED(victim, AFF_PARALYSIS)
    &&   victim->hit > 0
    &&   victim->hit <= victim->wimpy
    &&   victim->wait < PULSE_VIOLENCE / 2 )
	do_flee( victim, "" );

    tail_chain();
    return TRUE;
}






/*
 * Inflict damage from a hit.
 */
bool damage_old( CHAR_DATA *ch, CHAR_DATA *victim, int dam, int dt, int 
                 dam_type, bool show, OBJ_DATA *weapon ) 
{
    bool immune;
    
    if ( victim->position == POS_DEAD )
	return FALSE;

    /*
     * Stop up any residual loopholes.
     */

    if ( dam > 1200 && dt >= TYPE_HIT )
    {
        bug( "Damage: %d: more than 1200 points!", dam );
        dam = 1200;
        if (!IS_IMMORTAL(ch))
        {
            send_to_char("You really shouldn't cheat.\n\r",ch);
            if (weapon != NULL)
                extract_obj(weapon);
        }
 
    }

    
    /* damage reduction */
    if ( dam > 35)
	dam = (dam - 35)/2 + 35;
    if ( dam > 80)
	dam = (dam - 80)/2 + 80; 



   
    if ( victim != ch )
    {
	/*
	 * Certain attacks are forbidden.
	 * Most other attacks are returned.
	 */
	if ( is_safe( ch, victim ) )
	    return FALSE;
	check_killer( ch, victim );

	if ( victim->position > POS_STUNNED )
	{
	    if ( victim->fighting == NULL )
		set_fighting( victim, ch, FALSE );
	    if (victim->timer <= 4)
		victim->position = POS_FIGHTING;
	}

	if ( victim->position > POS_STUNNED )
	{
	    if ( ch->fighting == NULL )
		set_fighting( ch, victim, TRUE );

	    /*
	     * If victim is charmed, ch might attack victim's master.
	     */
	    if ( IS_NPC(ch)
	    &&   IS_NPC(victim)
	    &&   IS_AFFECTED(victim, AFF_CHARM)
	    &&   victim->master != NULL
	    &&   victim->master->in_room == ch->in_room
	    &&   ((victim->mounted_by != NULL
	    &&     number_bits(2) == 0)
	    ||    number_bits(3) == 0) )
	    {
		stop_fighting( ch, FALSE );
		multi_hit( ch, victim->master, TYPE_UNDEFINED );
		return FALSE;
	    }
	}

	/*
	 * More charm stuff.
	 */
	if ( victim->master == ch )
	    stop_follower( victim );
    }

    /*
     * Inviso attacks ... not.
     */
    if ( IS_AFFECTED(ch, AFF_INVISIBLE) )
    {
	affect_strip( ch, gsn_invis );
	affect_strip( ch, gsn_mass_invis );
	REMOVE_BIT( ch->affected_by, AFF_INVISIBLE );
	act_color( "$n fades into existence.", ch, NULL, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );
    }

    /*
     * Damage modifiers.
     */

    if ( dam > 1 && !IS_NPC(victim)
    &&   victim->pcdata->condition[COND_DRUNK]  > 10 )
        dam = 9 * dam / 10;
 
    if ( dam > 1 && IS_AFFECTED(victim, AFF_SANCTUARY) )
        dam /= 2;
 
    if ( dam > 1 && ((IS_AFFECTED(victim, AFF_PROTECT_EVIL) && IS_EVIL(ch) )
    ||               (IS_AFFECTED(victim, AFF_PROTECT_GOOD) && IS_GOOD(ch) )))
        dam -= dam / 4;

    /* Non-piercing weapons underwater do less damage. */
    if ( dam > 1 && weapon != NULL
    &&   ch->in_room->sector_type == SECT_WATER_NOSWIM
    &&   dam_type != DAM_PIERCE )
	dam /= 2;

    immune = FALSE;


    /*
     * Check for parry, and dodge.
     */
    if ( dt >= TYPE_HIT && ch != victim)
    {
        if ( check_parry( ch, weapon, victim ) )
	    return FALSE;
	if ( check_dodge( ch, victim ) )
	    return FALSE;
	if ( check_shield_block(ch, weapon, victim) )
	    return FALSE;
	if ( check_fade(ch, weapon, victim) )
	    return FALSE;
    }
    else if ( ( dt == gsn_backstab || dt == gsn_circle ) && ch != victim )
    {
	if ( check_fade(ch, weapon, victim) )
	    return FALSE;
    }
    else if ( dt == gsn_joust && ch != victim )
    {
	if ( check_shield_block(ch, weapon, victim) )
	    return FALSE;
	if ( check_fade(ch, weapon, victim) )
	    return FALSE;
    }
    else if ( ch != victim )
    {
	if ( check_mirror_image(ch, weapon, victim) )
	    return FALSE;
    }

    switch(check_immune(victim,dam_type))
    {
	case(IS_IMMUNE):
	    immune = TRUE;
	    dam = 0;
	    break;
	case(IS_RESISTANT):	
	    dam -= dam/3;
	    break;
	case(IS_VULNERABLE):
	    dam += dam/2;
	    break;
    }

    if (show)
    	dam_message_new( ch, victim, dam, dt, immune );

    if ( !IS_NPC(ch)
    &&   IS_SET(ch->act, PLR_SINGING) )
	remove_bard_song( ch, FALSE );

    if (dam == 0)
	return FALSE;

    /*
     * Hurt the victim.
     * Inform the victim of his new state.
     */
    victim->hit -= dam;
    if ( !IS_NPC(victim)
    &&   victim->level >= LEVEL_IMMORTAL
    &&   victim->hit < 1 )
	victim->hit = 1;
    update_pos( victim );

    switch( victim->position )
    {
    case POS_MORTAL:
	act( "$n is mortally wounded, and will die soon, if not aided.",
	    victim, NULL, NULL, TO_ROOM );
	send_to_char( 
	    "You are mortally wounded, and will die soon, if not aided.\n\r",
	    victim );
	break;

    case POS_INCAP:
	act( "$n is incapacitated and will slowly die, if not aided.",
	    victim, NULL, NULL, TO_ROOM );
	send_to_char(
	    "You are incapacitated and will slowly die, if not aided.\n\r",
	    victim );
	break;

    case POS_STUNNED:
	act( "$n is stunned, but will probably recover.",
	    victim, NULL, NULL, TO_ROOM );
	send_to_char("You are stunned, but will probably recover.\n\r",
	    victim );
	break;

    case POS_DEAD:
	act( "$n is DEAD!!", victim, 0, 0, TO_ROOM );
	send_to_char( "You have been KILLED!!\n\r\n\r", victim );
	break;

    default:
	if ( dam > victim->max_hit / 4 )
	    act_color( "That really did HURT!", victim, NULL, NULL, TO_CHAR, POS_RESTING, OTHER_COLOR(ch) );
	if ( victim->hit < victim->max_hit / 4 )
	    act_color( "You sure are BLEEDING!", victim, NULL, NULL, TO_CHAR, POS_RESTING, OTHER_COLOR(ch) );
	break;
    }

    /*
     * Sleep spells and extremely wounded folks.
     */
    if ( !IS_AWAKE(victim) )
	stop_fighting( victim, FALSE );

    /* Kill them. */
    if ( victim->position == POS_DEAD )
    {
	handle_death( ch, victim, dam_type, TRUE );
	return TRUE;
    }

    if ( victim == ch )
	return TRUE;

    /*
     * Take care of link dead people.
     */
    if ( !IS_NPC(victim) && victim->desc == NULL
    &&   !IS_AFFECTED(victim, AFF_PARALYSIS) )
    {
	if ( number_range( 0, victim->wait ) == 0 )
	{
	    do_recall( victim, "" );
	    return TRUE;
	}
    }

    /*
     * Wimp out?
     */
    if ( IS_NPC(victim) && dam > 0 && victim->wait < PULSE_VIOLENCE / 2)
    {
	if ( ( IS_SET(victim->act, ACT_WIMPY) && number_bits( 2 ) == 0
	&&   !IS_AFFECTED(victim, AFF_PARALYSIS)
	&&   victim->hit < victim->max_hit / 5) 
	||   ( IS_AFFECTED(victim, AFF_CHARM) && victim->master != NULL
	&&     victim->master->in_room != victim->in_room ) )
	    do_flee( victim, "" );
    }

    if ( !IS_NPC(victim)
    &&   !IS_AFFECTED(victim, AFF_PARALYSIS)
    &&   victim->hit > 0
    &&   victim->hit <= victim->wimpy
    &&   victim->wait < PULSE_VIOLENCE / 2 )
	do_flee( victim, "" );

    tail_chain();
    return TRUE;
}

bool is_safe(CHAR_DATA *ch, CHAR_DATA *victim)
{
    if (victim->in_room == NULL || ch->in_room == NULL)
	return TRUE;

    if (victim->fighting == ch || victim == ch)
	return FALSE;

    if (IS_IMMORTAL(ch) && ch->level > LEVEL_IMMORTAL)
	return FALSE;

    /* killing mobiles */
    if (IS_NPC(victim))
    {

	/* safe room? */
	if (IS_SET(victim->in_room->room_flags,ROOM_SAFE))
	{
	    send_to_char("Not in this room.\n\r",ch);
	    return TRUE;
	}

	if (victim->pIndexData->pShop != NULL)
	{
	    send_to_char("The shopkeeper wouldn't like that.\n\r",ch);
	    return TRUE;
	}

	if ( IS_SET(victim->act, ACT_META) )
	{
	    send_to_char( "That wouldn't be very nice.\n\r", ch );
	    return TRUE;
	}

	/* no killing healers, trainers, etc */
	if (IS_SET(victim->act,ACT_TRAIN)
	||  IS_SET(victim->act,ACT_PRACTICE)
	||  IS_SET(victim->act,ACT_IS_HEALER)
	||  IS_SET(victim->act,ACT_IS_CHANGER))
	{
	    send_to_char("I don't think The Triad would approve.\n\r",ch);
	    return TRUE;
	}

	if (!IS_NPC(ch))
	{
	    /* no pets */
	    if (IS_SET(victim->act,ACT_PET))
	    {
		act("But $N looks so cute and cuddly...",
		    ch,NULL,victim,TO_CHAR);
		return TRUE;
	    }

	    /* no charmed creatures unless owner */
	    if (IS_AFFECTED(victim,AFF_CHARM) && ch != victim->master)
	    {
		send_to_char("You don't own that monster.\n\r",ch);
		return TRUE;
	    }
	}
    }
    /* killing players */
    else
    {
	/* NPC doing the killing */
	if (IS_NPC(ch))
	{
	    /* safe room check */
	    if ( IS_SET(victim->in_room->room_flags, ROOM_SAFE) )
	    {
		send_to_char( "Not in this room.\n\r", ch );
		return TRUE;
	    }

	    /* charmed mobs and pets cannot attack players while owned */
	    if ( IS_AFFECTED(ch, AFF_CHARM) && ch->master != NULL
	    &&   ch->master->fighting != victim)
	    {
		send_to_char( "Players are your friends!\n\r", ch );
		return TRUE;
	    }
	}
	/* player doing the killing */
	else
	{
	/*
	 *   if ( !is_clan(ch)
	 *   &&   !IS_SET(ch->in_room->area->flags, AREA_LAWLESS|AREA_ARENA) )
	 *   {
	 *	send_to_char( 
	 *	    "Join a clan if you want to kill players.\n\r", ch );
	 *	return TRUE;
	 *   }
	 */

	/*
	 *   if ( IS_SET(victim->act,PLR_KILLER)
	 *   ||   IS_SET(victim->act,PLR_THIEF)
	 *   ||   victim->pcdata->heresy >= HERESY_CURSED )
	 *	return FALSE;
	 */
	
	/*
	 *   if ( !is_clan(victim)
	 *   &&   !IS_SET(ch->in_room->area->flags, AREA_LAWLESS|AREA_ARENA) )
	 *   {
	 *	send_to_char("They aren't in a clan, leave them alone.\n\r",ch);
	 *	return TRUE;
	 *   }
	 */

	    /* safe room check */
	    if ( IS_SET(victim->in_room->room_flags, ROOM_SAFE) )
	    {
		send_to_char( "Not in this room.\n\r", ch );
		return TRUE;
	    }

	    if ( ch->level > victim->level + 8
	    &&   !IS_SET(ch->in_room->area->flags, AREA_ARENA) )
	    {
		send_to_char( "Pick on someone your own size.\n\r", ch );
		return TRUE;
	    }
	}
    }
    return FALSE;
}
 
bool is_safe_spell(CHAR_DATA *ch, CHAR_DATA *victim, bool area )
{
    if (victim->in_room == NULL || ch->in_room == NULL)
        return TRUE;

    if (victim == ch && area)
	return TRUE;

    if (victim->fighting == ch || victim == ch)
	return FALSE;

    if (IS_IMMORTAL(ch) && ch->level > LEVEL_IMMORTAL && !area)
	return FALSE;

    /* killing mobiles */
    if (IS_NPC(victim))
    {
	/* safe room? */
	if (IS_SET(victim->in_room->room_flags,ROOM_SAFE))
	    return TRUE;

	if (victim->pIndexData->pShop != NULL)
	    return TRUE;
	    
	if ( IS_SET(victim->act, ACT_META) )
	    return TRUE;

	/* no killing healers, trainers, etc */
	if (IS_SET(victim->act,ACT_TRAIN)
	||  IS_SET(victim->act,ACT_PRACTICE)
	||  IS_SET(victim->act,ACT_IS_HEALER)
	||  IS_SET(victim->act,ACT_IS_CHANGER))
	    return TRUE;

	if (!IS_NPC(ch))
	{
	    /* no pets */
	    if (IS_SET(victim->act,ACT_PET))
	   	return TRUE;

	    /* no charmed creatures unless owner */
	    if (IS_AFFECTED(victim,AFF_CHARM) && (area || ch != victim->master))
		return TRUE;

	    /* legal kill? -- cannot hit mob fighting non-group member */
	    if (victim->fighting != NULL && !is_same_group(ch,victim->fighting))
		return TRUE;
	}
	else
	{
	    /* area effect spells do not hit other mobs */
	    if (area && !is_same_group(victim,ch->fighting))
		return TRUE;
	}
    }
    /* killing players */
    else
    {
	if (area && IS_IMMORTAL(victim) && victim->level > LEVEL_IMMORTAL)
	    return TRUE;

	/* NPC doing the killing */
	if (IS_NPC(ch))
	{
	    /* charmed mobs and pets cannot attack players while owned */
	    if (IS_AFFECTED(ch,AFF_CHARM) && ch->master != NULL
	    &&  ch->master->fighting != victim)
		return TRUE;
	
	    /* safe room? */
	    if (IS_SET(victim->in_room->room_flags,ROOM_SAFE))
		return TRUE;

	    /* legal kill? -- mobs only hit players grouped with opponent*/
	    if (ch->fighting != NULL && !is_same_group(ch->fighting,victim))
		return TRUE;
	}

	/* player doing the killing */
	else
	{
	/*
	 *   if ( !is_clan(ch)
	 *   &&   !IS_SET(ch->in_room->area->flags, AREA_LAWLESS|AREA_ARENA) )
	 *   {
	 *	send_to_char( 
	 *	    "Join a clan if you want to kill players.\n\r", ch );
	 *	return TRUE;
	 *   }
	 */
	 
	/*
	 *   if ( IS_SET(victim->act,PLR_KILLER)
	 *   ||   IS_SET(victim->act,PLR_THIEF)
	 *   ||   victim->pcdata->heresy >= HERESY_CURSED )
	 *	return FALSE;
	 */

	/*
         *   if ( !is_clan(victim)
         *   &&   !IS_SET(ch->in_room->area->flags, AREA_LAWLESS|AREA_ARENA) )
	 *	return TRUE;
	 */

	    /* safe room check */
	    if ( IS_SET(victim->in_room->room_flags, ROOM_SAFE) )
	    {
		send_to_char( "Not in this room.\n\r", ch );
		return TRUE;
	    }

	    if ( ch->level > victim->level + 8
	    &&   !IS_SET(ch->in_room->area->flags, AREA_ARENA) )
		return TRUE;
	}

    }
    return FALSE;
}
/*
 * See if an attack justifies a KILLER flag.
 */
void check_killer( CHAR_DATA *ch, CHAR_DATA *victim )
{
    char buf[MAX_STRING_LENGTH];

    if ( IS_SET(ch->in_room->area->flags, AREA_ARENA) )
	return;

    if ( !IS_NPC(ch)
    &&   !IS_NPC(victim) )
    {
        int i;
        for ( i = 0; i < MAX_PC_FIGHT_RECORD; ++i )
        {
            if ( victim->pcdata->fight_records[i] == ch->id )
                return;
        }
    }
	
    if ( !IS_NPC(ch)
    &&   !IS_NPC(victim)
    &&   ch != victim
    &&   ch->pcdata->religion == victim->pcdata->religion
    &&   victim->pcdata->heresy == 0
    &&   IS_SET(class_table[victim->class].flags, CLASS_RELIGIOUS)
    &&   !IS_SET(victim->in_room->area->flags, AREA_ARENA) )
    {
	char buf[MAX_STRING_LENGTH];

	if ( ch->pcdata->heresy == 0 )
	{
	    send_to_char( "You are branded a heretic!\n\r", ch );
	    
	    sprintf( buf, "$N is branded a heretic for attacking %s.", 
	    	victim->name );
	    wiznet( buf, ch, NULL, WIZ_PENALTIES, WIZ_SECURE, 0 );
	    
	    ch->pcdata->heresy = 1;
	}
	else if ( ch->pcdata->heresy < HERESY_CURSED )
	{
	    act( "$t's displeasure with you increases.", 
	        ch, religion_table[GET_GOD(ch)].head, NULL, TO_CHAR );
	        
	    sprintf( buf, "$N's heresy is increased for attacking %s.",
	        victim->name );
	    wiznet( buf, ch, NULL, WIZ_PENALTIES, WIZ_SECURE, 0 );
	    
	    ch->pcdata->heresy++;
	}
    }

    // Social status check. Guards can attack people with social class
    // 0 or 1 within their own territory.
    if ( !IS_NPC(victim)
    &&   victim->pcdata->social_classes[ch->in_room->area->territory] <= 1
    &&   (has_territory_flag(ch, ch->in_room->area->territory, TERR_CAPTAIN_GUARD)
    ||    has_territory_flag(ch, ch->in_room->area->territory, TERR_GUARD)) )
        return;

/*
 *   if ( IS_SET(ch->in_room->area->flags, AREA_LAWLESS) )
 *	return;
 */
 
    if ( !IS_SET(territories[ch->in_room->area->territory].crimes, CRIME_PKILLING) )
        return;
 
    /*
     * Follow charm thread to responsible character.
     * Attacking someone's charmed char is hostile!
     */
    while ( IS_AFFECTED(victim, AFF_CHARM) && victim->master != NULL )
	victim = victim->master;

    /*
     * NPC's are fair game.
     * So are killers and thieves.
     */
    if ( IS_NPC(victim)
    ||   is_killer(victim)
    ||   is_thief(victim)
    ||   victim->pcdata->heresy >= HERESY_CURSED
    ||   victim->pcdata->social_classes[victim->in_room->area->territory] <= 1 )
	return;

    /*
     * Charm-o-rama.
     */
    if ( IS_SET(ch->affected_by, AFF_CHARM) )
    {
	if ( ch->master == NULL )
	{
	    char buf[MAX_STRING_LENGTH];

	    sprintf( buf, "Check_killer: %s bad AFF_CHARM",
		IS_NPC(ch) ? ch->short_descr : ch->name );
	    bug( buf, 0 );
	    affect_strip( ch, gsn_charm_person );
	    REMOVE_BIT( ch->affected_by, AFF_CHARM );
	    return;
	}
/*
	send_to_char( "*** You are now a KILLER!! ***\n\r", ch->master );
  	SET_BIT(ch->master->act, PLR_KILLER);
*/

	stop_follower( ch );
	return;
    }

    /*
     * NPC's are cool of course (as long as not charmed).
     * Hitting yourself is cool too (bleeding).
     * So is being immortal (Alander's idea).
     * And current killers stay as they are.
     */
    if ( IS_NPC(ch)
    ||   ch == victim
    ||   ch->level >= LEVEL_IMMORTAL
    /*||   !is_clan(ch)*/
    ||   ch->fighting == victim)
	return;

    if ( !is_killer(ch) )
	send_to_char( "*** You are now a KILLER!! ***\n\r", ch );
    make_criminal( ch, CRIME_PKILLING, victim );
    /*SET_BIT(ch->act, PLR_KILLER);*/
    
    sprintf( buf, "$N is attempting to murder %s", victim->name );
    wiznet( buf, ch, NULL,WIZ_FLAGS, 0, 0 );
    save_char_obj( ch, FALSE );
    set_fighting( ch, victim, TRUE );
    return;
}



/*
 * Check for parry.
 */
bool check_parry( CHAR_DATA *ch, OBJ_DATA *wield, CHAR_DATA *victim )
{
    static int recursion_level = 0;
    
    int defender_bonus = 0, defensive_dual = 0, blind_bonus = 0, known_weapon = 0, martial_bonus = 0;
    OBJ_DATA *defending_weapon, *secondary_weapon;
    bool will_riposte;
    bool martial;
    int chance;
    int roll;
    bool hth;

    if ( !IS_AWAKE(victim)
    ||   IS_AFFECTED(victim, AFF_PARALYSIS)
    ||   recursion_level != 0 )
	return FALSE;

    chance = get_skill(victim, gsn_parry) / 2;

    defending_weapon = get_eq_char( victim, WEAR_WIELD );
    secondary_weapon = get_eq_char( victim, WEAR_SECONDARY );
    if ( defending_weapon == NULL )
	defending_weapon = secondary_weapon;

    hth = get_skill( victim, gsn_hand_to_hand ) > 0;

    if ( defending_weapon == NULL )
    {
	if ( (wield != NULL
	&&    (!IS_NPC(victim)
	||     !IS_SET(victim->off_flags, OFF_PARRY)))
	||    !hth )
	    return FALSE;
    }
else if ( IS_WEAPON_STAT(defending_weapon, WEAPON_DEFENDER) || (secondary_weapon && IS_WEAPON_STAT(secondary_weapon, WEAPON_DEFENDER)) )
    {
	chance         += 10;
        defender_bonus  = 10;
    }
	
    if ( !IS_NPC(victim)
    &&   defending_weapon != secondary_weapon
    &&   secondary_weapon != NULL )
    {
	will_riposte = FALSE;
	
	if ( IS_SET(victim->pcdata->style, STYLE_DUAL_RIPOSTE) )
	    will_riposte = TRUE;
	else if ( !IS_SET(victim->pcdata->style, STYLE_DUAL_OFFENSIVE) )
        {
	    chance         += 10;
            defensive_dual  = 10;
        }
    }
    else
        will_riposte = FALSE;

    martial = martial_artist( victim );

    if ( !can_see(victim, ch) )
    {
	int bf = get_skill( victim, gsn_blind_fighting ) +
                 per_app[get_curr_stat(victim, STAT_PER)/4].defense_bonus * 2;
	bf = URANGE( 5, bf, 95 );

	if ( !IS_NPC(victim) && number_percent() < bf )
	{
            blind_bonus    = chance / 4;
            defender_bonus = defender_bonus * 3 / 4;
            defensive_dual = defensive_dual * 3 / 4;
	    chance         = chance * 3 / 4;
	    
	    check_improve( victim, gsn_blind_fighting, TRUE, 10, TRUE );
	}
	else
        {
	    chance         /= 2;
            defender_bonus /= 2;
            defensive_dual /= 2;
        }
    }
    else if ( wield != NULL )
    {
	int sn;
	
	if ( wield->wear_loc == WEAR_WIELD )
	    sn = get_weapon_sn( ch );
	else
	    sn = get_secondary_sn( ch );
	
	if ( sn != -1 && get_skill(victim, sn) > 0 )
        {
	    chance       += 10;
            known_weapon  = 10;
        }
    }
    else if ( get_skill(victim, gsn_hand_to_hand) > 0
    &&        martial && !martial_artist(ch)
    &&        CHAR_SIZE(victim) >= CHAR_SIZE(ch) )
    {
	chance        += 5;
        martial_bonus  = 5;
    }

    if ( wield != NULL )
    {
	if ( wield->value[0] == WEAPON_WHIP )
        {
	     chance         /= 2;
             defender_bonus /= 2;
             defensive_dual /= 2;
             blind_bonus    /= 2;
             known_weapon   /= 2;
             martial_bonus  /= 2;
        }
	if ( wield->value[0] == WEAPON_FLAIL )
        {
	     chance         = chance * 3/4;
             defender_bonus = defender_bonus * 3/4;
             defensive_dual = defensive_dual * 3/4;
             blind_bonus    = blind_bonus * 3/4;
             known_weapon   = known_weapon * 3/4;
             martial_bonus  = martial_bonus * 3/4;
        }
	 
	if ( defending_weapon != NULL
	&&   defending_weapon->value[0] == WEAPON_WHIP )
        {
	     chance         = chance * 3/4;
             defender_bonus = defender_bonus * 3/4;
             defensive_dual = defensive_dual * 3/4;
             blind_bonus    = blind_bonus * 3/4;
             known_weapon   = known_weapon * 3/4;
             martial_bonus  = martial_bonus * 3/4;
        }

        if ( wield->wear_loc == WEAR_SECONDARY
        &&   COMPLEX_STYLE(ch, STYLE_RAINBOW) )
        {
            chance         /= 2;
            defender_bonus /= 2;
            defensive_dual /= 2;
            blind_bonus    /= 2;
            known_weapon   /= 2;
            martial_bonus  /= 2;
        }
    }

    if ( victim->fighting != NULL 
    &&   victim->fighting != ch
    &&   !martial
    &&   (!IS_NPC(victim)
    ||    !IS_SET(victim->off_flags, OFF_AREA_ATTACK))
    &&   (IS_NPC(victim)
    ||    !per_app[get_curr_stat(victim, STAT_PER)/4].secondary_defense) )
    {
        if ( martial )
            martial_bonus += chance / 3;
        else
        {
	     chance         = chance * 2/3;
             defender_bonus = defender_bonus * 2/3;
             defensive_dual = defensive_dual * 2/3;
             blind_bonus    = blind_bonus * 2/3;
             known_weapon   = known_weapon * 2/3;
             martial_bonus  = martial_bonus * 2/3;
        }
    }

    roll = number_percent() - chance - effective_level(victim, gsn_parry) + ch->level;
    if ( roll >= 0 )
	return FALSE;

    if ( wield != NULL
    &&   IS_WEAPON_STAT(wield, WEAPON_FADING)
    &&   (defending_weapon == NULL
    ||    !IS_WEAPON_STAT(defending_weapon, WEAPON_DEFENDER)) )
    {
	act_color( "$p passes straight through your attempt to parry!",
	     victim, wield, NULL, TO_CHAR, POS_RESTING, OTHER_COLOR(ch) );
	act_color( "$p phases through $n's weapon!",
	     victim, wield, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );
	return FALSE;
    }

    if ( will_riposte && number_bits(1) == 0 )
    {
	act_color( "You block $ns attack and strike back!", ch, NULL, victim, TO_VICT, POS_RESTING, C_COMBAT_SELF );
	act_color( "$N blocks your attack and counters!", ch, NULL, victim, TO_CHAR, POS_RESTING, OTHER_COLOR(ch) );
	
	recursion_level++;
	check_improve( ch, gsn_dual_wield, 
	    one_hit( victim, ch, TYPE_UNDEFINED, ATT_RIPOSTE ),
	    6, TRUE );
	recursion_level--;
	check_improve( victim, gsn_parry, TRUE, 6, TRUE );
    }
    else
    {
        if ( (roll += defender_bonus) >= 0 )
        {
            act_color( "$p comes alive in your hand, twisting to block $ns attack.", ch, defending_weapon, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
            act_color( "$p twists in $Ns hand to block your attack!", ch, defending_weapon, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
        }
        else if ( (roll += defensive_dual) >= 0 )
        {
            act_color( "Using your two weapons in combination you turn aside $ns attack with ease.", ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
            act_color( "Using $S two weapons in combination, $N turns aside your attack.", ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	    check_improve( victim, gsn_parry, TRUE, 6, TRUE );
	    check_improve( victim, gsn_dual_wield, TRUE, 6, TRUE );
        }
        else if ( (roll += known_weapon) >= 0 )
        {
            if ( number_bits(1) == 0 )
              act_color( "$n swings $s weapon at you using a move you know well. You block it easily.", ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
            else
              act_color( "Your weapons training kicks in at the last moment, knocking aside $ns attack.", ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
            act_color( "$N parries your attack easily, almost as if $E knows your every move before you make them.", ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	    check_improve( victim, gsn_parry, TRUE, 6, TRUE );
        }
        else if ( (roll += blind_bonus) >= 0 )
        {
            if ( number_bits(1) == 0 )
              act_color( "A whisper of noise alerts you to $ns attack, allowing you to knock it aside at the last moment.", ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
            else
              act_color( "A momentary silence and a brush of air alert you to $ns attack. You block it easily.", ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	    act_color( "$N parries your attack.", ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
            check_improve( victim, gsn_blind_fighting, TRUE, 6, TRUE );
	    check_improve( victim, gsn_parry, TRUE, 6, TRUE );
        }
        else if ( (roll += martial_bonus) >= 0 )
        {
            act_color( "Without thought you slip into a defensive kata, knocking aside $ns clumsy attack.", ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
            act_color( "$N knocks your wrist as you start your attack, sending your swing wide.", ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	    check_improve( victim, gsn_parry, TRUE, 6, TRUE );
        }
        else
        {
	    act_color( "You parry $n's attack.",  ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	    act_color( "$N parries your attack.", ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	    check_improve( victim, gsn_parry, TRUE, 6, TRUE );
        }
    }
    
    return TRUE;
}

/*
 * Check for shield block.
 */
bool check_shield_block( CHAR_DATA *ch, OBJ_DATA *wield, CHAR_DATA *victim )
{
    int spec_bonus = 0;
    OBJ_DATA *shield;
    int chance;
    int roll;

    if ( !IS_AWAKE(victim) || IS_AFFECTED(victim, AFF_PARALYSIS) )
        return FALSE;
    
    if ( wield != NULL 
    &&   wield->value[0] == WEAPON_FLAIL )
	return FALSE;

    chance = get_skill(victim, gsn_shield_block) / 4;

    if ( (shield = get_eq_char(victim, WEAR_SHIELD)) == NULL )
        return FALSE;

    if ( has_specialization(victim, gsn_single_weapon_combat) )
    {
	chance     += 10;
        spec_bonus  = 10;
    }

    if (victim->fighting != NULL && victim->fighting != ch
    &&  ((IS_NPC(victim) && !IS_SET(victim->off_flags, OFF_AREA_ATTACK))
    ||   !IS_NPC(victim)) )
	chance = chance * 2/3;
	
    if ( wield != NULL
    &&   wield->value[0] == WEAPON_WHIP )
	chance /= 2;

    roll = number_percent() - chance - effective_level(victim, gsn_shield_block) + ch->level;
    if ( roll >= 0 )
        return FALSE;

    if ( wield != NULL
    &&   wield->item_type == ITEM_WEAPON
    &&   wield->value[0] == WEAPON_AXE )
    {
	int break_chance;
	
	break_chance = UMAX( 1, wield->level - shield->level );
	if ( IS_OBJ_STAT(wield, ITEM_MAGIC) )
	    break_chance *= 2;
	if ( IS_OBJ_STAT(shield, ITEM_MAGIC) )
	    break_chance /= 2;
	
	if ( number_percent() <= break_chance )
	{
	    /*shield->condition -= number_range( 1, 4 );*/
	    if ( number_bits(3) == 0/*shield->condition <= 0*/ )
	    {
	        act_color( "$N's powerful blow shatters $p!",
	            ch, shield, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	        act_color( "Your powerful blow shatters $p!",
	            ch, shield, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	        act_color( "$n's powerful blow shatters $p!",
	            ch, shield, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
	        
	        extract_obj( shield );
	    }
	    else
	    {
		act_color( "$N's powerful blow slightly splinters $p.",
		    ch, shield, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
		act_color( "$N blocks your attack with a shield.", 
		    ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	    }
	}
	else
	{
            if ( (roll += spec_bonus) >= 0 )
	        act_color( "Without thinking you duck behind your shield, using it to block $n's attack.",
		    ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
            else
	        act_color( "You block $n's attack with your shield.",
		    ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	    act_color( "$N blocks your attack with a shield.", 
		ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	}
    }
    else
    {
        if ( (roll += spec_bonus) >= 0 )
	    act_color( "Without thinking you duck behind your shield, using it to block $n's attack.",
		ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
        else
	    act_color( "You block $n's attack with your shield.",
		ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	act_color( "$N blocks your attack with a shield.", 
	    ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
    }

    check_improve(victim,gsn_shield_block,TRUE,6,TRUE);
    return TRUE;
}


/*
 * Check for dodge.
 */
bool check_dodge( CHAR_DATA *ch, CHAR_DATA *victim )
{
    int martial_bonus = 0, spec_bonus = 0, blind_bonus = 0;
    bool martial;
    int chance;
    int roll;

    if ( !IS_AWAKE(victim) || IS_AFFECTED(victim, AFF_PARALYSIS) )
	return FALSE;

    chance = get_skill(victim, gsn_dodge) / 2;
    
    martial = martial_artist( victim );

    if ( (martial = martial_artist(victim)) )
    {
	chance       += 20;
        martial_bonus = 20;
    }

    chance -= dex_app[get_curr_stat(victim, STAT_DEX)/10].defensive / 10;

    if ( has_specialization(victim, gsn_single_weapon_combat)
    &&   get_eq_char(victim, WEAR_WIELD) != NULL
    &&   get_eq_char(victim, WEAR_SECONDARY) == NULL
    &&   get_eq_char(victim, WEAR_HOLD) == NULL
    &&   get_eq_char(victim, WEAR_SHIELD) == NULL )
    {
	chance    += 15;
        spec_bonus = 15;
    }
    
    if ( !can_see(victim, ch) )
    {
	int bf = get_skill( victim, gsn_blind_fighting ) +
                 per_app[get_curr_stat(victim, STAT_PER)/4].defense_bonus * 2;

	if ( !IS_NPC(victim) && number_percent() < bf )
	{
            blind_bonus   = chance / 4;
            martial_bonus = martial_bonus * 3/4;
            spec_bonus    = spec_bonus * 3/4;
	    chance        = chance * 3/4;
	    
	    check_improve( victim, gsn_blind_fighting, TRUE, 10, TRUE );
	}
	else
        {
	    chance        /= 2;
            martial_bonus /= 2;
            spec_bonus    /= 2;
        }
    }

    if ( victim->fighting != NULL
    &&   victim->fighting != ch
    &&   (!IS_NPC(victim) 
    ||    !IS_SET(victim->off_flags, OFF_AREA_ATTACK))
    &&   (IS_NPC(victim)
    ||    !per_app[get_curr_stat(victim, STAT_PER)/4].secondary_defense) )
    {
        if ( martial )
            martial_bonus += chance / 3;
        else
        {
	    chance      = chance * 2/3;
            spec_bonus  = spec_bonus * 2/3;
            blind_bonus = blind_bonus * 2/3;
        }
    }

    roll = number_percent() - chance - effective_level(victim, gsn_dodge) + ch->level;
    if ( roll >= 0 )
        return FALSE;

    if ( (roll += martial_bonus) >= 0 || (roll += spec_bonus) >= 0 )
    {
        act_color( "Without thought you spin effortlessly to the side, dodging $ns clumsy attack.", ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
        act_color( "$N spins to the side, dodging your attack.", ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
    }
    else if ( (roll += blind_bonus) >= 0 )
    {
        if ( number_bits(1) == 0 )
          act_color( "A whisper of noise alerts you to $ns attack, allowing you to tumble out of the way at the last moment.", ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
        else
          act_color( "A momentary silence and a brush of air alert you to $ns attack. You leap to the side and avoid it.", ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
        check_improve( victim, gsn_blind_fighting, TRUE, 6, TRUE );
    }
    else
    {
        act_color( "You dodge $n's attack.", ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
        act_color( "$N dodges your attack.", ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
    }
    check_improve(victim,gsn_dodge,TRUE,6,TRUE);
    return TRUE;
}


bool check_fade( CHAR_DATA *ch, OBJ_DATA *wield, CHAR_DATA *victim )
{
    if ( !IS_NPC(victim) 
    ||   IS_NPC(ch)
    ||   !IS_SET(victim->off_flags, OFF_FADE) )
	return FALSE;

    if ( wield == NULL || wield->item_type != ITEM_WEAPON )
    {
	const char *noun = attack_noun( CHAR_DAM_TYPE(ch), ch, victim );
	
	act_color( "$n's $t passes harmlessly through your body!",
	    ch, noun, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	act_color( "Your $t passes harmlessly through $N's body!",
	    ch, noun, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	
	return TRUE;
    }

    if ( IS_OBJ_STAT(wield, ITEM_MAGIC)
    ||   (IS_OBJ_STAT(wield, ITEM_BLESS) && IS_EVIL(victim))
    ||   (IS_OBJ_STAT(wield, ITEM_EVIL)  && IS_GOOD(victim))
    ||   (IS_WEAPON_STAT(wield, WEAPON_FLAMING) 
    &&    IS_SET(CHAR_VULN(victim), VULN_FIRE))
    ||   (IS_WEAPON_STAT(wield, WEAPON_FROST)
    &&    IS_SET(CHAR_VULN(victim), VULN_COLD))
    ||   (IS_WEAPON_STAT(wield, WEAPON_SHOCKING)
    &&    IS_SET(CHAR_VULN(victim), VULN_LIGHTNING))
    ||   (IS_WEAPON_STAT(wield, WEAPON_POISON)
    &&    IS_SET(CHAR_VULN(victim), VULN_POISON)) )
	return FALSE;

    act_color( "$p passes harmlessly through your body!",
	ch, wield, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
    act_color( "$p passes harmlessly through $N's body!",
	ch, wield, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
    return TRUE;
}


bool check_mirror_image( CHAR_DATA *ch, OBJ_DATA *wield, CHAR_DATA *victim )
{
    if ( IS_AFFECTED2(victim, AFF2_MIRROR_IMAGE)
    &&   !IS_AFFECTED2(ch, AFF2_DETECT_ILLUSION) )
    {
	AFFECT_DATA *paf = affect_find( victim->affected, gsn_mirror_image );
	
	if ( paf == NULL )
	    return FALSE;

	if ( !number_range(0, paf->modifier) )
	    return FALSE;
    
	if ( wield == NULL || wield->item_type != ITEM_WEAPON )
	{
	    const char *noun = attack_noun( CHAR_DAM_TYPE(ch), ch, victim );
	
	    act_color( "$n's $t passes harmlessly through your mirror image!",
		ch, noun, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	    act_color( "$N disappears in a flash of light as your $t passes harmlessly through $S body!",
		ch, noun, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	}
	else
	{
	    act_color( "$p passes harmlessly through your mirror image!",
		ch, wield, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	    act_color( "$N disappears in a flash of light as $p passes harmlessly through $M!",
		ch, wield, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	}

	if ( --paf->modifier == 0 )
	    affect_strip( victim, gsn_mirror_image );

	return TRUE;
    }

    return FALSE;
}


/*
 * Set position of a victim.
 */
void update_pos( CHAR_DATA *victim )
{
    if ( victim->hit > 0 )
    {
    	if ( victim->position <= POS_STUNNED )
	    victim->position = POS_STANDING;
	return;
    }

    if ( IS_NPC(victim) && victim->hit < 1 )
    {
	victim->position = POS_DEAD;
	return;
    }

    if ( victim->hit <= -11 )
    {
	victim->position = POS_DEAD;
	return;
    }

         if ( victim->hit <= -6 ) victim->position = POS_MORTAL;
    else if ( victim->hit <= -3 ) victim->position = POS_INCAP;
    else                          victim->position = POS_STUNNED;

    return;
}



/*
 * Start fights.
 */
void set_fighting( CHAR_DATA *ch, CHAR_DATA *victim, bool record )
{
/*
 *  if ( ch->fighting != NULL )
 *  {
 *	bug( "Set_fighting: already fighting", 0 );
 *	return;
 *  }
 */
	
    /* William - changed arg from victim to ch. */
    if ( IS_AFFECTED(victim, AFF_SLEEP) )
	affect_strip( victim, gsn_sleep );

    if ( ch->fighting == NULL )
    {
        ch->fighting = victim;
        ch->position = POS_FIGHTING;
    }
    music_to_char( "battle* L=1 C=1 T=music_combat", ch );
    music_to_char( "battle* L=1 C=1 T=music_combat", victim );

    if ( record && !IS_NPC(ch) && !IS_NPC(victim) )
    {
        bool found = FALSE;
        int i, slot = 0;
        for ( i = 0; i < MAX_PC_FIGHT_RECORD; ++i )
        {
            if ( ch->pcdata->fight_records[i] == victim->id )
                return;
            if ( ch->pcdata->fight_records[i] == 0 && !found )
            {
                found = TRUE;
                slot = i;
            }
        }

        if ( found )
            ch->pcdata->fight_records[slot] = victim->id;
        else
            bug( "set_fighting: fight records full for %s.", (long) ch->name );
    }

    return;
}



/*
 * Stop fights.
 */
void stop_fighting( CHAR_DATA *ch, bool fBoth )
{
    CHAR_DATA *fch;

    for ( fch = char_list; fch != NULL; fch = fch->next )
    {
	if ( fch == ch || ( fBoth && fch->fighting == ch ) )
	{
	    fch->fighting	= NULL;
	    fch->position	= IS_NPC(fch) ? fch->default_pos : POS_STANDING;
	    update_pos( fch );

            music_to_char( "off", fch );

            if ( !IS_NPC(fch) && !IS_NPC(ch) && ch != fch )
            {
                int i;
                for ( i = 0; i < MAX_PC_FIGHT_RECORD; ++i )
                {
                    if ( fch->pcdata->fight_records[i] == ch->id )
                        fch->pcdata->fight_records[i] = 0;
                }
            }
	}
    }

    return;
}



/*
 * Make a corpse out of a character.
 */
void make_corpse( CHAR_DATA *ch, int damage_type )
{
    char buf[MAX_STRING_LENGTH];
    OBJ_DATA *corpse; OBJ_DATA *obj; OBJ_DATA *obj_next; char *name; int	corpse_vnum; 
    if ( IS_SET(CHAR_FORM(ch), FORM_INSTANT_DECAY) )
    {
	for ( obj = ch->carrying; obj != NULL; obj = obj_next )
	{
	    obj_next = obj->next_content;

	    obj_from_char( obj );
	    
	    if ( obj->item_type == ITEM_POTION )
		obj->timer = number_range( 500, 1000 );
	    else if ( obj->item_type == ITEM_SCROLL )
		obj->timer = number_range( 1000, 2500 );

	    if ( IS_SET(obj->extra_flags, ITEM_ROT_DEATH) )
	    {
		obj->timer = number_range( 5, 10 );
		REMOVE_BIT( obj->extra_flags, ITEM_ROT_DEATH );
	    }
	    REMOVE_BIT( obj->extra_flags, ITEM_VIS_DEATH );

	    if ( IS_SET(obj->extra_flags, ITEM_INVENTORY) )
		extract_obj( obj );
	    else
	    {
		act( "$p falls to the floor.", ch, obj, NULL, TO_ROOM );
		obj_to_room( obj, ch->in_room );
	    }
	}

	return;
    }

    if ( IS_NPC(ch) )
    {
	name        = CHAR_SHORT(ch);
	corpse_vnum = OBJ_VNUM_CORPSE_NPC;

	switch (damage_type)
	{
		case DAM_BASH:
		case DAM_PIERCE:
		case DAM_SLASH:
			corpse_vnum = OBJ_VNUM_CORPSE_NPC_BASH;
			break;
		case DAM_FIRE:
			corpse_vnum = OBJ_VNUM_CORPSE_NPC_FIRE;
			break;
		case DAM_COLD:
			corpse_vnum = OBJ_VNUM_CORPSE_NPC_COLD;
			break;
		case DAM_LIGHTNING:
			corpse_vnum = OBJ_VNUM_CORPSE_NPC_LIGHTNING;
			break;
		case DAM_ACID:
			corpse_vnum = OBJ_VNUM_CORPSE_NPC_ACID;
			break;
		case DAM_POISON:
			corpse_vnum = OBJ_VNUM_CORPSE_NPC_POISON;
			break;
		case DAM_NEGATIVE:
			corpse_vnum = OBJ_VNUM_CORPSE_NPC_NEGATIVE;
			break;
		case DAM_HOLY:
			corpse_vnum = OBJ_VNUM_CORPSE_NPC_HOLY;
			break;
		case DAM_ENERGY:
		case DAM_LIGHT:
			corpse_vnum = OBJ_VNUM_CORPSE_NPC_ENERGY;
			break;
		case DAM_DISEASE:
			corpse_vnum = OBJ_VNUM_CORPSE_NPC_DISEASE;
			break;
		case DAM_DROWNING:
			corpse_vnum = OBJ_VNUM_CORPSE_NPC_DROWNING;
			break;
		case DAM_SLAY:
			corpse_vnum = OBJ_VNUM_CORPSE_NPC_SLAY;
			break;
		default:
			corpse_vnum = OBJ_VNUM_CORPSE_NPC;
			break;
	}

	corpse = create_object(get_obj_index(corpse_vnum), 0, FALSE);
	corpse->timer	= number_range( 3, 6 );
	if ( ch->gold > 0 )
	{
	    obj_to_obj( create_money( ch->gold, ch->silver ), corpse );
	    ch->gold = 0;
	    ch->silver = 0;
	}
	corpse->cost = 0;
    }
    else
    {
	name        = ch->polymorphed ? ch->polymorphed->short_descr : ch->name;
	corpse_vnum = OBJ_VNUM_CORPSE_PC;

	switch (damage_type)
	{
		case DAM_BASH:
		case DAM_PIERCE:
		case DAM_SLASH:
			corpse_vnum = OBJ_VNUM_CORPSE_PC_BASH;
			break;
		case DAM_FIRE:
			corpse_vnum = OBJ_VNUM_CORPSE_PC_FIRE;
			break;
		case DAM_COLD:
			corpse_vnum = OBJ_VNUM_CORPSE_PC_COLD;
			break;
		case DAM_LIGHTNING:
			corpse_vnum = OBJ_VNUM_CORPSE_PC_LIGHTNING;
			break;
		case DAM_ACID:
			corpse_vnum = OBJ_VNUM_CORPSE_PC_ACID;
			break;
		case DAM_POISON:
			corpse_vnum = OBJ_VNUM_CORPSE_PC_POISON;
			break;
		case DAM_NEGATIVE:
			corpse_vnum = OBJ_VNUM_CORPSE_PC_NEGATIVE;
			break;
		case DAM_HOLY:
			corpse_vnum = OBJ_VNUM_CORPSE_PC_HOLY;
			break;
		case DAM_ENERGY:
		case DAM_LIGHT:
			corpse_vnum = OBJ_VNUM_CORPSE_PC_ENERGY;
			break;
		case DAM_DISEASE:
			corpse_vnum = OBJ_VNUM_CORPSE_PC_DISEASE;
			break;
		case DAM_DROWNING:
			corpse_vnum = OBJ_VNUM_CORPSE_PC_DROWNING;
			break;
		case DAM_SLAY:
			corpse_vnum = OBJ_VNUM_CORPSE_PC_SLAY;
			break;
		default:
			corpse_vnum = OBJ_VNUM_CORPSE_PC;
			break;
	}

	corpse		= create_object(get_obj_index(corpse_vnum), 0, FALSE);
	corpse->timer	= TICKS_IN_HOUR * HOURS_IN_DAY * DAYS_IN_WEEK;
	                  /*number_range( 25, 40 );*/
	REMOVE_BIT(ch->act, PLR_CANLOOT);
	corpse->owner	= str_dup(ch->name);
/*
 *	if (!is_clan(ch))
 *	    corpse->owner = str_dup(ch->name);
 *	else
 *	{
 *	    corpse->owner = NULL;
 *	    if (ch->gold > 1 || ch->silver > 1)
 *	    {
 *		obj_to_obj(create_money(ch->gold / 2, ch->silver/2), corpse);
 *		ch->gold -= ch->gold/2;
 *		ch->silver -= ch->silver/2;
 *	    }
 *	}
 */		
	corpse->cost = 0;
    }

    corpse->level = ch->level;

    sprintf( buf, corpse->short_descr, name );
    free_string( corpse->short_descr );
    corpse->short_descr = str_dup( buf );

    sprintf( buf, corpse->description, name );
    free_string( corpse->description );
    corpse->description = str_dup( buf );

    for ( obj = ch->carrying; obj != NULL; obj = obj_next )
    {
	bool floating = FALSE;

	obj_next = obj->next_content;
	if (obj->wear_loc == WEAR_FLOAT)
	    floating = TRUE;
	obj_from_char( obj );
	if (obj->item_type == ITEM_POTION)
	    obj->timer = number_range(500,1000);
	if (obj->item_type == ITEM_SCROLL)
	    obj->timer = number_range(1000,2500);
	if (IS_SET(obj->extra_flags,ITEM_ROT_DEATH) && !floating)
	{
	    obj->timer = number_range(5,10);
	    REMOVE_BIT(obj->extra_flags,ITEM_ROT_DEATH);
	}
	REMOVE_BIT(obj->extra_flags,ITEM_VIS_DEATH);

	if ( IS_SET( obj->extra_flags, ITEM_INVENTORY ) )
	    extract_obj( obj );
	else if (floating)
	{
	    if (IS_OBJ_STAT(obj,ITEM_ROT_DEATH)) /* get rid of it! */
	    { 
		if (obj->contains != NULL)
		{
		    OBJ_DATA *in, *in_next;

		    act("$p evaporates,scattering its contents.",
			ch,obj,NULL,TO_ROOM);
		    for (in = obj->contains; in != NULL; in = in_next)
		    {
			in_next = in->next_content;
			obj_from_obj(in);
			obj_to_room(in,ch->in_room);
		    }
		 }
		 else
		    act("$p evaporates.",
			ch,obj,NULL,TO_ROOM);
		 extract_obj(obj);
	    }
	    else
	    {
		act("$p falls to the floor.",ch,obj,NULL,TO_ROOM);
		obj_to_room(obj,ch->in_room);
	    }
	}
	else
	    obj_to_obj( obj, corpse );
    }

    obj_to_room( corpse, ch->in_room );
    return;
}



/*
 * Improved Death_cry contributed by Diavolo.
 */
void death_cry( CHAR_DATA *ch, bool make_parts )
{
    ROOM_INDEX_DATA *was_in_room;
    char *msg;
    int door;

    msg = "You hear $n's death cry.";

    if ( IS_SET(CHAR_FORM(ch), FORM_INSTANT_DECAY) )
	msg = "$n cries out as $e fades from existence.";
    else if ( make_parts )
    {
	int vnum = 0;

	switch ( number_range(0, 8) )
	{
	    case  0:
		msg  = "$n hits the ground ... DEAD.";
		break;
	    case  1: 
		if ( ch->material == 0 )
		{
                    if ( CAN_BLEED(ch) )
                    {
                        if ( BLEEDS_ICHOR(ch) )
                            msg = "$n splatters yellow ooze on your armor.";
                        else
		            msg  = "$n splatters blood on your armor.";
                    }
		    break;
		}
	    case  2:
		if ( IS_SET(CHAR_PARTS(ch), PART_GUTS) )
		{
		    msg = "$n spills $s guts all over the floor.";
		    vnum = OBJ_VNUM_GUTS;
		}
		break;
	    case  3: 
		if ( IS_SET(CHAR_PARTS(ch), PART_HEAD) )
		{
		    msg  = "$n's severed head plops on the ground.";
		    vnum = OBJ_VNUM_SEVERED_HEAD;
		}
		break;
	    case  4: 
		if ( IS_SET(CHAR_PARTS(ch), PART_HEART) )
		{
		    msg  = "$n's heart is torn from $s chest.";
		    vnum = OBJ_VNUM_TORN_HEART;
		}
		break;
	    case  5: 
		if ( IS_SET(CHAR_PARTS(ch), PART_ARMS) )
		{
		    msg  = "$n's arm is sliced from $s dead body.";
		    vnum = OBJ_VNUM_SLICED_ARM;				
		}
		break;
	    case  6: 
		if ( IS_SET(CHAR_PARTS(ch), PART_LEGS) )
		{
		    msg  = "$n's leg is sliced from $s dead body.";
		    vnum = OBJ_VNUM_SLICED_LEG;				
		}
		break;
	    case 7:
		if ( IS_SET(CHAR_PARTS(ch), PART_BRAINS) )
		{
		    msg = "$n's head is shattered, and $s brains splash all over you.";
		    vnum = OBJ_VNUM_BRAINS;
		}
	    case 8:
		if ( IS_SET(CHAR_PARTS(ch), PART_WINGS) )
		{
		    msg  = "$n's wing is severed from $s body.";
		    vnum = OBJ_VNUM_WING;
		}
	}

	if ( vnum != 0 )
	{
	    char buf[MAX_STRING_LENGTH];
	    OBJ_DATA *obj;
	    char *name;

	    name	= ch->polymorphed 
                        ? ch->polymorphed->short_descr 
                        : (IS_NPC(ch) ? ch->short_descr : ch->name);
	    obj		= create_object( get_obj_index( vnum ), 0, FALSE );
	    obj->timer	= number_range( 4, 7 );

	    sprintf( buf, obj->short_descr, name );
	    free_string( obj->short_descr );
	    obj->short_descr = str_dup( buf );

	    sprintf( buf, obj->description, name );
	    free_string( obj->description );
	    obj->description = str_dup( buf );

	    if ( obj->item_type == ITEM_FOOD )
	    {
		if ( IS_SET(ch->form, FORM_POISON) )
		    obj->value[3] = 1;
		else if ( !IS_SET(ch->form, FORM_EDIBLE) )
		    obj->item_type = ITEM_TRASH;
	    }

	    obj_to_room( obj, ch->in_room );
	}
    }

    act( msg, ch, NULL, NULL, TO_ROOM );

    if ( IS_NPC(ch) )
	msg = "You hear something's death cry.";
    else
	msg = "You hear someone's death cry.";

    was_in_room = ch->in_room;
    for ( door = 0; door <= 11; door++ )
    {
	EXIT_DATA *pexit;

	if ( ( pexit = was_in_room->exit[door] ) != NULL
	&&   pexit->u1.to_room != NULL
	&&   pexit->u1.to_room != was_in_room )
	{
	    ch->in_room = pexit->u1.to_room;
	    act( msg, ch, NULL, NULL, TO_ROOM );
	}
    }
    ch->in_room = was_in_room;

    return;
}



void raw_kill( CHAR_DATA *victim, bool make_parts, int damage_type )
{
    CUST_DATA *cust;
    int i;

    stop_fighting( victim, TRUE );
    death_cry( victim, make_parts );

    if ( victim->mounted_by != NULL )
    {
        act( "$n dies suddenly, and you fall to the ground.",
             victim, NULL, victim->mounted_by, TO_VICT );
        victim->mounted_by->riding	= NULL;
        victim->mounted_by->position	= POS_RESTING;
        victim->mounted_by		= NULL;
    }

    if ( victim->riding != NULL )
    {
        act( "$n falls off you, dead.", 
             victim, NULL, victim->riding, TO_VICT );
        victim->riding->mounted_by	= NULL;
        victim->riding			= NULL;
    }

    make_corpse( victim, damage_type );

    if ( IS_NPC(victim) )
    {
	victim->pIndexData->killed++;
	kill_table[URANGE(0, victim->level, MAX_LEVEL-1)].killed++;
	extract_char( victim, TRUE );
	return;
    }

    extract_char( victim, FALSE );
    while ( victim->affected )
	affect_remove( victim, victim->affected );
    victim->affected_by	= race_table[victim->race].aff;
    victim->affected_by2= race_table[victim->race].aff2;
    victim->parts	= race_table[victim->race].parts;
    for (i = 0; i < 4; i++)
	victim->armor[i]= 100;
    victim->position	= POS_RESTING;
    victim->hit		= UMAX( 1, victim->hit  );
    victim->mana	= UMAX( 1, victim->mana );
    victim->move	= UMAX( 1, victim->move );
    free_string( victim->appearance );
    victim->appearance	= str_dup( "" );

    /* Add in any imm, res or vuln affects from customizations. */
    for ( cust = victim->pcdata->customizations;
          cust != NULL;
          cust = cust->next )
    {
        if ( cust->affect.where == TO_IMMUNE )
            victim->imm_flags |= cust->affect.bitvector;
        else if ( cust->affect.where == TO_RESIST )
            victim->res_flags |= cust->affect.bitvector;
        else
            victim->vuln_flags |= cust->affect.bitvector;
    }
/*  save_char_obj( victim ); we're stable enough to not need this :) */
    return;
}



void group_gain( CHAR_DATA *ch, CHAR_DATA *victim )
{
    char buf[MAX_STRING_LENGTH];
    CHAR_DATA *gch;
    CHAR_DATA *lch;
    int xp;
    int members;
    int max_level;
    int group_levels;

    /*
     * Monsters don't get kill xp's or alignment changes.
     * P-killing doesn't help either.
     * Dying of mortal wounds or poison doesn't give xp to anyone!
     */
    if ( victim == ch )
	return;
    
    members = 0;
    max_level = 0;
    group_levels = 0;
    for ( gch = ch->in_room->people; gch != NULL; gch = gch->next_in_room )
    {
	if ( is_same_group( gch, ch ) )
        {
            if ( !IS_NPC(gch) )
	        members++;
	    
	    if ( gch->mounted_by != NULL )
		group_levels += gch->level / 5;
	    else
		group_levels += gch->level;
		
	    max_level = UMAX( max_level, gch->level );
	}
    }

    if ( members == 0 )
    {
	bug( "Group_gain: members.", members );
	members = 1;
	group_levels = max_level = ch->level;
    }

    lch = (ch->leader != NULL) ? ch->leader : ch;

    for ( gch = ch->in_room->people; gch != NULL; gch = gch->next_in_room )
    {
/*
 *	OBJ_DATA *obj;
 *	OBJ_DATA *obj_next;
 */

	if ( !is_same_group( gch, ch ) || IS_NPC(gch))
	    continue;

/*	Taken out, add it back if you want it
	if ( gch->level - lch->level >= 5 )
	{
	    send_to_char( "You are too high for this group.\n\r", gch );
	    continue;
	}

	if ( gch->level - lch->level <= -5 )
	{
	    send_to_char( "You are too low for this group.\n\r", gch );
	    continue;
	}
*/

	xp = xp_compute( gch, victim, group_levels, max_level, members );  
	sprintf( buf, "You receive %d experience point%s.\n\r", xp, ((xp == 1) ? "" : "s"));
	send_to_char( buf, gch );
	gain_exp( gch, xp );
	
	if ( victim->level != group_levels )
	    adjust_kill_renown( ch, victim->level );

/*
 *	for ( obj = ch->carrying; obj != NULL; obj = obj_next )
 *	{
 *	    obj_next = obj->next_content;
 *	    if ( obj->wear_loc == WEAR_NONE )
 *		continue;
 *
 *	    if ( ( IS_OBJ_STAT(obj, ITEM_ANTI_EVIL)    && IS_EVIL(ch)    )
 *	    ||   ( IS_OBJ_STAT(obj, ITEM_ANTI_GOOD)    && IS_GOOD(ch)    )
 *	    ||   ( IS_OBJ_STAT(obj, ITEM_ANTI_NEUTRAL) && IS_NEUTRAL(ch) ) )
 *	    {
 *		act( "You are zapped by $p.", ch, obj, NULL, TO_CHAR );
 *		act( "$n is zapped by $p.",   ch, obj, NULL, TO_ROOM );
 *		obj_from_char( obj );
 *		obj_to_room( obj, ch->in_room );
 *	    }
 *	}
 */
    }

    return;
}



/*
 * Compute xp for a kill.
 * Also adjust alignment of killer.
 * Edit this function to change xp computations.
 */
int xp_compute( CHAR_DATA *gch, CHAR_DATA *victim, int total_levels, int max_level, int members )
{
    int level_range;
    int base_exp;
    int change;
    int align;
    int flag;
    int xp;
    
    if ( !IS_NPC(victim) || IS_AFFECTED(victim, AFF_PARALYSIS) )
	return 0;

    if ( IS_SET(victim->in_room->area->flags, AREA_ARENA) )
	return 0;

    if ( victim->pIndexData->area != NULL
    &&   IS_SET(victim->pIndexData->area->flags, AREA_VIRTUAL) )
	return 0;

    if ( victim->pIndexData->vnum == MOB_VNUM_SWARM
    ||   victim->pIndexData->vnum == MOB_VNUM_STONE_ELEMENTAL
    ||   victim->pIndexData->vnum ==  MOB_VNUM_DEMON_1
    ||   victim->pIndexData->vnum ==  MOB_VNUM_DEMON_2
    ||   victim->pIndexData->vnum ==  MOB_VNUM_DEMON_3
    ||   victim->pIndexData->vnum ==  MOB_VNUM_DEMON_4
    ||   victim->pIndexData->vnum ==  MOB_VNUM_DEMON_5
    ||   victim->pIndexData->vnum ==  MOB_VNUM_DEMON_6 )
	return 0;

/*    level_range = victim->level - gch->level; */
    /* 
     * Find the victim/attacker level difference. This is adjusted
     * for the difference in levels between the highest leveled member
     * of the group and the attacker so that low level group members
     * cannot coat tail on high level group members.
     */
    if ( gch->level == max_level )
        level_range = victim->level - max_level;
    else if ( gch->level >= max_level - 5 )
        level_range = victim->level - gch->level;
    else if ( gch->level >= max_level * 3 / 4 )
        level_range = victim->level - (max_level + gch->level) / 2;
    else
        level_range = victim->level - max_level;
 
    /* compute the base exp */
    switch (level_range)
    {
 	default : 	base_exp =   0;		break;
	case -9 :	base_exp =   1;		break;
	case -8 :	base_exp =   2;		break;
	case -7 :	base_exp =   5;		break;
	case -6 : 	base_exp =   9;		break;
	case -5 :	base_exp =  11;		break;
	case -4 :	base_exp =  22;		break;
	case -3 :	base_exp =  33;		break;
	case -2 :	base_exp =  50;		break;
	case -1 :	base_exp =  66;		break;
	case  0 :	base_exp =  83;		break;
	case  1 :	base_exp =  99;		break;
	case  2 :	base_exp = 121;		break;
	case  3 :	base_exp = 143;		break;
	case  4 :	base_exp = 165;		break;
    }
    
    if ( level_range > 4)
	base_exp = 160 + 20 * (level_range - 4);
    
    /* do alignment computations */
    
    align = victim->alignment - gch->alignment;

    if (IS_SET(victim->act,ACT_NOALIGN))
    {
	/* no change */
    }

    else if (align > 500) /* monster is more good than slayer */
    {
	change = (align - 500) * base_exp / 500 * gch->level/total_levels; 
        if ( has_customization(gch, ANCESTOR_YRNGRATH) ||
             has_customization(gch, ANCESTOR_ONDOVAL) )
            change *= 2;
	change = UMAX(1,change);
        gch->alignment = UMAX(-1000,gch->alignment - change);
    }

    else if (align < -500) /* monster is more evil than slayer */
    {
	change =  ( -1 * align - 500) * base_exp/500 * gch->level/total_levels;
	change = UMAX(1,change);
	gch->alignment = UMIN(1000,gch->alignment + change);
    }

    else /* improve this someday */
    {
	change =  gch->alignment * base_exp/500 * gch->level/total_levels; 
        if ( change > 0 
        &&   (has_customization(gch, ANCESTOR_YRNGRATH) ||
              has_customization(gch, ANCESTOR_ONDOVAL)) )
            change *= 2;
	gch->alignment -= change;
    }
    
    /* calculate exp multiplier */
    if (IS_SET(victim->act,ACT_NOALIGN))
	xp = base_exp;

    else if (gch->alignment > 500)  /* for goodie two shoes */
    {
	if (victim->alignment < -750)
	    xp = (base_exp *4)/3;
   
 	else if (victim->alignment < -500)
	    xp = (base_exp * 5)/4;

        else if (victim->alignment > 750)
	    xp = base_exp / 4;

   	else if (victim->alignment > 500)
	    xp = base_exp / 2;

        else if (victim->alignment > 250)
	    xp = (base_exp * 3)/4; 

	else
	    xp = base_exp;
    }

    else if (gch->alignment < -500) /* for baddies */
    {
	if (victim->alignment > 750)
	    xp = (base_exp * 5)/4;
	
  	else if (victim->alignment > 500)
	    xp = (base_exp * 11)/10; 

   	else if (victim->alignment < -750)
	    xp = base_exp/2;

	else if (victim->alignment < -500)
	    xp = (base_exp * 3)/4;

	else if (victim->alignment < -250)
	    xp = (base_exp * 9)/10;

	else
	    xp = base_exp;
    }

    else if (gch->alignment > 200)  /* a little good */
    {

	if (victim->alignment < -500)
	    xp = (base_exp * 6)/5;

 	else if (victim->alignment > 750)
	    xp = base_exp/2;

	else if (victim->alignment > 0)
	    xp = (base_exp * 3)/4; 
	
	else
	    xp = base_exp;
    }

    else if (gch->alignment < -200) /* a little bad */
    {
	if (victim->alignment > 500)
	    xp = (base_exp * 6)/5;
 
	else if (victim->alignment < -750)
	    xp = base_exp/2;

	else if (victim->alignment < 0)
	    xp = (base_exp * 3)/4;

	else
	    xp = base_exp;
    }

    else /* neutral */
    {

	if (victim->alignment > 500 || victim->alignment < -500)
	    xp = (base_exp * 4)/3;

	else if (victim->alignment < 200 && victim->alignment > -200)
	    xp = base_exp/2;

 	else
	    xp = base_exp;
    }

    /* more exp at the low levels */
    if (gch->level < 6)
    	xp = 10 * xp / (gch->level + 4);


    if ( base_exp )
    {
        int modification = 0;

	for ( flag = 0; act_table[flag].bit != 0; flag++ )
	    if ( IS_SET(victim->act, act_table[flag].bit) )
		modification += act_table[flag].modifier;

	if ( IS_SET(victim->form,FORM_SENTIENT) )
	     modification += 5;

	for ( flag = 0; off_table[flag].bit != 0; flag++ )
	    if ( IS_SET(victim->off_flags, off_table[flag].bit) )
		modification += off_table[flag].modifier;

	for ( flag = 0; aff_table[flag].bit != 0; flag++ )
	    if ( IS_AFFECTED(victim, aff_table[flag].bit) )
		modification += aff_table[flag].modifier;

        if ( victim->level > LEVEL_HERO )
            modification *= UMAX( 1, (victim->level - LEVEL_HERO) / 2 );
        xp += modification;
    }

    /* less at high */
    if (gch->level > 35 && gch->level < LEVEL_HERO )
	xp =  15 * xp / (gch->level - 25);

    /* reduce for playing time */

#if 0    
    {
	/* compute quarter-hours per level */
	time_per_level = 4 *
			 (gch->played + (int) (current_time - gch->logon))/3600
			 / gch->level;

	time_per_level = URANGE(2,time_per_level,12);
	if (gch->level < 15)  /* make it a curve */
	    time_per_level = UMAX(time_per_level,(15 - gch->level));
	xp = xp * time_per_level / 12;
  }
#endif   
    /* randomize the rewards */
    xp = number_range (xp * 3/4, xp * 5/4);

    /* adjust for grouping */
    xp = xp * gch->level / ( UMAX(1, total_levels - 1) );

    /* Adjust group XP upwards. */
    if ( level_range > 8 )
    {
        if ( members >= 4 )
            xp = xp * 140 / 100;
        else if ( members == 3 )
            xp = xp * 150 / 100;
        else if ( members == 2 )
            xp = xp * 130 / 100;
    }
    else if ( level_range > 5 )
    {
        if ( members >= 4 )
            xp = xp * 130 / 100;
        else if ( members == 3 )
            xp = xp * 140 / 100;
        else if ( members == 2 )
            xp = xp * 120 / 100;
    }
    else 
    {
        if ( members >= 4 )
            xp = xp * 120 / 100;
        else if ( members == 3 )
            xp = xp * 130 / 100;
        else if ( members == 2 )
            xp = xp * 110 / 100;
    }

    /* adjust for tanking */
    if ( victim->fighting != NULL && victim->fighting == gch )
	xp = xp * 11 / 10;
    
    if ( IS_AFFECTED2(gch, AFF2_HALF_XP) )
        xp /= 2;

    return xp;
}


void dam_message( CHAR_DATA *ch, CHAR_DATA *victim,int dam,int dt,bool immune )
{
    char buf1[256], buf2[256], buf3[256];
    const char *vs;
    const char *vp;
    const char *attack;
    char punct;

    if (ch == NULL || victim == NULL)
	return;

	 if ( dam ==   0 ) { vs = "miss";	vp = "misses";		}
    else if ( dam <=   4 ) { vs = "scratch";	vp = "scratches";	}
    else if ( dam <=   8 ) { vs = "graze";	vp = "grazes";		}
    else if ( dam <=  12 ) { vs = "hit";	vp = "hits";		}
    else if ( dam <=  16 ) { vs = "injure";	vp = "injures";		}
    else if ( dam <=  20 ) { vs = "wound";	vp = "wounds";		}
    else if ( dam <=  24 ) { vs = "maul";       vp = "mauls";		}
    else if ( dam <=  28 ) { vs = "decimate";	vp = "decimates";	}
    else if ( dam <=  32 ) { vs = "devastate";	vp = "devastates";	}
    else if ( dam <=  36 ) { vs = "maim";	vp = "maims";		}
    else if ( dam <=  40 ) { vs = "MUTILATE";	vp = "MUTILATES";	}
    else if ( dam <=  44 ) { vs = "DISEMBOWEL";	vp = "DISEMBOWELS";	}
    else if ( dam <=  48 ) { vs = "DISMEMBER";	vp = "DISMEMBERS";	}
    else if ( dam <=  52 ) { vs = "MASSACRE";	vp = "MASSACRES";	}
    else if ( dam <=  56 ) { vs = "MANGLE";	vp = "MANGLES";		}
    else if ( dam <=  60 ) { vs = "*** DEMOLISH ***";
			     vp = "*** DEMOLISHES ***";			}
    else if ( dam <=  75 ) { vs = "*** DEVASTATE ***";
			     vp = "*** DEVASTATES ***";			}
    else if ( dam <= 100)  { vs = "=== OBLITERATE ===";
			     vp = "=== OBLITERATES ===";		}
    else if ( dam <= 125)  { vs = ">>> ANNIHILATE <<<";
			     vp = ">>> ANNIHILATES <<<";		}
    else if ( dam <= 150)  { vs = "<<< ERADICATE >>>";
			     vp = "<<< ERADICATES >>>";			}
    else                   { vs = "do UNSPEAKABLE things to";
			     vp = "does UNSPEAKABLE things to";		}

    punct   = (dam <= 24) ? '.' : '!';

    if ( dt == TYPE_HIT )
    {
	if (ch  == victim)
	{
	    sprintf( buf1, "$n %s $melf%c",vp,punct);
	    sprintf( buf2, "You %s yourself%c",vs,punct);
	}
	else
	{
	    sprintf( buf1, "$n %s $N%c",  vp, punct );
	    sprintf( buf2, "You %s $N%c", vs, punct );
	    sprintf( buf3, "$n %s you%c", vp, punct );
	}
    }
    else
    {
	if ( dt >= 0 && dt < MAX_SKILL )
	    attack	= skill_table[dt].noun_damage;
	else if ( dt >= TYPE_HIT
	&& dt < TYPE_HIT + MAX_DAMAGE_MESSAGE) 
	    attack	= attack_noun( dt - TYPE_HIT, ch, victim );
	else
	{
	    bug( "Dam_message: bad dt %d.", dt );
	    dt  = TYPE_HIT;
	    attack  = attack_table[0].name;
	}

	if (immune)
	{
	    if (ch == victim)
	    {
		sprintf(buf1,"$n is unaffected by $s own %s.",attack);
		sprintf(buf2,"Luckily, you are immune to that.");
	    } 
	    else
	    {
	    	sprintf(buf1,"$N is unaffected by $n's %s!",attack);
	    	sprintf(buf2,"$N is unaffected by your %s!",attack);
	    	sprintf(buf3,"$n's %s is powerless against you.",attack);
	    }
	}
	else
	{
	    if (ch == victim)
	    {
		sprintf( buf1, "$n's %s %s $m%c",attack,vp,punct);
		sprintf( buf2, "Your %s %s you%c",attack,vp,punct);
	    }
	    else
	    {
	    	sprintf( buf1, "$n's %s %s $N%c",  attack, vp, punct );
	    	sprintf( buf2, "Your %s %s $N%c",  attack, vp, punct );
	    	sprintf( buf3, "$n's %s %s you%c", attack, vp, punct );
	    }
	}
    }

    if (ch == victim)
    {
	act_color( buf1, ch, NULL, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );
	act_color( buf2, ch, NULL, NULL, TO_CHAR, POS_RESTING, C_COMBAT_SELF);
    }
    else
    {
    	act_color( buf1, ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
    	act_color( buf2, ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
    	act_color( buf3, ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
    }

    return;
}



/*
 * Disarm a creature.
 * Caller must check for successful attack.
 */
void disarm( CHAR_DATA *ch, CHAR_DATA *victim )
{
    OBJ_DATA *obj;

    if ( (obj = get_eq_char(victim, WEAR_WIELD)) == NULL )
        if ( (obj = get_eq_char(victim, WEAR_SECONDARY)) == NULL )
	    return;

    if ( IS_OBJ_STAT(obj,ITEM_NOREMOVE))
    {
	act_color("$S weapon won't budge!",ch,NULL,victim,TO_CHAR,POS_RESTING,C_COMBAT_SELF);
	act_color("$n tries to disarm you, but your weapon won't budge!",
	    ch,NULL,victim,TO_VICT,POS_RESTING,OTHER_COLOR(ch));
	act_color("$n tries to disarm $N, but fails.",
            ch,NULL,victim,TO_NOTVICT,POS_RESTING,OTHER_COLOR(ch));
	return;
    }

    act_color( "$n DISARMS you and sends your weapon flying!", 
	 ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
    act_color( "You disarm $N!",
        ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF);
    act_color( "$n disarms $N!",
        ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );

    obj_from_char( obj );
    if ( IS_OBJ_STAT(obj,ITEM_NODROP) || IS_OBJ_STAT(obj,ITEM_INVENTORY) )
	obj_to_char( obj, victim );
    else
    {
	obj_to_room( obj, victim->in_room );
	if (IS_NPC(victim) && victim->wait == 0 && can_see_obj(victim,obj))
	    get_obj(victim,obj,NULL);
    }

    return;
}

void do_berserk( CHAR_DATA *ch, char *argument)
{
    int chance, hp_percent;

    if ((chance = get_skill(ch,gsn_berserk)) == 0
    ||  (IS_NPC(ch) && !IS_SET(ch->off_flags,OFF_BERSERK))
    ||  (!IS_NPC(ch)
    &&   ch->level < skill_level(ch, gsn_berserk)))
    {
	send_to_char("You turn red in the face, but nothing happens.\n\r",ch);
	return;
    }

    if (IS_AFFECTED(ch,AFF_BERSERK) || is_affected(ch,gsn_berserk)
    ||  is_affected(ch,skill_lookup("frenzy")))
    {
	send_to_char("You get a little madder.\n\r",ch);
	return;
    }

    if (IS_AFFECTED(ch,AFF_CALM))
    {
	send_to_char("You're feeling to mellow to berserk.\n\r",ch);
	return;
    }

    if (ch->mana < 50)
    {
	send_to_char("You can't get up enough energy.\n\r",ch);
	return;
    }

    /* modifiers */

    /* fighting */
    if (ch->position == POS_FIGHTING)
	chance += 10;

    /* damage -- below 50% of hp helps, above hurts */
    hp_percent = 100 * ch->hit/ch->max_hit;
    chance += 25 - hp_percent/2;

    if (number_percent() < chance)
    {
	AFFECT_DATA af;

	WAIT_STATE(ch,PULSE_VIOLENCE);
	ch->mana -= 50;
	ch->move /= 2;

	/* heal a little damage */
	ch->hit += effective_level(ch, gsn_berserk) * 2;
	ch->hit = UMIN(ch->hit,ch->max_hit);

	act_color("Your pulse races as you are consumed by rage!",
            ch, NULL, NULL, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	act_color("$n gets a wild look in $s eyes.",
            ch, NULL, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );
	check_improve(ch,gsn_berserk,TRUE,2,TRUE);

	af.where	= TO_AFFECTS;
	af.type		= gsn_berserk;
	af.level	= effective_level(ch, gsn_berserk);
	af.duration	= number_fuzzy(ch->level / 8);
	af.modifier	= UMAX(1,effective_level(ch, gsn_berserk)/5);
	af.bitvector 	= AFF_BERSERK;
	af.procNo	= 0;
	af.flags	= 0;
	af.name		= "";
	af.appearance	= "";

	af.location	= APPLY_HITROLL;
	affect_to_char(ch,&af);

	af.location	= APPLY_DAMROLL;
	affect_to_char(ch,&af);

	af.modifier	= UMAX(10,10 * (effective_level(ch, gsn_berserk)/5));
	af.location	= APPLY_AC;
	affect_to_char(ch,&af);

        act_sound( "berserk* V=100", ch, NULL, TO_ALL );
    }

    else
    {
	WAIT_STATE(ch,3 * PULSE_VIOLENCE);
	ch->mana -= 25;
	ch->move /= 2;

	act_color( "Your pulse speeds up, but nothing happens.",
            ch, NULL, NULL, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	check_improve(ch,gsn_berserk,FALSE,2,TRUE);
    }
}

void do_bash( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    bool shield_bonus;
    int chance;
    int roll;

    one_argument(argument,arg);
 
    if ( (chance = get_skill(ch,gsn_bash)) == 0
    ||   (IS_NPC(ch) && !IS_SET(ch->off_flags,OFF_BASH))
    ||   (!IS_NPC(ch)
    &&	  ch->level < skill_level(ch, gsn_bash)))
    {	
	send_to_char("Bashing? What's that?\n\r",ch);
	return;
    }

    if ( IS_AFFECTED2(ch, AFF2_PACIFIST) )
    {
	send_to_char( "You don't quite feel that violent.", ch );
	return;
    }

    if ( ch->riding != NULL )
    {
	send_to_char( "Not while you're riding, use trample instead.\n\r", ch );
	return;
    }
 
    if ( arg[0] == '\0' )
    {
	victim = ch->fighting;
	if ( victim == NULL )
	{
	    send_to_char( "But you aren't fighting anyone!\n\r", ch );
	    return;
	}
	
	if ( victim->riding != NULL )
	    victim = victim->riding;
    }
    else if ( (victim = get_char_room(ch, arg)) == NULL )
    {
	send_to_char( "They aren't here.\n\r", ch );
	return;
    }
    
    if ( victim->riding != NULL )
    {
	send_to_char( "You can't bash someone who's mounted!\n\r", ch );
	return;
    }

    if ( victim->position < POS_FIGHTING )
    {
	act("You'll have to let $M get back up first.",ch,NULL,victim,TO_CHAR);
	return;
    } 

    if (victim == ch)
    {
	send_to_char( "You try to bash your brains out, but fail.\n\r", ch );
	return;
    }

    if ( is_safe(ch,victim) )
	return;

    if ( IS_NPC(victim) && 
	victim->fighting != NULL && 
	!is_same_group(ch, victim->fighting))
    {
        send_to_char( "Kill stealing is not permitted.\n\r", ch );
        return;
    }

    if ( IS_AFFECTED(ch, AFF_CHARM) && ch->master == victim )
    {
	act( "But $N is your friend!", ch, NULL, victim, TO_CHAR );
	return;
    }

    /* modifiers */

    /* size  and weight */
    chance += ch->carry_weight / 250;
    chance -= victim->carry_weight / 200;

    if ( CHAR_SIZE(ch) < CHAR_SIZE(victim) )
	chance += (CHAR_SIZE(ch) - CHAR_SIZE(victim)) * 15;
    else
	chance += (CHAR_SIZE(ch) - CHAR_SIZE(victim)) * 10;
	
    if ( IS_SET(CHAR_FORM(victim), FORM_MIST) )
	chance -= 15;
    if ( IS_SET(CHAR_FORM(victim), FORM_INTANGIBLE) )
	chance -= 20;

    /* stats */
    chance += get_curr_stat(ch, STAT_STR) / 4;
    chance -= get_curr_stat(victim, STAT_DEX) / 3;
    chance -= get_ac(victim, AC_BASH) / 25;
    
    /* speed */
    if ( IS_SET(ch->off_flags, OFF_FAST) || IS_AFFECTED(ch, AFF_HASTE) )
        chance += 10;
    if ( IS_SET(victim->off_flags, OFF_FAST) || IS_AFFECTED(victim, AFF_HASTE) )
        chance -= 30;

    /* level */
    chance += (effective_level(ch, gsn_bash) - victim->level);

    /* shield and style */
    if ( get_eq_char(ch, WEAR_SHIELD) != NULL )
    {
	shield_bonus = 10;
	
	if ( has_specialization(ch, gsn_weapon_and_shield_combat) )
	    shield_bonus += 10;
    }
    else
	shield_bonus = 0;
    chance += shield_bonus;
    
    if ( !IS_NPC(victim) 
	 && chance < get_skill(victim, gsn_dodge) )
    {
/*
 *	act("$n tries to bash you, but you dodge it.",ch,NULL,victim,TO_VICT);
 *	act("$N dodges your bash, you fall flat on your face.",ch,NULL,victim,TO_CHAR);
 *	WAIT_STATE(ch,skill_table[gsn_bash].beats);
 *	return;
 */
	chance -= 3 * (get_skill(victim, gsn_dodge) - chance);
    }

    /* now the attack */
    if ( (roll = number_percent()) < chance )
    {
	if ( shield_bonus && roll >= chance - shield_bonus )
	{
	act_color( "$n slams $s shield into your face with stunning force!",
	    ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	act_color( "You slam your shield into $Ns face with stunning force!",
	    ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	act_color( "$n slams $s shield into $Ns face with stunning force!",
	    ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
	}
	else
	{
	act_color("$n sends you sprawling with a powerful bash!",
	    ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	act_color("You slam into $N, and send $M flying!",
            ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	act_color("$n sends $N sprawling with a powerful bash.",
	    ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
	}
	check_improve(ch,gsn_bash,TRUE,1,TRUE);

	DAZE_STATE(victim, 3 * PULSE_VIOLENCE);
	WAIT_STATE(ch,skill_table[gsn_bash].beats);
	victim->position = POS_RESTING;

	damage( ch, victim,
	        number_range( 2, 2 + 2 * ch->size + chance / 20 ),
	        gsn_bash, DAM_BASH, FALSE, NULL );
	if ( !CHAR_KILLED(victim) )
	    check_dismount( victim );
    }
    else
    {
	damage(ch,victim,0,gsn_bash,DAM_BASH,FALSE,NULL);
	act_color("You fall flat on your face!",
	    ch,NULL,victim,TO_CHAR,POS_RESTING,C_COMBAT_SELF);
	act_color("$n falls flat on $s face.",
	    ch,NULL,victim,TO_NOTVICT,POS_RESTING,OTHER_COLOR(ch));
	act_color("You evade $n's bash, causing $m to fall flat on $s face.",
	    ch,NULL,victim,TO_VICT,POS_RESTING,OTHER_COLOR(ch));
	check_improve(ch,gsn_bash,FALSE,1,TRUE);
	ch->position = POS_RESTING;
	WAIT_STATE(ch,skill_table[gsn_bash].beats * 3/2);

	check_dismount( ch );
    }
}

void do_dirt( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    int chance;

    one_argument(argument,arg);

    if ( (chance = get_skill(ch,gsn_dirt)) == 0
    ||   (IS_NPC(ch) && !IS_SET(ch->off_flags,OFF_KICK_DIRT))
    ||   (!IS_NPC(ch)
    &&    ch->level < skill_level(ch, gsn_dirt)))
    {
	send_to_char("You get your feet dirty.\n\r",ch);
	return;
    }

    if ( IS_AFFECTED2(ch, AFF2_PACIFIST) )
    {
	send_to_char( "You don't quite feel that violent.", ch );
	return;
    }

    if ( ch->riding != NULL )
    {
	send_to_char( "You can't do that while riding!\n\r", ch );
	return;
    }

    if (arg[0] == '\0')
    {
	victim = ch->fighting;
	if (victim == NULL)
	{
	    send_to_char("But you aren't in combat!\n\r",ch);
	    return;
	}
    }

    else if ((victim = get_char_room(ch,arg)) == NULL)
    {
	send_to_char("They aren't here.\n\r",ch);
	return;
    }

    if (IS_AFFECTED(victim,AFF_BLIND))
    {
	act("$E's already been blinded.",ch,NULL,victim,TO_CHAR);
	return;
    }
    
    if ( !IS_SET(CHAR_PARTS(victim), PART_EYE) )
    {
	act( "$E doesn't have any eyes to kick dirt into.",
	    ch, NULL, victim, TO_CHAR );
	return;
    }

    if (victim == ch)
    {
	send_to_char("Very funny.\n\r",ch);
	return;
    }

    if (is_safe(ch,victim))
	return;

    if (IS_NPC(victim) &&
	 victim->fighting != NULL && 
	!is_same_group(ch,victim->fighting))
    {
        send_to_char("Kill stealing is not permitted.\n\r",ch);
        return;
    }

    if (IS_AFFECTED(ch,AFF_CHARM) && ch->master == victim)
    {
	act("But $N is such a good friend!",ch,NULL,victim,TO_CHAR);
	return;
    }

    /* modifiers */

    /* dexterity */
    chance = chance + get_curr_stat(ch,STAT_DEX)/4;
    chance = chance - get_curr_stat(victim,STAT_DEX)/2;

    /* speed  */
    if (IS_SET(ch->off_flags,OFF_FAST) || IS_AFFECTED(ch,AFF_HASTE))
	chance += 10;
    if (IS_SET(victim->off_flags,OFF_FAST) || IS_AFFECTED(victim,AFF_HASTE))
	chance -= 25;

    /* level */
    chance += (effective_level(ch, gsn_dirt) - victim->level) * 2;

    /* sloppy hack to prevent false zeroes */
    if (chance % 5 == 0)
	chance += 1;

    /* terrain */

    switch(ch->in_room->sector_type)
    {
	case(SECT_INSIDE):		chance -= 20;	break;
	case(SECT_CITY):		chance -= 10;	break;
	case(SECT_FIELD):		chance +=  5;	break;
	case(SECT_FOREST):				break;
	case(SECT_HILLS):				break;
	case(SECT_MOUNTAIN):		chance -= 10;	break;
	case(SECT_WATER_SWIM):		chance  =  0;	break;
	case(SECT_WATER_NOSWIM):	chance  =  0;	break;
	case(SECT_AIR):			chance  =  0;  	break;
	case(SECT_DESERT):		chance += 10;   break;
    }

    if (chance == 0)
    {
	send_to_char("There isn't any dirt to kick.\n\r",ch);
	return;
    }

    /* now the attack */
    if (number_percent() < chance)
    {
	AFFECT_DATA af;
	act_color("$n is blinded by the dirt in $s eyes!",
            victim,NULL,NULL,TO_ROOM,POS_RESTING,OTHER_COLOR(ch));
	act_color("$n kicks dirt in your eyes!",
            ch,NULL,victim,TO_VICT,POS_RESTING,C_COMBAT_SELF);
	damage(ch,victim,number_range(2,5),gsn_dirt,DAM_NONE,FALSE,NULL);
	act_color("You can't see a thing!",
            victim,NULL,NULL,TO_CHAR,POS_RESTING,OTHER_COLOR(ch));
	check_improve(ch,gsn_dirt,TRUE,2,TRUE);
	WAIT_STATE(ch,skill_table[gsn_dirt].beats);

	af.where	= TO_AFFECTS;
	af.type 	= gsn_dirt;
	af.level 	= effective_level(ch, gsn_dirt);
	af.duration	= 0;
	af.location	= APPLY_HITROLL;
	af.modifier	= -4;
	af.bitvector 	= AFF_BLIND;
	af.procNo	= 0;
	af.flags	= 0;
	af.name		= "";
	af.appearance	= "";

	affect_to_char(victim,&af);
    }
    else
    {
	damage(ch,victim,0,gsn_dirt,DAM_NONE,TRUE,NULL);
	check_improve(ch,gsn_dirt,FALSE,2,TRUE);
	WAIT_STATE(ch,skill_table[gsn_dirt].beats);
    }
}

void do_trip( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    OBJ_DATA *wield;
    bool polearm;
    int chance;
    int roll;

    one_argument(argument,arg);

    if ( (chance = get_skill(ch,gsn_trip)) == 0
    ||   (IS_NPC(ch) && !IS_SET(ch->off_flags,OFF_TRIP))
    ||   (!IS_NPC(ch) 
    &&    ch->level < skill_level(ch, gsn_trip)) )
    {
	send_to_char("Tripping?  What's that?\n\r",ch);
	return;
    }

    if ( IS_AFFECTED2(ch, AFF2_PACIFIST) )
    {
	send_to_char( "You don't quite feel that violent.", ch );
	return;
    }

    if ( ch->riding != NULL )
    {
	send_to_char( "You can't do that while riding!\n\r", ch );
	return;
    }

    if ( arg[0] == '\0' )
    {
	victim = ch->fighting;
	if (victim == NULL)
	{
	    send_to_char("But you aren't fighting anyone!\n\r",ch);
	    return;
	}
	
	if ( victim->riding != NULL )
	    victim = victim->riding;
    }
    else if ( (victim = get_char_room(ch, arg)) == NULL )
    {
	send_to_char("They aren't here.\n\r",ch);
	return;
    }

    if ( victim->riding != NULL )
    {
	send_to_char( "You can't trip someone who's mounted!\n\r", ch );
	return;
    }

    if ( is_safe(ch, victim) )
	return;

    if (IS_NPC(victim) &&
	 victim->fighting != NULL && 
	!is_same_group(ch,victim->fighting))
    {
	send_to_char("Kill stealing is not permitted.\n\r",ch);
	return;
    }
    
    if (IS_AFFECTED(victim,AFF_FLYING))
    {
	act("$S feet aren't on the ground.",ch,NULL,victim,TO_CHAR);
	return;
    }
    
    if (victim->position < POS_FIGHTING)
    {
	act("$N is already down.",ch,NULL,victim,TO_CHAR);
	return;
    }

    if (victim == ch)
    {
	send_to_char("You fall flat on your face!\n\r",ch);
	WAIT_STATE(ch,2 * skill_table[gsn_trip].beats);
	act("$n trips over $s own feet!",ch,NULL,NULL,TO_ROOM);
	return;
    }

    if (IS_AFFECTED(ch,AFF_CHARM) && ch->master == victim)
    {
	act("$N is your beloved master.",ch,NULL,victim,TO_CHAR);
	return;
    }

    if ( IS_SET(CHAR_FORM(victim), FORM_WORM)
    ||   IS_SET(CHAR_FORM(victim), FORM_BLOB)
    ||   IS_SET(CHAR_FORM(victim), FORM_SNAKE)
    ||   IS_SET(CHAR_FORM(victim), FORM_FISH) )
    {
	act( "You can't even begin to work out how to trip $N!",
	    ch, NULL, victim, TO_CHAR );
	return;
    }

    /* modifiers */

    /* size */
    if (CHAR_SIZE(ch) < CHAR_SIZE(victim))
        chance += (CHAR_SIZE(ch) - CHAR_SIZE(victim)) * 10;  /* bigger = harder to trip */

    /* dex */
    chance = chance + get_curr_stat(ch,STAT_DEX)/4;
    chance = chance - get_curr_stat(victim,STAT_DEX) * 3 / 8;

    /* speed */
    if (IS_SET(ch->off_flags,OFF_FAST) || IS_AFFECTED(ch,AFF_HASTE))
	chance += 10;
    if (IS_SET(victim->off_flags,OFF_FAST) || IS_AFFECTED(victim,AFF_HASTE))
	chance -= 20;

    /* level */
    chance += (effective_level(ch, gsn_trip) - victim->level) * 2;

    /* form */
    if ( IS_SET(CHAR_FORM(victim), FORM_ANIMAL)
    ||   IS_SET(CHAR_FORM(victim), FORM_CENTAUR) )
	chance -= 20;
    if ( IS_SET(CHAR_FORM(victim), FORM_INSECT)
    ||   IS_SET(CHAR_FORM(victim), FORM_SPIDER) 
    ||   IS_SET(CHAR_FORM(victim), FORM_CRUSTACEAN) )
	chance -= 40;
    if ( IS_SET(CHAR_FORM(victim), FORM_MIST)
    ||   IS_SET(CHAR_FORM(victim), FORM_INTANGIBLE) )
	chance = 0;
	
    if ( martial_artist(ch) )
	chance += 10;
    else if ( martial_artist(victim) )
	chance -= 10;

    if ( (wield = get_eq_char(ch, WEAR_WIELD)) != NULL
    &&   (wield->value[0] == WEAPON_POLEARM 
    ||    wield->value[0] == WEAPON_SPEAR) )
    {
	chance += 10;
	polearm = TRUE;
    }
    else
	polearm = FALSE;

    /* now the attack */
    if ( (roll = number_percent()) < chance )
    {
	if ( polearm && roll >= chance - 10 )
	{
	act_color( "$n hooks $p around your ankles and pulls you off your feet!", 
            ch, wield, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	act_color( "You hook $p around $Ns ankles, pulling $M off $S feet!", 
            ch, wield, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	act_color( "$n hooks $s $p around $Ns ankles, pulling $M off $S feet!", 
            ch, wield, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
	}
	else
	{
	act_color("$n trips you and you go down!",
            ch,NULL,victim,TO_VICT, POS_RESTING, OTHER_COLOR(ch));
	act_color("You trip $N and $N goes down!",
            ch,NULL,victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF);
	act_color("$n trips $N, sending $M to the ground.",
            ch,NULL,victim,TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch));
	}
	check_improve(ch,gsn_trip,TRUE,1,TRUE);

	DAZE_STATE(victim,2 * PULSE_VIOLENCE);
        WAIT_STATE(ch,skill_table[gsn_trip].beats);
	victim->position = POS_RESTING;
	
	damage( ch,victim,
	        number_range( 2, 2 +  2 * victim->size ),
	        gsn_trip, DAM_BASH, TRUE, NULL );
	if ( !CHAR_KILLED(victim) )
	    check_dismount( victim );
    }
    else
    {
	if ( IS_SET(victim->form, FORM_MIST)
	||   IS_SET(victim->form, FORM_INTANGIBLE) )
	{
	    act_color( "You can't seem to make contact with $M!",
		ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	    damage(ch,victim,0,gsn_trip,DAM_BASH,FALSE,NULL);
	}
	else
	    damage(ch,victim,0,gsn_trip,DAM_BASH,TRUE,NULL);
	    
	WAIT_STATE(ch,skill_table[gsn_trip].beats*2/3);
	check_improve(ch,gsn_trip,FALSE,1,TRUE);
    }
}



void do_kill( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	send_to_char( "Kill whom?\n\r", ch );
	return;
    }

    if ( ( victim = get_char_room( ch, arg ) ) == NULL )
    {
	send_to_char( "They aren't here.\n\r", ch );
	return;
    }
/*  Allow player killing */
    if ( !IS_NPC(victim) )
    {
        if ( !is_criminal(victim, FALSE) )
        {
            send_to_char( "You must MURDER a player.\n\r", ch );
            return;
        }
    }
/**/
    if ( victim == ch )
    {
	send_to_char( "You hit yourself.  Ouch!\n\r", ch );
	multi_hit( ch, ch, TYPE_UNDEFINED );
	return;
    }

    if ( is_safe( ch, victim ) )
	return;

    if ( victim->fighting != NULL && 
	!is_same_group(ch,victim->fighting))
    {
        send_to_char("Kill stealing is not permitted.\n\r",ch);
        return;
    }

    if ( IS_AFFECTED(ch, AFF_CHARM) && ch->master == victim )
    {
	act( "$N is your beloved master.", ch, NULL, victim, TO_CHAR );
	return;
    }

    if ( IS_AFFECTED2(ch, AFF2_PACIFIST) )
    {
	send_to_char( "You don't quite feel that violent.", ch );
	return;
    }

    if ( ch->position == POS_FIGHTING )
    {
	send_to_char( "You do the best you can!\n\r", ch );
	return;
    }

    WAIT_STATE( ch, 1 * PULSE_VIOLENCE );
    check_killer( ch, victim );
    multi_hit( ch, victim, TYPE_UNDEFINED );
    return;
}



void do_murde( CHAR_DATA *ch, char *argument )
{
    send_to_char( "If you want to MURDER, spell it out.\n\r", ch );
    return;
}



void do_murder( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    bool no_shout = FALSE;
    CHAR_DATA *victim;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	send_to_char( "Murder whom?\n\r", ch );
	return;
    }

    if (IS_AFFECTED(ch,AFF_CHARM) || (IS_NPC(ch) && IS_SET(ch->act,ACT_PET)))
	return;

    if ( ( victim = get_char_room( ch, arg ) ) == NULL )
    {
	send_to_char( "They aren't here.\n\r", ch );
	return;
    }

    if ( victim == ch )
    {
	send_to_char( "Suicide is a mortal sin.\n\r", ch );
	return;
    }

    if ( is_safe( ch, victim ) )
	return;

    if (IS_NPC(victim) &&
	 victim->fighting != NULL && 
	!is_same_group(ch,victim->fighting))
    {
        send_to_char("Kill stealing is not permitted.\n\r",ch);
        return;
    }

    if ( IS_AFFECTED(ch, AFF_CHARM) && ch->master == victim )
    {
	act( "$N is your beloved master.", ch, NULL, victim, TO_CHAR );
	return;
    }

    if ( IS_AFFECTED2(ch, AFF2_PACIFIST) )
    {
	send_to_char( "You don't quite feel that violent.", ch );
	return;
    }

    if ( ch->position == POS_FIGHTING )
    {
	send_to_char( "You do the best you can!\n\r", ch );
	return;
    }

    WAIT_STATE( ch, 1 * PULSE_VIOLENCE );
    
    if ( IS_SET(CHAR_FORM(victim), FORM_SENTIENT) )
    {
	sprintf( buf, "Help! I am being attacked by %s!", PERS(ch, victim) );
	if ( IS_SET(victim->comm, COMM_NOSHOUT) )
	    no_shout = TRUE;
	REMOVE_BIT( victim->comm, COMM_NOSHOUT );
	do_yell( victim, buf );
	if ( no_shout )
	    SET_BIT( victim->comm, COMM_NOSHOUT );
    }
    
    check_killer( ch, victim );
    multi_hit( ch, victim, TYPE_UNDEFINED );
    return;
}



void do_backstab( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    OBJ_DATA *obj;

    one_argument( argument, arg );

    if (arg[0] == '\0')
    {
        send_to_char("Backstab whom?\n\r",ch);
        return;
    }

    if (ch->fighting != NULL)
    {
	send_to_char("You're facing the wrong end.\n\r",ch);
	return;
    }
    else if ( (victim = get_char_room(ch, arg)) == NULL )
    {
        send_to_char("They aren't here.\n\r",ch);
        return;
    }

    if ( victim == ch )
    {
	send_to_char( "How can you sneak up on yourself?\n\r", ch );
	return;
    }

    if ( ch->riding != NULL )
    {
	send_to_char( "You can't do that while riding!\n\r", ch );
	return;
    }

    if ( IS_AFFECTED2(ch, AFF2_PACIFIST) )
    {
	send_to_char( "You don't quite feel that violent.", ch );
	return;
    }
    
    if ( victim->riding != NULL
    &&   CHAR_SIZE(victim->riding) > CHAR_SIZE(ch)
    &&   !IS_AFFECTED(ch, AFF_FLYING) )
    {
	act( "You can't reach $S back while $Es mounted.",
	    ch, NULL, victim, TO_CHAR );
	return;
    }

    if ( is_safe( ch, victim ) )
      return;

    if (IS_NPC(victim) &&
	 victim->fighting != NULL && 
	!is_same_group(ch,victim->fighting))
    {
        send_to_char("Kill stealing is not permitted.\n\r",ch);
        return;
    }

    if ( ( obj = get_eq_char( ch, WEAR_WIELD ) ) == NULL)
    {
	send_to_char( "You need to wield a weapon to backstab.\n\r", ch );
	return;
    }

    if ( victim->hit < victim->max_hit / 3 && can_see(victim, ch) )
    {
	act( "$N is hurt and suspicious ... you can't sneak up.",
	    ch, NULL, victim, TO_CHAR );
	return;
    }

    if ( has_customization(victim, ADVANTAGE_DANGER_SENSE) )
    {
        act( "$N is far too alert for you to sneak up that close.",
             ch, NULL, victim, TO_CHAR );
        return;
    }

    check_killer( ch, victim );
    WAIT_STATE( ch, skill_table[gsn_backstab].beats );
    if ( number_percent( ) < get_skill(ch,gsn_backstab)
    || ( get_skill(ch,gsn_backstab) >= 2 && !IS_AWAKE(victim) ) )
    {
	check_improve(ch,gsn_backstab,TRUE,1,TRUE);
	multi_hit( ch, victim, gsn_backstab );
    }
    else
    {
	check_improve(ch,gsn_backstab,FALSE,1,TRUE);
	damage( ch, victim, 0, gsn_backstab,DAM_NONE,TRUE,NULL);
    }

    return;
}



void do_flee( CHAR_DATA *ch, char *argument )
{
    ROOM_INDEX_DATA *was_in;
    ROOM_INDEX_DATA *now_in;
    CHAR_DATA *victim;
    int attempt;

    if ( ( victim = ch->fighting ) == NULL )
    {
        if ( ch->position == POS_FIGHTING )
            ch->position = POS_STANDING;
	send_to_char( "You aren't fighting anyone.\n\r", ch );
	return;
    }

    if ( number_percent() <= 50
    &&   has_customization(ch, DISADVANTAGE_OVERCONFIDENT) )
    {
        send_to_char( "Flee? No way! Things are just getting exciting.\n\r", ch );
        return;
    }

    was_in = ch->in_room;
    for ( attempt = 0; attempt < 12; attempt++ )
    {
	EXIT_DATA *pexit;
	int door;

	door = number_door( );
	if ( ( pexit = was_in->exit[door] ) == 0
	||   pexit->u1.to_room == NULL
	||   IS_SET(pexit->exit_info, EX_CLOSED)
	||   number_range(0,ch->daze) != 0
	|| ( IS_NPC(ch)
	&&   IS_SET(pexit->u1.to_room->room_flags, ROOM_NO_MOB) ) )
	    continue;

	move_char( ch, door, FALSE );
	if ( (now_in = ch->in_room) == was_in )
	    continue;

	char_from_room( ch, FALSE );
	char_to_room( ch, was_in );

	save_hostility( ch );
	act( "$n has fled!", ch, NULL, NULL, TO_ROOM );

	char_from_room( ch, FALSE );
	char_to_room( ch, now_in );

	if ( !IS_NPC(ch) )
	{
	    send_to_char( "You flee from combat!\n\r", ch );
	    send_to_char( "You lost 10 exp.\n\r", ch); 
	    gain_exp( ch, -10 );
	}

	adjust_flee_renown( ch );

	check_improve( ch, gsn_consider, TRUE, 3, FALSE );
	stop_fighting( ch, TRUE );
	return;
    }

    send_to_char( "PANIC! You couldn't escape!\n\r", ch );
    return;
}



void do_rescue( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    CHAR_DATA *fch;

    one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
	send_to_char( "Rescue whom?\n\r", ch );
	return;
    }

    if ( ( victim = get_char_room( ch, arg ) ) == NULL )
    {
	send_to_char( "They aren't here.\n\r", ch );
	return;
    }

    if ( victim == ch )
    {
	send_to_char( "What about fleeing instead?\n\r", ch );
	return;
    }

    if ( !IS_NPC(ch) && IS_NPC(victim) )
    {
	send_to_char( "Doesn't need your help!\n\r", ch );
	return;
    }

    if ( victim->riding == ch || ch->riding == victim )
    {
	send_to_char( "You can't rescue them!\n\r", ch );
	return;
    }

    if ( ch->fighting == victim )
    {
	send_to_char( "Too late.\n\r", ch );
	return;
    }

    if ( ( fch = victim->fighting ) == NULL )
    {
	send_to_char( "That person is not fighting right now.\n\r", ch );
	return;
    }

    if ( IS_NPC(fch) && !is_same_group(ch, victim) )
    {
        send_to_char( "Kill stealing is not permitted.\n\r", ch );
        return;
    }

    WAIT_STATE( ch, skill_table[gsn_rescue].beats );
    if ( number_percent() > get_skill(ch, gsn_rescue) )
    {
	send_to_char( "You fail the rescue.\n\r", ch );
	check_improve(ch,gsn_rescue,FALSE,1,TRUE);
	return;
    }

    act_color( "You rescue $N!",
        ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF);
    act_color( "$n rescues you!",
        ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
    act_color( "$n rescues $N!",
        ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
    check_improve(ch,gsn_rescue,TRUE,1,TRUE);

    stop_fighting( fch, FALSE );
    stop_fighting( victim, FALSE );

    check_killer( ch, fch );
    set_fighting( ch, fch, FALSE );
    set_fighting( fch, ch, FALSE );
    return;
}



void do_kick( CHAR_DATA *ch, char *argument )
{
    CHAR_DATA *victim;

    if ( !IS_NPC(ch)
    &&   ch->level < skill_level(ch, gsn_kick) )
    {
	send_to_char(
	    "You better leave the martial arts to fighters.\n\r", ch );
	return;
    }

    if ( ch->riding != NULL )
    {
	send_to_char( "You can't do that while riding!\n\r", ch );
	return;
    }

    if (IS_NPC(ch) && !IS_SET(ch->off_flags,OFF_KICK))
	return;

    if ( ( victim = ch->fighting ) == NULL )
    {
	send_to_char( "You aren't fighting anyone.\n\r", ch );
	return;
    }

    WAIT_STATE( ch, skill_table[gsn_kick].beats );
    if ( get_skill(ch,gsn_kick) > number_percent() )
    {
        int max_dam;
        
	max_dam = effective_level( ch, gsn_kick );
	if ( IS_SET(ch->form, FORM_CENTAUR) )
	    max_dam += number_range( 1, 5 );
	if ( martial_artist(ch) )
	    max_dam += number_range( 1, 5 );

	damage( ch, victim, number_range(1, max_dam), gsn_kick, 
	    DAM_BASH, TRUE, NULL );
	check_improve(ch,gsn_kick,TRUE,1,TRUE);
    }
    else
    {
	damage( ch, victim, 0, gsn_kick, DAM_BASH, TRUE, NULL );
	check_improve(ch,gsn_kick,FALSE,1,TRUE);
    }
    return;
}


void do_disarm( CHAR_DATA *ch, char *argument )
{
    CHAR_DATA *victim;
    OBJ_DATA *obj;
    int chance,hth,ch_weapon,vict_weapon,ch_vict_weapon;

    hth = 0;

    if ( (chance = get_skill(ch, gsn_disarm)) == 0 )
    {
	send_to_char( "You don't know how to disarm opponents.\n\r", ch );
	return;
    }

    if ( get_eq_char(ch, WEAR_WIELD) == NULL
    &&   get_eq_char(ch, WEAR_SECONDARY) == NULL
    &&   ((hth = get_skill(ch, gsn_hand_to_hand)) == 0
    ||    (IS_NPC(ch) && !IS_SET(ch->off_flags,OFF_DISARM))))
    {
	send_to_char( "You must wield a weapon to disarm.\n\r", ch );
	return;
    }

    if ( ( victim = ch->fighting ) == NULL )
    {
	send_to_char( "You aren't fighting anyone.\n\r", ch );
	return;
    }

    if ( (obj = get_eq_char(victim, WEAR_WIELD)) == NULL )
    {
	if ( (obj = get_eq_char(victim, WEAR_SECONDARY)) == NULL )
	{
	send_to_char( "Your opponent is not wielding a weapon.\n\r", ch );
	return;
	}
    }

    /* find weapon skills */
    ch_weapon = get_weapon_skill( ch, get_weapon_sn(ch) );
    if ( obj->wear_loc == WEAR_WIELD )
    {
	int sn = get_weapon_sn( victim );

	vict_weapon	= get_weapon_skill( victim, sn );
	ch_vict_weapon	= get_weapon_skill( ch, sn );
    }
    else
    {
	int sn = get_secondary_sn( victim );

	vict_weapon	= get_weapon_skill( victim, sn );
	ch_vict_weapon	= get_weapon_skill( ch, sn );
    }

    /* modifiers */

    /* skill */
    if ( get_eq_char(ch, WEAR_WIELD) == NULL )
	chance = chance * hth/150;
    else
	chance = chance * ch_weapon/100;

    chance += (ch_vict_weapon/2 - vict_weapon) / 2; 

    /* dex vs. strength */
    chance = chance + get_curr_stat(ch,STAT_DEX)/4;
    chance = chance - get_curr_stat(victim,STAT_STR)/2;

    /* level */
    chance += (effective_level(ch, gsn_disarm) - victim->level) * 2;

    /* style */
    if ( COMPLEX_STYLE(victim, STYLE_FAN) )
        chance += 20;

    /* Always a chance of disarm or hanging on to weapon. */
    chance = URANGE( 5, chance, 95 );

    /* Reduce for two handed weapons. */
    if ( IS_WEAPON_STAT(obj, WEAPON_TWO_HANDS) )
	chance /= 2;

    /*check_killer( ch, victim );*/

    /* and now the attack */
    if ( number_percent() < chance )
    {
	WAIT_STATE( ch, skill_table[gsn_disarm].beats );
	disarm( ch, victim );
	check_improve(ch,gsn_disarm,TRUE,1,TRUE);
    }
    else
    {
	WAIT_STATE(ch,skill_table[gsn_disarm].beats);
	act_color("You fail to disarm $N.",
            ch,NULL,victim,TO_CHAR,POS_RESTING,C_COMBAT_SELF);
	act_color("$n tries to disarm you, but fails.",
            ch,NULL,victim,TO_VICT,POS_RESTING,OTHER_COLOR(ch));
	act_color("$n tries to disarm $N, but fails.",
            ch,NULL,victim,TO_NOTVICT,POS_RESTING,OTHER_COLOR(ch));
	check_improve(ch,gsn_disarm,FALSE,1,TRUE);
    }
    return;
}



void do_sla( CHAR_DATA *ch, char *argument )
{
    send_to_char( "If you want to SLAY, spell it out.\n\r", ch );
    return;
}



void do_slay( CHAR_DATA *ch, char *argument )
{
    CHAR_DATA *victim;
    char arg[MAX_INPUT_LENGTH];

    one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
	send_to_char( "Slay whom?\n\r", ch );
	return;
    }

    if ( ( victim = get_char_room( ch, arg ) ) == NULL )
    {
	send_to_char( "They aren't here.\n\r", ch );
	return;
    }

    if ( ch == victim )
    {
	send_to_char( "Suicide is a mortal sin.\n\r", ch );
	return;
    }

    if ( !IS_NPC(victim) && victim->level >= get_trust(ch) )
    {
	send_to_char( "You failed.\n\r", ch );
	return;
    }

    if ( !trusted_command(ch, "slay")
    &&   (!IS_NPC(victim)
    ||    !is_creator(ch, victim->pIndexData->area)) )
    {
	send_to_char( "You can only slay mobiles in areas you have creator privileges on.\n\r", ch );
	return;
    }

    act( "You slay $M in cold blood!",  ch, NULL, victim, TO_CHAR    );
    act( "$n slays you in cold blood!", ch, NULL, victim, TO_VICT    );
    act( "$n slays $N in cold blood!",  ch, NULL, victim, TO_NOTVICT );
    raw_kill( victim, TRUE, DAM_SLAY );
    return;
}


bool martial_artist( CHAR_DATA *ch )
{
    int max_armor;

    max_armor = MAX_MARTIAL_ARMOR + get_curr_stat( ch, STAT_STR ) - 50;

    if ( !IS_NPC(ch)
    &&   IS_SET(class_table[ch->class].flags, CLASS_MARTIAL_ARTIST)
    &&   ch->armor_weight <= max_armor
    &&   ch->riding == NULL )
    {
	OBJ_DATA *obj;
	
	for ( obj = ch->carrying; obj != NULL; obj = obj->next_content )
	{
	    if ( obj->wear_loc == WEAR_HOLD
	    ||   obj->wear_loc == WEAR_WIELD
	    ||   obj->wear_loc == WEAR_SHIELD
	    ||   obj->wear_loc == WEAR_SECONDARY )
		return FALSE;
	}
	
	return TRUE;
    }

    return FALSE;
}


void do_ki_punch( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    int chance;
    int roll;

    one_argument(argument,arg);
 
    if ( (chance = get_skill(ch,gsn_ki_punch)) == 0
    ||   (!IS_NPC(ch)
    &&    ch->level < skill_level(ch, gsn_ki_punch)) )
    {
	send_to_char( "You wave your fist around threateningly.\n\r", ch );
	return;
    }

    if ( IS_AFFECTED2(ch, AFF2_PACIFIST) )
    {
	send_to_char( "You don't quite feel that violent.", ch );
	return;
    }

    if ( arg[0] == '\0' )
    {
	victim = ch->fighting;
	if (victim == NULL)
	{
	    send_to_char( "But you aren't fighting anyone!\n\r", ch );
	    return;
	}
    }
    else if ( (victim = get_char_room(ch,arg)) == NULL )
    {
	send_to_char("They aren't here.\n\r",ch);
	return;
    }

    if ( ch->mana < skill_table[gsn_ki_punch].min_mana )
    {
	send_to_char( "You don't seem to have the energy.\n\r", ch );
	return;
    }
    
    if ( ch->riding != NULL )
    {
	send_to_char( "You can't do that while mounted.\n\r", ch );
	return;
    }

    if ( get_eq_char(ch, WEAR_WIELD) )
    {
	send_to_char( "You can't do that while wielding a weapon.\n\r", ch );
	return;
    }

    if ( victim->position < POS_FIGHTING )
    {
	act( "You'll have to let $M get back up first.", 
	    ch, NULL, victim, TO_CHAR );
	return;
    } 

    if ( victim == ch )
    {
	send_to_char( "Very funny.\n\r", ch );
	return;
    }

    if ( is_safe(ch, victim) )
	return;

    if ( IS_NPC(victim) 
    &&   victim->fighting != NULL
    &&   !is_same_group(ch, victim->fighting) )
    {
        send_to_char( "Kill stealing is not permitted.\n\r", ch );
        return;
    }

    if ( IS_AFFECTED(ch, AFF_CHARM) && ch->master == victim )
    {
	act( "But $N is your friend!", ch, NULL, victim, TO_CHAR );
	return;
    }

    /* Calculate modifiers. */
    
    /* stats */
    chance += get_curr_stat( ch, STAT_DEX ) / 4;
    chance -= get_curr_stat( victim, STAT_DEX ) / 3;

    /* level */
    chance += (effective_level(ch, gsn_ki_punch) - victim->level);

    /* dodge and parry */
    if ( !IS_NPC(victim) 
    &&   chance < get_skill(victim, gsn_dodge) )
	chance -= (get_skill(victim, gsn_dodge) - chance);
    if ( !IS_NPC(victim)
    &&   chance < get_skill(victim, gsn_parry) )
	chance -= (get_skill(victim, gsn_parry) - chance);

    /* now the attack */
    if ( (roll = number_percent()) < chance )
    {
	/* Modify to see how well they hit taking into account size */
	chance -= victim->carry_weight / 100;
	if ( ch->size < victim->size )
	    chance += (ch->size - victim->size) * 10;
	else
	    chance += (ch->size - victim->size) * 5;

	if ( roll < chance )
	{
	    EXIT_DATA *pexit;
	    int door = number_range( 0, 5 );

	    act_color( "$n's powerful punch sends you flying!",
		ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	    act_color( "$n's powerful punch sends $N flying!",
		ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
	    act_color( "Your powerful punch sends $M flying!",
		ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );

	    damage( ch, victim,
		    number_range(5, effective_level(ch, gsn_hand_to_hand)),
		    TYPE_HIT + ch->dam_type, DAM_BASH, FALSE, NULL );
	    if ( !CHAR_KILLED(victim) )
	    {
		if ((pexit = ch->in_room->exit[door]) != NULL
		&&  pexit->u1.to_room != NULL
		&&  can_see_room(victim, pexit->u1.to_room) 
		&&  !IS_SET(pexit->exit_info, EX_CLOSED)
		&&  victim->in_room != pexit->u1.to_room
		&&  (!IS_NPC(victim)
		||   !IS_SET(pexit->u1.to_room->room_flags, ROOM_NO_MOB)) )
		{
		    if ( victim->riding )
		    {
			victim->riding->mounted_by = NULL;
			victim->riding = NULL;
		    }
		    else
			check_dismount( victim );
		    
		    move_char( victim, door, FALSE );
		    DAZE_STATE( victim, 3 * PULSE_VIOLENCE );
		}
		else
		{
		    DAZE_STATE( victim, 2 * PULSE_VIOLENCE );
		    
		    check_dismount( victim );
		}

		victim->position = POS_RESTING;
	    }
	}
	else
	{
	    act_color( "With a loud shout $n drive $s fist into your midriff!",
		ch, NULL, victim, TO_VICT, POS_RESTING, C_COMBAT_SELF );
	    act_color( "With a loud shout $n drives $s fist into $N's midriff!",
		ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
	    act_color( "With a loud shout you drive your fist into $S midriff!",
		ch, NULL, victim, TO_CHAR, POS_RESTING, OTHER_COLOR(ch) );

	    damage( ch, victim,
		    number_range(5, effective_level(ch, gsn_hand_to_hand)),
		    TYPE_HIT + ch->dam_type, DAM_BASH, FALSE, NULL );
	    if ( !CHAR_KILLED(victim) )
		check_dismount( victim );
	}

	WAIT_STATE( ch, skill_table[gsn_ki_punch].beats );
	check_improve( ch, gsn_ki_punch, TRUE, 1, TRUE );
    }
    else
    {
	act_color( "You dodge $n's punch, leaving him off balance.",
	    ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	act_color( "$n shouts as he throws a wild punch at $N, who nimbly avoids it.",
	    ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
	act_color( "You completely missed $M!",
	    ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );

	damage( ch, victim, 0, TYPE_HIT + ch->dam_type, DAM_BASH, FALSE, NULL );
	WAIT_STATE( ch, skill_table[gsn_ki_punch].beats * 3 / 2 ); 
	check_improve( ch, gsn_ki_punch, FALSE, 1, TRUE );
    }

    ch->mana = UMAX( 0, ch->mana - skill_table[gsn_ki_punch].min_mana );
}


void do_ki_shout( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    int chance;

    one_argument( argument, arg );
 
    if ( (chance = get_skill(ch,gsn_ki_shout)) == 0
    ||   (!IS_NPC(ch)
    &&    ch->level < skill_level(ch, gsn_ki_shout)) )
    {
	send_to_char( "You shout out your rage!\n\r", ch );
	return;
    }

    if ( IS_AFFECTED2(ch, AFF2_PACIFIST) )
    {
	send_to_char( "You don't quite feel that violent.", ch );
	return;
    }

    if ( arg[0] == '\0' )
    {
	victim = ch->fighting;
	if (victim == NULL)
	{
	    send_to_char( "But you aren't fighting anyone!\n\r", ch );
	    return;
	}
    }
    else if ( (victim = get_char_room(ch,arg)) == NULL )
    {
	send_to_char("They aren't here.\n\r",ch);
	return;
    }

    if ( ch->mana < skill_table[gsn_ki_shout].min_mana )
    {
	send_to_char( "You don't seem to have the energy.\n\r", ch );
	return;
    }

    if ( victim == ch )
    {
	send_to_char( "Very funny.\n\r", ch );
	return;
    }

    if ( is_safe(ch, victim) )
	return;

    if ( IS_NPC(victim) 
    &&   victim->fighting != NULL
    &&   !is_same_group(ch, victim->fighting) )
    {
        send_to_char( "Kill stealing is not permitted.\n\r", ch );
        return;
    }

    if ( IS_AFFECTED(ch, AFF_CHARM) && ch->master == victim )
    {
	act( "But $N is your friend!", ch, NULL, victim, TO_CHAR );
	return;
    }

    if ( number_percent() < chance )
    {
	if ( saves_spell(effective_level(ch, gsn_ki_shout), victim, DAM_SOUND) )
	{
	    act_color( "$n shouts something at you.", 
		ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	    act_color( "$n shouts something at $N.",
		ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
	    act_color( "You shout at $N to no affect.",
		ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	}
	else
	{
	    act_color( "$n shouts a word of power sending you reeling!",
		ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	    act_color( "$n shouts a word of power at $N, sending $M reeling!",
		ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
	    act_color( "A powerful shout breaks from your lips, sending $N reeling.",
		ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );

	    DAZE_STATE( victim, 2 * PULSE_VIOLENCE );
	    victim->position = POS_RESTING;
	}

	check_improve( ch, gsn_ki_shout, TRUE, 1, TRUE );
    }
    else
    {
	act_color( "$n opens his mouth as if to shout something but nothing comes out.",
	    ch, NULL, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );
	act_color( 
	    "Your shout seems to catch in your throat before it can escape your lips.",
	    ch, NULL, NULL, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	    
	check_improve( ch, gsn_ki_shout, FALSE, 1, TRUE );
    }

    ch->mana = UMAX( 0, ch->mana - skill_table[gsn_ki_punch].min_mana );
    WAIT_STATE( ch, skill_table[gsn_ki_shout].beats ); 

    damage( ch, victim, 0, gsn_ki_punch, DAM_PIERCE, FALSE, NULL );
}


void do_eagle_claw( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *chest_armor;
    CHAR_DATA *victim;
    int chance;

    one_argument( argument, arg );
 
    if ( (chance = get_skill(ch,gsn_eagles_claw)) == 0
    ||   (!IS_NPC(ch)
    &&    ch->level < skill_level(ch, gsn_eagles_claw)) )
    {	
	send_to_char( 
	    "You form a claw with your hand and wave it about menacingly!\n\r",
	    ch );
	return;
    }

    if ( IS_AFFECTED2(ch, AFF2_PACIFIST) )
    {
	send_to_char( "You don't quite feel that violent.", ch );
	return;
    }

    if ( arg[0] == '\0' )
    {
	victim = ch->fighting;
	if (victim == NULL)
	{
	    send_to_char( "But you aren't fighting anyone!\n\r", ch );
	    return;
	}
    }
    else if ( (victim = get_char_room(ch,arg)) == NULL )
    {
	send_to_char("They aren't here.\n\r",ch);
	return;
    }

    if ( ch->mana < skill_table[gsn_eagles_claw].min_mana )
    {
	send_to_char( "You don't seem to have the energy.\n\r", ch );
	return;
    }

    if ( ch->riding != NULL )
    {
	send_to_char( "You can't do that while mounted.\n\r", ch );
	return;
    }

    if ( get_eq_char(ch, WEAR_HOLD) || get_eq_char(ch, WEAR_SECONDARY) )
    {
	send_to_char( "You can't do that while wielding a weapon.\n\r", ch );
	return;
    }

    if ( victim == ch )
    {
	send_to_char( "Very funny.\n\r", ch );
	return;
    }

    if ( is_safe(ch, victim) )
	return;

    if ( IS_NPC(victim) 
    &&   victim->fighting != NULL
    &&   !is_same_group(ch, victim->fighting) )
    {
        send_to_char( "Kill stealing is not permitted.\n\r", ch );
        return;
    }

    if ( IS_AFFECTED(ch, AFF_CHARM) && ch->master == victim )
    {
	act( "But $N is your friend!", ch, NULL, victim, TO_CHAR );
	return;
    }

    /* Apply modifiers. */
    
    /* scales */
    if ( IS_SET(CHAR_PARTS(victim), PART_SCALES) )
	chance -= 10;
	
    /* armor */
    chest_armor = get_eq_char( victim, WEAR_BODY );
    if ( chest_armor && chest_armor->item_type == ITEM_ARMOR )
	chance -= chest_armor->value[AC_PIERCE];

    /* size */
    if ( ch->size > victim->size )
	chance +=  5 * (ch->size - victim->size);
    else
	chance -= 10 * (ch->size - victim->size);

    /* stats */
    chance += get_curr_stat( ch, STAT_STR ) / 10;
    chance -= get_curr_stat( victim, STAT_DEX ) / 10;

    /* affects */
    if ( IS_AFFECTED(ch, AFF_HASTE) )
	chance += 10;
    if ( IS_AFFECTED(victim, AFF_HASTE) )
	chance -= 20;

    /* level */
    chance += ch->level - victim->level;

    /* Now for the exciting part! */
    if ( number_percent() < chance / 10
    ||   (ch->level >= LEVEL_IMMORTAL && victim->level <= LEVEL_HERO) )
    {
	if ( IS_SET(CHAR_PARTS(victim), PART_HEART)
	&&   (!IS_IMMORTAL(victim)
	||    get_trust(ch) > get_trust(victim)) )
	{
	    char buf[MAX_STRING_LENGTH];
	    OBJ_DATA *heart;
	    char *name;

	    act_color( "$n drives $s hand into your chest and tears out your still beating heart.",
		ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	    act_color( "$n drives $s hand into $N's chest tears out $S still beating heart.",
		ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
	    act_color( "You drive your hand into $S chest and tear out $S heart.",
		ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );

	    name  = IS_NPC(victim) ? victim->short_descr : victim->name;
	    heart = create_object( get_obj_index(OBJ_VNUM_TORN_HEART), 0, FALSE );
	    
	    sprintf( buf, heart->short_descr, name );
	    free_string( heart->short_descr );
	    heart->short_descr = str_dup( buf );

	    sprintf( buf, heart->description, name );
	    free_string( heart->description );
	    heart->description = str_dup( buf );

	    obj_to_char( heart, ch );
	    equip_char( ch, heart, WEAR_HOLD );

	    handle_death( ch, victim, DAM_PIERCE, FALSE );
	}
	else
	{
	    act_color( "Blood flies as $n drives $s hand into your midriff!",
		ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	    act_color( "Blood flies as $n drives $s hand into $N's midriff!",
		ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
	    act_color( "Blood flies as you drive your hand into $S midriff.",
		ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );

	    damage( ch, victim, dice(10, 20), 
	 	TYPE_HIT + ch->dam_type, DAM_PIERCE, TRUE, NULL );
	}

	check_improve( ch, gsn_eagles_claw, TRUE, 1, TRUE );
    }
    else
    {
	act_color( "$n swings his clawed hand wildly, missing you by miles.",
	    ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	act_color( "$n swings his clawed hand wildly.",
	    ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
	act_color( "$N nimbly avoids your claw.",
	    ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );

	damage( ch, victim, 0, gsn_eagles_claw, DAM_PIERCE, FALSE, NULL );
	check_improve( ch, gsn_eagles_claw, FALSE, 1, TRUE );
    }

    ch->mana = UMAX( 0, ch->mana - skill_table[gsn_eagles_claw].min_mana );
    WAIT_STATE( ch, skill_table[gsn_eagles_claw].beats ); 
}


/* Handle a death. */
void handle_death( CHAR_DATA *ch, CHAR_DATA *victim, int dam_type, bool make_parts )
{
    CHAR_DATA *leader = victim->leader;
    OBJ_DATA *corpse;

    if ( IS_AFFECTED2(victim, AFF2_DEATH_TRANCE) )
    {
	AFFECT_DATA *paf;
	AFFECT_DATA af;

	paf = affect_find( victim->affected, gsn_death_trance );
	if ( paf != NULL )
	{
	    int duration = paf->duration;
	    int level = paf->level;

	    affect_strip( victim, gsn_death_trance );

	    af.where            = TO_AFFECTS;
	    af.type             = gsn_death_trance;
	    af.level            = level;
	    af.duration         = duration;
	    af.location         = APPLY_NONE;
	    af.modifier         = 0;
	    af.bitvector        = AFF_PARALYSIS;
	    af.procNo           = affect_proc_lookup( "death trance" );
	    af.flags            = AFF_CONT_UPDATE;
	    af.name             = "";
	    af.appearance       = "";
	    affect_to_char( victim, &af );

	    stop_fighting( victim, TRUE );

	    act( "$n collapses to the ground unconscious!", victim, NULL, NULL, TO_ROOM );
	    send_to_char( "On the point of oblivion something pulls you back.", victim );

	    victim->position = POS_SLEEPING;
	    return;
	}
    }

    if ( ch != victim )
    {
	check_improve( ch, gsn_consider, TRUE, 1, FALSE );
	group_gain( ch, victim );
    }
    
    add_kill_message( ch, victim, dam_type );
    
    if ( !IS_NPC(victim) )
    {
        // If the killer was a punisher there's a change of a crime
        // being removed.
        DECLARE_SPEC_FUN( spec_cast_punisher );
        if ( IS_NPC(ch) && ch->spec_fun == spec_cast_punisher
        &&   number_percent() <= 50 )
        {
            int i;

            for ( i = 0; i < MAX_TERRITORIES; ++i )
            {
	        if ( IS_NULLSTR(territories[i].name) )
		    continue;

                if ( victim->pcdata->crimes[i] != NULL )
                {
                    CRIME_DATA *crime = victim->pcdata->crimes[i];
                    victim->pcdata->crimes[i] = crime->next;
                    free_crime( crime );
                    break;
                }
            }
        }

	sprintf( log_buf, "%s killed by %s at %d",
	    victim->name,
	    (IS_NPC(ch) ? ch->short_descr : ch->name),
	   ch->in_room->vnum );
	log_string( log_buf );

	/*
	 * Dying penalty:
	 * 2/3 way back to previous level.
	 */
/*
 *	if ( victim->exp > exp_per_level(victim,victim->pcdata->points) 
 *	                   * victim->level )
 *	    gain_exp( victim, (2 * (exp_per_level(victim,victim->pcdata->points)
 *	                            * victim->level - victim->exp)/3) + 50 );
 */
	if ( !IS_SET(victim->in_room->area->flags, AREA_ARENA) )
	    gain_exp( victim,
	              -exp_per_level(victim, victim->pcdata->points) / 3 );
    }

    sprintf( log_buf, "%s got toasted by %s at %s [room %d]",
	(IS_NPC(victim) ? victim->short_descr : victim->name),
	(IS_NPC(ch) ? ch->short_descr : ch->name),
	ch->in_room->name, 
        ch->in_room->vnum );

    if (IS_NPC(victim))
	wiznet(log_buf,NULL,NULL,WIZ_MOBDEATHS,0,0);
    else
	wiznet(log_buf,NULL,NULL,WIZ_DEATHS,0,0);

    victim->hit = 1;		/* Minimum post death hp. */

    /*
     * Death trigger
     */
    if ( IS_NPC(victim) && HAS_TRIGGER(victim, TRIG_DEATH) )
    {
	victim->position = POS_STANDING;
	mp_percent_trigger( victim, ch, NULL, NULL, TRIG_DEATH );
    }

    /*
     * Arena/renown/sounds.
     */
    if ( !IS_NPC(victim) )
    {
        if ( victim->sex == 2 )
            act_sound( "fpcdeath* V=100", ch, victim, TO_ALL );
        else
            act_sound( "mpcdeath* V=100", ch, victim, TO_ALL );

	if ( IS_SET(victim->in_room->area->flags, AREA_ARENA) )
	{
	    act( "$n collapses to the ground dead, the body fading quickly from existence.",
	        victim, NULL, NULL, TO_ROOM );
	    act( "Everything goes black as you realise $N has KILLED you!!",
	        ch, NULL, victim, TO_VICT );

	    stop_fighting( ch, TRUE );
	    
            affect_strip( ch, gsn_plague );
            affect_strip( ch, gsn_poison );

	    char_from_room( victim, TRUE );
	    char_to_room( victim, get_room_index(clan_table[victim->clan].hall) );
	    victim->position = POS_STANDING;
	    return;
	}
	
	adjust_death_renown( victim );
    }
    else
        act_sound( "mobdth* V=100", victim, NULL, TO_ROOM );

    /* dump the flags */
/*
 *   if ( ch != victim
 *   &&   !IS_NPC(ch)
 *   &&   !is_same_clan(ch, victim)
 *   &&   !IS_SET(victim->in_room->area->flags, AREA_ARENA) )
 *   {
 *	if ( IS_SET(victim->act, PLR_KILLER) )
 *	    REMOVE_BIT( victim->act, PLR_KILLER );
 *	else
 *	    REMOVE_BIT( victim->act, PLR_THIEF );
 *   }
 */

    raw_kill( victim, make_parts, dam_type );

    /*
     * Check for familier loss.
     */
    if ( IS_NPC(victim)
    &&   IS_SET(victim->act, ACT_PET)
    &&   IS_SET(victim->act, ACT_FAMILIER)
    &&   leader != NULL
    &&   !IS_NPC(leader) )
    {
        send_to_char( "You are suddenly filled with an overwelming sense of loss!\n\r", leader );
        leader->hit  = 1;
        leader->mana = 1;
    }


    music_to_char( "victory* C=1 L=1", ch );

    /* RT new auto commands */
    if (!IS_NPC(ch)
    &&  (corpse = get_obj_list(ch,"corpse",ch->in_room->contents)) != NULL
    &&  corpse->item_type == ITEM_CORPSE_NPC && can_see_obj(ch,corpse))
    {
	OBJ_DATA *coins;

	corpse = get_obj_list( ch, "corpse", ch->in_room->contents ); 

	if ( IS_SET(ch->act, PLR_AUTOLOOT) 
	&&   corpse && corpse->contains) /* exists and not empty */
	    do_get( ch, "all corpse" );

	if ( IS_SET(ch->act, PLR_AUTOGOLD) 
	&&   corpse && corpse->contains  /* exists and not empty */
	&&   !IS_SET(ch->act, PLR_AUTOLOOT) )
	    if ( (coins = get_obj_list(ch,"gcash",corpse->contains)) != NULL)
		do_get(ch, "all.gcash corpse");
            
	if ( IS_SET(ch->act, PLR_AUTOSAC) )
	    if ( !(IS_SET(ch->act,PLR_AUTOLOOT) && corpse && corpse->contains) )
		do_sacrifice( ch, "corpse" );
    }
}


void dam_message_new( CHAR_DATA *ch, CHAR_DATA *victim, int dam, int dt, bool immune )
{
    char buf1[256], buf2[256], buf3[256];
    const char *vp1;
    const char *vp2;
    const char *vp3;
    const char *attack;

    if (ch == NULL || victim == NULL)
	return;
   
         if ( dam ==   0 ) { vp1 = "awkward";
                             vp2 = "misses";
                             vp3 = " harmlessly."; }
    else if ( dam <=   4 ) { vp1 = "awkward";
                             vp2 = "gives";
                             vp3 = " a slight bruise."; }
    else if ( dam <=   8 ) { vp1 = "uncertain";    
                             vp2 = "hits";
                             vp3 = " causing minor scrapes."; }
    else if ( dam <=  12 ) { vp1 = "hesitant";  
                             vp2 = "hits";
                             vp3 = " making scratches."; }
    else if ( dam <=  16 ) { vp1 = "promising"; 
                             vp2 = "hits";
                             vp3 = " causing minor wounds."; }
    else if ( dam <=  20 ) { vp1 = "promising";    
                             vp2 = "strikes";
                             if ( !CAN_BLEED(victim) )
                                 vp3 = " causing minor wounds.";
                             else if ( BLEEDS_ICHOR(victim) )
                                 vp3 = ", icor seeps from the wound.";
                             else 
                                 vp3 = ", the wound bleeds."; }
    else if ( dam <=  26 ) { vp1 = "proficient";    
                             vp2 = "strikes";
                             vp3 = ", hitting an organ."; }
    else if ( dam <=  32 ) { vp1 = "proficient";
                             vp2 = "causes";
                             vp3 = " to gasp sharply in pain."; }
    else if ( dam <=  38 ) { vp1 = "adept";
                             vp2 = "causes";
                             vp3 = " to nearly buckle in pain!"; }
    else if ( dam <=  44 ) { vp1 = "adept";
                             vp2 = "has a ravaging effect on";
                             vp3 = "."; }
    else if (dam <=   50 ) { vp1 = "crafty";
                             vp2 = "tears into";
                             vp3 = ", shredding flesh."; }
    else if ( dam <=  60 ) { vp1 = "mighty";
                             vp2 = "causes";
                             if ( !CAN_BLEED(victim) )
                                 vp3 = ", shredding flesh.";
                             else if ( BLEEDS_ICHOR(victim) )
                                 vp3 = " to spurt yellow ooze.";
                             else
                                 vp3 = " to spurt blood!"; }
    else if ( dam <=  70 ) { vp1 = "studied";
                             vp2 = "leaves large, open gashes on";
                             vp3 = "!"; }
    else if ( dam <=  80 ) { vp1 = "studied";
                             vp2 = "tears";
                             vp3 = " leaving a GAPING hole!"; }
    else if ( dam <=  90 ) { vp1 = "well directed";
                             vp2 = "DISEMBOWELS";
                             if ( IS_SET(victim->parts, PART_GUTS) )
                                 vp3 = ". Guts spill out!!";
                             else
                                 vp3 = " leaving a gaping hole."; }
    else if ( dam <= 100 ) { vp1 = "poised";
                             vp2 = "DISMEMBERS";
                             if ( !CAN_BLEED(victim) )
                                 vp3 = "!";
                             else if ( BLEEDS_ICHOR(victim) )
                                 vp3 = "! Yellow ooze splatters!";
                             else
                                 vp3 = "! Blood splatters!"; }
    else if ( dam <= 115 ) { vp1 = "vicious";
                             vp2 = "ANNIHILATES";
                             vp3 = " nearly out of existence!!"; }
    else if ( dam <= 130 ) { vp1 = "vicious";
                             vp2 = "OBLITERATES";
                             vp3 = " utterly!!"; }
    else if ( dam <= 145 ) { vp1 = "savage";
                             vp2 = "MASSACRES";
                             if ( !CAN_BLEED(victim) )
                                 vp3 = " utterly!!";
                             else if ( BLEEDS_ICHOR(victim) )
                                 vp3 = ". Yellow ooze flies everywhere!";
                             else
                                 vp3 = ". Blood flies everywhere!"; }
    else if ( dam <= 160 ) { vp1 = "strategic";
                             vp2 = "ERADICATES";
                             vp3 = " into tiny bits of nothing!!"; }
    else                   { vp1 = "monumental";
	                     vp2 = "conjures the COMPLETE and ABSOLUTE POWER on"; 
	                     vp3 = "!"; }

    if ( dt == TYPE_HIT )
    {
	if (ch  == victim)
	{
	    sprintf( buf1, "$n fumbles and hits $melf!");
	    sprintf( buf2, "You fumble and hit yourself!");
	}
	else
        {
            sprintf( buf1, "$n's %s strike %s $N%s",vp1,vp2,vp3);
            sprintf( buf2, "Your %s strike %s $N%s",vp1,vp2,vp3);
            sprintf( buf3, "$n's %s strike %s you%s",vp1,vp2,vp3);
	}
    }
    else
    {
	if ( dt >= 0 && dt < MAX_SKILL )
	    attack	= skill_table[dt].noun_damage;
	else if ( dt >= TYPE_HIT
	&& dt <= TYPE_HIT + MAX_DAMAGE_MESSAGE)
	    attack  = attack_noun( dt - TYPE_HIT, ch, victim );
	else
	{
	    bug( "Dam_message: bad dt %d.", dt );
	    dt  = TYPE_HIT;
	    attack  = attack_table[0].name;
	}

	if (immune)
	{
	    if (ch == victim)
	    {
		sprintf(buf1,"$n is unaffected by $s own %s.",attack);
		sprintf(buf2,"Luckily, you are immune to that.");
	    }
	    else
	    {
	    	sprintf(buf1,"$N is unaffected by $n's %s!",attack);
	    	sprintf(buf2,"$N is unaffected by your %s!",attack);
	    	sprintf(buf3,"$n's %s is powerless against you.",attack);
	    }
	}
	else
	{
	    if (ch == victim)
	    {
                sprintf( buf1, "$n's %s %s %s $m%s", vp1, attack, vp2, vp3 );
                sprintf( buf2, "Your %s %s %s you%s", vp1, attack, vp2, vp3 );
	    }
	    else
	    {
                sprintf( buf1, "$n's %s %s %s $N%s", vp1, attack, vp2, vp3 );
                sprintf( buf2, "Your %s %s %s $N%s", vp1, attack, vp2, vp3 );
                sprintf( buf3, "$n's %s %s %s you%s",vp1, attack, vp2, vp3 );
	    }
	}
    }

    if (ch == victim)
    {
	act_color( buf1, 
            ch, NULL, NULL, TO_ROOM, POS_RESTING, OTHER_COLOR(ch) );
	act_color( buf2, 
            ch, NULL, NULL, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
    }
    else
    {
    	act_color( buf1, 
            ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
    	act_color( buf2, 
            ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
    	act_color( buf3, 
            ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
    }

    return;
}


int get_ac( CHAR_DATA *ch, int ac_type )
{
    int ac;

    ac = ch->armor[ac_type];

    if ( IS_AWAKE(ch) )
    {
	ac += dex_app[get_curr_stat(ch, STAT_DEX)/4].defensive;
    
	if ( martial_artist(ch) )
	{
	    if ( ac_type != AC_PIERCE
	    &&   ac_type != AC_BASH
	    &&   ac_type != AC_SLASH )
		ac -= 2 * effective_level( ch, gsn_hand_to_hand );
	    else
		ac -= 4 * effective_level( ch, gsn_hand_to_hand );
	}
    }
    
    return ac;
}


void save_hostility( CHAR_DATA *victim )
{
    char buf[MAX_STRING_LENGTH];
    CHAR_DATA *rch;

    if ( IS_NPC(victim) )
	return;

    for ( rch = victim->in_room->people; rch != NULL; rch = rch->next_in_room )
    {
	if ( rch == victim )
	    continue;

	if ( IS_NPC(rch)
	&&   rch->fighting == victim
	&&   (IS_SET(rch->off_flags, OFF_HUNTER)
        ||    IS_SET(rch->off_flags, OFF_NASTY_HUNTER)) )
	{
	    MEM_DATA *mem;

	    mem			= new_mem_data();

	    mem->next		= rch->memory;
	    mem->id		= victim->id;
	    mem->reaction	= MEM_HOSTILE;
	    mem->duration	= 18 + number_range( 1, 12 );

	    rch->memory = mem;

	    sprintf( buf, "$N has started to hunt %s", victim->name );
	    wiznet( buf, rch, NULL, WIZ_MOBACTIONS, 0, 0 );
	    
	    if ( IS_SET(CHAR_FORM(rch), FORM_SENTIENT)
	    &&   number_bits(1) == 0 )
	    {
		bool no_shout = FALSE;
		const char *msg;

		if ( number_bits(1) == 0 ) 
		    msg = "Don't think you'll escape me that easily %s.";
		else
		    msg = "I'll get you %s!";
		sprintf( buf, msg, victim->name );
		if ( IS_SET(rch->comm, COMM_NOSHOUT) )
		    no_shout = TRUE;
		REMOVE_BIT( rch->comm, COMM_NOSHOUT );
		do_yell( rch, buf );
		if ( no_shout )
		    SET_BIT( rch->comm, COMM_NOSHOUT );
	    }
	    else if ( IS_SET(CHAR_FORM(rch), FORM_ANIMAL)
	    &&        number_bits(1) == 0 )
		act( "$n snarls menacingly.", rch, NULL, victim, TO_NOTVICT );
	}
    }
}


bool check_dismount( CHAR_DATA *victim )
{
    CHAR_DATA *rider;
    int chance;

    if ( victim->riding != NULL )
	rider = victim;
    else
	rider = victim->mounted_by;

    if ( rider == NULL )
	return FALSE;

    chance = get_skill( rider, gsn_ride );
    if ( rider->daze )
	chance -= 20;

    chance = URANGE( 5, chance, 95 );

    if ( number_percent() > chance )
    {
	send_to_char( "You fall from the back of your mount.\n\r", rider );
	act( "$n falls from the back of $s mount!", 
	    rider, NULL, victim, TO_NOTVICT );
	if ( victim != rider )
	    act( "$n falls from your back!", rider, NULL, victim, TO_VICT );
	
	check_improve( rider, gsn_ride, FALSE, 3, TRUE );
	
	damage( rider, rider, dice(1, 10), 0, DAM_BASH, FALSE, NULL );

	DAZE_STATE( rider, 3 * PULSE_VIOLENCE );
	rider->position = POS_RESTING;

	if ( rider != victim )
	{
	    rider->riding		= NULL;
	    victim->mounted_by		= NULL;
	}
	else
	{
	    rider->riding->mounted_by	= NULL;
	    rider->riding		= NULL;
	}
	
	return TRUE;
    }
    
    check_improve( rider, gsn_ride, TRUE, 3, TRUE );
    return FALSE;
}


void do_trample( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    int chance;

    one_argument( argument, arg );

    if ( (chance = get_skill(ch, gsn_trample)) == 0
    ||   (!IS_NPC(ch)
    &&    ch->level < skill_level(ch, gsn_trample)) )
    {
	send_to_char("Grrrr - you stamp your feet up and down!\n\r",ch);
	return;
    }

    if ( ch->riding == NULL )
    {
	send_to_char( "You can only do that when mounted!\n\r", ch );
	return;
    }

    if ( IS_AFFECTED2(ch, AFF2_PACIFIST) )
    {
	send_to_char( "You don't quite feel that violent.", ch );
	return;
    }

    if ( arg[0] == '\0' )
    {
	victim = ch->fighting;
	if ( victim == NULL )
	{
	    send_to_char( "But you aren't fighting anyone!\n\r", ch );
	    return;
	}
    }
    else if ( (victim = get_char_room(ch,arg)) == NULL )
    {
	send_to_char( "They aren't here.\n\r", ch );
	return;
    }

    if ( victim->riding != NULL )
        victim = victim->riding;
    
    if ( victim == ch )
    {
	send_to_char( "You try to trample yourself!!?\n\r", ch );
	return;
    }
    
    if ( victim == ch->riding )
    {
	act( "And just how is your mount to trample $Mself?",
	    ch, NULL, ch->riding, TO_CHAR );
	return;
    }

    if ( is_safe(ch, victim) )
	return;

    if ( IS_NPC(victim)
    &&   victim->fighting != NULL
    &&   !is_same_group(ch, victim->fighting) )
    {
        send_to_char( "Kill stealing is not permitted.\n\r", ch );
        return;
    }

    if ( IS_AFFECTED(ch, AFF_CHARM) && ch->master == victim )
    {
	act( "But $N is your friend!", ch, NULL, victim, TO_CHAR );
	return;
    }

    if ( CHAR_SIZE(ch->riding) <= CHAR_SIZE(victim) )
    {
	send_to_char( "Try trampling someone a bit smaller.\n\r", ch );
	return;
    }
    
    /* Modifiers. */
    chance += (CHAR_SIZE(ch->riding) - CHAR_SIZE(victim)) * 10;

    if ( IS_SET(CHAR_FORM(victim), FORM_MIST) )
	chance -= 15;
    if ( IS_SET(CHAR_FORM(victim), FORM_INTANGIBLE) )
	chance -= 20;

    chance -= get_curr_stat( victim, STAT_DEX ) / 5;
    
    if ( IS_SET(ch->riding->off_flags, OFF_FAST) )
	chance += 10;
    if ( IS_AFFECTED(ch->riding, AFF_HASTE) )
	chance += 10;
    if ( (IS_NPC(victim)
    &&    IS_SET(victim->off_flags, OFF_FAST))
    ||   IS_AFFECTED(victim, AFF_HASTE) )
	chance -= 30;

    chance = URANGE( 5, chance, 95 );

    if ( number_percent() < chance )
    {
	act_color( "$n rears up and tramples you!",
	    ch->riding, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	act_color( "You rear up and trample $N underfoot!",
	    ch->riding, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	act_color( "$n rears up, then tramples $N underfoot!",
	    ch->riding, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
	
	check_improve( ch, gsn_trample, TRUE, 1, TRUE );

	DAZE_STATE( victim, 3 * PULSE_VIOLENCE );

	victim->position = POS_RESTING;

	damage( ch->riding, victim, 
	        10 + number_range(1, 20*(victim->size - ch->riding->size)),
	        gsn_trample, DAM_BASH, FALSE, NULL );
	if ( !CHAR_KILLED(victim) )
	    check_dismount( victim );

	check_dismount( ch );
    }
    else
    {
	act_color( "$n rears up, trying to trample you!",
	    ch->riding, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	act_color( "$N nimbly steps aside as you reat up trying to trample $M.",
	    ch->riding, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	act_color( "$n rears up, trying to trample $N!",
	    ch->riding, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );

	check_improve( ch, gsn_trample, FALSE, 1, TRUE );

	damage( ch->riding, victim, 0, gsn_trample, DAM_BASH, FALSE, NULL );

	check_dismount( ch );
    }

    WAIT_STATE( ch, skill_table[gsn_trample].beats );
    if ( ch->riding != NULL )
        WAIT_STATE( ch->riding, skill_table[gsn_trample].beats );

    if ( !CHAR_KILLED(victim)
    &&   victim->fighting != NULL )
	set_fighting( victim, ch, FALSE );
}


void do_joust( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    OBJ_DATA *weapon;
    int chance;

    one_argument( argument, arg );
 
    if ( (chance = get_skill(ch, gsn_joust)) == 0
    ||   (!IS_NPC(ch)
    &&    ch->level < skill_level(ch, gsn_joust)) )
    {
	send_to_char("You fling yourself into glorious battle!\n\r",ch);
	return;
    }

    if ( ch->fighting != NULL )
    {
	send_to_char( "You're already fighting!\n\r",ch);
	return;
    }

    if ( ch->riding == NULL )
    {
	send_to_char( "You can only do that when mounted!\n\r", ch );
	return;
    }

    if ( arg[0] == '\0' )
    {
	send_to_char( "Joust whom?\n\r", ch );
	return;
    }
    
    if ( (victim = get_char_room(ch, arg)) == NULL )
    {
	send_to_char( "They aren't here.\n\r", ch );
	return;
    }
    
    if ( victim == ch->riding )
    {
	send_to_char( "You can't joust your own mount.\n\r", ch );
	return;
    }

    if ( victim == ch )
    {
	send_to_char( "You can't joust yourself!\n\r", ch );
	return;
    }

    if ( is_safe(ch, victim) )
	return;

    if ( IS_NPC(victim)
    &&   victim->fighting != NULL
    &&   !is_same_group(ch, victim->fighting) )
    {
        send_to_char( "Kill stealing is not permitted.\n\r", ch );
        return;
    }

    if ( IS_AFFECTED(ch, AFF_CHARM) && ch->master == victim )
    {
	act( "But $N is your friend!", ch, NULL, victim, TO_CHAR );
	return;
    }
    
    if ( ( weapon = get_eq_char(ch, WEAR_WIELD)) == NULL )
    {
	send_to_char( "You need a weapon to joust.\n\r", ch );
	return;
    }
/*    
    if ( (weapon = get_eq_char(ch, WEAR_WIELD)) == NULL
    ||   weapon->value[0] != WEAPON_SPEAR )
    {
	send_to_char( "You need a spear to joust.\n\r", ch );
	return;
    }
*/  
    check_killer( ch, victim );
    
    WAIT_STATE( ch, skill_table[gsn_joust].beats );
    
    if ( number_percent() < chance )
    {
	check_improve(ch, gsn_joust, TRUE, 2, TRUE) ;
	multi_hit( ch, victim, gsn_joust );
    }
    else
    {
	check_improve(ch, gsn_joust, FALSE, 1, TRUE );
	damage( ch, victim, 0, gsn_joust, DAM_NONE, TRUE, NULL );
    }
}


void decapitate( CHAR_DATA *victim, CHAR_DATA *killer )
{
    char buf[MAX_STRING_LENGTH];
    const char *name;
    OBJ_DATA *head;

    name = victim->polymorphed
         ? victim->polymorphed->short_descr
         : (IS_NPC(victim) ? victim->short_descr : victim->name);

    head = create_object( get_obj_index(OBJ_VNUM_SEVERED_HEAD), 0, FALSE );
    head->timer = number_range( 4, 7 );

    sprintf( buf, head->short_descr, name );
    free_string( head->short_descr );
    head->short_descr = str_dup( buf );

    sprintf( buf, head->description, name );
    free_string( head->description );
    head->description = str_dup( buf );

    act_color( "$N's severed head plops on the ground.",
         killer, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(killer) );
    act_color( "With a powerful slash you sever $N's head!",
         killer, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
    act_color( "$n's superb slash slices your head from your shoulders!",
         killer, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(killer) );

    obj_to_room( head, killer->in_room );

    handle_death( killer, victim, DAM_SLASH, FALSE );
}


char *attack_noun( int dam_type, CHAR_DATA *ch, CHAR_DATA *victim )
{
    if ( attack_table[dam_type].fun == NULL )
	return attack_table[dam_type].noun;
    else
	return (*attack_table[dam_type].fun) ( ch, victim );
}


void do_surrender( CHAR_DATA *ch, char *argument )
{
    CHAR_DATA *mob;

    if ( (mob = ch->fighting) == NULL )
    {
	send_to_char( "But you're not fighting!\n\r", ch );
	return;
    }

    act_color( "You surrender to $N!", 
        ch, NULL, mob, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
    act_color( "$n surrenders to you!", 
        ch, NULL, mob, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
    act_color( "$n tries to surrender to $N!", 
        ch, NULL, mob, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
    stop_fighting( ch, TRUE );

    if ( !IS_NPC(mob) )
    {
	char buf[ MAX_STRING_LENGTH ];
	
	if ( IS_SET(mob->act, PLR_NOMERCY) )
	{
	    act_color( "You ignore their cowardly act!", 
                mob, NULL, NULL, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	    act_color( "$N seems to ignore your cowardly act!", 
                ch, NULL, mob, TO_CHAR, POS_RESTING, OTHER_COLOR(mob) );
	    multi_hit( mob, ch, TYPE_UNDEFINED );
	    
	    sprintf( buf, "%s has ignored %ss surrender.",
	        mob->name,
	        IS_NPC(ch) ? ch->short_descr : ch->name );
	}
	else
	{
	    sprintf( buf, "%s has surrendered to %s.",
	        IS_NPC(ch) ? ch->short_descr : ch->name,
	        mob->name );
	}
	
	wiznet( buf, mob, NULL, WIZ_MOBACTIONS, 0, 0 );
	return;
    }

    if ( !IS_NPC(ch)
    &&   IS_NPC(mob)
    &&   (!HAS_TRIGGER(mob, TRIG_SURR) 
    ||    !mp_percent_trigger(mob, ch, NULL, NULL, TRIG_SURR)) )
    {
	act( "$N seems to ignore your cowardly act!", ch, NULL, mob, TO_CHAR );
	multi_hit( mob, ch, TYPE_UNDEFINED );
	return;
    }
}


void do_unmount( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    OBJ_DATA *weapon;
    int chance, roll;

    one_argument( argument, arg );
 
    if ( (chance = get_skill(ch, gsn_unmount)) == 0
    ||   (!IS_NPC(ch)
    &&    ch->level < skill_level(ch, gsn_unmount)) )
    {
	send_to_char( "You can't even begin to work out how to do that!\n\r",
	    ch );
	return;
    }

    if ( ch->fighting == NULL && arg[0] == '\0' )
    {
	send_to_char( "Unmount whom?\n\r",ch);
	return;
    }

    if ( ch->fighting != NULL )
	victim = ch->fighting;
    else if ( (victim = get_char_room(ch, arg)) == NULL )
    {
	send_to_char( "They aren't here.\n\r", ch );
	return;
    }

    if ( victim == ch )
    {
	send_to_char( "Use dismount to get off.\n\r", ch );
	return;
    }

    if ( victim->riding == NULL )
    {
	send_to_char( "But they're not mounted.\n\r", ch );
	return;
    }

    if ( victim->riding == ch )
    {
	send_to_char( "You can't do that.\n\r", ch );
	return;
    }

    if ( arg[0] == '\0' )
    {
	send_to_char( "Unmount whom?\n\r", ch );
	return;
    }

    if ( is_safe(ch, victim) )
	return;

    if ( IS_NPC(victim)
    &&   victim->fighting != NULL
    &&   !is_same_group(ch, victim->fighting) )
    {
        send_to_char( "Kill stealing is not permitted.\n\r", ch );
        return;
    }

    if ( IS_AFFECTED(ch, AFF_CHARM) && ch->master == victim )
    {
	act( "But $N is your friend!", ch, NULL, victim, TO_CHAR );
	return;
    }

    weapon = get_eq_char( ch, WEAR_WIELD );
    if ( weapon == NULL )
	weapon = get_eq_char( ch, WEAR_SECONDARY );

    check_killer( ch, victim );

    WAIT_STATE( ch, skill_table[gsn_unmount].beats );

    chance += (get_curr_stat(ch, STAT_STR) / 2) - 25;
    if ( weapon == NULL )
	chance -= 45;
    else if ( weapon->value[0] == WEAPON_POLEARM )
	chance += 5;
    else if ( weapon->value[0] == WEAPON_DAGGER )
	chance -= 25;
    else
	chance -= 15;

    if ( !can_see(victim, ch) )
	chance += 30;
    else if ( victim->fighting != NULL && victim->fighting != ch )
	chance += 15;

    roll = chance + number_percent();

    if ( roll > 100 )
    {
	int hold_on;

	hold_on = get_skill( victim, gsn_ride ) - roll + 101;
	
	if ( number_percent() >= hold_on )
	{
	    act_color( "You pull $N from the back of $S mount.",
	        ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	    act_color( "$n pulls $N from the back of $S mount.",
	        ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
	    act_color( "$n pulls you from the back of your mount!",
	        ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );

	    damage( ch, victim,
	        dice(1, 1+victim->armor_weight/100),
	        gsn_unmount, DAM_BASH, FALSE, NULL );

	    DAZE_STATE( victim, 3 * PULSE_VIOLENCE );
	    victim->position = POS_RESTING;

	    check_improve(ch, gsn_unmount, TRUE, 3, TRUE);

	    victim->riding->mounted_by = NULL;
	    victim->riding = NULL;

	    return;
	}
	else
	    check_improve(ch, gsn_unmount, TRUE, 3, TRUE);
    }
    else
	check_improve(ch, gsn_unmount, FALSE, 1, TRUE );

    act_color( "You try to pull $N from the back of $S mount.",
        ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
    act_color( "$n tries to pull $N from the back of $S mount.",
        ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
    act_color( "$n tries to pull you from the back of your mount.",
        ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );

    damage( ch, victim, 0, gsn_unmount, DAM_NONE, FALSE, NULL );
}


void crush( CHAR_DATA *ch, int hits )
{
    const char *msg_room, *msg_char, *msg_vict;
    int chance;

    if ( ch->fighting == NULL )
	return;

    chance  = 40 + ch->level;
    chance += get_curr_stat( ch, STAT_STR )
           -  get_curr_stat( ch->fighting, STAT_STR );
    chance += 5 * (ch->level - ch->fighting->level);
    chance += 10 * (hits - 1);
    chance += 20 * (CHAR_SIZE(ch) - CHAR_SIZE(ch->fighting));

    if ( number_percent() > chance )
	return;

    if ( IS_SET(CHAR_PARTS(ch), PART_ARMS) )
    {
	msg_room = "$n grabs $N and crushes $M in $s arms!";
	msg_char = "You grab $N and crush $M in your arms!";
	msg_vict = "$n grabs you and crushes you in $s arms!";
    }
    else if ( IS_SET(CHAR_PARTS(ch), PART_TENTACLES) )
    {
	msg_room = "$n wraps $s tentacles around $N and squeezes!";
	msg_char = "You wrap your tentacles around $N and squeeze!";
	msg_vict = "$n wraps $s tentacles around you and squeezes!";
    }
    else
    {
	msg_room = "$n crushes $N!";
	msg_char = "You crush $N!";
	msg_vict = "$n crushes you!";
    }
    
    act_color( msg_room, ch, NULL, ch->fighting, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
    act_color( msg_char, ch, NULL, ch->fighting, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
    act_color( msg_vict, ch, NULL, ch->fighting, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );

    DAZE_STATE( ch->fighting, 3 * PULSE_VIOLENCE );
    WAIT_STATE( ch->fighting, 3 * PULSE_VIOLENCE );
    
    WAIT_STATE( ch, 6 * PULSE_VIOLENCE );

    damage( ch, ch->fighting,
        dice(UMIN(hits, 4), UMAX(6, (ch->size - ch->fighting->size) * 10 * ch->level / MAX_LEVEL)),
        TYPE_UNDEFINED, DAM_BASH, FALSE, NULL );
}


void do_circle( CHAR_DATA *ch, char *argument )
{
    char		arg[MAX_INPUT_LENGTH];
    CHAR_DATA	*victim, *rch;
    OBJ_DATA	*obj;
    int			chance = 0;

    one_argument( argument, arg );

    if (arg[0] == '\0')
    {
        send_to_char("Circle whom?\n\r",ch);
        return;
    }

    if ( (victim = get_char_room(ch, arg)) == NULL )
    {
        send_to_char("They aren't here.\n\r",ch);
        return;
    }

    if ( victim == ch )
    {
	send_to_char( "How can you sneak up on yourself?\n\r", ch );
	return;
    }

    if ( victim->fighting == NULL )
    {
	act( "$e is not fighting, you can't circle $m.", ch, NULL, victim, TO_CHAR );
	return;
    }

    if ( ch->riding != NULL )
    {
	send_to_char( "You can't do that while riding!\n\r", ch );
	return;
    }
    
    if ( victim->riding != NULL
    &&   CHAR_SIZE(victim->riding) > CHAR_SIZE(ch)
    &&   !IS_AFFECTED(ch, AFF_FLYING) )
    {
	act( "You can't reach $S back while $Es mounted.",
	    ch, NULL, victim, TO_CHAR );
	return;
    }

    if ( is_safe( ch, victim ) )
      return;

    if (IS_NPC(victim) &&
	!is_same_group(ch,victim->fighting))
    {
        send_to_char("Kill stealing is not permitted.\n\r",ch);
        return;
    }

    if ( ( obj = get_eq_char( ch, WEAR_WIELD ) ) == NULL)
    {
	send_to_char( "You need to wield a weapon to backstab.\n\r", ch );
	return;
    }

	/* check to see if char is not being attacked by anyone */
	for (rch = ch->in_room->people; rch != NULL; rch = rch->next_in_room)
	{
		if ( rch->fighting == ch )
  		{
  			send_to_char( "No way! You are being attacked!\n\r", ch);
  			return;
  		}
	}
	
    check_killer( ch, victim );
    WAIT_STATE( ch, skill_table[gsn_circle].beats );
    
    /* calculate the chance of success */
    chance = ( get_skill( ch, gsn_circle ) / 2 ) + 10;
    if ( ch->level > victim->level )
    	chance += 10;
    if ( (IS_AFFECTED(ch, AFF_SNEAK) && !IS_AFFECTED(ch, AFF_DETECT_HIDDEN) )
      || (IS_AFFECTED(ch, AFF_INVIS) && !IS_AFFECTED(ch, AFF_DETECT_INVIS ) ) )
    	chance += 10;
    
    /* do it! */
    if ( number_percent() < chance )
    {
	check_improve(ch,gsn_circle,TRUE,1,TRUE);
	multi_hit( ch, victim, gsn_circle );
    }
    else
    {
	check_improve(ch,gsn_circle,FALSE,1,TRUE);
	//damage( ch, victim, 0, gsn_circle,DAM_NONE,TRUE,NULL);
	act( "You fail to get around behind $N.",
		ch, NULL, victim, TO_CHAR );
    }

    return;
}


void do_sap( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    OBJ_DATA *obj;
    int chance;
    int roll;

    if ( (chance = get_skill(ch, gsn_sap)) == 0
    ||   (!IS_NPC(ch) 
    &&    ch->level < skill_level(ch, gsn_sap)) )
    {
	send_to_char( "You don't know how to do that!\n\r", ch );
	return;
    }

    if ( IS_AFFECTED2(ch, AFF2_PACIFIST) )
    {
	send_to_char( "You don't quite feel that violent.", ch );
	return;
    }

    if ( ch->riding != NULL )
    {
	send_to_char( "You can't do that while riding!\n\r", ch );
	return;
    }

    if ( ch->fighting != NULL )
    {
	send_to_char( "Not while you're fighting!\n\r", ch );
	return;
    }
    
    one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
	send_to_char( "Sap whom?\n\r", ch );
	return;
    }
    
    if ( (victim = get_char_room(ch, arg)) == NULL )
    {
	send_to_char( "They aren't here.\n\r", ch );
	return;
    }

    if ( is_safe(ch, victim) )
	return;

    if ( IS_NPC(victim) &&
	 victim->fighting != NULL && 
	 !is_same_group(ch, victim->fighting) )
    {
	send_to_char( "Kill stealing is not permitted.\n\r", ch );
	return;
    }
        
    if ( victim->fighting != NULL )
    {
	send_to_char( "Not while they're fighting.\n\r", ch );
	return;
    }

    if ( victim == ch )
    {
	send_to_char( "Very funny.\n\r", ch );
	return;
    }

    if ( IS_AFFECTED(ch, AFF_CHARM) && ch->master == victim )
    {
	act( "$N is your beloved master.", ch, NULL, victim, TO_CHAR );
	return;
    }

    if ( IS_SET(CHAR_FORM(victim), FORM_WORM)
    ||   IS_SET(CHAR_FORM(victim), FORM_BLOB)
    ||   !IS_SET(CHAR_FORM(victim), PART_HEAD) )
    {
	act( "You can't even begin to work out how to sap $N!",
	    ch, NULL, victim, TO_CHAR );
	return;
    }
    
    /* weapon */
    obj = get_eq_char( ch, WEAR_WIELD );
    if ( obj == NULL )
	chance /= 2;
    else if ( attack_table[obj->value[3]].damage != DAM_BASH )
    {
	send_to_char( "You can only sap with a blunt weapon.\n\r", ch );
	return;
    }

    /* modifiers */

    if ( can_see(victim, ch) )
	chance = 0;

    /* level */
    chance += (effective_level(ch, gsn_sap) - victim->level) * 2;
    
    /* now the attack */
    roll = number_percent();
    if ( roll < chance
    &&   !saves_spell(effective_level(ch, gsn_sap) * 11/10, victim, DAM_BASH) )
    {
	if ( roll > 75 )
	{
	    AFFECT_DATA af;
	    
	    act_color( "A deadly blow to the back of your head sends you collapsing into darkness!",
	        ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	    act_color( "Your blow strikes home with deadly force! $N collapses unconscious.",
	        ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	    act_color( "$n strikes $N hard on the back of the head. $N collapses unconscious.",
	        ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
	    
	    af.where		= TO_AFFECTS;
	    af.type		= gsn_sleep;
	    af.level		= effective_level( ch, gsn_sap );
	    af.duration		= TICKS_IN_HOUR;
	    af.location		= APPLY_NONE;
	    af.modifier		= 0;
	    af.bitvector	= AFF_SLEEP;
	    af.procNo		= 0;
	    af.flags		= 0;
	    af.name		= "";
	    af.appearance	= "";
	    affect_join( victim, &af );
	    
	    victim->position = POS_SLEEPING;
	}
	else
	{
	    act_color( "A stunning blow strikes home on the back of your head!",
		ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	    act_color( "Your blow strikes with stunning force!",
	        ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	    act_color( "$n strikes a stunning blow to the back of $Ns head!",
	        ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
	}

	check_improve( ch, gsn_sap, TRUE, 1, TRUE );
	
	DAZE_STATE( victim, 10 * PULSE_VIOLENCE );
	check_dismount( victim );
    }
    else
    {
	act_color( "Oops, you didn't quite get that right.", 
            ch, NULL, NULL, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	act_color( "$n strikes you hard on the back of the head!", 
            ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	act_color( "$n strikes $N hard on the back of the head!", 
            ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
	damage( ch, victim, number_range(5, 10), gsn_sap, DAM_BASH, TRUE, NULL );
	check_improve( ch, gsn_sap, FALSE, 1, TRUE );
    }
    
    return;
}


void do_style( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    
    if ( IS_NPC(ch) )
    {
	send_to_char( "Only players can set their fighting style.\n\r", ch );
	return;
    }
    
    if ( argument[0] == '\0' )
    {
	char *complex;
	
	sprintf( buf, "%sStyles:\n\r",
	    (IS_SET(ch->comm, COMM_COLOR) ? P_COLOR(ch, C_TITLES) : "") );
	send_to_char( buf, ch );
	sprintf( buf, "  Use two hands where possible: %s%s\n\r",
	    (IS_SET(ch->comm, COMM_COLOR) ? P_COLOR(ch, C_INFORMATION) : "" ),
	    (IS_SET(ch->pcdata->style, STYLE_TWO_HANDED) ? "yes" : "no" ) );
	send_to_char( buf, ch );
	sprintf( buf, "    %s<--- style two <yes|no>\n\r\n\r",
	    (IS_SET(ch->comm, COMM_COLOR) ? P_COLOR(ch, C_INFORMATION) : "" ) );
	send_to_char( buf, ch );

	sprintf( buf, "  %sDual weapon style: %s%s\n\r",
	    (IS_SET(ch->comm, COMM_COLOR) ? P_COLOR(ch, C_TITLES) : ""),
	    (IS_SET(ch->comm, COMM_COLOR) ? P_COLOR(ch, C_INFORMATION) : "" ),
	    (IS_SET(ch->pcdata->style, STYLE_DUAL_OFFENSIVE)
	      ? "offensive"
	      : IS_SET(ch->pcdata->style, STYLE_DUAL_RIPOSTE)
	        ? "riposte"
	        : "defensive") );
	send_to_char( buf, ch );
	sprintf( buf, "    %s<--- style dual <offensive|defensive|riposte>\n\r\n\r",
	    (IS_SET(ch->comm, COMM_COLOR) ? P_COLOR(ch, C_INFORMATION) : "" ) );
	send_to_char( buf, ch );
	
	     if ( IS_SET(ch->pcdata->style, STYLE_WINDMILL) )
	    complex = "windmill";
	else if ( IS_SET(ch->pcdata->style, STYLE_VIPER) )
	    complex = "viper";
	else if ( IS_SET(ch->pcdata->style, STYLE_FAN) )
	    complex = "fan";
	else if ( IS_SET(ch->pcdata->style, STYLE_STONE_CLAW) )
	    complex = "stone claw";
	else if ( IS_SET(ch->pcdata->style, STYLE_RAINBOW) )
	    complex = "rainbow";
	else if ( IS_SET(ch->pcdata->style, STYLE_VERTIGO) )
	    complex = "vertigo";
	else
	    complex = "none";
	
	sprintf( buf, "  %sComplex style: %s%s\n\r",
	    (IS_SET(ch->comm, COMM_COLOR) ? P_COLOR(ch, C_TITLES) : ""),
	    (IS_SET(ch->comm, COMM_COLOR) ? P_COLOR(ch, C_INFORMATION) : "" ),
	    complex );
	send_to_char( buf, ch );
	sprintf( buf, "    %s<--- style complex <windmill|viper|fan|stone claw|rainbow|vertigo>%s\n\r\n\r",
	    (IS_SET(ch->comm, COMM_COLOR) ? P_COLOR(ch, C_INFORMATION) : "" ),
	    (IS_SET(ch->comm, COMM_COLOR) ? COLOR_OFF : "") );
	send_to_char( buf, ch );
	
	return;
    }
    
    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );
    
    if ( !str_prefix(arg1, "two") )
    {
	if ( arg2[0] == '\0' )
	{
	    send_to_char( "Please select either 'yes' or 'no'.\n\r", ch );
	    return;
	}
	
	if ( !str_prefix(arg2, "yes") )
	    SET_BIT( ch->pcdata->style, STYLE_TWO_HANDED );
	else if ( !str_prefix(arg2, "no") )
	    REMOVE_BIT( ch->pcdata->style, STYLE_TWO_HANDED );
	else
	{
	    do_style( ch, "two" );
	    return;
	}
	
	send_to_char( "OK.\n\r", ch );
	return;
    }
    else if ( !str_prefix(arg1, "dual") )
    {
	if ( arg2[0] == '\0' )
	{
	    send_to_char( "Please select one of: offensive, defensive, riposte.\n\r", ch );
	    return;
	}
	
	if ( !str_prefix(arg2, "offensive") )
	{
	    REMOVE_BIT( ch->pcdata->style, STYLE_DUAL_RIPOSTE );
	    SET_BIT( ch->pcdata->style, STYLE_DUAL_OFFENSIVE );
	}
	else if ( !str_prefix(arg2, "defensive") )
	{
	    REMOVE_BIT( ch->pcdata->style, STYLE_DUAL_RIPOSTE );
	    REMOVE_BIT( ch->pcdata->style, STYLE_DUAL_OFFENSIVE );
	}
	else if ( !str_prefix(arg2, "riposte") )
	{
	    SET_BIT( ch->pcdata->style, STYLE_DUAL_RIPOSTE );
	    REMOVE_BIT( ch->pcdata->style, STYLE_DUAL_OFFENSIVE );
	}
	else
	{
	    do_style( ch, "dual" );
	    return;
	}

	send_to_char( "OK.\n\r", ch );
	return;
    }
    else if ( !str_prefix(arg1, "complex") )
    {
        REMOVE_BIT( ch->pcdata->style, STYLE_WINDMILL );
        REMOVE_BIT( ch->pcdata->style, STYLE_VIPER );
        REMOVE_BIT( ch->pcdata->style, STYLE_FAN );
        REMOVE_BIT( ch->pcdata->style, STYLE_STONE_CLAW );
        REMOVE_BIT( ch->pcdata->style, STYLE_RAINBOW );
        REMOVE_BIT( ch->pcdata->style, STYLE_VERTIGO );

        if ( arg2[0] == '\0' )
        {
            send_to_char( "Complex style cleared.\n\r", ch );
	    return;
        }

        if ( !str_prefix(arg2, "windmill") )
        {
            if ( has_specialization(ch, gsn_windmill) )
            {
                SET_BIT( ch->pcdata->style, STYLE_WINDMILL );
                send_to_char( "OK.\n\r", ch );
            }
            else
                send_to_char( "You do not know that style.\n\r", ch );
        }
        else if ( !str_prefix(arg2, "viper") )
        {
            if ( has_specialization(ch, gsn_viper) )
            {
                SET_BIT( ch->pcdata->style, STYLE_VIPER );
                send_to_char( "OK.\n\r", ch );
            }
            else
                send_to_char( "You do not know that style.\n\r", ch );
        }
        else if ( !str_prefix(arg2, "fan") )
        {
            if ( has_specialization(ch, gsn_fan) )
            {
                SET_BIT( ch->pcdata->style, STYLE_FAN );
                send_to_char( "OK.\n\r", ch );
            }
            else
                send_to_char( "You do not know that style.\n\r", ch );
        }
        else if ( !str_prefix(arg2, "stoneclaw") )
        {
            if ( has_specialization(ch, gsn_stone_claw) )
            {
                SET_BIT( ch->pcdata->style, STYLE_STONE_CLAW );
                send_to_char( "OK.\n\r", ch );
            }
            else
                send_to_char( "You do not know that style.\n\r", ch );
        }
        else if ( !str_prefix(arg2, "rainbow") )
        {
            if ( has_specialization(ch, gsn_rainbow) )
            {
                SET_BIT( ch->pcdata->style, STYLE_RAINBOW );
                send_to_char( "OK.\n\r", ch );
            }
            else
                send_to_char( "You do not know that style.\n\r", ch );
        }
        else if ( !str_prefix(arg2, "vertigo") )
        {
            if ( has_specialization(ch, gsn_vertigo) )
            {
                SET_BIT( ch->pcdata->style, STYLE_VERTIGO );
                send_to_char( "OK.\n\r", ch );
            }
            else
                send_to_char( "You do not know that style.\n\r", ch );
        }
        else
            send_to_char( "That is not a valid complex style.\n\r", ch );
        return;
    }
    else
    {
	send_to_char( "Invalid option.\n\r", ch );
	return;
    }
}


void do_bite( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    int chance;
    int roll;

    one_argument( argument, arg );
 
    if ( (chance = get_skill(ch, gsn_bite)) == 0
    ||   IS_NPC(ch)
    ||   ch->level < skill_level(ch, gsn_bite) )
    {	
	send_to_char("Biting? What's that?\n\r",ch);
	return;
    }

    if ( IS_AFFECTED2(ch, AFF2_PACIFIST) )
    {
	send_to_char( "You don't quite feel that violent.", ch );
	return;
    }

    if ( arg[0] == '\0' )
    {
	victim = ch->fighting;
	if ( victim == NULL )
	{
	    send_to_char( "But you aren't fighting anyone!\n\r", ch );
	    return;
	}
    }
    else if ( (victim = get_char_room(ch, arg)) == NULL )
    {
	send_to_char( "They aren't here.\n\r", ch );
	return;
    }

    if ( victim == ch )
    {
	send_to_char( "You spin around and try to bite your tail, but you just can't catch it.\n\r", ch );
        act( "$n spins round and round, trying to bite $s tail!",
            ch, NULL, NULL, TO_ROOM );
	return;
    }

    if ( is_safe(ch, victim) )
	return;

    if ( IS_NPC(victim) && 
	victim->fighting != NULL && 
	!is_same_group(ch, victim->fighting))
    {
        send_to_char( "Kill stealing is not permitted.\n\r", ch );
        return;
    }

    if ( IS_AFFECTED(ch, AFF_CHARM) && ch->master == victim )
    {
	act( "But $N is your friend!", ch, NULL, victim, TO_CHAR );
	return;
    }

    /* modifiers */

    /* size  and weight */
    if ( IS_SET(CHAR_FORM(victim), FORM_MIST) )
	chance -= 15;
    if ( IS_SET(CHAR_FORM(victim), FORM_INTANGIBLE) )
	chance -= 20;
    
    /* speed */
    if ( IS_SET(ch->off_flags, OFF_FAST) || IS_AFFECTED(ch, AFF_HASTE) )
        chance += 10;
    if ( IS_SET(victim->off_flags, OFF_FAST) || IS_AFFECTED(victim, AFF_HASTE) )
        chance -= 30;

    /* level */
    chance += (effective_level(ch, gsn_bite) - victim->level);
    
    /* now the attack */
    if ( (roll = number_percent()) < chance )
    {
	act_color( "$n sinks $s fangs deep into $N!",
	    ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	act_color( "You sink your fangs deep into $N!",
            ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	act_color( "$n sinks $s fangs deep into $N!",
	    ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
	check_improve( ch, gsn_bite, TRUE, 1, TRUE );

	WAIT_STATE( ch, skill_table[gsn_bite].beats );

	damage( ch, victim,
	        number_range( 1, 5 ),
	        gsn_bite, DAM_PIERCE, FALSE, NULL );

        spell_slow( 
            gsn_slow, effective_level(ch, gsn_bite), 
            ch, (void *) victim,
            TARGET_CHAR );

	if ( !CHAR_KILLED(victim) )
	    check_dismount( victim );
    }
    else
    {
	damage( ch, victim, 0, gsn_bite, DAM_PIERCE, FALSE, NULL );
	act_color( "You snap your teeth together menacingly!",
	    ch, NULL, victim, TO_CHAR, POS_RESTING, C_COMBAT_SELF );
	act_color( "$n snaps $s teeth together, $s bite narrowly missing $N!",
	    ch, NULL, victim, TO_NOTVICT, POS_RESTING, OTHER_COLOR(ch) );
	act_color( "$n snaps $s teeth together, $s bite narrowly missing you!",
	    ch, NULL, victim, TO_VICT, POS_RESTING, OTHER_COLOR(ch) );
	check_improve( ch, gsn_bite, FALSE, 1, TRUE );
	WAIT_STATE( ch, skill_table[gsn_bite].beats * 3/2 );
    }
}

// launch an arrow
void do_fire( CHAR_DATA *ch, char *argument )
{
    char arg[MIL];
    CHAR_DATA *victim = NULL;
    OBJ_DATA *arrow;
    OBJ_DATA *bow;
    ROOM_INDEX_DATA *was_in_room;
    EXIT_DATA *pexit;
    int dam, door, chance;

    bow = get_eq_char(ch, WEAR_WIELD);
    if (bow == NULL) {
	send_to_char("You consider simply throwing the arrow, but it doesn't seem likely to have much of an effect.\n\r", ch);
	return;
    }

    if (bow->value[0] != WEAPON_BOW) {
	send_to_char("Your chances of firing an arrow seem much better when using a bow.\n\r", ch);
	return;
    }

    one_argument( argument, arg );
    if ( arg[0] == '\0' && ch->fighting == NULL ) {
	send_to_char( "Who do you want to fire at?\n\r", ch );
	return;
    }

    if (!str_cmp(arg, "none") || !str_cmp(arg, "self") || victim == ch) {
	send_to_char( "No, no, no. Suicide is baaaaad.\n\r", ch );
	return;
    }

    if ( ( arrow = get_eq_char( ch, WEAR_HOLD ) ) == NULL ) {
	send_to_char( "You aren't holding anything to fire.\n\r", ch );
	return;
    }

    if ( arrow->item_type != ITEM_ARROW ) {
	send_to_char( "You can only fire arrows.\n\r", ch );
	return;
    }

    if ( arg[0] == '\0' ) {
	if ( ch->fighting != NULL )
	    victim = ch->fighting;
	else {
	    send_to_char("Fire at whom or what?\n\r", ch);
	    return;
	}
    } else {
	if ( ( victim = get_char_room( ch, arg ) ) == NULL) {
	    was_in_room = ch->in_room;

	    for( door = 0; door <= 5 && victim == NULL; door++ ) {
		if ( ( pexit = was_in_room->exit[door] ) != NULL
		    && pexit->u1.to_room != NULL
		    && pexit->u1.to_room != was_in_room ) {
		    ch->in_room = pexit->u1.to_room;
		    victim = get_char_room ( ch, arg );		
		    }
	    }

	    ch->in_room = was_in_room;
	    if ( victim == NULL ) {
		send_to_char( "You can't find it.\n\r", ch);
		return;
	    } else {
		if (IS_SET(pexit->exit_info, EX_CLOSED)) {
		    send_to_char("You can't fire through a door.\n\r", ch);
		    return;
		}
	    }
	}
    }

    if ((ch->in_room) == (victim->in_room)) {
	send_to_char("Your target is too close to fire at.\n\r", ch);
	return;
    }

    WAIT_STATE( ch, 2 * PULSE_VIOLENCE );

    if (arrow->item_type == ITEM_ARROW ) {
	if ( victim != NULL ) {
	    act("$n fires $p at $N.", ch, arrow, victim, TO_NOTVICT );
	    act("You fire $p at $N.", ch, arrow, victim, TO_CHAR );
	    act("$n fires $p at you.", ch, arrow, victim, TO_VICT );
	}

    if ( ch->level < arrow->level || number_percent() >= 20 + get_skill(ch, gsn_bow) * 4/5 ) {
	act("$p flies from your bow, but misses, and lands harmlessly on the ground.", ch, arrow, NULL, TO_CHAR);
	act("$p flies from $n's bow, but misses, and lands harmlessly on the ground.", ch, arrow, NULL, TO_ROOM);
	obj_from_char( arrow );
	obj_to_room(arrow, victim->in_room);
	check_improve(ch, gsn_bow, FALSE, 2, TRUE);
    } else {
	chance = dice(1, 10);
	switch(chance) {
	    case 1:
	    case 2:
	    case 3:
	    case 4:
	    case 5:
	    case 6:
		obj_from_char(arrow);
		obj_to_char(arrow, victim);
		arrow->wear_flags = 2097153;
		wear_obj(victim, arrow, TRUE);
		arrow->wear_flags = 16385;
		arrow->extra_flags = arrow->extra_flags + 268435456;
		dam = dice(arrow->value[1], arrow->value[2]);
		damage( ch, victim, dam, gsn_bow, DAM_PIERCE, TRUE, arrow);
		check_improve(ch, gsn_bow, TRUE, 2, TRUE);
		break;

	    case 7:
	    case 8:
	    case 9:
		obj_from_char(arrow);
		obj_to_char(arrow, victim);
		arrow->wear_flags = 4194305;
		wear_obj(victim, arrow, TRUE);
		arrow->wear_flags = 16385;
		arrow->extra_flags = arrow->extra_flags + 268435456;
		dam = (dice(arrow->value[1], arrow->value[2]) * 1.5);
		damage( ch, victim, dam, gsn_bow, DAM_PIERCE, TRUE, arrow);
		check_improve(ch, gsn_bow, TRUE, 2, TRUE);
		break;

	    case 10:
		obj_from_char(arrow);
		obj_to_char(arrow, victim);
		arrow->wear_flags = 8388609;
		wear_obj(victim, arrow, TRUE);
		arrow->wear_flags = 16385;
		arrow->extra_flags = arrow->extra_flags + 268435456;
		dam = (dice(arrow->value[1], arrow->value[2]) * 2);
		damage( ch, victim, dam, gsn_bow, DAM_PIERCE, TRUE, arrow);
		check_improve(ch, gsn_bow, TRUE, 2, TRUE);
		break;
	    }
	}
    }
    return;
}


void daze_state( CHAR_DATA *victim, int numPulses )
{
    if ( has_customization(victim, ADVANTAGE_HIGH_PAIN_THRESHOLD) )
    {
        numPulses /= 2;
        numPulses = UMAX( 1, numPulses );
    }
    if ( has_customization(victim, DISADVANTAGE_LOW_PAIN_THRESHOLD) )
        numPulses *= 2;
    victim->daze = UMAX( victim->daze, numPulses );
}
// EOF

