/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "merc.h"
#include "olc.h"
#include "magic.h"
#include "tables.h"
#include "recycle.h"


DECLARE_DO_FUN(do_rescue);
DECLARE_DO_FUN(do_scan);
DECLARE_DO_FUN(do_wake);
DECLARE_DO_FUN(do_kill);
DECLARE_DO_FUN(do_look);
DECLARE_DO_FUN(do_flee);


int    find_door    args( ( CHAR_DATA *ch, char *arg ) );


extern char *target_name;


bool is_unique( int vnum, const char *name )
{
    OBJ_DATA *current;

    for ( current = get_obj_index(vnum)->instances;
          current != NULL;
          current = current->next_instance )
    {
        if ( !IS_NULLSTR(current->owner)
        &&   !str_cmp(current->owner, name) )
            return FALSE;
    }

    return TRUE;
}


bool is_mob_unique( int vnum, CHAR_DATA *ch )
{
    CHAR_DATA *current;

    for ( current = get_mob_index(vnum)->instances;
          current != NULL;
          current = current->next_instance )
    {
        if ( current->master == ch )
            return FALSE;
    }

    return TRUE;
}


OBJ_DATA *find_unique( int vnum, const char *name )
{
    OBJ_DATA *current;

    for ( current = get_obj_index(vnum)->instances;
          current != NULL;
          current = current->next_instance )
    {
        if ( !IS_NULLSTR(current->owner)
        &&   !str_cmp(current->owner, name) )
            return current;
    }

    return NULL;
}


void taint( CHAR_DATA *victim, int chance )
{
    AFFECT_DATA af;
    int tries;

    if ( IS_SET(race_table[victim->race].char_flags, CHAR_CAMBION) )
        chance /= 2;

    if ( number_percent() > chance )
        return;

    af.where        = TO_AFFECTS;
    af.level        = victim->level;
    af.duration     = -1;
    af.location     = APPLY_NONE;
    af.modifier     = 0;
    af.bitvector    = 0;
    af.flags        = 0;
    af.procNo       = 0;
    af.name         = "";
    af.appearance   = "";

    for ( tries = 0; tries < 20; tries++  ) {
        switch( number_range(1, 20) )
        {
            case 1:
            if ( is_affected(victim, gsn_aging) )
                break;

            send_to_char(
                "You near collapse from the pain as your body is aged.",
                victim );

            af.type        = gsn_aging;
            af.modifier    = -4;

            af.location = APPLY_STR;
            affect_to_char( victim, &af );
            af.location = APPLY_INT;
            affect_to_char( victim, &af );
            af.location = APPLY_WIS;
            affect_to_char( victim, &af );
            af.location = APPLY_DEX;
            affect_to_char( victim, &af );
            af.location = APPLY_CON;
            affect_to_char( victim, &af );
            af.location = APPLY_PER;
            affect_to_char( victim, &af );
            af.location = APPLY_CHA;
            affect_to_char( victim, &af );
            return;

            case 2:
            if ( is_affected(victim, gsn_cadaverous_appearance) )
                break;

            af.type        = gsn_cadaverous_appearance;
            af.location    = APPLY_CHA;
            af.modifier    = -16;

            if ( !IS_NPC(victim) )
            {
                af.appearance    = "%n %p, %s skin deathly pale.";
                af.flags        = AFF_PRI_LOW;
            }

            affect_to_char( victim, &af );
            return;

            case 3:
            if ( is_affected(victim, gsn_light_intolerance) )
                break;

            af.type        = gsn_light_intolerance;
            af.flags    = AFF_CONT_UPDATE;
            af.procNo    = affect_proc_lookup( "light intolerance" );
            affect_to_char( victim, &af );

            af.where    = TO_VULN;
            af.bitvector    = IMM_LIGHT;
            af.flags    = 0;
            af.procNo    = 0;
            affect_to_char( victim, &af );
            return;

            case 4:
            if ( is_affected(victim, gsn_agrophobia) )
                break;

            af.type        = gsn_agrophobia;
            af.flags    = AFF_CONT_UPDATE;
            af.procNo    = affect_proc_lookup( "agrophobia" );
            affect_to_char( victim, &af );
            return;

            case 5:
            if ( is_affected(victim, gsn_catatonia) )
                break;

            af.type        = gsn_catatonia;
            af.flags    = AFF_CONT_UPDATE;
            af.procNo    = affect_proc_lookup( "catatonia" );
            affect_to_char( victim, &af );
            return;

            case 6:
            if ( is_affected(victim, gsn_claustrophobia) )
                break;

            af.type        = gsn_claustrophobia;
            af.flags    = AFF_CONT_UPDATE;
            af.procNo    = affect_proc_lookup( "claustrophobia" );
            affect_to_char( victim, &af );
            return;

            case 7:
            if ( is_affected(victim, gsn_melancholia) )
                break;

            af.type        = gsn_melancholia;
            af.flags    = AFF_CONT_UPDATE;
            af.procNo    = affect_proc_lookup( "melancholia" );
            affect_to_char( victim, &af );
            return;

            case 8:
            if ( is_affected(victim, gsn_dipsomania) )
                break;

            af.type        = gsn_dipsomania;
            af.flags    = AFF_CONT_UPDATE;
            af.procNo    = affect_proc_lookup( "dipsomania" );
            affect_to_char( victim, &af );
            return;

            case 9:
            if ( is_affected(victim, gsn_introversion) )
                break;

            af.type        = gsn_introversion;
            af.location    = APPLY_CHA;
            af.modifier    = -16;
            affect_to_char( victim, &af );
            return;

            case 10:
            if ( is_affected(victim, gsn_mania) )
                break;

            af.type        = gsn_mania;
            af.location    = APPLY_DEX;
            af.modifier    = 4;
            affect_to_char( victim, &af );

            af.location    = APPLY_CHA;
            af.modifier    = -8;
            affect_to_char( victim, &af );
            return;
        }
    }
}


void taint_light_intolerance( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    AFFECT_DATA af;

    if ( ch == NULL )
        return;

    if ( reason != AFF_PROC_PERIODIC )
        return;

    if ( ch->in_room == NULL )
        return;

    if ( room_is_dark(ch, ch->in_room) )
        return;

    if ( ch->in_room->light < 3 )
        return;

    if ( !IS_SET(ch->in_room->room_flags, ROOM_INDOORS)
    &&   (weather_info.sunlight == SUN_SET
    ||    weather_info.sunlight == SUN_DARK) )
        return;

    if ( saves_spell(10, ch, DAM_LIGHT) )
        return;

    if ( paf->next && paf->next->type == gsn_light_intolerance )
        return;

    af.where        = TO_AFFECTS;
    af.type         = gsn_light_intolerance;
    af.level        = ch->level;
    af.duration     = 0;
    af.modifier     = -4;
    af.location     = APPLY_STR;
    af.bitvector    = 0;
    af.flags        = 0;
    af.procNo       = 0;
    af.name         = "";
    af.appearance   = "";
    affect_to_char_after( ch, &af, paf );

    af.location     = APPLY_DEX;
    affect_to_char_after( ch, &af, paf );

    af.location     = APPLY_PER;
    affect_to_char_after( ch, &af, paf );

    send_to_char(
        "A dull ache fills your head, the light here too bright.", ch );

    return;
}


void taint_agrophobia( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    AFFECT_DATA af;

    if ( ch == NULL )
        return;

    if ( reason != AFF_PROC_PERIODIC )
        return;

    if ( ch->in_room == NULL )
        return;

    if ( IS_SET(ch->in_room->room_flags, ROOM_INDOORS) )
        return;

    if ( paf->next && paf->next->type == gsn_agrophobia )
        return;

    af.where           = TO_AFFECTS;
    af.type            = gsn_agrophobia;
    af.level           = ch->level;
    af.duration        = 0;
    af.modifier        = -4;
    af.location        = APPLY_INT;
    af.bitvector       = 0;
    af.flags           = 0;
    af.procNo          = 0;
    af.name            = "";
    af.appearance      = "";
    affect_to_char_after( ch, &af, paf );

    af.location        = APPLY_WIS;
    affect_to_char_after( ch, &af, paf );

    send_to_char(
        "Terror fills you, to lack of surrounding walls too much to bear.\n\r",
        ch );
}


void taint_catatonia( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    AFFECT_DATA af;

    if ( ch == NULL )
        return;

    if ( reason != AFF_PROC_PERIODIC )
        return;

    if ( ch->position == POS_SLEEPING )
        return;

    if ( number_bits(4) != 0 )
        return;

    if ( paf->next && paf->next->type == gsn_catatonia )
        return;

    af.where        = TO_AFFECTS;
    af.type         = gsn_catatonia;
    af.level        = ch->level;
    af.duration     = 0;
    af.modifier     = 0;
    af.location     = APPLY_NONE;
    af.bitvector    = AFF_SLEEP;
    af.flags        = 0;
    af.procNo       = 0;
    af.name         = "";
    af.appearance   = "";
    affect_to_char_after( ch, &af, paf );

    send_to_char( "Everything suddenly goes black.\n\r", ch );
    act( "$n collapses unconcious.", ch, NULL, NULL, TO_ROOM );
    ch->position = POS_SLEEPING;
}


void taint_claustrophobia( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    AFFECT_DATA af;

    if ( ch == NULL )
        return;

    if ( reason != AFF_PROC_PERIODIC )
        return;

    if ( ch->in_room == NULL )
        return;

    if ( !IS_SET(ch->in_room->room_flags, ROOM_INDOORS) )
        return;

    if ( paf->next && paf->next->type == gsn_claustrophobia )
        return;

    af.where        = TO_AFFECTS;
    af.type         = gsn_claustrophobia;
    af.level        = ch->level;
    af.duration     = 0;
    af.modifier     = -4;
    af.location     = APPLY_INT;
    af.bitvector    = 0;
    af.flags        = 0;
    af.procNo       = 0;
    af.name         = "";
    af.appearance   = "";
    affect_to_char_after( ch, &af, paf );

    af.location     = APPLY_WIS;
    affect_to_char_after( ch, &af, paf );

    send_to_char(
        "You are filled with a feeling of dread, the walls too close!\n\r",
        ch );
}


void taint_melancholia( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    AFFECT_DATA af;

    if ( ch == NULL )
        return;

    if ( reason != AFF_PROC_PERIODIC )
        return;

    if ( number_bits(4) != 0 )
        return;

    if ( paf->next && paf->next->type == gsn_melancholia )
        return;

    stop_fighting( ch, FALSE );

    af.where        = TO_AFFECTS;
    af.type         = gsn_melancholia;
    af.level        = ch->level;
    af.duration     = 0;
    af.modifier     = -4;
    af.location     = APPLY_HITROLL;
    af.bitvector    = AFF_CALM;
    af.flags        = 0;
    af.procNo       = 0;
    af.name         = "";
    af.appearance   = "";
    affect_to_char_after( ch, &af, paf );

    af.location     = APPLY_DAMROLL;
    affect_to_char_after( ch, &af, paf );

    send_to_char(
        "You are filled with a sense of calm, the worries of the world far away.\n\r",
        ch );
}


void taint_dipsomania( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    AFFECT_DATA af;

    if ( ch == NULL || IS_NPC(ch) )
        return;

    if ( reason != AFF_PROC_PERIODIC )
        return;

    if ( ch->in_room == NULL )
        return;

    if ( ch->pcdata->condition[COND_DRUNK] != 0 )
    {
        AFFECT_DATA *raf, *raf_next;

        for ( raf = ch->affected; raf != NULL; raf = raf->next )
        {
            raf_next = raf->next;

            if ( raf->type == gsn_dipsomania )
                affect_remove( ch, raf );
        }

        return;
    }
    
    if ( paf->next && paf->next->type == gsn_dipsomania )
        return;

    af.where        = TO_AFFECTS;
    af.type         = gsn_dipsomania;
    af.level        = ch->level;
    af.duration     = 0;
    af.modifier     = -4;
    af.location     = APPLY_STR;
    af.bitvector    = 0;
    af.flags        = 0;
    af.procNo       = 0;
    af.name         = "";
    af.appearance   = "";
    affect_to_char_after( ch, &af, paf );

    af.location     = APPLY_INT;
    affect_to_char_after( ch, &af, paf );

    af.location     = APPLY_WIS;
    affect_to_char_after( ch, &af, paf );

    af.location     = APPLY_DEX;
    affect_to_char_after( ch, &af, paf );

    af.location     = APPLY_CON;
    affect_to_char_after( ch, &af, paf );

    af.location     = APPLY_PER;
    affect_to_char_after( ch, &af, paf );

    af.location     = APPLY_CHA;
    affect_to_char_after( ch, &af, paf );

    send_to_char(
        "A strong craving for a drink fills you... must have...\n\r",
        ch );
}


void nausea_affect_proc( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    if ( reason == AFF_PROC_PERIODIC )
    {
        if ( ch != NULL && !IS_AFFECTED(ch, AFF_SLOW) )
        {
            act( "$n shudders uncontrollably and looks ill.", ch, NULL, NULL, TO_ROOM );
            send_to_char( "You shudder uncontrollably as a sudden wave of nausea threatens to overwhelm you.\n\r", ch );

            damage_old( ch, ch, paf->level / 10 + 1,
                        gsn_poison, DAM_POISON, FALSE,
                        NULL );
        }
    }

    return;
}


void bleeding_affect_proc( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    if ( reason == AFF_PROC_PERIODIC )
    {
        if ( ch != NULL && !IS_AFFECTED(ch, AFF_SLOW) )
        {
            act( "$n is bleeding profusely.", ch, NULL, NULL, TO_ROOM );
            send_to_char( "You are bleeding profusely.\n\r", ch );

            damage_old( ch, ch, paf->level / 4 + 1,
                        gsn_poison, DAM_POISON, FALSE,
                        NULL );
        }
    }

    return;
}


/*
 * Spells.
 */
void spell_eyes_of_the_undead( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( IS_AFFECTED(victim, AFF_DARK_VISION) )
    {
        if (victim == ch)
            send_to_char( "You already possess night vision.\n\r", ch );
        else
            act( "$N already has night vision.", ch, NULL, victim, TO_CHAR );
        return;
    }

    af.where        = TO_AFFECTS;
    af.type         = sn;
    af.level        = level;
    af.duration     = level;
    af.modifier     = 0;
    af.location     = APPLY_NONE;
    af.bitvector    = AFF_DARK_VISION;
    af.procNo       = 0;
    af.flags        = 0;
    af.name         = "";
    af.appearance   = "";
    affect_to_char( victim, &af );

    send_to_char( "Your eyes burn briefly, everything turning slightly gray.\n\r", victim );
    if ( ch != victim )
        send_to_char( "Ok.\n\r", ch );
}


#if 0
void spell_farsight( int sn, int level, CHAR_DATA *ch, void *vo,int target)
{
    if (IS_AFFECTED(ch,AFF_BLIND))
    {
        send_to_char("Maybe it would help if you could see?\n\r",ch);
        return;
    }

    do_scan(ch,target_name);
}
#endif


void spell_antifreeze( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( is_affected(victim, sn) )
    {
        if (victim == ch)
            send_to_char(
                "You are already resistant to cold.\n\r", ch );
        else
            act( "$N is already resistant to cold.",
                ch, NULL, victim, TO_CHAR );
        return;
    }

    af.where        = TO_RESIST;
    af.type         = sn;
    af.level        = level;
    af.duration     = 24;
    af.modifier     = 0;
    af.location     = APPLY_NONE;
    af.bitvector    = IMM_COLD;
    af.procNo       = 0;
    af.flags        = 0;
    af.name         = "";
    af.appearance   = "";
    affect_to_char( victim, &af );

    send_to_char(
        "The blood flowing through your veins seems to warm momentarily.\n\r",
        victim );
    if ( ch != victim )
        act( "$N is granted resistance to cold.", ch, NULL, victim, TO_CHAR );
}


void spell_animal_call( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    char buf[ MAX_STRING_LENGTH ];
    char arg[ MAX_INPUT_LENGTH ];
    int door = BFS_NO_PATH;
    CHAR_DATA *victim;
    int number, count;
    AFFECT_DATA af;
    MEM_DATA *mem;

    if ( !str_cmp(target_name, "self") || !str_cmp(target_name, ch->name) )
    {
        send_to_char( "Very funny.\n\r", ch );
        return;
    }

    number = number_argument( target_name, arg );
    count  = 0;

    for ( victim = char_list; victim != NULL; victim = victim->next )
    {
        if ( !can_see(ch, victim)
        ||   !is_name(arg, CHAR_NAME(victim))
        ||   !IS_NPC(victim)
        ||   !IS_SET(CHAR_FORM(victim), FORM_ANIMAL)
        ||   IS_AFFECTED(victim, AFF_ANIMAL_CALL) )
            continue;

        door = find_first_step(
            victim,
            (void *) ch->id,
            IS_NPC(ch) ? TRACK_NPC_ID_BLIND : TRACK_PC_ID_BLIND,
            20,
            IS_AFFECTED(victim, AFF_FLYING) ? TRUE : FALSE,
            TRUE,
            IS_AFFECTED(victim, AFF_SWIM) ? TRUE : FALSE );

        if ( door == BFS_ERROR || door == BFS_NO_PATH )
            continue;

        if ( ++count == number )
            break;
    }

    if ( door == BFS_FOUND )
    {
        send_to_char( "They're already here!\n\r", ch );
        return;
    }

    if ( victim == NULL )
    {
        send_to_char( "You sense nothing.\n\r", ch );
        return;
    }

    if ( victim == NULL
    ||   IS_SET(victim->act, ACT_SENTINEL)
    ||   IS_SET(victim->in_room->room_flags, ROOM_SAFE)
    ||   IS_SET(victim->in_room->room_flags, ROOM_PRIVATE)
    ||   IS_SET(victim->in_room->room_flags, ROOM_SOLITARY)
    ||   victim->level > level + 5
    ||   victim->fighting != NULL
    ||   IS_SET(CHAR_IMM(victim), IMM_SUMMON)
    ||   IS_SET(victim->act, ACT_PET)
    ||   (victim->master != NULL && victim->master != ch)
    ||   saves_spell(level, victim, DAM_OTHER) )
    {
        send_to_char( "You failed.\n\r", ch );
        return;
    }

    mem              = new_mem_data();
    mem->next        = victim->memory;
    mem->id          = ch->id;
    mem->reaction    = IS_NPC(ch) ? MEM_SUMMONED_NPC : MEM_SUMMONED_PC;
    mem->duration    = level / 10;
    victim->memory   = mem;

    af.where         = TO_AFFECTS;
    af.type          = sn;
    af.level         = level;
    af.duration      = level / 10;
    af.modifier      = 0;
    af.location      = APPLY_NONE;
    af.bitvector     = AFF_ANIMAL_CALL;
    af.procNo        = 0;
    af.flags         = 0;
    af.name          = "";
    af.appearance    = "";
    affect_to_char( victim, &af );

    send_to_char( "You hear a distant cry as your call is answered!\n\r", ch );

    sprintf( buf, "$N is answering a call from %s",
        IS_NPC(ch) ? ch->short_descr : ch->name );
    wiznet( buf, victim, NULL, WIZ_MOBACTIONS, 0, 0 );
}


void spell_avatar( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA af;

    if ( is_affected(victim, sn) )
    {
        if (victim == ch)
            send_to_char(
                "You are already infused with divine power!\n\r", ch );
        else
            act( "$N is already infused with divine power!",
                ch, NULL, victim, TO_CHAR );
        return;
    }

    af.where        = TO_AFFECTS;
    af.type         = sn;
    af.level        = level;
    af.duration     = 1;
    af.modifier     = 10;
    af.location     = APPLY_HITROLL;
    af.bitvector    = 0;
    af.procNo       = 0;
    af.flags        = AFF_EXTENDED;
    af.name         = "";
    af.appearance   = "";
    affect_to_char( victim, &af );

    af.modifier     = -50;
    af.location     = APPLY_AC;
    affect_to_char( victim, &af );

    af.modifier     = 8;
    af.location     = APPLY_STR;
    affect_to_char( victim, &af );

    af.modifier     = 8;
    af.location     = APPLY_DEX;
    affect_to_char( victim, &af );

    af.where        = TO_IMMUNE;
    af.modifier     = 0;
    af.location     = APPLY_NONE;
    af.bitvector    = IMM_NEGATIVE;
    affect_to_char( victim, &af );

    sprintf( buf, "The power of %s courses through your veins!\n\r",
        religion_table[GET_GOD(ch)].head );
    send_to_char( buf, victim );
    if ( ch != victim )
    {
        sprintf( buf, "$N is granted the power of %s!",
            religion_table[GET_GOD(ch)].head );
        act( buf, ch, NULL, victim, TO_CHAR );
    }
}


void spell_awakening( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA *paf, *paf_next;

    if ( IS_AWAKE(victim) )
    {
        send_to_char( "They're already awake.\n\r", ch );
        return;
    }

    for ( paf = victim->affected; paf != NULL; paf = paf_next )
    {
        int chance;

        paf_next = paf->next;
        while ( paf_next != NULL && paf_next->type == paf->type )
            paf_next = paf_next->next;

        chance = UMAX( 5, 50 + 5 * (level - paf->level) );

        if ( paf->bitvector == AFF_SLEEP
        &&   number_percent() <= chance )
        {
            affect_strip( victim, paf->type );
        }
    }

    if ( IS_AFFECTED(victim, AFF_SLEEP) )
    {
        send_to_char( "You failed.\n\r", ch );
        return;
    }

    do_wake( victim, "" );
}


void spell_befriend( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( is_safe(ch, victim) )
        return;

    if ( !IS_SET(CHAR_FORM(victim), FORM_ANIMAL) )
    {
        send_to_char( "You can only befriend animals.\n\r", ch );
        return;
    }

    if ( victim == ch )
    {
        send_to_char( "You like yourself even better!\n\r", ch );
        return;
    }

    level = level - 5 + get_curr_stat(ch, STAT_CHA) / 10;

    if ( IS_AFFECTED(victim, AFF_CHARM)
    ||   IS_AFFECTED(ch, AFF_CHARM)
    ||   level < victim->level
    ||   IS_SET(CHAR_IMM(victim), IMM_CHARM)
    ||   saves_spell(level, victim, DAM_CHARM) )
        return;

    if ( IS_SET(victim->in_room->room_flags, ROOM_LAW) )
    {
        send_to_char(
            "The mayor does not allow charming in the city limits.\n\r",ch);
        return;
    }

    if ( victim->master )
        stop_follower( victim );
    add_follower( victim, ch );
    victim->leader = ch;

    af.where            = TO_AFFECTS;
    af.type             = sn;
    af.level            = level;
    af.duration         = number_fuzzy( level / 4 );
    af.location         = 0;
    af.modifier         = 0;
    af.bitvector        = AFF_CHARM;
    af.procNo           = 0;
    af.flags            = 0;
    af.name             = "";
    af.appearance       = "";
    affect_to_char( victim, &af );

    act( "Isn't $n just so nice?", ch, NULL, victim, TO_VICT );
    if ( ch != victim )
        act( "$N looks at you with adoring eyes.", ch, NULL, victim, TO_CHAR );
    return;
}


void spell_blade_of_darkness( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *blade = (OBJ_DATA *) vo;
    AFFECT_DATA af;

    if ( blade->pIndexData->vnum != OBJ_VNUM_SHADOWBLADE )
    {
        send_to_char( "That isn't a shadowblade!\n\r", ch );
        return;
    }

    if ( IS_NULLSTR(blade->owner) || str_cmp(blade->owner, ch->name) )
    {
        send_to_char( "You can only cast that on your own shadowblade.\n\r", ch );
        return;
    }

    if ( IS_WEAPON_STAT(blade, WEAPON_FLAMING)
    ||   IS_WEAPON_STAT(blade, WEAPON_FROST)
    ||   IS_WEAPON_STAT(blade, WEAPON_VAMPIRIC)
    ||   IS_WEAPON_STAT(blade, WEAPON_SHARP)
    ||   IS_WEAPON_STAT(blade, WEAPON_VORPAL)
    ||   IS_WEAPON_STAT(blade, WEAPON_POISON)
    ||   IS_WEAPON_STAT(blade, WEAPON_SHOCKING) )
    {
        send_to_char( "Nothing seems to happen.", ch );
        return;
    }

    if ( IS_WEAPON_STAT(blade, WEAPON_FADING) )
    {
        act( "$p is already shrouded in darkness.",
            ch, blade, NULL, TO_CHAR );
        return;
    }

    af.where            = TO_WEAPON;
    af.type             = sn;
    af.level            = level / 2;
    af.duration         = level / 5;
    af.location         = 0;
    af.modifier         = 0;
    af.bitvector        = WEAPON_FADING;
    af.procNo           = 0;
    af.flags            = 0;
    af.name             = "";
    af.appearance       = "";
    affect_to_obj( blade, &af );

    act( "You shroud $p in darkness.", ch, blade, NULL, TO_CHAR );
    act( "$n shrouds $p in darkness.", ch, blade, NULL, TO_ROOM );
}


void spell_cloud_of_darkness( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *rch;
    AFFECT_DATA af;

    act( "You summon forth a great cloud of darkness.",
         ch, NULL, NULL, TO_CHAR );
    act( "$n summons forth a great cloud of darkness.",
         ch, NULL, NULL, TO_ROOM );

    for ( rch = ch->in_room->people; rch != NULL; rch = rch->next_in_room )
    {
        if ( rch == ch
        ||   IS_AFFECTED2(rch, AFF2_DARKNESS_CLOUD)
        ||   !can_see(ch, rch)
        ||   !is_same_group(ch, rch) )
            continue;

        af.where            = TO_AFFECTS2;
        af.type             = sn;
        af.level            = level;
        af.duration         = level / 5;
        af.location         = APPLY_HITROLL;
        af.modifier         = -4;
        af.bitvector        = AFF2_DARKNESS_CLOUD;
        af.procNo           = 0;
        af.flags            = 0;
        af.name             = "";
        af.appearance       = "";
        affect_to_char( rch, &af );
    }
}


void spell_create_focus( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *orig_focus, *obj = (OBJ_DATA *) vo;
    bool found = FALSE;
    int power_level;

    for ( orig_focus = ch->carrying;
          orig_focus != NULL;
          orig_focus = orig_focus->next_content )
    {
        if ( orig_focus->item_type == ITEM_FOCUS
        &&   orig_focus->wear_loc  != WEAR_NONE )
        {
            found = TRUE;
            break;
        }
    }

    /* If you cast without an existing focus... you suffer! */
    if ( !found )
    {
        AFFECT_DATA af;

        send_to_char( "An intense pain shoots through your body, "
                      "the power required too much for you!\n\r", ch );
        ch->hit         = 1;
        ch->move        = 10;

        af.where        = TO_AFFECTS;
        af.type         = sn;
        af.level        = level;
        af.duration     = 24;
        af.location     = APPLY_MANA;
        af.modifier     = -ch->max_mana;
        af.bitvector    = 0;
        af.procNo       = 0;
        af.flags        = 0;
        af.name         = "magical burnout";
        af.appearance   = "";
        affect_to_char( ch, &af );

        return;
    }

    if ( obj->item_type != ITEM_GEM )
        act( "You cannot make a focus out of $p.", ch, obj, NULL, TO_CHAR );
    else
    {
        power_level = (obj->cost / 400) + (dice(2, 6) - 7) + (ch->level / 10);

        if ( number_percent() > UMIN(95, get_skill(ch, sn)) )
        {
            act( "$p flares brightly and is gone!", ch, obj, NULL, TO_CHAR );
            act( "$p flares brightly and is gone!", ch, obj, NULL, TO_ROOM );
            extract_obj( obj );
        }
        else
        {
            char *aura;

            obj->item_type   = ITEM_FOCUS;
            obj->value[0]    = power_level;
            obj->value[1]    = 0;
            obj->value[2]    = 0;
            obj->value[3]    = 0;
            if ( obj->wear_flags == ITEM_TAKE
            ||   (obj->wear_flags ^ (ITEM_HOLD | ITEM_TAKE)) == 0 )
            {
                obj->wear_flags = ITEM_TAKE | ITEM_HOLD;
            }
            SET_BIT( obj->extra_flags, ITEM_GLOW );

            if ( IS_EVIL(ch) )
                aura = "baleful";
            else if ( IS_GOOD(ch) )
                aura = "golden";
            else
                aura = "bright";

            act( "$p glows with a $T aura!", ch, obj, aura, TO_CHAR );
            act( "$p glows with a $T aura!", ch, obj, aura, TO_ROOM );
        }
    }

    act( "$p flares brightly and is gone, consumed by the power of the spell!",
        ch, orig_focus, NULL, TO_CHAR );
    act( "$p flares brightly and is gone!",
        ch, orig_focus, NULL, TO_ROOM );
    extract_obj( orig_focus );
}


void spell_create_symbol( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *symbol;

    if ( religion_table[GET_GOD(ch)].symbol_vnum == 0 )
    {
        send_to_char( "You failed!", ch );
        return;
    }

    symbol = create_object(
    get_obj_index(religion_table[GET_GOD(ch)].symbol_vnum), 0, FALSE );
    obj_to_char( symbol, ch );

    act( "You create $p!", ch, symbol, NULL, TO_CHAR );
    act( "$n creates $p!", ch, symbol, NULL, TO_ROOM );

    ch->hit     = UMAX( ch->hit / 2, 1 );
    ch->move    = UMAX( ch->move / 2, 1 );

    return;
}


void spell_defender_true( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *defender = (OBJ_DATA *) vo;
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA *paf;

    level = ch->level;     /* We use casters true level for these spells. */

    if ( defender->pIndexData->vnum != OBJ_VNUM_DEFENDER )
    {
        send_to_char( "That isn't a defender!\n\r", ch );
        return;
    }

    if ( IS_NULLSTR(defender->owner) || str_cmp(defender->owner, ch->name) )
    {
        send_to_char( "You can only cast that on your own defender.\n\r", ch );
        return;
    }

    if ( str_suffix("lord", defender->name) )
    {
        send_to_char( "You can only cast this on a lord defender.\n\r", ch );
        return;
    }

    sprintf( buf, "%s true", defender->pIndexData->name );
    free_string( defender->name );
    defender->name = str_dup( buf );

    for ( paf = defender->affected; paf != NULL; paf = paf->next )
    {
        if ( paf->location == APPLY_HITROLL )
            paf->modifier = UMAX( 8, paf->modifier );
        else if ( paf->location == APPLY_DAMROLL )
            paf->modifier = UMAX( 8, paf->modifier );
    }

    defender->value[1]  = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].number;
    defender->value[2]  = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].type;
    defender->level     = ch->level;

    act( "$p glows with a blinding golden light!", ch, defender, NULL, TO_CHAR );
    act( "$p glows with a blinding golden light!", ch, defender, NULL, TO_ROOM );
}


void spell_demonskin( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    AFFECT_DATA af;
    int dam;

    if ( !is_affected(ch, skill_lookup("lesser demonskin")) )
    {
        send_to_char(
            "You need to bind a lesser demonskin before attempting this.\n\r",
            ch );

        return;
    }

    affect_strip( ch, skill_lookup("lesser demonskin") );

    dam = dice( 3, 10 ) * dice( 3, 10 );

    act( "A searing pain shoots through your body as the demon binds to your skin.",
        ch, NULL, NULL, TO_CHAR );
    act( "$n collapses to the ground writhing in pain.",
        ch, NULL, NULL, TO_ROOM );

    damage_old( ch, ch, dam, sn, DAM_NEGATIVE, FALSE, NULL );

    if ( !saves_spell(30, ch, DAM_NEGATIVE) )
    {
        act( "Blackness overwelms you as the power of the demon proves too much!",
            ch, NULL, NULL, TO_CHAR );

        af.where        = TO_AFFECTS;
        af.type         = sn;
        af.level        = level;
        af.duration     = 72;
        af.location     = APPLY_NONE;
        af.modifier     = 0;
        af.bitvector    = AFF_SLEEP;
        af.procNo       = 0;
        af.flags        = 0;
        af.name         = "";
        af.appearance   = "";
        affect_to_char( ch, &af );

        ch->position    = POS_SLEEPING;

        return;
    }

    af.where            = TO_AFFECTS;
    af.type             = sn;
    af.level            = level;
    af.duration         = -1;
    af.location         = APPLY_AC;
    af.modifier         = -50;
    af.bitvector        = 0;
    af.procNo           = 0;
    af.flags            = 0;
    af.name             = "";
    af.appearance       = "";
    affect_to_char( ch, &af );

    af.location         = APPLY_HIT;
    af.modifier         = 90;
    affect_to_char( ch, &af );

    af.location        = APPLY_MOVE;
    af.modifier        = 20;
    affect_to_char( ch, &af );

    taint( ch, 70 );
}


void spell_demonskin_true( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    AFFECT_DATA af;
    int dam;

    if ( !is_affected(ch, skill_lookup("greater demonskin")) )
    {
        send_to_char(
            "You need to bind a greater demonskin before attempting this.\n\r",
            ch );

        return;
    }

    affect_strip( ch, skill_lookup("greater demonskin") );

    dam = dice( 5, 10 ) * dice( 5, 10 );

    act( "A searing pain shoots through your body as the demon binds to your skin.",
        ch, NULL, NULL, TO_CHAR );
    act( "$n collapses to the ground writhing in pain.",
        ch, NULL, NULL, TO_ROOM );

    damage_old( ch, ch, dam, sn, DAM_NEGATIVE, FALSE, NULL );

    if ( !saves_spell(50, ch, DAM_NEGATIVE) )
    {
        act( "Blackness overwelms you as the power of the demon proves too much!",
            ch, NULL, NULL, TO_CHAR );

        af.where        = TO_AFFECTS;
        af.type         = sn;
        af.level        = level;
        af.duration     = 120;
        af.location     = APPLY_NONE;
        af.modifier     = 0;
        af.bitvector    = AFF_SLEEP;
        af.procNo       = 0;
        af.flags        = 0;
        af.name         = "";
        af.appearance   = "";
        affect_to_char( ch, &af );

        ch->position    = POS_SLEEPING;

        return;
    }

    af.where            = TO_AFFECTS;
    af.type             = sn;
    af.level            = level;
    af.duration         = -1;
    af.location         = APPLY_AC;
    af.modifier         = -50;
    af.bitvector        = 0;
    af.procNo           = 0;
    af.flags            = 0;
    af.name             = "";
    af.appearance       = "";
    affect_to_char( ch, &af );

    af.location         = APPLY_HIT;
    af.modifier         = 300;
    affect_to_char( ch, &af );

    af.location         = APPLY_MOVE;
    af.modifier         = 20;
    affect_to_char( ch, &af );

    taint( ch, 90 );
}


void spell_detect_traps( int sn, int level, CHAR_DATA *ch, void *vo,int target)
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( IS_AFFECTED(victim, AFF_DETECT_TRAPS) )
    {
        if (victim == ch)
            send_to_char( "You can already see traps.\n\r", ch );
        else
            act( "$N can already see traps.", ch, NULL, victim, TO_CHAR );
        return;
    }

    af.where            = TO_AFFECTS;
    af.type             = sn;
    af.level            = level;
    af.duration         = level;
    af.modifier         = 0;
    af.location         = APPLY_NONE;
    af.bitvector        = AFF_DETECT_TRAPS;
    af.procNo           = 0;
    af.flags            = 0;
    af.name             = "";
    af.appearance       = "";
    affect_to_char( victim, &af );

    send_to_char( "Your eyes burn briefly.\n\r", victim );
    if ( ch != victim )
        send_to_char( "Ok.\n\r", ch );
    return;
}


void spell_druidstaff_true( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *staff = (OBJ_DATA *) vo;
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA *paf;

    level = ch->level;     /* We use casters true level for these spells. */

    if ( staff->pIndexData->vnum != OBJ_VNUM_DRUID_STAFF )
    {
        send_to_char( "That isn't a druidstaff!\n\r", ch );
        return;
    }

    if ( IS_NULLSTR(staff->owner) || str_cmp(staff->owner, ch->name) )
    {
        send_to_char( "You can only cast that on your own staff.\n\r", ch );
        return;
    }

    if ( str_suffix("lord", staff->name) )
    {
        send_to_char( "You can only cast this on a lord driudstaff.\n\r", ch );
        return;
    }

    sprintf( buf, "%s true", staff->pIndexData->name );
    free_string( staff->name );
    staff->name = str_dup( buf );

    for ( paf = staff->affected; paf != NULL; paf = paf->next )
    {
        if ( paf->location == APPLY_HITROLL )
            paf->modifier = UMAX( 8, paf->modifier );
        else if ( paf->location == APPLY_DAMROLL )
            paf->modifier = UMAX( 8, paf->modifier );
        else if ( paf->location == APPLY_MANA )
            paf->modifier = 40;
    }

    staff->value[1] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].number;
    staff->value[2] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].type;
    staff->level    = ch->level;

    act( "$p glows with a blinding golden light!", ch, staff, NULL, TO_CHAR );
    act( "$p glows with a blinding golden light!", ch, staff, NULL, TO_ROOM );
}


void spell_earth_elemental( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *elemental;
    AFFECT_DATA af;

    if ( ch->in_room->sector_type == SECT_WATER_SWIM
    ||   ch->in_room->sector_type == SECT_WATER_NOSWIM
    ||   ch->in_room->sector_type == SECT_AIR )
    {
        send_to_char( "Nothing seems to happen. Perhaps there is not enough earth here.\n\r",
            ch );
        return;
    }

    elemental           = create_mobile( get_mob_index(MOB_VNUM_STONE_ELEMENTAL) );
    elemental->level    = 3 * level / 4;
    elemental->max_hit *= elemental->level;
    elemental->hit      = elemental->max_hit;
    elemental->damage[DICE_NUMBER]
        = mobile_autos[UMIN(MAX_LEVEL-1, elemental->level)].damage[DICE_NUMBER];
    elemental->damage[DICE_TYPE]
        = mobile_autos[UMIN(MAX_LEVEL-1, elemental->level)].damage[DICE_TYPE];
    elemental->damage[DICE_BONUS]
        = mobile_autos[UMIN(MAX_LEVEL-1, elemental->level)].damage[DICE_BONUS];
    char_to_room( elemental, ch->in_room );

    act( "The ground vibrates briefly, then buckles as a huge stone form rises up.",
         ch, NULL, NULL, TO_ROOM );
    act( "The ground vibrates briefly, then buckles as a huge stone form rises up.",
         ch, NULL, NULL, TO_CHAR );

    if ( saves_spell(level, elemental, DAM_CHARM) )
    {
        multi_hit( elemental, ch, TYPE_UNDEFINED );
        return;
    }

    add_follower( elemental, ch );
    elemental->leader = ch;

    af.where            = TO_AFFECTS;
    af.type             = sn;
    af.level            = level;
    af.duration         = -1;
    af.location         = 0;
    af.modifier         = 0;
    af.bitvector        = AFF_CHARM;
    af.procNo           = affect_proc_lookup("earth elemental");
    af.flags            = AFF_CONT_UPDATE;
    af.name             = "";
    af.appearance       = "";
    affect_to_char( elemental, &af );

    if ( HAS_TRIGGER(elemental, TRIG_ENTRY) )
        mp_percent_trigger( elemental, NULL, NULL, NULL, TRIG_ENTRY );
}


void earth_elemental_affect_proc( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    if ( ch == NULL )
        return;

    if ( reason == AFF_PROC_PERIODIC )
    {
        damage_old( ch, ch, ch->max_hit / 8, paf->type,
                    DAM_NONE, FALSE, NULL );
        if ( !CHAR_KILLED(ch) )
            act( "The surface of $n cracks and splinters.",
                ch, NULL, NULL, TO_ROOM );

        return;
    }
}


void spell_empathic_healing( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af, *paf;
    int hp;

    if ( victim == ch )
    {
        send_to_char( "You can only use this spell on others.\n\r", ch );
        return;
    }

    hp = victim->max_hit - victim->hit;
    if ( hp < 0 )
        hp = 0;
    else if ( hp > ch->hit )
        hp = ch->hit - 1;

    victim->hit += hp;
    ch->hit     -= hp;

    if ( IS_AFFECTED(victim, AFF_PLAGUE)
    &&   (paf = affect_find(victim->affected, gsn_plague)) != NULL )
    {
        affect_join( ch, paf );
        send_to_char( "You feel hot and feverish.\n\r", ch );

        affect_strip( victim, gsn_plague );
    }

    if ( IS_AFFECTED(victim, AFF_POISON)
    &&   (paf = affect_find(victim->affected, gsn_poison)) != NULL )
    {
        affect_join( ch, paf );
        send_to_char( "You feel very sick.\n\r", ch );

        affect_strip( victim, gsn_poison );
    }

    af.where            = TO_AFFECTS;
    af.type             = sn;
    af.level            = level;
    af.duration         = hp / 100;
    af.location         = APPLY_NONE;
    af.modifier         = 0;
    af.bitvector        = AFF_REGENERATION;
    af.procNo           = affect_proc_lookup( "empathic healing" );
    af.flags            = AFF_CONT_UPDATE|AFF_INVIS;
    af.name             = "";
    af.appearance       = "";

    affect_join( ch, &af );

    act( "Concentrating deeply, you transfer $Ns wounds to your own body.",
        ch, NULL, victim, TO_CHAR );
    act( "Concentrating deeply, $n transfers the wounds on $Ns body to $s own.",
        ch, NULL, victim, TO_NOTVICT );
    act( "Concentrating deeply, $n transfers the wounds from your body to $s.",
        ch, NULL, victim, TO_VICT );
}


void empathic_healing_affect_proc( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    if ( ch == NULL )
        return;

    if ( reason == AFF_PROC_PERIODIC )
    {
        AFFECT_DATA *paf;
        bool removed = FALSE;

        if ( IS_AFFECTED(ch, AFF_PLAGUE)
        &&   (paf = affect_find(ch->affected, gsn_plague)) != NULL
        &&   saves_spell(paf->level, ch, DAM_DISEASE) )
        {
            removed = TRUE;

            affect_strip( ch, gsn_plague );
        }

        if ( IS_AFFECTED(ch, AFF_POISON)
        &&   (paf = affect_find(ch->affected, gsn_poison)) != NULL
        &&   saves_spell(paf->level, ch, DAM_POISON) )
        {
            removed = TRUE;

            affect_strip( ch, gsn_poison );
        }

        if ( removed )
            send_to_char( "A warm tingling feeling fills your body.\n\r", ch );
    }

    return;
}


void spell_fortitude( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( is_affected(victim, sn) )
    {
        if (victim == ch)
            send_to_char(
                "You are already blessed with divine protection.\n\r", ch );
        else
            act( "$N is already blessed with divine protection.",
                ch, NULL, victim, TO_CHAR );
        return;
    }

    af.where            = TO_AFFECTS;
    af.type             = sn;
    af.level            = level;
    af.duration         = 24;
    af.modifier         = level / -10;
    af.location         = APPLY_SAVES;
    af.bitvector        = 0;
    af.procNo           = 0;
    af.flags            = 0;
    af.name             = "";
    af.appearance    = "";
    affect_to_char( victim, &af );

    af.where        = TO_RESIST;
    af.modifier        = 0;
    af.location        = APPLY_NONE;
    af.bitvector    = IMM_COLD;
    affect_to_char( victim, &af );

    af.where        = TO_RESIST;
    af.modifier        = 0;
    af.location        = APPLY_NONE;
    af.bitvector    = IMM_NEGATIVE;
    affect_to_char( victim, &af );

    send_to_char( "You are granted resistance to evil.\n\r", victim );
    if ( ch != victim )
    act( "$N is granted resistance to evil.", ch, NULL, victim, TO_CHAR );
}


void spell_golden_defender( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *defender = (OBJ_DATA *) vo;
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA *paf;

    level = ch->level;     /* We use casters true level for these spells. */

    if ( defender->pIndexData->vnum != OBJ_VNUM_DEFENDER )
    {
    send_to_char( "That isn't a defender!\n\r", ch );
    return;
    }

    if ( IS_NULLSTR(defender->owner) || str_cmp(defender->owner, ch->name) )
    {
    send_to_char( "You can only cast that on your own defender.\n\r", ch );
    return;
    }

    if ( str_suffix("silver", defender->name) )
    {
    send_to_char( "You can only cast this on a silver defender.\n\r", ch );
    return;
    }

    sprintf( buf, "%s golden", defender->pIndexData->name );
    free_string( defender->name );
    defender->name = str_dup( buf );

    for ( paf = defender->affected; paf != NULL; paf = paf->next )
    {
    if ( paf->location == APPLY_HITROLL )
        paf->modifier = UMAX( 6, paf->modifier );
    else if ( paf->location == APPLY_DAMROLL )
        paf->modifier = UMAX( 6, paf->modifier );
    }

    defender->value[1]    = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].number;
    defender->value[2]    = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].type;
    defender->level    = ch->level;

    act( "$p glows with a golden light!", ch, defender, NULL, TO_CHAR );
    act( "$p glows with a golden light!", ch, defender, NULL, TO_ROOM );
}


void spell_golden_druidstaff( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *staff = (OBJ_DATA *) vo;
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA *paf;

    level = ch->level;     /* We use casters true level for these spells. */

    if ( staff->pIndexData->vnum != OBJ_VNUM_DRUID_STAFF )
    {
    send_to_char( "That isn't a druidstaff!\n\r", ch );
    return;
    }

    if ( IS_NULLSTR(staff->owner) || str_cmp(staff->owner, ch->name) )
    {
    send_to_char( "You can only cast that on your own staff.\n\r", ch );
    return;
    }

    if ( str_suffix("silver", staff->name) )
    {
    send_to_char( "You can only cast this on a silver driudstaff.\n\r", ch );
    return;
    }

    sprintf( buf, "%s golden", staff->pIndexData->name );
    free_string( staff->name );
    staff->name = str_dup( buf );

    for ( paf = staff->affected; paf != NULL; paf = paf->next )
    {
    if ( paf->location == APPLY_HITROLL )
        paf->modifier = UMAX( 6, paf->modifier );
    else if ( paf->location == APPLY_DAMROLL )
        paf->modifier = UMAX( 6, paf->modifier );
    else if ( paf->location == APPLY_MANA )
        paf->modifier = 20;
    }

    staff->value[1] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].number;
    staff->value[2] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].type;
    staff->level    = ch->level;

    act( "$p glows with a golden light!", ch, staff, NULL, TO_CHAR );
    act( "$p glows with a golden light!", ch, staff, NULL, TO_ROOM );
}


void spell_greater_defender( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *defender = (OBJ_DATA *) vo;
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA *paf;

    level = ch->level;     /* We use casters true level for these spells. */

    if ( defender->pIndexData->vnum != OBJ_VNUM_DEFENDER )
    {
    send_to_char( "That isn't a defender!\n\r", ch );
    return;
    }

    if ( IS_NULLSTR(defender->owner) || str_cmp(defender->owner, ch->name) )
    {
    send_to_char( "You can only cast that on your own defender.\n\r", ch );
    return;
    }

    if ( str_suffix("lesser", defender->name) )
    {
    send_to_char( "You can only cast this on a lesser defender.\n\r", ch );
    return;
    }

    sprintf( buf, "%s greater", defender->pIndexData->name );
    free_string( defender->name );
    defender->name = str_dup( buf );

    for ( paf = defender->affected; paf != NULL; paf = paf->next )
    {
    if ( paf->location == APPLY_HITROLL )
        paf->modifier = UMAX( 4, paf->modifier );
    else if ( paf->location == APPLY_DAMROLL )
        paf->modifier = UMAX( 4, paf->modifier );
    }

    defender->value[1] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].number;
    defender->value[2] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].type;
    defender->level    = ch->level;

    act( "$p glows with a bright light!", ch, defender, NULL, TO_CHAR );
    act( "$p glows with a bright light!", ch, defender, NULL, TO_ROOM );
}


void spell_greater_demonskin( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    AFFECT_DATA af;
    int dam;

    if ( !is_affected(ch, skill_lookup("demonskin")) )
    {
    send_to_char(
        "You need to bind a demonskin before attempting this.\n\r",
        ch );

    return;
    }

    affect_strip( ch, skill_lookup("demonskin") );

    dam = dice( 4, 10 ) * dice( 4, 10 );

    act( "A searing pain shoots through your body as the demon binds to your skin.",
        ch, NULL, NULL, TO_CHAR );
    act( "$n collapses to the ground writhing in pain.",
        ch, NULL, NULL, TO_ROOM );

    damage_old( ch, ch, dam, sn, DAM_NEGATIVE, FALSE, NULL );

    if ( !saves_spell(40, ch, DAM_NEGATIVE) )
    {
    act( "Blackness overwelms you as the power of the demon proves too much!",
        ch, NULL, NULL, TO_CHAR );

    af.where    = TO_AFFECTS;
    af.type        = sn;
    af.level    = level;
    af.duration    = 96;
    af.location    = APPLY_NONE;
    af.modifier    = 0;
    af.bitvector    = AFF_SLEEP;
    af.procNo    = 0;
    af.flags    = 0;
    af.name        = "";
    af.appearance    = "";
    affect_to_char( ch, &af );

    ch->position = POS_SLEEPING;

    return;
    }

    af.where        = TO_AFFECTS;
    af.type        = sn;
    af.level        = level;
    af.duration        = -1;
    af.location        = APPLY_AC;
    af.modifier        = -60;
    af.bitvector    = 0;
    af.procNo        = 0;
    af.flags        = 0;
    af.name        = "";
    af.appearance    = "";
    affect_to_char( ch, &af );

    af.location        = APPLY_HIT;
    af.modifier        = 120;
    affect_to_char( ch, &af );

    af.location        = APPLY_MOVE;
    af.modifier        = 40;
    affect_to_char( ch, &af );

    taint( ch, 80 );
}


void spell_greater_druidstaff( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *staff = (OBJ_DATA *) vo;
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA *paf;

    level = ch->level;     /* We use casters true level for these spells. */

    if ( staff->pIndexData->vnum != OBJ_VNUM_DRUID_STAFF )
    {
    send_to_char( "That isn't a druidstaff!\n\r", ch );
    return;
    }

    if ( IS_NULLSTR(staff->owner) || str_cmp(staff->owner, ch->name) )
    {
    send_to_char( "You can only cast that on your own staff.\n\r", ch );
    return;
    }

    if ( str_suffix("lesser", staff->name) )
    {
    send_to_char( "You can only cast this on a lesser driudstaff.\n\r", ch );
    return;
    }

    sprintf( buf, "%s greater", staff->pIndexData->name );
    free_string( staff->name );
    staff->name = str_dup( buf );

    for ( paf = staff->affected; paf != NULL; paf = paf->next )
    {
    if ( paf->location == APPLY_HITROLL )
        paf->modifier = UMAX( 4, paf->modifier );
    else if ( paf->location == APPLY_DAMROLL )
        paf->modifier = UMAX( 4, paf->modifier );
    }

    paf               = new_affect();
    paf->where        = TO_OBJECT;
    paf->type         = sn;
    paf->level        = level;
    paf->duration     = -1;
    paf->location     = APPLY_MANA;
    paf->modifier     = 10;
    paf->bitvector    = 0;
    paf->procNo       = 0;
    paf->flags        = 0;
    paf->next         = staff->affected;
    staff->affected   = paf;

    staff->value[1] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].number;
    staff->value[2] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].type;
    staff->level    = ch->level;

    act( "$p glows with a bright light!", ch, staff, NULL, TO_CHAR );
    act( "$p glows with a bright light!", ch, staff, NULL, TO_ROOM );
}


void spell_greater_shadowblade( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *blade = (OBJ_DATA *) vo;
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA *paf;

    level = ch->level;     /* We use casters true level for these spells. */

    if ( blade->pIndexData->vnum != OBJ_VNUM_SHADOWBLADE )
    {
    send_to_char( "That isn't a shadowblade!\n\r", ch );
    return;
    }

    if ( IS_NULLSTR(blade->owner) || str_cmp(blade->owner, ch->name) )
    {
    send_to_char( "You can only cast that on your own shadowblade.\n\r", ch );
    return;
    }

    if ( str_suffix("lesser", blade->name) )
    {
    send_to_char( "You can only cast this on a lesser shadowblade.\n\r", ch );
    return;
    }

    sprintf( buf, "%s greater", blade->pIndexData->name );
    free_string( blade->name );
    blade->name = str_dup( buf );

    for ( paf = blade->affected; paf != NULL; paf = paf->next )
    {
    if ( paf->location == APPLY_HITROLL )
        paf->modifier = UMAX( 4, paf->modifier );
    else if ( paf->location == APPLY_DAMROLL )
        paf->modifier = UMAX( 4, paf->modifier );
    }

    blade->value[1] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].number;
    blade->value[2] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].type;
    blade->level    = ch->level;

    act( "$p glows with a deep red aura!", ch, blade, NULL, TO_CHAR );
    act( "$p glows with a deep red aura!", ch, blade, NULL, TO_ROOM );
}


void spell_greater_healing( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    switch ( weather_info.sky )
    {
        case SKY_CLOUDLESS:
            victim->hit = UMIN( victim->hit + number_range(150, 300), victim->max_hit );
            break;
        case SKY_CLOUDY:
            victim->hit = UMIN( victim->hit + number_range(100, 200), victim->max_hit );
            break;
        case SKY_RAINING:
            victim->hit = UMIN( victim->hit + number_range(75, 150), victim->max_hit );
            break;
        default:
            victim->hit = UMIN( victim->hit + number_range(75, 100), victim->max_hit );
            ch->pcdata->condition[COND_HUNGER] = 0;
	    af.where        = TO_AFFECTS;
	    af.type         = sn;
	    af.level        = level;
	    af.duration     = 6;
	    af.location     = APPLY_STR;
	    af.modifier     = -4;
	    af.bitvector    = 0;
	    af.procNo       = 0;
	    af.flags        = 0;
	    af.name         = "";
	    af.appearance   = "";
	    affect_join( victim, &af );
            break;
    }

    update_pos( victim );

    send_to_char( "You are bathed in the golden light of a ray from the heavens.\n\r", victim );
    act( "$N is bathed in the golden light of a ray from the heavens.", ch, NULL, victim, TO_NOTVICT );
    if ( ch != victim )
        act( "$N is bathed in the golden light of a ray from the heavens.",
            ch, NULL, victim, TO_CHAR );
    return;
}


void spell_holy_defender( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *defender = (OBJ_DATA *) vo;
    AFFECT_DATA af;

    if ( defender->pIndexData->vnum != OBJ_VNUM_DEFENDER )
    {
    send_to_char( "That isn't a defender!\n\r", ch );
    return;
    }

    if ( IS_NULLSTR(defender->owner) || str_cmp(defender->owner, ch->name) )
    {
    send_to_char( "You can only cast that on your own defender.\n\r", ch );
    return;
    }

    if ( IS_WEAPON_STAT(defender, WEAPON_FLAMING)
    ||   IS_WEAPON_STAT(defender, WEAPON_FROST)
    ||   IS_WEAPON_STAT(defender, WEAPON_VAMPIRIC)
    ||   IS_WEAPON_STAT(defender, WEAPON_SHARP)
    ||   IS_WEAPON_STAT(defender, WEAPON_VORPAL)
    ||   IS_WEAPON_STAT(defender, WEAPON_POISON)
    ||   IS_WEAPON_STAT(defender, WEAPON_SHOCKING) )
    {
    send_to_char( "Nothing seems to happen.", ch );
    return;
    }

    if ( IS_WEAPON_STAT(defender, WEAPON_DEFENDER) )
    {
    act( "$p is already a holy defender.",
        ch, defender, NULL, TO_CHAR );
    return;
    }

    af.where        = TO_WEAPON;
    af.type         = sn;
    af.level        = level / 2;
    af.duration     = level / 5;
    af.location     = 0;
    af.modifier     = 0;
    af.bitvector    = WEAPON_DEFENDER;
    af.procNo       = 0;
    af.flags        = 0;
    af.name         = "";
    af.appearance   = "";
    affect_to_obj( defender, &af );

    act( "$p is blessed as a holy defender.",
         ch, defender, NULL, TO_ALL );
}


void spell_lesser_defender( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *defender = (OBJ_DATA *) vo;
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA *paf;

    level = ch->level;     /* We use casters true level for these spells. */

    if ( defender->pIndexData->vnum != OBJ_VNUM_DEFENDER )
    {
    send_to_char( "That isn't a defender!\n\r", ch );
    return;
    }

    if ( IS_NULLSTR(defender->owner) || str_cmp(defender->owner, ch->name) )
    {
    send_to_char( "You can only cast that on your own defender.\n\r", ch );
    return;
    }

    if ( str_suffix("minor", defender->name) )
    {
    send_to_char( "You can only cast this on a minor defender.\n\r", ch );
    return;
    }

    sprintf( buf, "%s lesser", defender->pIndexData->name );
    free_string( defender->name );
    defender->name = str_dup( buf );

    for ( paf = defender->affected; paf != NULL; paf = paf->next )
    {
    if ( paf->location == APPLY_HITROLL )
        paf->modifier = UMAX( 3, paf->modifier );
    else if ( paf->location == APPLY_DAMROLL )
        paf->modifier = UMAX( 3, paf->modifier );
    }

    defender->value[1] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].number;
    defender->value[2] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].type;
    defender->level    = ch->level;

    act( "$p glows with a faint light!", ch, defender, NULL, TO_CHAR );
    act( "$p glows with a faint light!", ch, defender, NULL, TO_ROOM );
}


void spell_lesser_demonskin( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    AFFECT_DATA af;
    int dam;

    if ( !is_affected(ch, skill_lookup("minor demonskin")) )
    {
    send_to_char(
        "You need to bind a minor demonskin before attempting this.\n\r",
        ch );

    return;
    }

    affect_strip( ch, skill_lookup("minor demonskin") );

    dam = dice( 2, 10 ) * dice( 2, 10 );

    act( "A searing pain shoots through your body as the demon binds to your skin.",
        ch, NULL, NULL, TO_CHAR );
    act( "$n collapses to the ground writhing in pain.",
        ch, NULL, NULL, TO_ROOM );

    damage_old( ch, ch, dam, sn, DAM_NEGATIVE, FALSE, NULL );

    if ( !saves_spell(20, ch, DAM_NEGATIVE) )
    {
    act( "Blackness overwelms you as the power of the demon proves too much!",
        ch, NULL, NULL, TO_CHAR );

    af.where    = TO_AFFECTS;
    af.type        = sn;
    af.level    = level;
    af.duration    = 48;
    af.location    = APPLY_NONE;
    af.modifier    = 0;
    af.bitvector    = AFF_SLEEP;
    af.procNo    = 0;
    af.flags    = 0;
    af.name        = "";
    af.appearance    = "";
    affect_to_char( ch, &af );

    ch->position = POS_SLEEPING;

    return;
    }

    af.where        = TO_AFFECTS;
    af.type        = sn;
    af.level        = level;
    af.duration        = -1;
    af.location        = APPLY_AC;
    af.modifier        = -30;
    af.bitvector    = 0;
    af.procNo        = 0;
    af.flags        = 0;
    af.name        = "";
    af.appearance    = "";
    affect_to_char( ch, &af );

    af.location        = APPLY_HIT;
    af.modifier        = 60;
    affect_to_char( ch, &af );

    af.location        = APPLY_MOVE;
    af.modifier        = 10;
    affect_to_char( ch, &af );

    taint( ch, 60 );
}


void spell_lesser_druidstaff( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *staff = (OBJ_DATA *) vo;
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA *paf;

    level = ch->level;     /* We use casters true level for these spells. */

    if ( staff->pIndexData->vnum != OBJ_VNUM_DRUID_STAFF )
    {
    send_to_char( "That isn't a druidstaff!\n\r", ch );
    return;
    }

    if ( IS_NULLSTR(staff->owner) || str_cmp(staff->owner, ch->name) )
    {
    send_to_char( "You can only cast that on your own staff.\n\r", ch );
    return;
    }

    if ( str_suffix("minor", staff->name) )
    {
    send_to_char( "You can only cast this on a minor driudstaff.\n\r", ch );
    return;
    }

    sprintf( buf, "%s lesser", staff->pIndexData->name );
    free_string( staff->name );
    staff->name = str_dup( buf );

    for ( paf = staff->affected; paf != NULL; paf = paf->next )
    {
    if ( paf->location == APPLY_HITROLL )
        paf->modifier = UMAX( 3, paf->modifier );
    else if ( paf->location == APPLY_DAMROLL )
        paf->modifier = UMAX( 3, paf->modifier );
    }

    staff->value[1] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].number;
    staff->value[2] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].type;
    staff->level    = ch->level;

    act( "$p glows with a faint light!", ch, staff, NULL, TO_CHAR );
    act( "$p glows with a faint light!", ch, staff, NULL, TO_ROOM );
}


void spell_lesser_shadowblade( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *blade = (OBJ_DATA *) vo;
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA *paf;

    level = ch->level;     /* We use casters true level for these spells. */

    if ( blade->pIndexData->vnum != OBJ_VNUM_SHADOWBLADE )
    {
    send_to_char( "That isn't a shadowblade!\n\r", ch );
    return;
    }

    if ( IS_NULLSTR(blade->owner) || str_cmp(blade->owner, ch->name) )
    {
    send_to_char( "You can only cast that on your own shadowblade.\n\r", ch );
    return;
    }

    if ( str_suffix("minor", blade->name) )
    {
    send_to_char( "You can only cast this on a minor shadowblade.\n\r", ch );
    return;
    }

    sprintf( buf, "%s lesser", blade->pIndexData->name );
    free_string( blade->name );
    blade->name = str_dup( buf );

    for ( paf = blade->affected; paf != NULL; paf = paf->next )
    {
    if ( paf->location == APPLY_HITROLL )
        paf->modifier = UMAX( 3, paf->modifier );
    else if ( paf->location == APPLY_DAMROLL )
        paf->modifier = UMAX( 3, paf->modifier );
    }

    blade->value[1] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].number;
    blade->value[2] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].type;
    blade->level    = ch->level;

    act( "$p glows with a faint red aura!", ch, blade, NULL, TO_CHAR );
    act( "$p glows with a faint red aura!", ch, blade, NULL, TO_ROOM );
}


void spell_lord_defender( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *defender = (OBJ_DATA *) vo;
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA *paf;

    level = ch->level;     /* We use casters true level for these spells. */

    if ( defender->pIndexData->vnum != OBJ_VNUM_DEFENDER )
    {
    send_to_char( "That isn't a defender!\n\r", ch );
    return;
    }

    if ( IS_NULLSTR(defender->owner) || str_cmp(defender->owner, ch->name) )
    {
    send_to_char( "You can only cast that on your own defender.\n\r", ch );
    return;
    }

    if ( str_suffix("golden", defender->name) )
    {
    send_to_char( "You can only cast this on a golden defender.\n\r", ch );
    return;
    }

    sprintf( buf, "%s lord", defender->pIndexData->name );
    free_string( defender->name );
    defender->name = str_dup( buf );

    for ( paf = defender->affected; paf != NULL; paf = paf->next )
    {
    if ( paf->location == APPLY_HITROLL )
        paf->modifier = UMAX( 7, paf->modifier );
    else if ( paf->location == APPLY_DAMROLL )
        paf->modifier = UMAX( 7, paf->modifier );
    }

    defender->value[1] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].number;
    defender->value[2] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].type;
    defender->level    = ch->level;

    act( "$p glows with a bright golden light!", ch, defender, NULL, TO_CHAR );
    act( "$p glows with a bright golden light!", ch, defender, NULL, TO_ROOM );
}


void spell_lord_druidstaff( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *staff = (OBJ_DATA *) vo;
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA *paf;

    level = ch->level;     /* We use casters true level for these spells. */

    if ( staff->pIndexData->vnum != OBJ_VNUM_DRUID_STAFF )
    {
    send_to_char( "That isn't a druidstaff!\n\r", ch );
    return;
    }

    if ( IS_NULLSTR(staff->owner) || str_cmp(staff->owner, ch->name) )
    {
    send_to_char( "You can only cast that on your own staff.\n\r", ch );
    return;
    }

    if ( str_suffix("golden", staff->name) )
    {
    send_to_char( "You can only cast this on a golden driudstaff.\n\r", ch );
    return;
    }

    sprintf( buf, "%s lord", staff->pIndexData->name );
    free_string( staff->name );
    staff->name = str_dup( buf );

    for ( paf = staff->affected; paf != NULL; paf = paf->next )
    {
    if ( paf->location == APPLY_HITROLL )
        paf->modifier = UMAX( 7, paf->modifier );
    else if ( paf->location == APPLY_DAMROLL )
        paf->modifier = UMAX( 7, paf->modifier );
    else if ( paf->location == APPLY_MANA )
        paf->modifier = 30;
    }

    staff->value[1] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].number;
    staff->value[2] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].type;
    staff->level    = ch->level;

    act( "$p glows with a bright golden light!", ch, staff, NULL, TO_CHAR );
    act( "$p glows with a bright golden light!", ch, staff, NULL, TO_ROOM );
}


void spell_lord_shadowblade( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *blade = (OBJ_DATA *) vo;
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA *paf;

    level = ch->level;     /* We use casters true level for these spells. */

    if ( blade->pIndexData->vnum != OBJ_VNUM_SHADOWBLADE )
    {
    send_to_char( "That isn't a shadowblade!\n\r", ch );
    return;
    }

    if ( IS_NULLSTR(blade->owner) || str_cmp(blade->owner, ch->name) )
    {
    send_to_char( "You can only cast that on your own shadowblade.\n\r", ch );
    return;
    }

    if ( str_suffix("greater", blade->name) )
    {
    send_to_char( "You can only cast this on a greater shadowblade.\n\r", ch );
    return;
    }

    sprintf( buf, "%s lord", blade->pIndexData->name );
    free_string( blade->name );
    blade->name = str_dup( buf );

    for ( paf = blade->affected; paf != NULL; paf = paf->next )
    {
    if ( paf->location == APPLY_HITROLL )
        paf->modifier = UMAX( 7, paf->modifier );
    else if ( paf->location == APPLY_DAMROLL )
        paf->modifier = UMAX( 7, paf->modifier );
    }

    blade->value[1] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].number;
    blade->value[2] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].type;
    blade->level    = ch->level;

    act( "$p glows with a baleful aura!", ch, blade, NULL, TO_CHAR );
    act( "$p glows with a baleful aura!", ch, blade, NULL, TO_ROOM );
}


void spell_love( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( is_affected(ch, sn) )
    {
    if ( victim == ch )
        send_to_char( "You are already love sick.\n\r", ch );
    else
        act( "$N is already love sick.", ch, NULL, victim, TO_CHAR );

    return;
    }

    if (saves_spell(level, victim, DAM_MENTAL)
    ||  IS_SET(CHAR_IMM(victim), IMM_MENTAL)
    ||  IS_AFFECTED(victim, AFF_CHARM))
    {
    send_to_char( "Nothing seemed to happen.\n\r", ch );
        return;
    }

    af.where        = TO_AFFECTS;
    af.type        = sn;
    af.level        = level;
    af.duration        = level / 10;
    af.location        = APPLY_NONE;
    af.modifier        = 0;
    af.bitvector    = 0;
    af.procNo        = affect_proc_lookup("love");
    af.flags        = AFF_CONT_UPDATE|AFF_INVIS;
    af.name        = "";
    af.appearance    = "";

    affect_to_char( victim, &af );

    if ( ch != victim )
    {
    act( "$N gazes around the room with love hungry eyes.", ch, NULL, victim, TO_NOTVICT );
    act( "$N gazes around the room with love hungry eyes.", ch, NULL, victim, TO_CHAR );
    }
}


void love_affect_proc( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    if ( ch == NULL )
    return;

    if ( is_affected(ch, AFF_CHARM) )
    return;

    if ( ch->in_room == NULL )
    return;

    if ( reason == AFF_PROC_PERIODIC )
    {
    /* If their "true love" hasn't been found yet... */
    if ( !IS_AFFECTED(ch, AFF_CHARM) )
    {
        CHAR_DATA *vch;

        for ( vch=ch->in_room->people; vch; vch=vch->next_in_room )
        {
        int level;
        /* Eliminate unsuitable partners. */
        if (vch == ch
        ||  !IS_AWAKE(ch)
        ||  IS_AFFECTED(vch, AFF_CHARM)
        ||  !can_see(ch, vch)
        ||  CHAR_FORM(vch) != CHAR_FORM(ch))
            continue;

        level = paf->level;

        /* Modify level to save against. */
        if ( CHAR_RACE(ch) == CHAR_RACE(vch) )
            level += 2;
        if ( ch->sex == vch->sex )
            level -= 10;
        level -= abs( CHAR_SIZE(ch) - CHAR_SIZE(vch) ) * 5;

        /* Is vch their "true love". */
        if ( !saves_spell(level, ch, DAM_MENTAL) )
        {
            AFFECT_DATA af;

            act( "$n stares dreamily at you, completely lost in your eyes.",
            ch, NULL, vch, TO_VICT );
            act( "$n stares dreamily at $N.",
            ch, NULL, vch, TO_NOTVICT );
            act( "As your eyes wander you notice $N, and realise $E is your one true love.",
            ch, NULL, vch, TO_CHAR );

            if ( ch->master )
            stop_follower( ch );
            add_follower( ch, vch );
            ch->leader = vch;

            af.where       = TO_AFFECTS;
            af.type        = paf->type;
            af.level       = paf->level;
            af.duration    = paf->duration;
            af.location    = 0;
            af.modifier    = 0;
            af.bitvector   = AFF_CHARM;
            af.procNo      = 0;
            af.flags       = 0;
            af.name        = "";
            af.appearance  = "";
            affect_to_char_after( ch, &af, paf );
            break;
        }
        }
    }
    /* else they have found their love... make them do something embarrasing. */
    else
    {
        /* sanity check */
        if (ch->master == NULL
        ||    ch->in_room != ch->master->in_room
        ||    ch->position <= POS_SLEEPING )
        return;

        switch (number_range(0,4))
        {
        case 0:
        if ( ch->sex != 2 )
        {
            act( "$n wraps $s arms around your waist and kisses you passionately.",
            ch, NULL, ch->master, TO_VICT );
            act( "$n wraps $s arms around $N's waist and kisses $M passionately.",
            ch, NULL, ch->master, TO_NOTVICT );
            act( "You wrap your arms around $N's waist and kiss $M passionately.",
            ch, NULL, ch->master, TO_CHAR );
        }
        else
        {
            act( "$n slips her arms around your neck and kisses you passionately.",
            ch, NULL, ch->master, TO_VICT );
            act( "$n slips her arms around $N's neck and kisses $M passionately.",
            ch, NULL, ch->master, TO_NOTVICT );
            act( "You slips her arms around $N's neck and kiss $M passionately.",
            ch, NULL, ch->master, TO_CHAR );
        }
        break;
        case 1:
        act( "$n softly nuzzles your neck.",
            ch, NULL, ch->master, TO_VICT );
        act( "$n softly nuzzles $N's neck.",
            ch, NULL, ch->master, TO_NOTVICT );
        act( "You nuzzle $S neck softly.",
            ch, NULL, ch->master, TO_CHAR );
        break;
        case 2:
        act( "$n whispers to you sweet words of love.",
            ch, NULL, ch->master, TO_VICT );
        act( "$n whispers softly to $N.",
            ch, NULL, ch->master, TO_NOTVICT );
        act( "You tell your true feelings to $N.",
            ch, NULL, ch->master, TO_CHAR );
        break;
        case 3:
        act( "$n winks suggestively at you.",
            ch, NULL, ch->master, TO_VICT );
        act( "$n winks at $N.",
            ch, NULL, ch->master, TO_NOTVICT );
        act( "You wink suggestively at $N.",
            ch, NULL, ch->master, TO_CHAR );
        break;
        case 4:
        act( "$n flirts with you.",
            ch, NULL, ch->master, TO_VICT );
        act( "$n flirts with $N, hmmm wonder what $e wants? *wink*",
            ch, NULL, ch->master, TO_NOTVICT );
        act( "You flirt with $N trying to get $S attention.",
            ch, NULL, ch->master, TO_CHAR );
        break;
        }
    }
    }

    return;
}


void spell_minor_defender( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA *paf;
    OBJ_DATA *defender;

    level = ch->level;     /* We use casters true level for these spells. */

    if ( !is_unique(OBJ_VNUM_DEFENDER, ch->name) )
    {
    send_to_char( "You already have a defender.\n\r", ch );
    return;
    }

    defender = create_object( get_obj_index(OBJ_VNUM_DEFENDER), 0, FALSE );

    sprintf( buf, "%s minor", defender->pIndexData->name );
    free_string( defender->name );
    defender->name    = str_dup( buf );

    defender->owner    = str_dup( ch->name );
    defender->enchanted    = TRUE;

    paf                = new_affect();
    paf->where         = TO_OBJECT;
    paf->type          = sn;
    paf->level         = level;
    paf->duration      = -1;
    paf->location      = APPLY_DAMROLL;
    paf->modifier      = 2;
    paf->bitvector     = 0;
    paf->procNo        = 0;
    paf->flags         = 0;
    paf->next          = defender->affected;
    defender->affected = paf;

    paf                = new_affect();
    paf->where         = TO_OBJECT;
    paf->type          = sn;
    paf->level         = level;
    paf->duration      = -1;
    paf->location      = APPLY_HITROLL;
    paf->modifier      = 2;
    paf->bitvector     = 0;
    paf->procNo        = 0;
    paf->flags         = 0;
    paf->next          = defender->affected;
    defender->affected = paf;

    obj_to_char( defender, ch );

    act( "You create $p!", ch, defender, NULL, TO_CHAR );
    act( "$n creates $p!", ch, defender, NULL, TO_ROOM );
}


void spell_minor_demonskin( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    AFFECT_DATA af;
    int dam;

    if ( is_affected(ch, skill_lookup("lesser demonskin"))
    ||   is_affected(ch, skill_lookup("demonskin"))
    ||   is_affected(ch, skill_lookup("greater demonskin"))
    ||   is_affected(ch, skill_lookup("demonskin true")) )
    {
    send_to_char(
        "You can't cast this while a greater demonskin is bound.\n\r",
        ch );

    return;
    }

    dam = dice( 1, 10 ) * dice( 1, 10 );

    act( "A searing pain shoots through your body as the demon binds to your skin.",
        ch, NULL, NULL, TO_CHAR );
    act( "$n collapses to the ground writhing in pain.",
        ch, NULL, NULL, TO_ROOM );

    damage_old( ch, ch, dam, sn, DAM_NEGATIVE, FALSE, NULL );

    if ( !saves_spell(10, ch, DAM_NEGATIVE) )
    {
    act( "Blackness overwelms you as the power of the demon proves too much!",
        ch, NULL, NULL, TO_CHAR );

    af.where    = TO_AFFECTS;
    af.type        = sn;
    af.level    = level;
    af.duration    = 24;
    af.location    = APPLY_NONE;
    af.modifier    = 0;
    af.bitvector    = AFF_SLEEP;
    af.procNo    = 0;
    af.flags    = 0;
    af.name        = "";
    af.appearance    = "";
    affect_to_char( ch, &af );

    ch->position = POS_SLEEPING;

    return;
    }

    af.where        = TO_AFFECTS;
    af.type        = sn;
    af.level        = level;
    af.duration        = -1;
    af.location        = APPLY_AC;
    af.modifier        = -20;
    af.bitvector    = 0;
    af.procNo        = 0;
    af.flags        = 0;
    af.name        = "";
    af.appearance    = "";
    affect_to_char( ch, &af );

    af.location        = APPLY_HIT;
    af.modifier        = 35;
    affect_to_char( ch, &af );

    af.location        = APPLY_MOVE;
    af.modifier        = 10;
    affect_to_char( ch, &af );

    taint( ch, 50 );
}


void spell_minor_druidstaff( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA *paf;
    OBJ_DATA *staff;

    level = ch->level;     /* We use casters true level for these spells. */

    if ( !is_unique(OBJ_VNUM_DRUID_STAFF, ch->name) )
    {
    send_to_char( "You already have a druidstaff.\n\r", ch );
    return;
    }

    staff = create_object( get_obj_index(OBJ_VNUM_DRUID_STAFF), 0, FALSE );

    sprintf( buf, "%s minor", staff->pIndexData->name );
    free_string( staff->name );
    staff->name        = str_dup( buf );

    staff->owner       = str_dup( ch->name );
    staff->enchanted   = TRUE;

    paf                = new_affect();
    paf->where         = TO_OBJECT;
    paf->type          = sn;
    paf->level         = level;
    paf->duration      = -1;
    paf->location      = APPLY_DAMROLL;
    paf->modifier      = 2;
    paf->bitvector     = 0;
    paf->procNo        = 0;
    paf->flags         = 0;
    paf->next          = staff->affected;
    staff->affected    = paf;

    paf                = new_affect();
    paf->where         = TO_OBJECT;
    paf->type          = sn;
    paf->level         = level;
    paf->duration      = -1;
    paf->location      = APPLY_HITROLL;
    paf->modifier      = 2;
    paf->bitvector     = 0;
    paf->procNo        = 0;
    paf->flags         = 0;
    paf->next          = staff->affected;
    staff->affected    = paf;

    obj_to_char( staff, ch );

    act( "You create $p!", ch, staff, NULL, TO_CHAR );
    act( "$n creates $p!", ch, staff, NULL, TO_ROOM );
}


void spell_minor_shadowblade( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA *paf;
    OBJ_DATA *blade;

    level = ch->level;     /* We use casters true level for these spells. */

    if ( !is_unique(OBJ_VNUM_SHADOWBLADE, ch->name) )
    {
    send_to_char( "You already have a shadowblade.\n\r", ch );
    return;
    }

    blade = create_object( get_obj_index(OBJ_VNUM_SHADOWBLADE), 0, FALSE );

    sprintf( buf, "%s minor", blade->pIndexData->name );
    free_string( blade->name );
    blade->name        = str_dup( buf );

    blade->owner       = str_dup( ch->name );
    blade->enchanted   = TRUE;

    paf                = new_affect();
    paf->where         = TO_OBJECT;
    paf->type          = sn;
    paf->level         = level;
    paf->duration      = -1;
    paf->location      = APPLY_DAMROLL;
    paf->modifier      = 2;
    paf->bitvector     = 0;
    paf->procNo        = 0;
    paf->flags         = 0;
    paf->next          = blade->affected;
    blade->affected    = paf;

    paf                = new_affect();
    paf->where         = TO_OBJECT;
    paf->type          = sn;
    paf->level         = level;
    paf->duration      = -1;
    paf->location      = APPLY_HITROLL;
    paf->modifier      = 1;
    paf->bitvector     = 0;
    paf->procNo        = 0;
    paf->flags         = 0;
    paf->next          = blade->affected;
    blade->affected    = paf;

    obj_to_char( blade, ch );

    act( "You create $p!", ch, blade, NULL, TO_CHAR );
    act( "$n creates $p!", ch, blade, NULL, TO_ROOM );
}


void spell_nexus( int sn, int level, CHAR_DATA *ch, void *vo, int target)
{
    CHAR_DATA *victim;
    OBJ_DATA *portal, *stone;
    ROOM_INDEX_DATA *to_room, *from_room;

    from_room = ch->in_room;

        if ( ( victim = get_char_world( ch, target_name ) ) == NULL
    ||   victim == ch
    ||   (to_room = victim->in_room) == NULL
    ||   !can_see_room(ch,to_room) || !can_see_room(ch,from_room)
    ||   IS_SET(to_room->room_flags, ROOM_SAFE)
    ||     IS_SET(from_room->room_flags,ROOM_SAFE)
    ||   IS_SET(to_room->room_flags, ROOM_PRIVATE)
    ||   IS_SET(to_room->room_flags, ROOM_SOLITARY)
    ||   IS_SET(to_room->room_flags, ROOM_NO_RECALL)
    ||   IS_SET(from_room->room_flags,ROOM_NO_RECALL)
    ||   victim->level >= level + 3
    ||   (!IS_NPC(victim) && victim->level >= LEVEL_HERO)  /* NOT trust */
    ||   (IS_NPC(victim) && IS_SET(CHAR_IMM(victim),IMM_SUMMON))
    ||   (IS_NPC(victim) && saves_spell( level, victim,DAM_NONE) )
    /*||     (is_clan(victim) && !is_same_clan(ch,victim))*/)
    {
        send_to_char( "You failed.\n\r", ch );
        return;
    }

    stone = get_eq_char(ch,WEAR_HOLD);
    if (!IS_IMMORTAL(ch)
    &&  (stone == NULL || stone->item_type != ITEM_WARP_STONE))
    {
        send_to_char("You lack the proper component for this spell.\n\r",ch);
        return;
    }

    if (stone != NULL && stone->item_type == ITEM_WARP_STONE)
    {
        act("You draw upon the power of $p.",ch,stone,NULL,TO_CHAR);
        act("It flares brightly and vanishes!",ch,stone,NULL,TO_CHAR);
        extract_obj(stone);
    }

    /* portal one */
    portal = create_object(get_obj_index(OBJ_VNUM_PORTAL),0, FALSE);
    portal->timer = 1 + level / 10;
    portal->value[3] = to_room->vnum;

    obj_to_room(portal,from_room);

    act("$p rises up from the ground.",ch,portal,NULL,TO_ROOM);
    act("$p rises up before you.",ch,portal,NULL,TO_CHAR);

    /* no second portal if rooms are the same */
    if (to_room == from_room)
    return;

    /* portal two */
    portal = create_object(get_obj_index(OBJ_VNUM_PORTAL),0, FALSE);
    portal->timer = 1 + level/10;
    portal->value[3] = from_room->vnum;

    obj_to_room(portal,to_room);

    if (to_room->people != NULL)
    {
    act("$p rises up from the ground.",to_room->people,portal,NULL,TO_ROOM);
    act("$p rises up from the ground.",to_room->people,portal,NULL,TO_CHAR);
    }
}


void spell_petrify( int sn, int level, CHAR_DATA *ch, void *vo, int target)
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( is_safe(ch, victim) )
    return;

    if ( is_affected(victim, sn)
    ||   saves_spell(level, victim, DAM_NONE) )
    return;

    af.where        = TO_AFFECTS;
    af.type         = sn;
    af.level        = level;
    af.duration     = level * 2;
    af.modifier     = -9;
    af.location     = APPLY_HITROLL;
    af.bitvector    = 0;
    af.procNo       = affect_proc_lookup("petrify");
    af.flags        = AFF_CONT_UPDATE;
    af.name         = "";
    af.appearance   = "";
    affect_to_char( victim, &af );

    af.modifier     = -8;
    af.location     = APPLY_STR;
    af.procNo       = 0;
    af.flags        = 0;
    affect_to_char( victim, &af );

    af.modifier     = -8;
    af.location     = APPLY_DEX;
    affect_to_char( victim, &af );

    af.modifier     = -8;
    af.location     = APPLY_CON;
    affect_to_char( victim, &af );

    send_to_char( "A tearing pain shoots to the centre of your very being as your bones petrify.\n\r", victim );
    act( "$N near collaspes as a tearing cry of pain shudders from $S throat!",
        ch, NULL, victim, TO_CHAR );
    act( "$N near collaspes as a tearing cry of pain shudders from $S throat!",
        ch, NULL, victim, TO_NOTVICT );
}


void petrify_affect_proc( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    if ( ch == NULL )
    return;

    if ( reason == AFF_PROC_PERIODIC )
    {
    send_to_char( "A deep weariness seeps through your petrified skeleton.\n\r", ch );
    if ( ch->hit > 2 )
        ch->hit -= 2;
    }

    return;
}


/* RT plague spell, very nasty */
void spell_plague( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if (saves_spell(level,victim,DAM_DISEASE) ||
        (IS_NPC(victim) && IS_SET(victim->act,ACT_UNDEAD)))
    {
    if (ch == victim)
      send_to_char("You feel momentarily ill, but it passes.\n\r",ch);
    else
      act("$N seems to be unaffected.",ch,NULL,victim,TO_CHAR);
    return;
    }

    af.flags     = TO_AFFECTS;
    af.type      = sn;
    af.level     = level * 3/4;
    af.duration  = level;
    af.location  = APPLY_STR;
    af.modifier  = -20;
    af.bitvector = AFF_PLAGUE;
    af.procNo     = affect_proc_lookup("plague");
    af.flags     = AFF_CONT_UPDATE|AFF_INVIS|AFF_PRI_MEDIUM;
    af.name        = "";
    if ( IS_NPC(victim) )
    af.appearance    = "";
    else
    af.appearance    = "%n %p, a fevered sweat covering %s body.";
    affect_join(victim,&af);

    send_to_char( "You feel hot and feverish.\n\r", victim );
    act( "$n shivers and looks very ill.", victim, NULL, NULL, TO_ROOM);
}


void plague_affect_proc( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    if ( reason == AFF_PROC_PERIODIC )
    {
    AFFECT_DATA plague, plague_carrier;
    CHAR_DATA *vch;
    int dam;

    if ( ch->in_room == NULL )
        return;

    REMOVE_BIT( paf->flags, AFF_INVIS );

    act( "$n writhes in agony as plague sores erupt from $s skin.",
        ch, NULL, NULL, TO_ROOM );
    send_to_char( "You writhe in agony from the plague.\n\r", ch );

    if ( paf->level == 1 )
        return;

    plague.where        = TO_AFFECTS;
    plague.type         = gsn_plague;
    plague.level         = paf->level - 1;
    plague.duration     = number_range( 1, 2 * plague.level );
    plague.location        = APPLY_STR;
    plague.modifier     = -20;
    plague.bitvector     = AFF_PLAGUE;
    plague.procNo        = paf->procNo;
    plague.flags        = AFF_CONT_UPDATE|AFF_INVIS|AFF_PRI_MEDIUM;
    plague.name        = "";

    plague_carrier.where        = TO_AFFECTS;
    plague_carrier.type        = gsn_plague;
    plague_carrier.level        = paf->level / 2;
    plague_carrier.duration        = number_range( 1,
                                          2 * plague_carrier.level );
    plague_carrier.location        = APPLY_NONE;
    plague_carrier.modifier        = 0;
    plague_carrier.bitvector    = AFF_PLAGUE;
    plague_carrier.procNo        = 0;
    plague_carrier.flags        = AFF_INVIS;
    plague_carrier.name        = "";
    plague_carrier.appearance    = "";

    for ( vch = ch->in_room->people; vch != NULL; vch = vch->next_in_room )
    {
        if ( !saves_spell(plague.level - 2,vch,DAM_DISEASE)
        &&  !IS_IMMORTAL(vch)
        &&  !IS_AFFECTED(vch,AFF_PLAGUE) && number_bits(4) == 0)
        {
        if ( number_bits(2) == 0 )
        {
        affect_join( vch, &plague_carrier );
        }
        else
        {
        if ( IS_NPC(vch) )
            plague.appearance = "";
        else
            plague.appearance = "%n %p, a fevered sweat covering %s body.";
        send_to_char( "You feel hot and feverish.\n\r", vch );
        act( "$n shivers and looks very ill.", vch, NULL, NULL, TO_ROOM );
        affect_join( vch, &plague );
        }
        }
    }

    dam = UMIN( ch->level, paf->level / 5 + 1 );

    ch->mana -= dam;
    ch->move -= dam;

    damage_old( ch, ch, dam, gsn_plague,
                DAM_DISEASE, FALSE, NULL );
    }

    return;
}


void spell_poison( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim;
    OBJ_DATA *obj;
    AFFECT_DATA af;

    if (target == TARGET_OBJ)
    {
    obj = (OBJ_DATA *) vo;

    if (obj->item_type == ITEM_FOOD || obj->item_type == ITEM_DRINK_CON)
    {
        if (IS_OBJ_STAT(obj,ITEM_BLESS) || IS_OBJ_STAT(obj,ITEM_BURN_PROOF))
        {
        act("Your spell fails to corrupt $p.",ch,obj,NULL,TO_CHAR);
        return;
        }
        obj->value[3] = 1;
        act("$p is infused with poisonous vapors.",ch,obj,NULL,TO_ALL);
        return;
    }

    if (obj->item_type == ITEM_WEAPON)
    {
        if (IS_WEAPON_STAT(obj,WEAPON_FLAMING)
        ||  IS_WEAPON_STAT(obj,WEAPON_FROST)
        ||  IS_WEAPON_STAT(obj,WEAPON_VAMPIRIC)
        ||  IS_WEAPON_STAT(obj,WEAPON_SHARP)
        ||  IS_WEAPON_STAT(obj,WEAPON_VORPAL)
        ||  IS_WEAPON_STAT(obj,WEAPON_SHOCKING)
        ||  IS_WEAPON_STAT(obj,WEAPON_FADING)
        ||  IS_OBJ_STAT(obj,ITEM_BLESS) || IS_OBJ_STAT(obj,ITEM_BURN_PROOF))
        {
        act("You can't seem to envenom $p.",ch,obj,NULL,TO_CHAR);
        return;
        }

        if (IS_WEAPON_STAT(obj,WEAPON_POISON))
        {
        act("$p is already envenomed.",ch,obj,NULL,TO_CHAR);
        return;
        }

        af.where        = TO_WEAPON;
        af.type         = sn;
        af.level        = level / 2;
        af.duration     = level/8;
        af.location     = 0;
        af.modifier     = 0;
        af.bitvector    = WEAPON_POISON;
        af.procNo       = 0;
        af.flags        = 0;
        af.name         = "";
        af.appearance   = "";
        affect_to_obj(obj,&af);

        act("$p is coated with deadly venom.",ch,obj,NULL,TO_ALL);
        return;
    }

    act("You can't poison $p.",ch,obj,NULL,TO_CHAR);
    return;
    }

    victim = (CHAR_DATA *) vo;

    if ( saves_spell( level, victim,DAM_POISON) )
    {
    act("$n turns slightly green, but it passes.",victim,NULL,NULL,TO_ROOM);
    send_to_char("You feel momentarily ill, but it passes.\n\r",victim);
    return;
    }

    af.where        = TO_AFFECTS;
    af.type         = sn;
    af.level        = level;
    af.duration     = level;
    af.location     = APPLY_STR;
    af.modifier     = -8;
    af.bitvector    = AFF_POISON;
    af.procNo       = affect_proc_lookup("poison");
    af.flags        = AFF_CONT_UPDATE|AFF_PRI_LOW;
    af.name         = "";
    if ( IS_NPC(victim) )
    af.appearance   = "";
    else
    af.appearance   = "%n %p, a sickly green tinge coloring %s skin.";
    affect_join( victim, &af );
    send_to_char( "You feel very sick.\n\r", victim );
    act("$n looks very ill.",victim,NULL,NULL,TO_ROOM);
    return;
}


void spell_polar_wind( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    static const sh_int dam_each[] =
    {
      0,
      0,   0,   0,   0,   0,      0,   0,   0,   0,   0,
      0,   0,   0,   0,  30,     35,  40,  45,  50,  55,
     60,  65,  70,  75,  80,     82,  84,  86,  88,  90,
     92,  94,  96,  98, 100,    102, 104, 106, 108, 110,
    112, 114, 116, 118, 120,    122, 124, 126, 128, 130
    };
    CHAR_DATA *vch;
    int dam;

    level    = UMIN( level, sizeof(dam_each) / sizeof(dam_each[0]) - 1 );
    level    = UMAX( 0, level );
    dam        = number_range( dam_each[level] / 2, dam_each[level] * 2 );
    if ( saves_spell(level, victim, DAM_COLD) )
    dam /= 2;
    damage_old( ch, victim, dam, sn, DAM_COLD ,TRUE, NULL );

    for ( vch = ch->in_room->people; vch != NULL; vch = vch->next_in_room )
    {
    if ( !can_see(ch, vch) || vch->fighting != ch || vch == victim )
        continue;

    level    = UMIN( level, sizeof(dam_each) / sizeof(dam_each[0]) - 1 );
    level    = UMAX( 0, level );
    dam    = number_range( dam_each[level] / 4, 3 * dam_each[level] / 4 );
    if ( saves_spell(level, vch, DAM_COLD) )
        dam /= 2;
    damage_old( ch, vch, dam, sn, DAM_COLD ,TRUE, NULL );
    }

    return;
}


void poison_affect_proc( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    if ( reason == AFF_PROC_PERIODIC )
    {
        if ( ch != NULL && !IS_AFFECTED(ch, AFF_SLOW) )
        {
        act( "$n shivers and suffers.", ch, NULL, NULL, TO_ROOM );
        send_to_char( "You shiver and suffer.\n\r", ch );

        damage_old( ch, ch, paf->level / 10 + 1,
                    gsn_poison, DAM_POISON, FALSE,
                    NULL );
        }
    }

    tail_chain();
    return;
}


void spell_portal( int sn, int level, CHAR_DATA *ch, void *vo,int target)
{
    CHAR_DATA *victim;
    OBJ_DATA *portal, *stone;

    if ( (victim = get_char_world(ch, target_name)) == NULL
    ||   victim == ch
    ||   victim->in_room == NULL
    ||   !can_see_room(ch,victim->in_room)
    ||   IS_SET(victim->in_room->room_flags, ROOM_SAFE)
    ||   IS_SET(victim->in_room->room_flags, ROOM_PRIVATE)
    ||   IS_SET(victim->in_room->room_flags, ROOM_SOLITARY)
    ||   IS_SET(victim->in_room->room_flags, ROOM_NO_RECALL)
    ||   IS_SET(ch->in_room->room_flags, ROOM_NO_RECALL)
    ||   victim->level >= level + 3
    ||   (!IS_NPC(victim) && victim->level >= LEVEL_HERO)  /* NOT trust */
    ||   (IS_NPC(victim) && IS_SET(CHAR_IMM(victim),IMM_SUMMON))
    ||   (IS_NPC(victim) && saves_spell( level, victim,DAM_NONE) )
    /*||   (is_clan(victim) && !is_same_clan(ch,victim))*/)
    {
        send_to_char( "You failed.\n\r", ch );
        return;
    }

    stone = get_eq_char(ch,WEAR_HOLD);
    if (!IS_IMMORTAL(ch)
    &&  (stone == NULL || stone->item_type != ITEM_WARP_STONE))
    {
    send_to_char("You lack the proper component for this spell.\n\r",ch);
    return;
    }

    if (stone != NULL && stone->item_type == ITEM_WARP_STONE)
    {
         act("You draw upon the power of $p.",ch,stone,NULL,TO_CHAR);
         act("It flares brightly and vanishes!",ch,stone,NULL,TO_CHAR);
         extract_obj(stone);
    }

    portal = create_object(get_obj_index(OBJ_VNUM_PORTAL),0, FALSE);
    portal->timer = 2 + level / 25;
    portal->value[3] = victim->in_room->vnum;

    obj_to_room(portal,ch->in_room);

    act("$p rises up from the ground.",ch,portal,NULL,TO_ROOM);
    act("$p rises up before you.",ch,portal,NULL,TO_CHAR);
}


void spell_prayer( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    AFFECT_DATA af;
    CHAR_DATA *gch;

    for ( gch = ch->in_room->people; gch != NULL; gch = gch->next_in_room )
    {
    if ( !is_same_group( gch, ch ) || is_affected(gch, sn) )
        continue;

    act( "A golden aura briefly glows around $n.", gch, NULL, NULL, TO_ROOM );
    send_to_char( "An aura of gold briefly glows around your body.\n\r", gch );

    af.where    = TO_AFFECTS;
    af.type        = sn;
    af.level    = level;
    af.duration    = 24;
    af.location    = APPLY_HITROLL;
    af.modifier    = 5;
    af.bitvector    = 0;
    af.procNo    = 0;
    af.flags    = 0;
    af.name        = "";
    af.appearance    = "";
    affect_to_char( gch, &af );
    }

    send_to_char( "Ok.\n\r", ch );
}


void spell_recall_druidstaff( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *staff;

    staff = find_unique( OBJ_VNUM_DRUID_STAFF, ch->name );

    if ( staff == NULL )
    {
    send_to_char( "Nothing seems to happen.\n\r", ch );
    return;
    }

    if ( staff->carried_by )
    {
    act( "$p fades from view.", staff->carried_by, staff, NULL, TO_CHAR );
    act( "$p fades from view.", staff->carried_by, staff, NULL, TO_ROOM );

    obj_from_char( staff );
    }
    else if ( staff->in_room )
    {
    act( "$p fades from view.", staff->carried_by, staff, NULL, TO_CHAR );
    act( "$p fades from view.", staff->carried_by, staff, NULL, TO_ROOM );

    obj_from_room( staff );
    }
    else
    obj_from_obj( staff );

    obj_to_char( staff, ch );

    act( "$p shimmers into view.", ch, staff, NULL, TO_CHAR );
    act( "$p shimmers into view.", ch, staff, NULL, TO_ROOM );
}


void spell_recall_shadowblade( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *blade;

    blade = find_unique( OBJ_VNUM_SHADOWBLADE, ch->name );

    if ( blade == NULL )
    {
    send_to_char( "Nothing seems to happen.\n\r", ch );
    return;
    }

    if ( blade->carried_by )
    {
    act( "$p fades from view.", blade->carried_by, blade, NULL, TO_CHAR );
    act( "$p fades from view.", blade->carried_by, blade, NULL, TO_ROOM );

    obj_from_char( blade );
    }
    else if ( blade->in_room )
    {
    act( "$p fades from view.", blade->in_room->people, blade, NULL, TO_CHAR );
    act( "$p fades from view.", blade->in_room->people, blade, NULL, TO_ROOM );

    obj_from_room( blade );
    }
    else
    obj_from_obj( blade );

    obj_to_char( blade, ch );

    act( "$p shimmers into view.", ch, blade, NULL, TO_CHAR );
    act( "$p shimmers into view.", ch, blade, NULL, TO_ROOM );
}


void spell_shadow( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( is_affected(victim, sn) )
    {
    if (victim == ch)
        send_to_char(
            "You already walk the realm of shadows.\n\r", ch );
    else
        act( "$N already walks the realm of shadows.",
            ch, NULL, victim, TO_CHAR );
    return;
    }

    af.where        = TO_AFFECTS;
    af.type        = sn;
    af.level        = level;
    af.duration        = 24;
    af.modifier        = 0;
    af.location        = APPLY_NONE;
    af.bitvector    = AFF_SNEAK;
    af.procNo        = 0;
    af.flags        = AFF_PRI_MEDIUM;
    af.name        = "";
    if ( IS_NPC(ch) )
    af.appearance    = "";
    else
    af.appearance    = "%n %p, cloaked in shadows.";
    affect_to_char( victim, &af );

    af.where        = TO_VULN;
    af.bitvector    = IMM_LIGHT;
    af.appearance    = "";
    affect_to_char( victim, &af );

    send_to_char( "You enter the realm of shadows.", victim );
    if ( ch != victim )
    act( "$N enters the realm of shadows.", ch, NULL, victim, TO_CHAR );
}


void spell_shadowblade_true( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *blade = (OBJ_DATA *) vo;
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA *paf;

    level = ch->level;     /* We use casters true level for these spells. */

    if ( blade->pIndexData->vnum != OBJ_VNUM_SHADOWBLADE )
    {
    send_to_char( "That isn't a shadowblade!\n\r", ch );
    return;
    }

    if ( IS_NULLSTR(blade->owner) || str_cmp(blade->owner, ch->name) )
    {
    send_to_char( "You can only cast that on your own shadowblade.\n\r", ch );
    return;
    }

    if ( str_suffix("lord", blade->name) )
    {
    send_to_char( "You can only cast this on a lord shadowblade.\n\r", ch );
    return;
    }

    sprintf( buf, "%s true", blade->pIndexData->name );
    free_string( blade->name );
    blade->name = str_dup( buf );

    for ( paf = blade->affected; paf != NULL; paf = paf->next )
    {
    if ( paf->location == APPLY_HITROLL )
        paf->modifier = UMAX( 8, paf->modifier );
    else if ( paf->location == APPLY_DAMROLL )
        paf->modifier = UMAX( 8, paf->modifier );
    }

    blade->value[1] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].number;
    blade->value[2] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].type;
    blade->level    = ch->level;

    act( "You cloak it in utter darkness!",
         ch, blade, NULL, TO_CHAR );
    act( "$n cloaks an evil looking blade in utter darkness!",
         ch, blade, NULL, TO_ROOM );
}


void spell_shadowfire( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    int dam;

    dam = dice( level, 11 );
    if ( saves_spell(level, victim, DAM_NEGATIVE) )
    dam /= 2;

    cold_effect( victim, level/2, dam/4, TARGET_CHAR );
    damage_old( ch, victim, dam, sn, DAM_NEGATIVE, TRUE, NULL );
}


void spell_silver_defender( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *defender = (OBJ_DATA *) vo;
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA *paf;

    level = ch->level;     /* We use casters true level for these spells. */

    if ( defender->pIndexData->vnum != OBJ_VNUM_DEFENDER )
    {
    send_to_char( "That isn't a defender!\n\r", ch );
    return;
    }

    if ( IS_NULLSTR(defender->owner) || str_cmp(defender->owner, ch->name) )
    {
    send_to_char( "You can only cast that on your own defender.\n\r", ch );
    return;
    }

    if ( str_suffix("greater", defender->name) )
    {
    send_to_char( "You can only cast this on a greater defender.\n\r", ch );
    return;
    }

    sprintf( buf, "%s silver", defender->name );
    free_string( defender->name );
    defender->name = str_dup( buf );

    for ( paf = defender->affected; paf != NULL; paf = paf->next )
    {
    if ( paf->location == APPLY_HITROLL )
        paf->modifier = UMAX( 5, paf->modifier );
    else if ( paf->location == APPLY_DAMROLL )
        paf->modifier = UMAX( 5, paf->modifier );
    }

    defender->value[1] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].number;
    defender->value[2] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].type;
    defender->level    = ch->level;

    act( "$p glows with a silvery light!", ch, defender, NULL, TO_CHAR );
    act( "$p glows with a silvery light!", ch, defender, NULL, TO_ROOM );
}


void spell_silver_druidstaff( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *staff = (OBJ_DATA *) vo;
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA *paf;

    level = ch->level;     /* We use casters true level for these spells. */

    if ( staff->pIndexData->vnum != OBJ_VNUM_DRUID_STAFF )
    {
    send_to_char( "That isn't a druidstaff!\n\r", ch );
    return;
    }

    if ( IS_NULLSTR(staff->owner) || str_cmp(staff->owner, ch->name) )
    {
    send_to_char( "You can only cast that on your own staff.\n\r", ch );
    return;
    }

    if ( str_suffix("greater", staff->name) )
    {
    send_to_char( "You can only cast this on a greater driudstaff.\n\r", ch );
    return;
    }

    sprintf( buf, "%s silver", staff->name );
    free_string( staff->name );
    staff->name = str_dup( buf );

    for ( paf = staff->affected; paf != NULL; paf = paf->next )
    {
    if ( paf->location == APPLY_HITROLL )
        paf->modifier = UMAX( 5, paf->modifier );
    else if ( paf->location == APPLY_DAMROLL )
        paf->modifier = UMAX( 5, paf->modifier );
    else if ( paf->location == APPLY_MANA )
        paf->modifier = 20;
    }

    staff->value[1] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].number;
    staff->value[2] = weapon_autos[UMIN(MAX_LEVEL, UMAX(ch->level,level)) - 1].type;
    staff->level    = ch->level;

    act( "$p glows with a silvery light!", ch, staff, NULL, TO_CHAR );
    act( "$p glows with a silvery light!", ch, staff, NULL, TO_ROOM );
}


void spell_starvation( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA af;

    if ( is_affected(ch, sn) ) {
    if ( victim == ch )
        send_to_char( "You are already starving.\n\r", ch );
    else
        act( "$N is already starving.", ch, NULL, victim, TO_CHAR );

    return;
    }

    if (saves_spell(level, victim, DAM_OTHER)
    ||  IS_SET(CHAR_IMM(victim), IMM_MAGIC))
    {
    if ( victim != ch )
            send_to_char( "Nothing seemed to happen.\n\r", ch );
        send_to_char( "You feel a momentary craving for food.\n\r", victim );
        return;
    }

    af.where        = TO_AFFECTS;
    af.type         = sn;
    af.level        = level;
    af.duration     = level;
    af.location     = APPLY_NONE;
    af.modifier     = 0;
    af.bitvector    = 0;
    af.procNo       = affect_proc_lookup("starvation");
    af.flags        = AFF_CONT_UPDATE;
    sprintf( buf, "%s's curse of starvation.",
        PERS(ch, victim) );
    af.name         = buf;
    af.appearance   = "";

    affect_to_char( victim, &af );

    send_to_char( "You feel a gnawing hunger in your stomach.\n\r", victim );
    if ( ch != victim )
    {
    act( "$N looks hungry.", ch, NULL, victim, TO_NOTVICT );
    act( "$N looks hungry.", ch, NULL, victim, TO_CHAR );
    }
}


void spell_stone_to_flesh( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA *paf;

    if ( !is_affected(victim, gsn_petrification)
    &&   !is_affected(victim, gsn_petrify) )
    {
    send_to_char( "Their flesh is not petrified.\n\r", ch );
    return;
    }

    if ( (paf = affect_find(victim->affected, gsn_petrification)) != NULL )
    {
    if ( number_percent() < 50 + 5 * (level - paf->level) )
    {
        affect_strip( victim, gsn_petrification );
        send_to_char( "Their body slowly returns to normal.", ch );
    }
    else
        send_to_char( "Nothing seemed to happen.\n\r", ch );

    return;
    }

    if ( (paf = affect_find(victim->affected, gsn_petrify)) != NULL
    &&   number_percent() < 50 + 5 * (level - paf->level) )
    {
    affect_strip( victim, gsn_petrify );
    send_to_char(
        "The faint look of pain on their face seems to fade.\n\r", ch );
    return;
    }

    send_to_char( "Nothing seemed to happen.\n\r", ch );
}


void spell_swarm( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    CHAR_DATA *swarm;

    if ( victim == ch )
    {
    send_to_char( "That would be pretty silly wouldn't it?\n\r", ch );
    return;
    }

    swarm = create_mobile( get_mob_index(MOB_VNUM_SWARM) );
    swarm->level                 = level;
    swarm->max_hit              *= level;
    swarm->hit                   = swarm->max_hit;
    swarm->damage[DICE_NUMBER]   = mobile_autos[UMIN(MAX_LEVEL-1,
                                               level+10)].damage[DICE_NUMBER];
    swarm->damage[DICE_TYPE]     = mobile_autos[UMIN(MAX_LEVEL-1,
                                               level+10)].damage[DICE_TYPE];
    swarm->damage[DICE_BONUS]    = mobile_autos[UMIN(MAX_LEVEL-1,
                                               level+10)].damage[DICE_BONUS];
    char_to_room( swarm, ch->in_room );

    act( "The sky blackens as a huge swarm of insects descends.",
         ch, NULL, NULL, TO_ROOM );
    act( "The sky blackens as a huge swarm of insects descends.",
         ch, NULL, NULL, TO_CHAR );

    if ( HAS_TRIGGER(swarm, TRIG_ENTRY) )
    mp_percent_trigger( swarm, NULL, NULL, NULL, TRIG_ENTRY );

    add_follower( swarm, ch );
    swarm->leader = ch;

    multi_hit( swarm, victim, TYPE_UNDEFINED );
    if ( ch->fighting == victim )
    do_rescue( swarm, CHAR_NAME(ch) );
    multi_hit( ch, victim, TYPE_UNDEFINED );

    act_sound( "wasp* V=100", ch, NULL, TO_ALL );
}


void starvation_affect_proc( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    if ( ch == NULL )
    return;

    if ( reason == AFF_PROC_PERIODIC )
    {
    if ( !IS_NPC(ch) )
    {
        ch->pcdata->condition[COND_HUNGER]  = 0;
        ch->pcdata->condition[COND_FULL]    = 0;
    }
    }

    return;
}


void spell_thunderstaff( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *staff = (OBJ_DATA *) vo;
    AFFECT_DATA af;

    if ( staff->pIndexData->vnum != OBJ_VNUM_DRUID_STAFF )
    {
    send_to_char( "That isn't a druidstaff!\n\r", ch );
    return;
    }

    if ( IS_NULLSTR(staff->owner) || str_cmp(staff->owner, ch->name) )
    {
    send_to_char( "You can only cast that on your own staff.\n\r", ch );
    return;
    }

    if ( IS_WEAPON_STAT(staff, WEAPON_FLAMING)
    ||   IS_WEAPON_STAT(staff, WEAPON_FROST)
    ||   IS_WEAPON_STAT(staff, WEAPON_VAMPIRIC)
    ||   IS_WEAPON_STAT(staff, WEAPON_SHARP)
    ||   IS_WEAPON_STAT(staff, WEAPON_VORPAL)
    ||   IS_WEAPON_STAT(staff, WEAPON_POISON) )
    {
    send_to_char( "Nothing seems to happen.", ch );
    return;
    }

    if ( IS_WEAPON_STAT(staff, WEAPON_SHOCKING) )
    {
    act( "Arcing sparks already surround $p.",
        ch, staff, NULL, TO_CHAR );
    return;
    }

    af.where        = TO_WEAPON;
    af.type         = sn;
    af.level        = level / 2;
    af.duration     = level / 5;
    af.location     = 0;
    af.modifier     = 0;
    af.bitvector    = WEAPON_SHOCKING;
    af.procNo       = 0;
    af.flags        = 0;
    af.name         = "";
    af.appearance   = "";
    affect_to_obj( staff, &af );

    act( "$p is surrounded by an arcing layer of sparks.",
         ch, staff, NULL, TO_ALL );
}


void spell_winters_touch( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *obj = (OBJ_DATA *) vo;
    AFFECT_DATA af;

    if (obj->item_type == ITEM_WEAPON)
    {
        if ( IS_WEAPON_STAT(obj,WEAPON_FLAMING)
        ||  IS_WEAPON_STAT(obj,WEAPON_SHOCKING) )
        {
            send_to_char( "Nothing seems to happen.", ch );
            return;
        }

        if ( IS_WEAPON_STAT(obj,WEAPON_FROST) )
        {
            act( "$p is already surrounded by an icy frost.",
                ch, obj, NULL, TO_CHAR );
            return;
        }

        af.where        = TO_WEAPON;
        af.type         = sn;
        af.level        = level / 2;
        af.duration     = level/8;
        af.location     = 0;
        af.modifier     = 0;
        af.bitvector    = WEAPON_FROST;
        af.procNo       = 0;
        af.flags        = 0;
        af.name         = "";
        af.appearance   = "";
        affect_to_obj( obj, &af );

        act( "$p is surrounded by an icy frost.", ch, obj, NULL, TO_ALL );
        return;
    }

    act( "$p is not a weapon.", ch, obj, NULL, TO_CHAR );
}


void spell_withering_touch( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;
    int dam;

    if ( saves_spell(level, victim, DAM_NONE) )
    {
        act( "$N avoids your touch.", ch, NULL, victim, TO_CHAR );
        return;
    }

    if ( ch != victim )
    {
        act( "$n reaches out and grips your wrist, $s icy hand numbing you to the bone.",
            ch, NULL, victim, TO_VICT );
        act( "$N screams out in pain as $n reaches out and grabs $m.",
            ch, NULL, victim, TO_NOTVICT );
    }

    dam = number_range( 100, 150 );

    if ( !saves_spell(level, victim, DAM_COLD) )
    {
        af.where        = TO_AFFECTS;
        af.type         = sn;
        af.level        = level;
        af.duration     = 24;
        af.location     = APPLY_STR;
        af.modifier     = -4;
        af.bitvector    = 0;
        af.procNo       = 0;
        af.flags        = 0;
        af.name         = "";
        af.appearance   = "";
        affect_join( victim, &af );

        af.location     = APPLY_DEX;
        affect_join( victim, &af );
    }
    else
	dam /= 2;

    damage_old( ch, victim, dam, sn, DAM_COLD ,TRUE, NULL );
}







/*******************************************************************************/

void conjure( int sn, int spellLevel, int mobLevel, CHAR_DATA *ch, CHAR_DATA *victim, int target )
{
    CHAR_DATA    *demon;
    CHAR_DATA    *rch;
    AFFECT_DATA    af;
    int    mobVNUM;
    int count;

    // disallow if specified target
    if ( victim != NULL )
    {
    send_to_char( "This spell can't be cast on a target.\n\r", ch );
    return;
    }

    // disallow if wrong type of room
    if ( IS_SET(ch->in_room->room_flags, ROOM_LAW)
    ||     IS_SET(ch->in_room->room_flags, ROOM_SAFE)
    ||     IS_SET(ch->in_room->room_flags, ROOM_SOLITARY) )
    {
    send_to_char( "You can't cast that spell in here.\n\r", ch );
       return;
    }

    if ( IS_SET(ch->in_room->room_flags, ROOM_PRIVATE ))
    {
    count = 0;
    for (rch = ch->in_room->people; rch != NULL; rch = rch->next_in_room)
        count++;

    if (count > 1)
    {
        send_to_char( "This room's too crowded already.\n\r", ch );
        return;
    }
    }

    // check for existing demons
    if ( !is_mob_unique(MOB_VNUM_DEMON_1, ch)
    ||   !is_mob_unique(MOB_VNUM_DEMON_2, ch)
    ||   !is_mob_unique(MOB_VNUM_DEMON_3, ch)
    ||   !is_mob_unique(MOB_VNUM_DEMON_4, ch)
    ||   !is_mob_unique(MOB_VNUM_DEMON_5, ch)
    ||   !is_mob_unique(MOB_VNUM_DEMON_6, ch) )
    {
    send_to_char( "You can only have one demonic servant at once.\n\r", ch );
    return;
    }

    // check change of summoning higher level demon
    while ( dice(1, 10) == 10 && mobLevel < MAX_LEVEL )
    {
    mobLevel += 10;
    }

    // determine appropriate demon to summon
         if ( mobLevel <= 10 ) mobVNUM = MOB_VNUM_DEMON_1;
    else if ( mobLevel <= 20 ) mobVNUM = MOB_VNUM_DEMON_2;
    else if ( mobLevel <= 30 ) mobVNUM = MOB_VNUM_DEMON_3;
    else if ( mobLevel <= 40 ) mobVNUM = MOB_VNUM_DEMON_4;
    else if ( mobLevel <= 50 ) mobVNUM = MOB_VNUM_DEMON_5;
    else                       mobVNUM = MOB_VNUM_DEMON_6;

    ch->alignment = UMAX( -1000, ch->alignment - 50 );

    // create demon
    demon = create_mobile( get_mob_index(mobVNUM) );
    char_to_room( demon, ch->in_room );

    act( "The stench of brimstone assails you as a huge form steps through a gate of pure black.",
       ch, NULL, NULL, TO_ALL );

    af.where        = TO_AFFECTS;
    af.type        = sn;
    af.level        = spellLevel;
    af.duration        = 3;
    af.location        = 0;
    af.modifier        = APPLY_NONE;
    af.bitvector    = 0;
    af.procNo        = affect_proc_lookup("conjuration");
    af.flags        = AFF_CONT_UPDATE;
    af.name        = "";
    af.appearance    = "";

    affect_to_char( demon, &af );

    if ( !saves_spell(spellLevel, demon, DAM_CHARM) )
    {
    af.bitvector    = AFF_CHARM;
    af.procNo    = 0;
    af.flags    = 0;
    affect_to_char( demon, &af );

    af.where    = TO_AFFECTS2;
    af.bitvector    = AFF2_DOMINATION;
    affect_to_char( demon, &af );

    act( "$N glares at you, snarling as $E strains against the bonds of your control.",
        ch, NULL, demon, TO_CHAR );

    add_follower( demon, ch );
    demon->leader = ch;

    if ( ch->fighting != NULL )
        do_rescue( demon, CHAR_NAME(ch) );
    }
    else
    {
    multi_hit( demon, ch, TYPE_UNDEFINED );
    }
}

void spell_conjuration( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA    *victim = (CHAR_DATA *) vo;

    conjure( sn, level, 30, ch, victim, target);
    return;
}

void spell_conjuration_true( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA    *victim = (CHAR_DATA *) vo;

    conjure( sn, level, 50, ch, victim, target);
    return;
}

void spell_minor_conjuration( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA    *victim = (CHAR_DATA *) vo;

    conjure( sn, level, 10, ch, victim, target);
    return;
}

void spell_lesser_conjuration( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA    *victim = (CHAR_DATA *) vo;

    conjure( sn, level, 20, ch, victim, target);
    return;
}

void spell_greater_conjuration( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA    *victim = (CHAR_DATA *) vo;

    conjure( sn, level, 40, ch, victim, target);
    return;
}

void conjuration_affect_proc( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    if ( ch == NULL )
        return;

    if ( reason == AFF_PROC_FINISHED )
    {
        act( "There is a bright flash, the demon is drawn into it and is gone.",
            ch, NULL, NULL, TO_ROOM );
        extract_char(ch, TRUE);
    }
    return;
}

//----------------------------------------------------------------------------------

void spell_banishment( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA    *victim = (CHAR_DATA *) vo;

    // is the target a demon?
    if ( !IS_SET(race_table[CHAR_RACE(victim)].char_flags, CHAR_DEMON) )
    {
    act( "$N is not a demon.", ch, NULL, victim, TO_CHAR );
    return;
    }

    // does the demon save?
    if ( saves_spell(level, victim, DAM_NONE) )
    {
    act( "$N seems unaffected.", ch, NULL, victim, TO_CHAR );
    return;
    }

    // the demon is about to be banished
    act( "There is a bright flash, the demon is drawn into it and is gone.",
    ch, NULL, NULL, TO_ROOM );
    act( "There is a bright flash, the demon is drawn into it and is gone.",
    ch, NULL, NULL, TO_CHAR );
    extract_char(victim, TRUE);
    return;
}


//----------------------------------------------------------------------------------

void spell_fix_soul( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    AFFECT_DATA    af;

    /* check if we've already cast fix soul */
    if ( IS_AFFECTED2(ch, AFF2_FIX_SOUL) )
    {
	send_to_char( "Your soul is already spread too thin to fix here.\n\r", ch );
	return;
    }

    /* don't allow in no_recall or safe zones */
/*
 *    if ( IS_SET(ch->in_room->room_flags, ROOM_NO_RECALL)
 *    ||   IS_SET(ch->in_room->room_flags, ROOM_SAFE) )
 *    {
 *	send_to_char( "You can't seem to fix this spot in your mind.\n\r", ch );
 *	return;
 *    }
 */
 
    /* otherwise, store current location */
    af.where		= TO_AFFECTS2;
    af.type		= sn;
    af.level		= level;
    af.duration		= -1;
    af.location		= APPLY_NONE;
    af.modifier		= ch->in_room->vnum;
    af.bitvector	= AFF2_FIX_SOUL;
    af.procNo		= 0;
    af.flags		= AFF_INVIS;
    af.name		= "";
    af.appearance	= "";
    affect_to_char( ch, &af );

    af.where		= TO_AFFECTS2;
    af.type		= sn;
    af.level		= level;
    af.duration		= -1;
    af.location		= APPLY_CON;
    af.modifier		= -4;
    af.bitvector	= AFF2_FIX_SOUL;
    af.procNo		= 0;
    af.flags		= AFF_INVIS;
    af.name		= "";
    af.appearance	= "";
    affect_to_char( ch, &af );

    send_to_char( "You feel a slight loss as part of your soul leaves you.\n\r", ch );
    return;
}

void spell_merge_soul( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    AFFECT_DATA    *af;

    /* must have already cast fix soul */
    if ( !IS_AFFECTED2(ch, AFF2_FIX_SOUL) )
    {
	send_to_char( "You must have fixed your soul somewhere to cast this spell.\n\r", ch );
	return;
    }

    /* locate the appropriate affect proc */
    af = affect_find( ch->affected, gsn_fix_soul );
    if ( af->location == APPLY_CON )
    {
	af = af->next;
	if ( af == NULL )
	{
	    bug( "merge_soul: missing affect data.", 0 );
	    send_to_char( "Nothing seems to happen.\n\r", 0 );
	    return;
	}
    }

    /* check we are in the room where our soul is fixed */
    if ( af->modifier != ch->in_room->vnum )
    {
	send_to_char( "You can't seem to sense the missing part of your soul here.\n\r", ch );
	return;
    }

    /* finally remove the fix soul affect */
    affect_strip( ch, gsn_fix_soul );

    send_to_char( "A sense of wholeness fills you as your soul reunites.\n\r", ch );
    return;
}

void spell_rejoin_soul( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    ROOM_INDEX_DATA *dest;
    AFFECT_DATA	*af;
    bool gate_pet;

    /* must have already cast fix soul */
    if ( !IS_AFFECTED2(ch, AFF2_FIX_SOUL) )
    {
	send_to_char( "You must have fixed your soul somewhere to cast this spell.\n\r", ch );
	return;
    }

    /* first, locate the appropriate affect proc */
    af = affect_find( ch->affected, gsn_fix_soul );
    if ( af->location == APPLY_CON )
    {
	af = af->next;
	if( af == NULL )
        {
	    bug( "merge_soul:missing affect data.", 0 );
	    send_to_char( "Nothing seems to happen.\n\r", 0 );
	    return;
	}
    }

    if ( (dest = get_room_index(af->modifier)) == NULL
    ||   !can_see_room(ch, dest) )
    {
	send_to_char( "You sense nothingness where there used to be...\n\r", ch );
	return;
    }

    /* otherwise, gate character and possibly pet */
    if ( ch->pet != NULL && ch->in_room == ch->pet->in_room )
	gate_pet = TRUE;
    else
	gate_pet = FALSE;

    /* first, locate the appropriate affect proc */
    af = affect_find( ch->affected, gsn_fix_soul );

    /* do the gate */
    act( "$n steps through a gate and vanishes.", ch, NULL,NULL,TO_ROOM);
    send_to_char("You step through a gate and vanish.\n\r",ch);
    char_from_room( ch, FALSE );

    char_to_room( ch, dest );
    act( "$n has arrived through a gate.", ch, NULL, NULL, TO_ROOM );
    do_look( ch, "auto" );

    if ( gate_pet )
    {
    act( "$n steps through a gate and vanishes.", ch->pet, NULL, NULL, TO_ROOM );
    send_to_char( "You step through a gate and vanish.\n\r", ch->pet );
    char_from_room( ch->pet, FALSE );
    char_to_room( ch->pet, ch->in_room );
    act( "$n has arrived through a gate.", ch->pet, NULL, NULL, TO_ROOM );
    do_look( ch->pet, "auto" );

    if ( HAS_TRIGGER(ch->pet, TRIG_ENTRY) )
        mp_percent_trigger( ch->pet, NULL, NULL, NULL, TRIG_ENTRY );
    }

    if ( IS_NPC(ch) && HAS_TRIGGER(ch, TRIG_ENTRY) )
	mp_percent_trigger( ch, NULL, NULL, NULL, TRIG_ENTRY );
    if ( !IS_NPC(ch) )
	mp_greet_trigger( ch );

    return;
}

void spell_soul_vision( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    ROOM_INDEX_DATA *room, *dest;
    CHAR_DATA *rider, *mount;
    AFFECT_DATA *af;
    OBJ_DATA *on;

    /* must have already cast fix soul */
    if ( !IS_AFFECTED2(ch, AFF2_FIX_SOUL) )
    {
	send_to_char( "You must have fixed your soul somewhere to cast this spell.\n\r", ch );
	return;
    }

    /* first, locate the appropriate affect proc */
    af = affect_find( ch->affected, gsn_fix_soul );
    if ( af->location == APPLY_CON )
    {
	af = af->next;
	if ( af == NULL )
        {
	    bug( "merge_soul: missing affect data.", 0 );
	    send_to_char( "Nothing seems to happen.\n\r", 0 );
	    return;
	}
    }


    if ( (dest = get_room_index(af->modifier)) == NULL
    ||   !can_see_room(ch, dest) )
    {
	send_to_char( "You sense nothingness where there used to be...\n\r", ch );
	return;
    }

    /* do the look at the saved position */
    send_to_char( "You feel a momentary sense of wholeness.\n\r", ch );
    room		= ch->in_room;
    rider		= ch->mounted_by;
    mount		= ch->riding;
    on			= ch->on;
    char_from_room( ch, FALSE );
    char_to_room( ch, dest );
    do_look( ch, "auto" );

    /* return to our proper position now */
    char_from_room( ch, FALSE );
    char_to_room( ch, room );

    ch->mounted_by	= rider;
    ch->riding		= mount;
    ch->on		= on;

    return;
}

void spell_mark_tree( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    AFFECT_DATA    af;

    /* check if we've already cast fix soul */
    if ( IS_AFFECTED2(ch, AFF2_MARK_TREE) )
    {
	send_to_char( "You're already thinking of another tree.\n\r", ch );
	return;
    }

    /* don't allow in no_recall or safe zones */
    if ( IS_SET(ch->in_room->room_flags, ROOM_NO_RECALL)
    ||   IS_SET(ch->in_room->room_flags, ROOM_SAFE) )
    {
	send_to_char( "You can't seem to fix this spot in your mind.\n\r", ch );
	return;
    }

    /* The sector must be forest */
    if ( ch->in_room->sector_type != SECT_FOREST )
    {
    send_to_char( "You look around for a nearby tree in vain.\n\r", ch );
    return;
    }

    /* otherwise, store current location */
    af.where		= TO_AFFECTS2;
    af.type		= sn;
    af.level		= level;
    af.duration		= -1;
    af.location		= 0;
    af.modifier		= ch->in_room->vnum;
    af.bitvector	= AFF2_MARK_TREE;
    af.procNo		= 0;
    af.flags		= AFF_INVIS;
    af.name		= "";
    af.appearance	= "";
    affect_to_char( ch, &af ); 
	
    send_to_char( "You fix the location of this tree in your mind.\n\r", ch );
    return;
}

void spell_unmark_tree( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    AFFECT_DATA    *af;

    /* must have already cast fix soul */
    if ( !IS_AFFECTED2(ch, AFF2_MARK_TREE ) )
    {
	send_to_char( "You aren't remembering any tree locations at present.\n\r", ch );
	return;
    }

    /* locate the appropriate affect proc */
    af = affect_find( ch->affected, gsn_mark_tree );

    /* finally remove the fix soul affect */
    affect_strip( ch, gsn_mark_tree );

    send_to_char( "You wipe the memory of this tree from your mind.\n\r", ch );
    return;
}

void spell_tree_transport( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    ROOM_INDEX_DATA *dest;
    CHAR_DATA	*rch;
    AFFECT_DATA	*af;
    bool gate_pet;
    int count;

    /* must have already cast fix soul */
    if ( !IS_AFFECTED2(ch, AFF2_MARK_TREE ) )
    {
	send_to_char( "You can't remember any trees to travel to.\n\r", ch );
	return;
    }
		
    /* first, locate the appropriate affect proc */
    af = affect_find( ch->affected, gsn_mark_tree );

    if ( (dest = get_room_index(af->modifier)) == NULL
    ||   !can_see_room(ch, dest) )
    {
	send_to_char( "You sense nothingness where there was...\n\r", ch );
	return;
    }

    /* don't allow in no_recall or safe zones */
    if ( IS_SET(dest->room_flags, ROOM_SAFE)
    ||   IS_SET(dest->room_flags, ROOM_NO_RECALL) )
    {
	send_to_char( "Spell failed.\n\r", ch );
	return;
    }

    /* The sector must be forest */
    if ( ch->in_room->sector_type != SECT_FOREST )
    {
	send_to_char( "You look around for a nearby tree in vain.\n\r", ch );
	return;
    }

    /* don't allow if the stored room is private */
    if ( IS_SET(dest->room_flags, ROOM_PRIVATE) )
    {
	count = 0;
	for ( rch = dest->people; rch != NULL; rch = rch->next_in_room )
	    count++;

	if (count > 1)
	{
	    send_to_char( "That room's too crowded already.\n\r", ch );
	    return;
	}
    }

    /* otherwise, gate character and possibly pet */
    if ( ch->pet != NULL && ch->in_room == ch->pet->in_room )
	gate_pet = TRUE;
    else
	gate_pet = FALSE;

    /* do the gate */
    act( "$n steps into a tree and vanishes.", ch, NULL, NULL, TO_ROOM );
    send_to_char( "You step into a tree and vanish.\n\r", ch );
    char_from_room( ch, FALSE );

    char_to_room( ch, dest );
    act("$n has stepped out of a tree.", ch, NULL, NULL, TO_ROOM );
    do_look( ch, "auto" );

    if ( gate_pet )
    {
	act( "$n steps into a tree and vanishes.", ch->pet, NULL, NULL, TO_ROOM );
	send_to_char( "You step into a tree and vanish.\n\r", ch->pet );
	char_from_room( ch->pet, FALSE );
	char_to_room( ch->pet, ch->in_room );
	act( "$n has stepped out of a tree.", ch->pet, NULL, NULL, TO_ROOM );
	do_look( ch->pet, "auto" );

	if ( HAS_TRIGGER(ch->pet, TRIG_ENTRY) )
	    mp_percent_trigger( ch->pet, NULL, NULL, NULL, TRIG_ENTRY );
    }

    if ( IS_NPC(ch) && HAS_TRIGGER(ch, TRIG_ENTRY) )
	mp_percent_trigger( ch, NULL, NULL, NULL, TRIG_ENTRY );
    if ( !IS_NPC(ch) )
	mp_greet_trigger( ch );

    return;
}


void spell_water_speech( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( IS_AFFECTED2(victim, AFF2_WATER_SPEECH) )
    {
    send_to_char( "They can already talk under water.\n\r", ch );
    return;
    }

    af.where        = TO_AFFECTS2;
    af.type        = sn;
    af.level        = level;
    af.duration        = level / 2;
    af.location        = APPLY_NONE;
    af.modifier        = 0;
    af.bitvector    = AFF2_WATER_SPEECH;
    af.procNo        = 0;
    af.flags        = 0;
    af.name        = "";
    af.appearance    = "";
    affect_to_char( victim, &af );

    if ( ch != victim )
    send_to_char( "They can now talk under water.\n\r", ch );
    send_to_char( "You can now talk under water.\n\r", victim );
    return;
}


void spell_water_breathing( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( IS_AFFECTED2(victim, AFF2_WATER_BREATHING) )
    {
    send_to_char( "They can already breath under water.\n\r", ch );
    return;
    }

    af.where        = TO_AFFECTS2;
    af.type        = sn;
    af.level        = level;
    af.duration        = level / 2;
    af.location        = APPLY_NONE;
    af.modifier        = 0;
    af.bitvector    = AFF2_WATER_BREATHING;
    af.procNo        = 0;
    af.flags        = 0;
    af.name        = "";
    af.appearance    = "";
    affect_to_char( victim, &af );

    if ( ch != victim )
    send_to_char( "They can now breath under water.\n\r", ch );
    send_to_char( "You feel a warm tingling through your chest.\n\r", victim );
    return;
}



void spell_water_ways( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( IS_AFFECTED2(victim, AFF2_WATERS_WAYS) )
    {
    send_to_char( "They can already move freely under water.\n\r", ch );
    return;
    }

    af.where        = TO_AFFECTS2;
    af.type        = sn;
    af.level        = level;
    af.duration        = level / 2;
    af.location        = APPLY_NONE;
    af.modifier        = 0;
    af.bitvector    = AFF2_WATERS_WAYS;
    af.procNo        = 0;
    af.flags        = 0;
    af.name        = "";
    af.appearance    = "";
    affect_to_char( victim, &af );

    if ( ch != victim )
    send_to_char( "They can now move freely under water.\n\r", ch );
    send_to_char( "You can now move freely under water.\n\r", victim );
    return;
}


void spell_lightning_storm( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *vch;
    CHAR_DATA *vch_next;
    int dam;

    act( "$n raises $s arms upwards, summoning a storm of lightning bolts.",
        ch, NULL, NULL, TO_ROOM );
    act( "You raise you arms upwards, summoning a storm of lightning bolts.",
        ch, NULL, NULL, TO_CHAR );

    for ( vch = ch->in_room->people; vch != NULL; vch = vch_next)
    {
    vch_next = vch->next_in_room;

    if ( is_safe_spell(ch, vch, TRUE)
    ||   (IS_NPC(ch) && IS_NPC(vch)
    &&    (ch->fighting == vch
    ||     vch->fighting == ch)) )
        continue;

    dam = dice( level, 12 );

    if ( saves_spell(level, vch, DAM_LIGHTNING) )
    {
        damage_old( ch, vch, dam / 2, sn, DAM_LIGHTNING, TRUE, NULL);
    }
    else
    {
        shock_effect( vch, level, dam / 4, TARGET_CHAR );
        damage_old( ch, vch, dam, sn, DAM_LIGHTNING, TRUE, NULL);
    }
    }
}


void spell_raze( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *vch;
    CHAR_DATA *vch_next;
    int dam;

    act( "$n raises $s arms upwards, summoning forth a blazing inferno.",
        ch, NULL, NULL, TO_ROOM );
    act( "You raise you arms upwards, summoning forth a blazing inferno.",
        ch, NULL, NULL, TO_CHAR );

    for ( vch = ch->in_room->people; vch != NULL; vch = vch_next)
    {
    vch_next = vch->next_in_room;

    if ( is_safe_spell(ch, vch, TRUE)
    ||   (IS_NPC(ch) && IS_NPC(vch)
    &&    (ch->fighting == vch
    ||     vch->fighting == ch)) )
        continue;

    dam = dice( level, 12 );

    if ( saves_spell(level, vch, DAM_FIRE) )
    {
        damage_old( ch, vch, dam / 2, sn, DAM_FIRE, TRUE, NULL);
    }
    else
    {
        fire_effect( vch, level, dam / 4, TARGET_CHAR );
        damage_old( ch, vch, dam, sn, DAM_FIRE, TRUE, NULL);
    }
    }
}


void spell_fossilize( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *vch;
    CHAR_DATA *vch_next;
    AFFECT_DATA af;

    for ( vch = ch->in_room->people; vch != NULL; vch = vch_next )
    {
    vch_next = vch->next_in_room;

    if ( is_safe_spell(ch, vch, TRUE)
    ||   (IS_NPC(ch) && IS_NPC(vch)
    &&    (ch->fighting == vch
    ||     vch->fighting == ch)) )
        continue;
    
    if ( is_affected(vch, sn) )
        continue;

    if ( saves_spell(level, vch, DAM_FIRE) )
        continue;

    /* Lets rock! */
    af.where        = TO_AFFECTS;
    af.type         = sn;
    af.level        = ch->level;
    af.duration     = 24;
    af.location     = APPLY_NONE;
    af.modifier     = 0;
    af.bitvector    = AFF_PARALYSIS;
    af.procNo       = 0;
    af.flags        = AFF_PRI_HIGH;
    af.name         = "";
    af.appearance   =
        "A statue of %n stands here, an expression of horror on %s face.";
    affect_to_char( vch, &af );

    stop_fighting( vch, TRUE );

    act( "$n screams in agony as $s flesh turns to stone!",
        vch, NULL, NULL, TO_ROOM );
    send_to_char(
        "A tearing pain shoots through your body as your flesh pretifies.\n\r",
        vch );
    }
}


void spell_absolution( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;

    if ( saves_spell(level, victim, DAM_SLAY) && victim->level <= LEVEL_HERO )
    return;

    act( "$n reaches out and touches you on the forehead sending you into blackness.",
        ch, NULL, victim, TO_VICT );
    act( "$n reaches out and touches $N on the forehead.",
        ch, NULL, victim, TO_NOTVICT );
    act( "You reach out and touch $M on the forehead.",
        ch, NULL, victim, TO_CHAR );

    handle_death( ch, victim, DAM_NONE, FALSE );
}


void spell_ray_of_dissolution( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    OBJ_DATA *obj_next, *obj;

    if ( saves_spell(level, victim, DAM_ENERGY) )
    {
    act( "A thin ray of blackness shoots from $ns finger, missing you narrowly.",
        ch, NULL, victim, TO_VICT );
    act( "A thin ray of blackness shoots from $ns finger, missing $N narrowly.",
        ch, NULL, victim, TO_NOTVICT );
    act( "A thin ray of blackness misses $N narrowly.",
        ch, NULL, victim, TO_CHAR );
    return;
    }

    act( "A thin ray of blackness shoots from $ns finger, hitting YOU!",
        ch, NULL, victim, TO_VICT );
    act( "A thin ray of blackness shoots from $ns finger, hitting $N.",
        ch, NULL, victim, TO_NOTVICT );
    act( "A thin ray of blackness hits $N.",
        ch, NULL, victim, TO_CHAR );

    act( "You disintergrate in a fine spray of blood!",
        ch, NULL, victim, TO_VICT );
    act( "$N dissolves in a fine spray of blood!",
        ch, NULL, victim, TO_NOTVICT );
    act( "$N dissolves in a fine spray of blood!",
        ch, NULL, victim, TO_CHAR );

    for ( obj = victim->carrying; obj != NULL; obj = obj_next )
    {
    obj_next = obj->next_content;

        if ( IS_OBJ_STAT(obj, ITEM_BURN_PROOF)
        ||   IS_OBJ_STAT(obj, ITEM_NOPURGE)
        ||   !IS_OBJ_STAT(obj, ITEM_NONMETAL)
        ||   number_range(0, 3) != 0 )
            continue;

        act( "$p is consumed by a burst of blackness.",
            ch, obj, NULL, TO_ALL );
        extract_obj( obj );
    }

    handle_death( ch, victim, DAM_ENERGY, FALSE );
}


void spell_brittle_bones( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( saves_spell(level, victim, DAM_ENERGY) )
    return;

    act( "A pained look comes over $Ns face.",
    ch, NULL, victim, TO_CHAR );

    af.where            = TO_VULN;
    af.type             = sn;
    af.level            = ch->level;
    af.duration         = 24;
    af.location         = APPLY_NONE;
    af.modifier         = 0;
    af.bitvector        = VULN_BASH;
    af.procNo           = 0;
    af.flags            = 0;
    af.name             = "";
    af.appearance       = "";
    affect_to_char( victim, &af );
}


void spell_smoke_parasite(  int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( saves_spell(level, victim, DAM_NONE) )
    {
    act( "A thin tendril of smoke coils briefly about you before disapating.",
        ch, NULL, victim, TO_VICT );
    act( "A thin tendril of smoke coils briefly about $N before disapating.",
        ch, NULL, victim, TO_NOTVICT );
    act( "A thin tendril of smoke coils briefly about $N before disapating.",
        ch, NULL, victim, TO_CHAR );
    return;
    }

    af.where            = TO_AFFECTS2;
    af.type             = sn;
    af.level            = level;
    af.duration         = -1;
    af.location         = APPLY_STR;
    af.modifier         = -20;
    af.bitvector        = AFF2_SMOKE_PARASITE;
    af.procNo           = affect_proc_lookup("smoke parasite");
    af.flags            = AFF_CONT_UPDATE;
    af.name             = "";
    af.appearance       = "";

    affect_to_char( victim, &af );

    act( "A thin tendril of smoke shoots up your nose!",
        ch, NULL, victim, TO_VICT );
    act( "A thin tendril of smoke shoots up $Ns nose!",
        ch, NULL, victim, TO_NOTVICT );
    act( "A thin tendril of smoke shoots up $Ns nose!",
        ch, NULL, victim, TO_CHAR );
}



void smoke_parasite_affect_proc( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    int dam;

    if ( ch == NULL )
        return;

    if ( reason != AFF_PROC_PERIODIC )
        return;

    if ( ch->in_room == NULL )
        return;

    if ( ch->position == POS_FIGHTING )
        dam = 20;
    else if ( ch->position == POS_STANDING )
        dam = 10;
    else if ( ch->position == POS_RESTING
    ||        ch->position == POS_SITTING )
        dam = 5;
    else
        dam = 2;

    dam = UMIN( dam, ch->hit - 1 );

    if ( number_percent() < dam )
    {
        switch( number_range(0,1) )
        {
            case 0:
                act( "$n starts to cough up smoke!", ch, NULL, NULL, TO_ROOM );
                act( "You start to cough up smoke!", ch, NULL, NULL, TO_CHAR );
                break;
            default:
                act( "A tendril of smoke drifts lazily from $ns mouth.",
                    ch, NULL, NULL, TO_ROOM );
                act( "A tendril of smoke drifts lazily from your mouth.",
                    ch, NULL, NULL, TO_CHAR );
                break;
        }
    }

    damage_old( ch, ch, dam, paf->type, DAM_DISEASE, FALSE, NULL );
}


void spell_spell_steal(  int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( IS_AFFECTED2(victim, AFF2_SPELL_STEAL) )
    {
        if ( ch == victim )
            send_to_char( "You are already affected by this spell.\n\r", ch );
        else
            send_to_char( "They are already affected by this spell.\n\r", ch );
        return;
    }

    if ( IS_AFFECTED2(victim, AFF2_STOLEN_SPELL) )
    {
        if ( ch == victim )
            send_to_char( "You already have a stolen spell stored.\n\r", ch );
        else
            send_to_char( "They already have a stolen spell stored.\n\r", ch );
        return;
    }

    af.where            = TO_AFFECTS2;
    af.type             = sn;
    af.level            = level;
    af.duration         = level / 10;
    af.location         = APPLY_NONE;
    af.modifier         = 0;
    af.bitvector        = AFF2_SPELL_STEAL;
    af.procNo           = 0;
    af.flags            = AFF_INVIS;
    af.name             = "";
    af.appearance       = "";

    affect_to_char( victim, &af );

    send_to_char( "Your body tingles with power as a magical trap is set upon it.\n\r", victim );
    if ( ch != victim )
        send_to_char( "OK.\n\r", ch );
}


void spell_reflection(  int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( IS_AFFECTED2(victim, AFF2_REFLECTION) )
    {
        if ( ch == victim )
            send_to_char( "You are already affected by this spell.\n\r", ch );
        else
            send_to_char( "They are already affected by this spell.\n\r", ch );
        return;
    }

    af.where            = TO_AFFECTS2;
    af.type             = sn;
    af.level            = level;
    af.duration         = level / 15;
    af.location         = APPLY_NONE;
    af.modifier         = 0;
    af.bitvector        = AFF2_REFLECTION;
    af.procNo           = 0;
    af.flags            = AFF_INVIS;
    af.name             = "";
    af.appearance       = "";

    affect_to_char( victim, &af );

    send_to_char( "A reflective magical shield forms about your body.\n\r", ch );

    if ( ch != victim )
    send_to_char( "OK.\n\r", ch );
}


void spell_mana_stabilization(  int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( IS_AFFECTED2(victim, AFF2_STABILIZE_MANA) )
    {
    if ( ch == victim )
        send_to_char( "You are already affected by this spell.\n\r", ch );
    else
        send_to_char( "They are already affected by this spell.\n\r", ch );
    return;
    }

    af.where        = TO_AFFECTS2;
    af.type        = sn;
    af.level        = level;
    af.duration        = level / 15;
    af.location        = APPLY_NONE;
    af.modifier        = 0;
    af.bitvector    = AFF2_STABILIZE_MANA;
    af.procNo        = 0;
    af.flags        = 0;
    af.name        = "";
    af.appearance    = "";

    affect_to_char( victim, &af );

    send_to_char( "The mana has been stabilized against your health.\n\r", ch );

    if ( ch != victim )
    send_to_char( "OK.\n\r", ch );
}


void spell_turn_undead( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *vch;
    CHAR_DATA *vch_next;
    int dam;

    act( "$n raises $s arms upwards, holding $s holy symbol high.",
        ch, NULL, NULL, TO_ROOM );
    act( "You raise your arms upwards, holding your holy symbol high.",
        ch, NULL, NULL, TO_CHAR );

    for ( vch = ch->in_room->people; vch != NULL; vch = vch_next)
    {
    vch_next = vch->next_in_room;

    if ( is_safe_spell(ch, vch, TRUE)
    ||   (IS_NPC(ch) && IS_NPC(vch) ) )
    //&&    (ch->fighting == vch
    //||     vch->fighting == ch)) )
        continue;

    if ( !IS_SET(CHAR_FORM(vch), FORM_UNDEAD) )
        continue;

    if ( saves_spell(level, vch, DAM_HOLY) )
    {
        act( "$n shrugs off your feeble attempt at turning and rushes to attack",
            vch, NULL, NULL, TO_ROOM );
         act( "You shrug off the feeble turn attempt.",
            vch, NULL, NULL, TO_CHAR );
        damage_old( ch, vch, 0, sn, DAM_NONE, TRUE, NULL);
       //do_kill( vch, ch->name );
    }
    else
    {
        act( "$n is overcome by a rush of holy power and attempts to flee in terror.",
            vch, NULL, NULL, TO_ROOM );
        act( "You are overcome by a rush of holy power and try to flee for your very existence.",
            vch, NULL, NULL, TO_CHAR );

        dam = dice( level, 6 );
        damage_old( ch, vch, dam, sn, DAM_HOLY, TRUE, NULL);
        do_flee( vch, "" );
    }
    }

}


void spell_dark_vision( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( IS_AFFECTED(victim, AFF_DARK_VISION) )
    {
        if ( victim == ch )
          send_to_char( "You already have dark vision.\n\r", ch );
        else
          act( "$N already has dark vision.", ch, NULL, victim, TO_CHAR );
        return;
    }

    af.where		= TO_AFFECTS;
    af.type		= sn;
    af.level		= level;
    af.duration		= level;
    af.modifier		= 0;
    af.location		= APPLY_NONE;
    af.bitvector	= AFF_DARK_VISION;
    af.procNo		= 0;
    af.flags		= 0;
    af.name		= "";
    af.appearance	= "";
    affect_to_char( victim, &af );

    send_to_char( "Your glow a deep red.\n\r", victim );
    if ( ch != victim )
	send_to_char( "Ok.\n\r", ch );
    return;
}


void spell_long_eye( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    ROOM_INDEX_DATA *original, *location;
    CHAR_DATA *rider, *mount;
    OBJ_DATA *on;
    int door;

    if ( IS_NULLSTR(target_name) )
    {
        send_to_char( "Scan in which direction?\n\r", ch );
        return;
    }

         if ( !str_cmp( target_name, "n"   ) || !str_cmp( target_name, "north"		  ) ) door = 0;
    else if ( !str_cmp( target_name, "e"   ) || !str_cmp( target_name, "east"		  ) ) door = 1;
    else if ( !str_cmp( target_name, "s"   ) || !str_cmp( target_name, "south"		  ) ) door = 2;
    else if ( !str_cmp( target_name, "w"   ) || !str_cmp( target_name, "west"		  ) ) door = 3;
    else if ( !str_cmp( target_name, "u"   ) || !str_cmp( target_name, "up"		  ) ) door = 4;
    else if ( !str_cmp( target_name, "d"   ) || !str_cmp( target_name, "down"		  ) ) door = 5;
    else if ( !str_cmp( target_name, "ne"  ) || !str_cmp( target_name, "northeast"	  ) ) door = 6;
    else if ( !str_cmp( target_name, "se"  ) || !str_cmp( target_name, "southeast"	  ) ) door = 7;
    else if ( !str_cmp( target_name, "sw"  ) || !str_cmp( target_name, "southwest"	  ) ) door = 8;
    else if ( !str_cmp( target_name, "nw"  ) || !str_cmp( target_name, "northwest"	  ) ) door = 9;
    else if ( !str_cmp( target_name, "cw"  ) || !str_cmp( target_name, "clockwise"	  ) ) door = 10;
    else if ( !str_cmp( target_name, "ccw" ) || !str_cmp( target_name, "counterclockwise" ) ) door = 11;
    else door = -1;

    if ( door < 0
    ||   (location = ch->in_room->exit[door]->u1.to_room) == NULL
    ||   !can_see_room(ch, ch->in_room->exit[door]->u1.to_room) )
    {
        send_to_char( "There doesn't seem to be anything in that direction.\n\r", ch );
        return;
    }

    original        = ch->in_room;
    rider            = ch->mounted_by;
    mount            = ch->riding;
    on                = ch->on;

    char_from_room( ch, TRUE );
    char_to_room( ch, location );

    do_look( ch, "auto" );

    char_from_room( ch, TRUE );
    char_to_room( ch, original );
    ch->mounted_by    = rider;
    ch->riding        = mount;
    ch->on            = on;
}


void spell_detect_illusion( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( IS_AFFECTED2(victim, AFF2_DETECT_ILLUSION) )
    {
        if ( victim == ch )
            send_to_char( "You can already detect illusions.\n\r", ch );
        else
            act( "$N can already detect illusions.", ch, NULL, victim, TO_CHAR );
        return;
    }

    af.where        = TO_AFFECTS2;
    af.type            = sn;
    af.level        = level;
    af.duration        = level;
    af.modifier        = 0;
    af.location        = APPLY_NONE;
    af.bitvector    = AFF2_DETECT_ILLUSION;
    af.procNo        = 0;
    af.flags        = 0;
    af.name            = "";
    af.appearance    = "";
    affect_to_char( victim, &af );

    send_to_char( "Your eyes glow yellow.\n\r", victim );
    if ( ch != victim )
        send_to_char( "Ok.\n\r", ch );
    return;
}


void spell_presence( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    char buf[MAX_STRING_LENGTH];
    CHAR_DATA *victim;
    bool found;
    int door;

    send_to_char( "You scan the immediate area, detecting...\n\r", ch );

    found = FALSE;
    for ( victim = ch->in_room->people; victim != NULL; victim = victim->next_in_room )
    {
        if ( victim == ch )
            continue;

        if ( IS_SET(CHAR_FORM(victim), FORM_SENTIENT)
        &&   get_trust(ch) > victim->invis_level )
        {
            found = TRUE;

            sprintf( buf, "%-28s Right here!\n\r",
                PERS(victim, ch) );
        }
    }

    for ( door = 0; door < 12; door++ )
    {
        EXIT_DATA *pExit = ch->in_room->exit[door];

        if ( pExit == NULL
        ||   pExit->u1.to_room == NULL
        ||   pExit->u1.to_room == ch->in_room
        ||   !can_see_room(ch, pExit->u1.to_room) )
            continue;

        for ( victim = pExit->u1.to_room->people; victim != NULL; victim = victim->next_in_room )
        {
            if ( IS_SET(CHAR_FORM(victim), FORM_SENTIENT)
            &&   get_trust(ch) > victim->invis_level
            &&   get_trust(ch) > victim->incog_level )
            {
                found = TRUE;

                sprintf( buf, "%-28s %s\n\r",
                    PERS(victim, ch),
                    victim->in_room->name );
                send_to_char( buf, ch );
            }
        }
    }

    if ( !found )
        send_to_char( "Nothing.\n\r", ch );
}


char *agg_name( int bit )
{
    switch ( bit )
    {
    case CHAR_CAST_MAGE:	return "essense manipulators";
    case CHAR_CAST_CLERIC:	return "channelers";
    case CHAR_NON_CASTER:	return "non-casters";
    case CHAR_SEMI_CASTER:	return "semi-casters";
    case CHAR_PURE_CASTER:	return "pure casters";
    case CHAR_MALE:		return "men";
    case CHAR_FEMALE:		return "women";
    case CHAR_EUNICH:		return "eunichs";
    case CHAR_GOOD:		return "followers of light";
    case CHAR_NEUTRAL:		return "the neutral";
    case CHAR_EVIL:		return "followers of darkness";
    case CHAR_HUMAN:		return "humans";
    case CHAR_ELF:		return "elves";
    case CHAR_DWARF:		return "dwarves";
    case CHAR_GIANT:		return "giants";
    case CHAR_CENTAUR:		return "centaurs";
    case CHAR_GNOME:		return "gnomes";
    case CHAR_ORC:		return "orcs";
    case CHAR_CAMBION:		return "cambion";
    case CHAR_DEMON:		return "demons";
    default:			return "<???>";
    }
}


CHAR_DATA *get_char_near( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *current;
    int number;
    int count;

    number = number_argument( argument, arg );
    count  = 0;

    if ( !str_cmp(arg, "self") )
        return ch;

    for ( current = char_list; current != NULL; current = current->next )
    {
        bool nearby;
        int door;

        if ( current->in_room == NULL
        ||   ch->in_room == NULL
        ||   ch->in_room->area != current->in_room->area
        ||   !is_name(arg, CHAR_NAME(current)) )
            continue;

        nearby = ch->in_room == current->in_room;
        for ( door = 0; door < 12 && !nearby; door++ )
        {
            EXIT_DATA *pExit = ch->in_room->exit[door];

            if ( pExit == NULL
            ||   !IS_SET(pExit->exit_info, EX_ISDOOR)
            ||   current->in_room != pExit->u1.to_room )
                continue;

            nearby = TRUE;
        }

        if ( ++count == number )
            return current;
    }

    return NULL;
}


void spell_mind_scan( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    char last_buf[MAX_STRING_LENGTH];
    char buf1[MAX_STRING_LENGTH];
    char buf2[MAX_STRING_LENGTH];
    CHAR_DATA *victim = get_char_near( ch, target_name );
    int level_diff;
    char *lev;
    int i;

    if ( victim == NULL
    ||   !IS_SET(CHAR_FORM(victim), FORM_SENTIENT) )
    {
        send_to_char( "You sense nothing.\n\r", ch );
        return;
    }

    if ( saves_spell(level, victim, DAM_MENTAL) )
    {
        send_to_char( "Your probe is blocked.\n\r", ch );
        return;
    }

    level_diff = ch->level - victim->level;
         if ( level_diff >   9 ) lev = "insignificant";
    else if ( level_diff <=  5 ) lev = "minor";
    else if ( level_diff <= -5 ) lev = "equivilant";
    else if ( level_diff <=  9 ) lev = "greater";
    else                         lev = "massive";

    sprintf( buf1, "You sense a presence of %s power.\n\r", lev );
    send_to_char( buf1, ch );

    if ( !IS_NPC(victim)
    ||   !IS_SET(victim->act, ACT_AGGRESSIVE) )
    {
	send_to_char( "They hold no obvious prejudices.\n\r", ch );
        return;
    }

    if ( victim->pIndexData->aggression == 0 )
    {
        sprintf( buf1, "%s will attack anyone of equivilant power or less.\n\r",
            capitalize(CHAR_SHORT(victim)) );
        send_to_char( buf1, ch );
        return;
    }

    last_buf[0]    = '\0';
    buf2[0]    = '\0';

    for ( i = 0;
        !IS_NULLSTR(aggression_flags[i].name) &&
        str_cmp(aggression_flags[i].name, "everyone");
        i++ )
    {
        if ( IS_SET(victim->pIndexData->aggression, aggression_flags[i].bit) )
        {
            if ( last_buf[0] != '\0' )
            {
                if ( buf2[0] != '\0' )
                    strcat( buf2, ", " );
                strcat( buf2, last_buf );
            }

            strcpy( last_buf, agg_name(aggression_flags[i].bit) );
        }
    }

    if ( last_buf[0] != '\0' )
    {
        if ( buf2[0] != '\0' )
            strcat( buf2, " and " );
        strcat( buf2, last_buf );
    }

    sprintf( buf1, "%s will attack %s.\n\r", capitalize(victim->short_descr), buf2 );
    send_to_char( buf1, ch );
}


void spell_bright_moonkiss_touch( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    AFFECT_DATA af;
    int duration;
    int roll;

    if ( IS_NPC(ch)
    ||   IS_AFFECTED(ch, AFF_CALM)
    ||   IS_AFFECTED(ch, AFF_BERSERK)
    ||   is_affected(ch, gsn_bright_moonkiss_touch)
    ||   is_affected(ch, gsn_the_voice_of_nations_calling)
    ||   is_affected(ch, gsn_the_dragon_flight_thunder)
    ||   is_affected(ch, gsn_come_clarion_rage)
    ||   is_affected(ch, gsn_ere_faerie_glamour) )
    {
        send_to_char( "You fail to summon the magic of the song.\n\r", ch );
        return;
    }

    roll = number_percent();
    if ( roll < 95 )
        duration = level / 5;
    else
        duration = 2 * level / 5;

    af.where        = TO_AFFECTS;
    af.type         = sn;
    af.level        = level;
    af.duration     = duration;
    af.location     = APPLY_DAMROLL;
    af.modifier     = -5;
    af.bitvector    = AFF_CALM;
    af.procNo       = affect_proc_lookup( "bright moonkiss touch" );
    af.flags        = AFF_CONT_UPDATE;
    af.name         = "";
    af.appearance   = "";
    affect_to_char( ch, &af );

    af.location        = APPLY_HITROLL;
    af.bitvector    = 0;
    af.procNo        = 0;
    affect_to_char( ch, &af );
    SET_BIT( ch->act, PLR_SINGING );

    send_to_char( "An overwelming sense of calmness fills you as you open yourself to the music.\n\r", ch );
    if ( ch->fighting || ch->position == POS_FIGHTING )
        stop_fighting( ch, FALSE );

    /*
     * Call the affect proc to do initial room calm. This relies on the affect
     * proc not modifying paf.
     */
    bright_moonkiss_touch_affect_proc( ch, NULL, &af, AFF_PROC_PERIODIC );

    return;
}


void bright_moonkiss_touch_affect_proc( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    if ( IS_NPC(ch) || !IS_SET(ch->act, PLR_SINGING) )
        return;

    if ( !check_bard_song( ch, paf->type ) )
        return;

    if ( reason == AFF_PROC_PERIODIC )
    {
        AFFECT_DATA af;
        CHAR_DATA *vch;

        if ( ch->in_room == NULL )
            return;

        act( "$n sings a soft, soothing song, placing you at ease.", ch, NULL, NULL, TO_ROOM );

        af.where        = TO_AFFECTS;
        af.type            = paf->type;
        af.level        = paf->level;
        af.duration        = 2;
        af.modifier     = -3;
        af.bitvector    = AFF_CALM;
        af.procNo        = 0;
        af.flags        = 0;
        af.name            = "";
        af.appearance    = "";

        for ( vch = ch->in_room->people; vch != NULL; vch = vch->next_in_room )
        {
            if ( ch == vch
            ||   IS_AFFECTED(vch, AFF_CALM)
            ||   IS_AFFECTED(vch, AFF_BERSERK)
            ||   is_safe_spell(ch, vch, TRUE)
            ||   saves_spell(paf->level, vch, DAM_NONE) )
                continue;

            af.location    = APPLY_DAMROLL;
            affect_to_char( vch, &af );

            af.location = APPLY_HITROLL;
            affect_to_char( vch, &af );

            send_to_char( "A wave of calm passes over you.\n\r", vch );

            if ( vch->fighting || vch->position == POS_FIGHTING )
                stop_fighting( vch, FALSE );
        }
    }

    return;
}


void spell_the_voice_of_nations_calling( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    AFFECT_DATA af;
    int duration;
    int roll;

    if ( IS_NPC(ch)
    ||   IS_AFFECTED(ch, AFF_CALM)
    ||   IS_AFFECTED(ch, AFF_BERSERK)
    ||   is_affected(ch, gsn_bright_moonkiss_touch)
    ||   is_affected(ch, gsn_the_voice_of_nations_calling)
    ||   is_affected(ch, gsn_the_dragon_flight_thunder)
    ||   is_affected(ch, gsn_come_clarion_rage)
    ||   is_affected(ch, gsn_ere_faerie_glamour) )
    {
        send_to_char( "You fail to summon the magic of the song.\n\r", ch );
        return;
    }

    roll = number_percent();
    if ( roll < 95 )
        duration = level / 5;
    else
        duration = 2 * level / 5;

    af.where        = TO_AFFECTS;
    af.type         = sn;
    af.level        = level;
    af.duration     = duration;
    af.location     = APPLY_DAMROLL;
    af.modifier     = -5;
    af.bitvector    = AFF_CALM;
    af.procNo       = affect_proc_lookup( "the voice of nations calling" );
    af.flags        = AFF_CONT_UPDATE;
    af.name         = "";
    af.appearance   = "";
    affect_to_char( ch, &af );

    af.where        = TO_AFFECTS2;
    af.bitvector    = AFF2_PACIFIST;
    af.location        = APPLY_HITROLL;
    affect_to_char( ch, &af );
    SET_BIT( ch->act, PLR_SINGING );

    send_to_char( "An overwelming sense of peace and harmony fills you as you open yourself to the music.\n\r", ch );
    if ( ch->fighting || ch->position == POS_FIGHTING )
        stop_fighting( ch, FALSE );

    /*
     * Call the affect proc to do initial room calm. This relies on the affect
     * proc not modifying paf.
     */
    the_voice_of_nations_calling_affect_proc( ch, NULL, &af, AFF_PROC_PERIODIC );

    return;
}


void the_voice_of_nations_calling_affect_proc( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    if ( IS_NPC(ch) || !IS_SET(ch->act, PLR_SINGING) )
        return;

    if ( !check_bard_song( ch, paf->type ) )
        return;

    if ( reason == AFF_PROC_PERIODIC )
    {
        AFFECT_DATA af1, af2;
        CHAR_DATA *vch;

        if ( ch->in_room == NULL )
            return;

        act( "$n sings a soft, soothing song, placing you at ease.", ch, NULL, NULL, TO_ROOM );

        af1.where       = TO_AFFECTS;
        af1.type        = paf->type;
        af1.level       = paf->level;
        af1.duration    = 2;
        af1.location    = APPLY_DAMROLL;
        af1.modifier    = -3;
        af1.bitvector   = AFF_CALM;
        af1.procNo      = 0;
        af1.flags       = 0;
        af1.name        = "";
        af1.appearance  = "";

        af2.where       = TO_AFFECTS2;
        af2.type        = paf->type;
        af2.level       = paf->level;
        af2.duration    = 2;
        af2.location    = APPLY_HITROLL;
        af2.modifier    = -3;
        af2.bitvector   = AFF2_PACIFIST;
        af2.procNo      = 0;
        af2.flags       = 0;
        af2.name        = "";
        af2.appearance  = "";

        for ( vch = ch->in_room->people; vch != NULL; vch = vch->next_in_room )
        {
            if ( ch == vch
            ||   IS_AFFECTED(vch, AFF_CALM)
            ||   IS_AFFECTED(vch, AFF_BERSERK)
            ||   is_safe_spell(ch, vch, TRUE)
            ||   saves_spell(paf->level, vch, DAM_NONE) )
                continue;

            affect_to_char( vch, &af1 );
            affect_to_char( vch, &af2 );

            send_to_char( "A sudden feeling of peace and harmony passes over you.\n\r", vch );

            if ( vch->fighting || vch->position == POS_FIGHTING )
                stop_fighting( vch, FALSE );
        }
    }

    return;
}


void spell_the_dragon_flight_thunder( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    AFFECT_DATA af;
    int duration;
    int roll;

    if ( IS_NPC(ch)
    ||   IS_AFFECTED(ch, AFF_CALM)
    ||   IS_AFFECTED(ch, AFF_BERSERK)
    ||   is_affected(ch, gsn_bright_moonkiss_touch)
    ||   is_affected(ch, gsn_the_voice_of_nations_calling)
    ||   is_affected(ch, gsn_the_dragon_flight_thunder)
    ||   is_affected(ch, gsn_come_clarion_rage)
    ||   is_affected(ch, gsn_ere_faerie_glamour) )
    {
        send_to_char( "You fail to summon the magic of the song.\n\r", ch );
        return;
    }

    roll = number_percent();
    if ( roll < 95 )
        duration = level / 5;
    else
        duration = 2 * level / 5;

    af.where        = TO_AFFECTS;
    af.type         = sn;
    af.level        = level;
    af.duration     = duration;
    af.location     = APPLY_NONE;
    af.modifier     = 0;
    af.bitvector    = 0;
    af.procNo       = affect_proc_lookup( "the dragon flight thunder" );
    af.flags        = AFF_CONT_UPDATE;
    af.name         = "";
    af.appearance   = "";
    affect_to_char( ch, &af );
    SET_BIT( ch->act, PLR_SINGING );

    send_to_char( "The thunder of dragon flight overwelms your senses as you open yourself to the music.\n\r", ch );

    /*
     * Call the affect proc to do initial room calm. This relies on the affect
     * proc not modifying paf.
     */
    the_dragon_flight_thunder_affect_proc( ch, NULL, &af, AFF_PROC_PERIODIC );

    return;
}


void the_dragon_flight_thunder_affect_proc( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    if ( IS_NPC(ch) || !IS_SET(ch->act, PLR_SINGING) )
        return;

    if ( !check_bard_song( ch, paf->type ) )
        return;

    if ( reason == AFF_PROC_PERIODIC )
    {
        CHAR_DATA *vch, *vch_next;

        if ( ch->in_room == NULL )
            return;

        act( "$n sings a thunderous song of dragon rage.", ch, NULL, NULL, TO_ROOM );

        for ( vch = ch->in_room->people; vch != NULL; vch = vch_next )
        {
            vch_next = vch->next_in_room;

            if ( ch == vch )
                continue;

            if ( IS_AFFECTED(vch, AFF_CALM)
            ||   IS_AFFECTED(vch, AFF_BERSERK)
            ||   is_safe_spell(ch, vch, TRUE)
            ||   saves_spell(paf->level, vch, DAM_NONE) )
            {
                send_to_char( "You are filled with a vague sense of unease.\n\r", vch );
                continue;
            }

            send_to_char( "A sense of complete terror stabs a chilling blow to your heart.\n\r", vch );

            if ( vch->fighting || vch->position == POS_FIGHTING )
                do_flee( vch, "" );
            else
            {
                ROOM_INDEX_DATA *was_in = vch->in_room;
                int attempt;

                for ( attempt = 0; attempt < 12; attempt++ )
                {
                    EXIT_DATA *pexit;
                    int door;

                    door = number_door();
                    if ( (pexit = was_in->exit[door]) == 0
                    ||   pexit->u1.to_room == NULL
                    ||   IS_SET(pexit->exit_info, EX_CLOSED)
                    ||   (IS_NPC(ch)
                    &&    IS_SET(pexit->u1.to_room->room_flags, ROOM_NO_MOB)) )
                        continue;

                    act( "$N runs from the room, screaming in terror!", ch, NULL, vch, TO_NOTVICT );
                    act( "$N runs from the room, screaming in terror!", ch, NULL, vch, TO_CHAR );
                    move_char( vch, door, FALSE );

                    send_to_char( "You flee from the room!\n\r", vch );
                    break;
                }
            }
        }
    }

    return;
}


void spell_come_clarion_rage( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    AFFECT_DATA af;
    int duration;
    int roll;

    if ( IS_NPC(ch)
    ||   IS_AFFECTED(ch, AFF_CALM)
    ||   IS_AFFECTED(ch, AFF_BERSERK)
    ||   is_affected(ch, gsn_bright_moonkiss_touch)
    ||   is_affected(ch, gsn_the_voice_of_nations_calling)
    ||   is_affected(ch, gsn_the_dragon_flight_thunder)
    ||   is_affected(ch, gsn_come_clarion_rage)
    ||   is_affected(ch, gsn_ere_faerie_glamour) )
    {
        send_to_char( "You fail to summon the magic of the song.\n\r", ch );
        return;
    }

    roll = number_percent();
    if ( roll < 95 )
        duration = level / 5;
    else
        duration = 2 * level / 5;

    af.where            = TO_AFFECTS;
    af.type             = sn;
    af.level            = level;
    af.duration         = duration;
    af.location         = APPLY_DAMROLL;
    af.modifier         = 5;
    af.bitvector        = AFF_BERSERK;
    af.procNo           = affect_proc_lookup( "come clarion rage" );
    af.flags            = AFF_CONT_UPDATE;
    af.name             = "";
    af.appearance       = "";
    affect_to_char( ch, &af );

    af.location         = APPLY_HITROLL;
    af.bitvector        = 0;
    af.procNo           = 0;
    affect_to_char( ch, &af );
    SET_BIT( ch->act, PLR_SINGING );

    send_to_char( "Your blood boils with rage as you open yourself to the music.\n\r", ch );

    /*
     * Call the affect proc to do initial room calm. This relies on the affect
     * proc not modifying paf.
     */
    come_clarion_rage_affect_proc( ch, NULL, &af, AFF_PROC_PERIODIC );

    return;
}


void come_clarion_rage_affect_proc( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    if ( IS_NPC(ch) || !IS_SET(ch->act, PLR_SINGING) )
        return;

    if ( !check_bard_song( ch, paf->type ) )
        return;

    if ( reason == AFF_PROC_PERIODIC )
    {
        AFFECT_DATA af1, af2;
        CHAR_DATA *vch;

        if ( ch->in_room == NULL )
            return;

        af1.where           = TO_AFFECTS;
        af1.type            = paf->type;
        af1.level           = paf->level;
        af1.duration        = 2;
        af1.location        = APPLY_DAMROLL;
        af1.modifier        = 5;
        af1.bitvector       = AFF_BERSERK;
        af1.procNo          = 0;
        af1.flags           = 0;
        af1.name            = "";
        af1.appearance      = "";

        af2.where           = TO_AFFECTS;
        af2.type            = paf->type;
        af2.level           = paf->level;
        af2.duration        = 2;
        af2.location        = APPLY_HITROLL;
        af2.modifier        = 5;
        af2.bitvector       = 0;
        af2.procNo          = 0;
        af2.flags           = 0;
        af2.name            = "";
        af2.appearance      = "";

        act( "$n sings a stirring song of battle.", ch, NULL, NULL, TO_ROOM );

        for ( vch = ch->in_room->people; vch != NULL; vch = vch->next_in_room )
        {
            if ( ch == vch
            ||   IS_AFFECTED(vch, AFF_CALM)
            ||   IS_AFFECTED(vch, AFF_BERSERK)
            ||   is_safe_spell(ch, vch, TRUE)
            ||   saves_spell(paf->level, vch, DAM_NONE) )
                continue;

            send_to_char( "Your blood boils with rage!\n\r", vch );

            affect_to_char( vch, &af1 );
            affect_to_char( vch, &af2 );
        }
    }

    return;
}


void spell_ere_faerie_glamour( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    AFFECT_DATA af;
    int duration;
    int roll;

    if ( IS_NPC(ch)
    ||   IS_AFFECTED(ch, AFF_BERSERK)
    ||   is_affected(ch, gsn_bright_moonkiss_touch)
    ||   is_affected(ch, gsn_the_voice_of_nations_calling)
    ||   is_affected(ch, gsn_the_dragon_flight_thunder)
    ||   is_affected(ch, gsn_come_clarion_rage)
    ||   is_affected(ch, gsn_ere_faerie_glamour) )
    {
        send_to_char( "You fail to summon the magic of the song.\n\r", ch );
        return;
    }

    roll = number_percent();
    if ( roll < 95 )
        duration = level / 5;
    else
        duration = 2 * level / 5;

    af.where        = TO_AFFECTS;
    af.type         = sn;
    af.level        = level;
    af.duration     = duration;
    af.location     = APPLY_NONE;
    af.modifier     = 0;
    af.bitvector    = 0;
    af.procNo       = affect_proc_lookup( "ere faerie glamour" );
    af.flags        = AFF_CONT_UPDATE;
    af.name         = "";
    af.appearance   = "";
    affect_to_char( ch, &af );
    SET_BIT( ch->act, PLR_SINGING );

    send_to_char( "You feel the fae glamour flowing through your veins.\n\r", ch );

    /*
     * Call the affect proc to do initial room calm. This relies on the affect
     * proc not modifying paf.
     */
    ere_faerie_glamour_affect_proc( ch, NULL, &af, AFF_PROC_PERIODIC );

    return;
}


void ere_faerie_glamour_affect_proc( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    if ( IS_NPC(ch) || !IS_SET(ch->act, PLR_SINGING) )
        return;

    if ( !check_bard_song( ch, paf->type ) )
        return;

    if ( reason == AFF_PROC_PERIODIC )
    {
        AFFECT_DATA af;
        CHAR_DATA *vch;

        if ( ch->in_room == NULL )
            return;

        af.where        = TO_AFFECTS;
        af.type         = paf->type;
        af.level        = paf->level;
        af.duration     = 2;
        af.location     = APPLY_NONE;
        af.modifier     = 0;
        af.bitvector    = AFF_CHARM;
        af.procNo       = 0;
        af.flags        = 0;
        af.name         = "";
        af.appearance   = "";

        act( "$n sings slow song of love.", ch, NULL, NULL, TO_ROOM );

        for ( vch = ch->in_room->people; vch != NULL; vch = vch->next_in_room )
        {
            if ( ch == vch
            ||   IS_AFFECTED(vch, AFF_BERSERK)
            ||   IS_AFFECTED(vch, AFF_CHARM)
            ||   IS_SET(CHAR_IMM(vch), IMM_CHARM)
            ||   IS_SET(CHAR_FORM(vch), FORM_UNDEAD)
            ||   IS_SET(CHAR_FORM(vch), FORM_CONSTRUCT)
            ||   IS_SET(CHAR_FORM(vch), FORM_MIST)
            ||   IS_SET(CHAR_FORM(vch), FORM_INTANGIBLE)
            ||   vch->level > LEVEL_HERO
            ||   is_safe_spell(ch, vch, TRUE)
            ||   saves_spell(paf->level, vch, DAM_NONE) )
                continue;

            act( "Your heart is filled with longing for $n.", ch, NULL, vch, TO_VICT );
            act( "$N looks at you with adoring eyes.", ch, NULL, vch, TO_CHAR );

            affect_to_char( vch, &af );

            if ( vch->master )
                stop_follower( vch );
            add_follower( vch, ch );
            vch->leader = ch;
        }
    }

    return;
}


void spell_capture( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    char buf[MAX_STRING_LENGTH];
    ROOM_INDEX_DATA *jail;
    bool captainCapture = FALSE;

    if ( saves_spell(level, victim, DAM_NONE) )
    {
        act( "$N shakes off your attempted capture.", ch, NULL, victim, TO_CHAR );
        return;
    }

    if ( !is_criminal(victim, TRUE) )
    {
        // Check if ch is the captain of the guard here and that the victim is a PC.
        if ( !IS_NPC(victim)
        &&   has_territory_flag(ch, ch->in_room->area->territory, TERR_CAPTAIN_GUARD) )
        {
            // If the victim has social class > 0 then the captain can't capture them.
            if ( victim->pcdata->social_classes[ch->in_room->area->territory] > 0 )
            {
                send_to_char( "You can't capture them!\n\r", ch );
                return;
            }
            captainCapture = TRUE;
        }
        // Else ch isn't the captain so they must be a justicar.
        // The victim needs to be a criminal in this case.
        else
        {
            send_to_char( "They are not even a criminal!\n\r", ch );
            return;
        }
    }

    if ( !IS_SET(territories[ch->in_room->area->territory].crimes, TERRITORY_JUSTICAR_ALIGNED)
    &&   !captainCapture )
    {
        send_to_char( "Your magic has no power here!\n\r", ch );
        return;
    }

    if ( territories[ch->in_room->area->territory].jail == 0
    ||   (jail = get_room_index(territories[ch->in_room->area->territory].jail)) == NULL )
    {
        send_to_char( "This territory has no jail in which to hold them!\n\r", ch );
        return;
    }

    if ( jail == victim->in_room )
    {
        send_to_char( "They are already in jail!\n\r", ch );
        return;
    }

    if ( victim != ch )
        send_to_char( "You have been captured!\n\r", victim );

    act( "$n vanishes!", victim, NULL, NULL, TO_ROOM );
    char_from_room( victim, TRUE );
    char_to_room( victim, jail );
    act( "$n slowly fades into existence.", victim, NULL, NULL, TO_ROOM );
    do_look( victim, "auto" );

    if ( IS_NPC(ch) )
    {
        sprintf( buf, "%s has been captured by $N.", victim->name );
        wiznet( buf, ch, NULL, WIZ_MOBACTIONS, 0, 0 );
    }

    if ( IS_NPC(victim) && HAS_TRIGGER(victim, TRIG_ENTRY) )
        mp_percent_trigger( victim, NULL, NULL, NULL, TRIG_ENTRY );
    if ( !IS_NPC(victim) )
        mp_greet_trigger( victim );
}


CRIME_DATA *get_crime( CHAR_DATA *ch, CHAR_DATA *victim, char *argument )
{
    char crimeName[MAX_INPUT_LENGTH], victimName[MAX_INPUT_LENGTH];
    CRIME_DATA *crime;
    int crimeFlag;

    argument = one_argument( argument, crimeName );
    argument = one_argument( argument, victimName );

    if ( crimeName[0] == '\0' )
    {
        send_to_char( "You need to specify a crime (\"assault\" or \"theft\").\n\r", ch );
        return NULL;
    }

    if ( !str_prefix(crimeName, "assault") )
        crimeFlag = CRIME_PKILLING;
    else if ( !str_prefix(crimeName, "theft") )
        crimeFlag = CRIME_PSTEALING;
    else
    {
        send_to_char( "That is not a valid crime. Try either \"assault\" or \"theft\"\n\r", ch );
        return NULL;
    }

    if ( victimName[0] == '\0' )
    {
        send_to_char( "Who was the victim of the crime?\n\r", ch );
        return NULL;
    }

    for ( crime = victim->pcdata->crimes[victim->in_room->area->territory];
          crime != NULL;
          crime = crime->next )
    {
        if ( IS_SET(crime->crime, crimeFlag)
        &&   !str_cmp(crime->victim, victimName) )
            return crime;
    }

    send_to_char( "They have not commited that crime!\n\r", ch );
    return NULL;
}


void remove_crime( CHAR_DATA *victim, CRIME_DATA *crime, int territory )
{
    CRIME_DATA *current;

    current = victim->pcdata->crimes[territory];

    if ( current == crime )
    {
        victim->pcdata->crimes[territory] = current->next;
        free_crime( current );
    }
    else
    {
        for ( ; current != NULL; current = current->next )
        {
            if ( current->next == NULL )
            {
            bug( "remove_crime: crime not found in crime list.", 0 );
            return;
            }

            if ( current->next == crime )
            {
            current->next = crime->next;
            free_crime( crime );
            return;
            }
        }
    }
}


void spell_execute( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    char arg[MAX_STRING_LENGTH];
    CHAR_DATA *victim;
    CRIME_DATA *crime;

    target_name = one_argument( target_name, arg );
    if ( arg[0] == '\0' ) 
    {
        send_to_char( "Execute whom?\n\r", ch );
        return;
    }
    if ( (victim = get_char_room(ch, arg)) == NULL )
    {
        send_to_char( "They're not here.\n\r", ch );
        return;
    }

    if ( !IS_SET(victim->in_room->room_flags, ROOM_COURT) )
    {
        send_to_char( "This isn't a court of law!\n\r", ch );
        return;
    }

    if ( !is_criminal(victim, FALSE) )
    {
        send_to_char( "They are not a criminal here!\n\r", ch );
        return;
    }

    if ( (crime = get_crime(ch, victim, target_name)) == NULL )
        return;

    decapitate( victim, ch );

    act( "$n vanishes!", victim, NULL, NULL, TO_ROOM );
    char_from_room( victim, TRUE );
    char_to_room( victim, ch->in_room );
    act( "$n slowly fades into existence.", victim, NULL, NULL, TO_ROOM );
    send_to_char( "You are drawn back to the place of your execution.\n\r", victim );
    do_look( victim, "auto" );

    remove_crime( victim, crime, victim->in_room->area->territory );
}


void spell_fine( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    char amount_buf[MAX_INPUT_LENGTH];
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    CRIME_DATA *crime;
    int fine, amount;

    target_name = one_argument( target_name, arg );
    if ( arg[0] == '\0' ) 
    {
        send_to_char( "Execute whom?\n\r", ch );
        return;
    }
    if ( (victim = get_char_room(ch, arg)) == NULL )
    {
        send_to_char( "They're not here.\n\r", ch );
        return;
    }

    if ( !IS_SET(victim->in_room->room_flags, ROOM_COURT) )
    {
        send_to_char( "This isn't a court of law!\n\r", ch );
        return;
    }

    if ( !is_criminal(victim, FALSE) )
    {
        send_to_char( "They are not a criminal here!\n\r", ch );
        return;
    }

    target_name = one_argument( target_name, amount_buf );
    if ( amount_buf[0] == '\0' || !is_number(amount_buf) )
    {
        send_to_char( "Fine them how much?\n\r", ch );
        return;
    }

    fine = amount = atoi( amount_buf );

    if ( (crime = get_crime(ch, victim, target_name)) == NULL )
        return;

    if ( amount >= victim->silver )
    {
        amount -= victim->silver;
        victim->silver = 0;

        if ( amount >= victim->gold * 100 )
        {
            amount -= victim->gold * 100;
            victim->gold = 0;

            if ( amount >= victim->pcdata->balance )
            {
            amount -= victim->pcdata->balance;
            victim->pcdata->balance = 0;
            }
            else
            {
            victim->pcdata->balance -= amount;
            amount = 0;
            }
        }
        else
        {
            victim->gold   -= (amount / 100) + 1;
            victim->silver += ((amount / 100) + 1) * 100;
            victim->silver -= amount;
            amount = 0;
        }
    }
    else
    {
        victim->silver -= amount;
        amount = 0;
    }

    send_to_char( "You are stripped of your wealth!\n\r", victim );
    if ( amount == 0 )
        send_to_char( "You strip them of their wealth!\n\r", ch );
    else
    {
        sprintf( buf, "You only obtain %d silver coins from them!\n\r", fine - amount );
        send_to_char( buf, ch );
    }

    remove_crime( victim, crime, victim->in_room->area->territory );
}


void spell_confiscate( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    OBJ_DATA *obj_lose, *obj_next;
    char arg[MAX_INPUT_LENGTH];
    CRIME_DATA *crime;

    target_name = one_argument( target_name, arg );
    if ( arg[0] == '\0' ) 
    {
        send_to_char( "Confiscate equipment from whom?\n\r", ch );
        return;
    }
    if ( (victim = get_char_room(ch, arg)) == NULL )
    {
        send_to_char( "They're not here.\n\r", ch );
        return;
    }

    if ( !IS_SET(victim->in_room->room_flags, ROOM_COURT) )
    {
        send_to_char( "This isn't a court of law!\n\r", ch );
        return;
    }

    if ( !is_criminal(victim, FALSE) )
    {
        send_to_char( "They are not a criminal here!\n\r", ch );
        return;
    }

    if ( (crime = get_crime(ch, victim, target_name)) == NULL )
        return;

    send_to_char( "An overwelming compulsion compels you to drop all off your possessions!\n\r", ch );

    for ( obj_lose = victim->carrying;
          obj_lose != NULL;
          obj_lose = obj_next)
    {
        obj_next = obj_lose->next_content;

        if ( obj_lose->wear_loc != -1 )
        {
            unequip_char( victim, obj_lose );
            act( "$n removes $p and throws it to the ground!",
            victim, obj_lose, NULL, TO_ROOM );
            act( "You remove and drop $p before it burns you.",
            victim, obj_lose, NULL, TO_CHAR );
        }
        else
        {
            act( "$n throws $p to the ground!",
                    victim, obj_lose, NULL, TO_ROOM );
            act( "You drop $p.",
                victim, obj_lose, NULL, TO_CHAR );
        }

        obj_from_char( obj_lose );
        obj_to_room( obj_lose, victim->in_room );
    }

    remove_crime( victim, crime, victim->in_room->area->territory );
}


void spell_extradite( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    char arg[MAX_INPUT_LENGTH];
    ROOM_INDEX_DATA *jail;
    int territory;

    target_name = one_argument( target_name, arg );
    if ( arg[0] == '\0' ) 
    {
        send_to_char( "Extradite whom?\n\r", ch );
        return;
    }
    if ( (victim = get_char_room(ch, arg)) == NULL )
    {
        send_to_char( "They're not here.\n\r", ch );
        return;
    }

    if ( !IS_SET(victim->in_room->room_flags, ROOM_COURT) )
    {
        send_to_char( "This isn't a court of law!\n\r", ch );
        return;
    }

    if ( !is_criminal(victim, TRUE) )
    {
        send_to_char( "They are not even a criminal!\n\r", ch );
        return;
    }

    if ( !IS_SET(territories[ch->in_room->area->territory].crimes, TERRITORY_JUSTICAR_ALIGNED) )
    {
        send_to_char( "Your magic has no power here!\n\r", ch );
        return;
    }

    if ( target_name[0] == '\0' )
    {
        send_to_char( "Extradite them to which territory?\n\r", ch );
        return;
    }

    territory = territory_lookup( target_name );
    if ( territory == -1 || territory == 0 )
    {
        send_to_char( "That is not a territory.\n\r", ch );
        return;
    }

    if ( !IS_SET(territories[territory].crimes, TERRITORY_JUSTICAR_ALIGNED) )
    {
        send_to_char( "Your magic has no power there!\n\r", ch );
        return;
    }

    if ( territories[territory].jail == 0
    ||   (jail = get_room_index(territories[territory].jail)) == NULL )
    {
        send_to_char( "That territory has no jail in which to hold them!\n\r", ch );
        return;
    }

    if ( victim != ch )
        send_to_char( "You have been extradited!\n\r", victim );

    act( "$n vanishes!", victim, NULL, NULL, TO_ROOM );
    char_from_room( victim, TRUE );
    char_to_room( victim, jail );
    act( "$n slowly fades into existence.", victim, NULL, NULL, TO_ROOM );
    do_look( victim, "auto" );

    if ( IS_NPC(ch) )
    {
        char buf[MAX_STRING_LENGTH];
        sprintf( buf, "%s has been extradited by $N.", victim->name );
        wiznet( buf, ch, NULL, WIZ_MOBACTIONS, 0, 0 );
    }

    if ( IS_NPC(victim) && HAS_TRIGGER(victim, TRIG_ENTRY) )
        mp_percent_trigger( victim, NULL, NULL, NULL, TRIG_ENTRY );
    if ( !IS_NPC(victim) )
        mp_greet_trigger( victim );
}


void spell_mirror_image( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;
    int images, i;

    if ( IS_AFFECTED2(victim, AFF2_MIRROR_IMAGE) )
    {
        if ( victim == ch )
            send_to_char( "You already have mirror images.\n\r", ch );
        else
            send_to_char( "They already have mirror images.\n\r", ch );
        return;
    }

    images = number_range( 1, 4 ) + level / 15;

    af.where            = TO_AFFECTS2;
    af.type             = sn;
    af.level            = level;
    af.duration         = level / 10;
    af.location         = APPLY_NONE;
    af.modifier         = images;
    af.bitvector        = AFF2_MIRROR_IMAGE;
    af.procNo           = 0;
    af.flags            = 0;
    af.name             = "";
    af.appearance       = "";
    affect_to_char( ch, &af );

    for ( i = 0; i < images; i++ )
        act( "$n appears in a puff of yellowish green smoke.", victim, NULL, NULL, TO_ROOM );
    send_to_char( "OK.\n\r", ch );
}


void spell_delude( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( IS_AFFECTED2(victim, AFF2_DELUDE) )
    {
        if ( victim == ch )
            send_to_char( "Your aura is already masked.\n\r", ch );
        else
            send_to_char( "Their aura is already masked.\n\r", ch );
        return;
    }

    af.where            = TO_AFFECTS2;
    af.type             = sn;
    af.level            = level;
    af.duration         = level / 5;
    af.location         = APPLY_NONE;
    af.modifier         = 0;
    af.bitvector        = AFF2_DELUDE;
    af.procNo           = 0;
    af.flags            = 0;
    af.name             = "";
    af.appearance       = "";
    affect_to_char( ch, &af );

    send_to_char( "A faint tingle runs through your body.\n\r", victim );
    if ( ch != victim )
	send_to_char( "OK.\n\r", ch );
}


void spell_improved_invisibility( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( IS_AFFECTED2(victim, AFF2_IMPROVED_INVIS) )
    {
        if ( victim == ch )
            send_to_char( "You are already invisible.\n\r", ch );
        else
            send_to_char( "Their are already invisible.\n\r", ch );
        return;
    }

    af.where            = TO_AFFECTS2;
    af.type             = sn;
    af.level            = level;
    af.duration         = level / 15;
    af.location         = APPLY_NONE;
    af.modifier         = 0;
    af.bitvector        = AFF2_IMPROVED_INVIS;
    af.procNo           = 0;
    af.flags            = 0;
    af.name             = "";
    af.appearance       = "";
    affect_to_char( ch, &af );

    send_to_char( "You fade from view.\n\r", victim );
    act( "$N fades from view.", ch, NULL, victim, TO_NOTVICT );
    if ( ch != victim )
	send_to_char( "OK.\n\r", ch );
}


void spell_alkar( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( IS_AFFECTED2(victim, AFF2_ALKAR) )
    {
        if ( victim == ch )
            send_to_char( "An aura of power already surrounds you.\n\r", ch );
        else
            send_to_char( "They are already surrounded by an aura of power.\n\r", ch );
        return;
    }

    af.where            = TO_AFFECTS2;
    af.type             = sn;
    af.level            = level;
    af.duration         = level / 4;
    af.location         = APPLY_NONE;
    af.modifier         = level / 10;
    af.bitvector        = AFF2_ALKAR;
    af.procNo           = 0;
    af.flags            = 0;
    af.name             = "";
    af.appearance       = "";
    affect_to_char( ch, &af );

    send_to_char( "You are surrounded by an aura of power.\n\r", victim );
    act( "$N is surrounded by an aura of power.", ch, NULL, victim, TO_NOTVICT );
    if ( ch != victim )
	send_to_char( "OK.\n\r", ch );
}


void spell_embalm( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *part = (OBJ_DATA *) vo;

    if ( part->pIndexData->vnum != OBJ_VNUM_SEVERED_HEAD
    &&   part->pIndexData->vnum != OBJ_VNUM_TORN_HEART
    &&   part->pIndexData->vnum != OBJ_VNUM_SLICED_ARM
    &&   part->pIndexData->vnum != OBJ_VNUM_SLICED_LEG
    &&   part->pIndexData->vnum != OBJ_VNUM_GUTS
    &&   part->pIndexData->vnum != OBJ_VNUM_BRAINS )
    {
        send_to_char( "You can only preserve body parts.\n\r", ch );
        return;
    }

    part->timer = -1;

    act( "$p briefly glows with a faint purplish light!", ch, part, NULL, TO_CHAR );
    act( "$p briefly glows with a faint purplish light!", ch, part, NULL, TO_ROOM );

    ch->alignment = UMAX( -1000, ch->alignment - 10 );
}


void spell_death_trance( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( IS_AFFECTED2(victim, AFF2_DEATH_TRANCE) || is_affected(victim, sn) )
    {
        if ( victim == ch )
            send_to_char( "You are already in a death trance.\n\r", ch );
        else
            send_to_char( "They are already in a death trance.\n\r", ch );
        return;
    }

    af.where            = TO_AFFECTS2;
    af.type             = sn;
    af.level            = level;
    af.duration         = level / 5;
    af.location         = APPLY_NONE;
    af.modifier         = 0;
    af.bitvector        = AFF2_DEATH_TRANCE;
    af.procNo           = 0;
    af.flags            = 0;
    af.name             = "";
    af.appearance       = "";
    affect_to_char( ch, &af );

    send_to_char( "You are protected from death.\n\r", victim );
    if ( ch != victim )
	send_to_char( "OK.\n\r", ch );

    ch->alignment = UMAX( -1000, ch->alignment - 10 );
}


void death_trance_affect_proc( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    if ( ch == NULL )
        return;

    if ( reason == AFF_PROC_PERIODIC )
    {
        if ( ch->hit > 0 )
	{
	    affect_strip( ch, paf->type );
	    do_wake( ch, "" );
	}

        return;
    }
    else if ( reason == AFF_PROC_FINISHED )
    {
        damage_old( ch, ch, 2 * paf->level, gsn_death_trance, DAM_DISEASE, FALSE, NULL );
    }
}


void spell_control_undead( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if (is_safe(ch,victim)) return;

    if ( victim == ch )
    {
	send_to_char( "You like yourself even better!\n\r", ch );
	return;
    }
    
    if ( IS_AFFECTED(victim, AFF_CHARM)
    ||   IS_AFFECTED(ch, AFF_CHARM)
    ||   level < victim->level
    ||   IS_SET(CHAR_IMM(victim),IMM_CHARM)
    ||   !IS_SET(CHAR_FORM(victim), FORM_UNDEAD)
    ||   saves_spell( level, victim, DAM_CHARM) )
	return;

    if ( IS_SET(victim->in_room->room_flags, ROOM_LAW) )
    {
	send_to_char(
	    "The mayor does not allow charming in the city limits.\n\r",ch);
	return;
    }
  
    if ( victim->master )
	stop_follower( victim );
    add_follower( victim, ch );
    victim->leader = ch;
    
    af.where		= TO_AFFECTS;
    af.type		= sn;
    af.level		= level;
    af.duration		= number_fuzzy( level / 2 );
    af.location		= 0;
    af.modifier		= 0;
    af.bitvector	= AFF_CHARM;
    af.procNo		= 0;
    af.flags		= 0;
    af.name		= "";
    af.appearance	= "";
    affect_to_char( victim, &af );
    act( "Isn't $n just so nice?", ch, NULL, victim, TO_VICT );
    if ( ch != victim )
	act("$N looks at you with hatred, trying to break your control.",ch,NULL,victim,TO_CHAR);
    return;
}


void spell_corpse_form( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( is_affected(victim, sn) )
    {
	if ( ch == victim )
	    send_to_char( "You already possess the form of a corpse!\n\r", ch );
	else
	    send_to_char( "They already possess the form of a corpse!\n\r", ch );
	return;
    }
    
    af.where		= TO_AFFECTS2;
    af.type	        = sn;
    af.level		= level;
    af.duration		= level;
    af.location		= APPLY_CHA;
    af.modifier		= level / 10;
    af.bitvector	= AFF2_CORPSE_FORM;
    af.procNo		= 0;
    af.flags		= AFF_PRI_LOW;
    af.name	        = "";
    af.appearance	= "The corpse of %n %p.";
    affect_to_char( victim, &af );

    send_to_char( "OK.\n\r", ch );
}


void spell_distraction( int sn, int level, CHAR_DATA *ch, void *vo,int target)
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( is_affected(victim, sn) || saves_spell(level, victim, DAM_MENTAL) )
	return;

    af.where		= TO_AFFECTS;
    af.type		= sn;
    af.level		= level;
    af.duration		= level * 50 / get_curr_stat( victim, STAT_WIS );
    af.location		= APPLY_CHA;
    af.modifier		= -2 * (level / 5);
    af.bitvector	= 0;
    af.procNo	 	= 0;
    af.flags	 	= AFF_INVIS;
    af.name		= "";
    af.appearance	= "";
    affect_to_char( victim, &af );

    af.location         = APPLY_WIS;
    affect_to_char( victim, &af );

    send_to_char( "You feel a strange itch on the back of your neck, as if someone were watching you.\n\r", victim );
    act( "$n suddenly looks all around, a suspicious look on $s face.", victim, NULL, NULL, TO_ROOM );
}


void spell_numb( int sn, int level, CHAR_DATA *ch, void *vo, int target)
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( is_affected(victim, sn) || saves_spell(level, victim, DAM_MENTAL) )
	return;

    af.where		= TO_AFFECTS;
    af.type		= sn;
    af.level		= level;
    af.duration		= level * 50 / get_curr_stat( victim, STAT_WIS );
    af.location		= APPLY_STR;
    af.modifier		= -2 * (level / 5);
    af.bitvector	= 0;
    af.procNo	 	= 0;
    af.flags	 	= AFF_INVIS;
    af.name		= "";
    af.appearance	= "";
    affect_to_char( victim, &af );

    send_to_char( "You feel a sudden numbness in your limbs.\n\r", victim );
    act( "$n hesitates suddenly, $s movements becoming leaden and weak.", victim, NULL, NULL, TO_ROOM );
}


void spell_eye_bite( int sn, int level, CHAR_DATA *ch, void *vo, int target)
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( is_affected(victim, sn) 
    ||   IS_AFFECTED(victim, AFF_BLIND) 
    ||   saves_spell(level, victim, DAM_MENTAL) )
	return;

    af.where		= TO_AFFECTS;
    af.type		= sn;
    af.level		= level;
    af.duration		= level * 25 / get_curr_stat( victim, STAT_WIS );
    af.location		= 0;
    af.modifier		= 0;
    af.bitvector	= AFF_BLIND;
    af.procNo	 	= 0;
    af.flags	 	= AFF_INVIS;
    af.name		= "";
    af.appearance	= "";
    affect_to_char( victim, &af );

    send_to_char( "You feel a sudden stabbing pain in the back of your eyes.\n\r"
                  "Everything turns black.\n\r", victim );
    act( "A agonized look crosses $ns face, $s eyes rolling up until only the whites are showing.", victim, NULL, NULL, TO_ROOM );
}


void spell_lethargy( int sn, int level, CHAR_DATA *ch, void *vo, int target)
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( is_affected(victim, sn) 
    ||   saves_spell(level, victim, DAM_MENTAL) )
	return;

    af.where		= TO_AFFECTS;
    af.type		= sn;
    af.level		= level;
    af.duration		= level * 50 / get_curr_stat( victim, STAT_WIS );
    af.location		= APPLY_DEX;
    af.modifier		= -2 * (level / 5);
    af.bitvector	= AFF_SLOW;
    af.procNo	 	= 0;
    af.flags	 	= AFF_INVIS;
    af.name		= "";
    af.appearance	= "";
    affect_to_char( victim, &af );

    send_to_char( "You are filled with an overwelming feeling of lethargy.\n\r", victim );
    act( "$n suddenly starts to move in exagerated slow motion.", victim, NULL, NULL, TO_ROOM );
}


void spell_embed( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    char arg[MAX_INPUT_LENGTH];
    CRIME_DATA *crime;
    AFFECT_DATA af;
    int days;

    target_name = one_argument( target_name, arg );
    if ( arg[0] == '\0' ) 
    {
        send_to_char( "Embed whom in rock?\n\r", ch );
        return;
    }
    if ( (victim = get_char_room(ch, arg)) == NULL )
    {
        send_to_char( "They're not here.\n\r", ch );
        return;
    }

    if ( !IS_SET(victim->in_room->room_flags, ROOM_COURT) )
    {
        send_to_char( "This isn't a court of law!\n\r", ch );
        return;
    }

    if ( !is_criminal(victim, FALSE) )
    {
        send_to_char( "They are not a criminal here!\n\r", ch );
        return;
    }

    target_name = one_argument( target_name, arg );
    if ( arg[0] == '\0' || !is_number(arg) )
    {
        send_to_char( "Embed them in rock for how many days?", ch );
        return;
    }
    if ( (days = atoi(arg)) == 0 )
    {
        send_to_char( "The duration must be either a positive integer or -1 (forever).\n\r", ch );
        return;
    }
    if ( days < -1 )
        days = -1;

    if ( (crime = get_crime(ch, victim, target_name)) == NULL )
        return;

    af.where		= TO_AFFECTS;
    af.type		= sn;
    af.level		= level;
    af.duration		= 3600 * 24 * days * PULSE_PER_SECOND / PULSE_TICK;
    af.location		= APPLY_NONE;
    af.modifier		= 0;
    af.bitvector	= AFF_PARALYSIS;
    af.flags	 	= AFF_INVIS | AFF_CONT_UPDATE;
    af.procNo           = affect_proc_lookup( "embed" );
    af.name		= "";
    af.appearance	= "";
    affect_to_char( victim, &af );

    act( "A shimmering field of blackness envelops $n.", victim, NULL, NULL, TO_ROOM );
    send_to_char( "A shimmering field of blackness envelops you.\n\r", victim );
    char_from_room( victim, TRUE );
    char_to_room( victim, get_room_index(ROOM_VNUM_EMBED) );

    remove_crime( victim, crime, victim->in_room->area->territory );
}


void embed_affect_proc( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    if ( reason == AFF_PROC_FINISHED )
    {
        char_from_room( ch, TRUE );
        char_to_room( ch, get_room_index(ROOM_VNUM_TEMPLE) );
        do_wake( ch, "" );
    }
}


void spell_hallucinate( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( IS_AFFECTED2(victim, AFF2_HALLUCINATION) )
    {
        if ( victim == ch )
            send_to_char( "You're already hallucinating.\n\r", ch );
        else
            send_to_char( "They're already hallucinating.\n\r", ch );
	return;
    }

    if ( saves_spell(level, victim, DAM_CHARM) )
    {
        send_to_char( "Nothing seems to happen.\n\r", ch );
        return;
    }

    /* otherwise, store current location */
    af.where		= TO_AFFECTS2;
    af.type		= sn;
    af.level		= level;
    af.duration		= level / 2;
    af.location		= APPLY_NONE;
    af.modifier		= 0;
    af.bitvector	= AFF2_HALLUCINATION;
    af.procNo		= 0;
    af.flags		= 0;
    af.name		= "";
    af.appearance	= "";
    affect_to_char( victim, &af );

    send_to_char( "Your perspective on things suddenly seems radically different.\n\r", victim );
    return;
}
